@@ -3,24 +3,22 @@
    Free Software Foundation, Inc.
    Copyright (C) 1995 Board of Trustees, University of Illinois.
    Copyright (C) 1998, 1999 J. Kean Johnston.
-   Copyright (C) 2001, 2002 Ben Wing.
+   Copyright (C) 2001, 2002, 2010 Ben Wing.
 
 This file is part of XEmacs.
 
-XEmacs is free software; you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 2, or (at your option)
-any later version.
+XEmacs is free software: you can redistribute it and/or modify it
+under the terms of the GNU General Public License as published by the
+Free Software Foundation, either version 3 of the License, or (at your
+option) any later version.
 
-XEmacs is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
+XEmacs is distributed in the hope that it will be useful, but WITHOUT
+ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
+for more details.
 
 You should have received a copy of the GNU General Public License
-along with XEmacs; see the file COPYING.  If not, write to
-the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-Boston, MA 02111-1307, USA.  */
+along with XEmacs.  If not, see <http://www.gnu.org/licenses/>. */
 
 /* Synched up with: FSF 21.3. */
 
@@ -533,6 +531,8 @@ read_c_string (FILE *infile, int printflag, int c_docstring)
 /* Write to file OUT the argument names of function FUNC, whose text is in BUF.
    MINARGS and MAXARGS are the minimum and maximum number of arguments.  */
 
+#define SKIPWHITE do { while (isspace ((unsigned char) (*p))) p++; } while (0)
+
 static void
 write_c_args (FILE *out, const char *UNUSED (func), char *buf,
 	      int minargs, int maxargs)
@@ -540,6 +540,7 @@ write_c_args (FILE *out, const char *UNUSED (func), char *buf,
   register char *p;
   int in_ident = 0;
   int just_spaced = 0;
+  int need_paren = 0;
 #if 0
   int need_space = 1;
 
@@ -560,76 +561,68 @@ write_c_args (FILE *out, const char *UNUSED (func), char *buf,
   for (p = buf; *p; p++)
     {
       char c = *p;
+#if 0
       int ident_start = 0;
+#endif
 
-      /* XEmacs addition:  add support for ANSI prototypes and the UNUSED
-	 macros.  Hop over them.  "Lisp_Object" is the only C type allowed
-	 in DEFUNs.  For the UNUSED macros we need to eat parens, too. */
+      /* XEmacs addition:  used for ANSI prototypes and UNUSED macros. */
       static char uu [] = "UNUSED";
       static char ui [] = "USED_IF_";
-      static char lo[] = "Lisp_Object";
+      static char lo [] = "Lisp_Object";
 
-      /* aren't these all vulnerable to buffer overrun?  I guess that
-	 means that the .c is busted, so we may as well just die ... */
-      /* skip over "Lisp_Object" */
-      if ((C_IDENTIFIER_CHAR_P (c) != in_ident) && !in_ident &&
-	  (strncmp (p, lo, sizeof (lo) - 1) == 0) &&
-	  isspace ((unsigned char) p[sizeof (lo) - 1]))
-	{
-	  p += (sizeof (lo) - 1);
-	  while (isspace ((unsigned char) (*p)))
-	    p++;
-	  c = *p;
-	}
-
-      /* skip over "UNUSED" invocation */
-      if ((C_IDENTIFIER_CHAR_P (c) != in_ident) && !in_ident &&
-	  (strncmp (p, uu, sizeof (uu) - 1) == 0))
-	{
-	  char *here = p;
-	  p += (sizeof (uu) - 1);
-	  while (isspace ((unsigned char) (*p)))
-	    p++;
-	  if (*p == '(')
-	    {
-	      while (isspace ((unsigned char) (*++p)))
-		;
-	      c = *p;
-	    }
-	  else
-	    p = here;
-	}
-
-      /* skip over "USED_IF_*" invocation (only if USED failed) */
-      else if ((C_IDENTIFIER_CHAR_P (c) != in_ident) && !in_ident &&
-	  (strncmp (p, ui, sizeof (ui) - 1) == 0))
-	{
-	  char *here = p;
-	  p += (sizeof (ui) - 1);
-	  /* There should be a law against parsing in C:
-	     this allows a broken USED_IF call, skipping to next macro's
-	     parens.  *You* can fix that, I don't see how offhand. ;-) */
-	  while (*p && *p++ != '(')
-	    ;
-	  if (*p)
-	    {
-	      while (isspace ((unsigned char) (*p)))
-		p++;
-	      c = *p;
-	    }
-	  else
-	    p = here;
-	}
-
-      /* Notice when we start printing a new identifier.  */
+      /* Notice when we enter or leave an identifier.  */
       if (C_IDENTIFIER_CHAR_P (c) != in_ident)
 	{
 	  if (!in_ident)
 	    {
+	      /* Entering identifier.  Print as we parse. */
+	      char *here;     	/* Target for backtracking. */
+
+	      /* XEmacs addition:  add support for ANSI prototypes and the
+		 UNUSED macros.  Hop over them.  "Lisp_Object" is the only
+		 C type allowed in DEFUNs.  For the UNUSED macros we need
+		 to eat parens, too. */
+	      /* Aren't these all vulnerable to buffer overrun?  I guess that
+		 means that the .c is busted, so we may as well just die ... */
+
+	      /* Skip over "Lisp_Object". */
+	      if ((strncmp (p, lo, sizeof (lo) - 1) == 0) &&
+		  isspace ((unsigned char) p[sizeof (lo) - 1]))
+		{
+		  p += (sizeof (lo) - 1);
+		  SKIPWHITE;
+		}
+	      /* Skip over "UNUSED" or "USED_IF_*" invocation. */
+	      need_paren = 1;
+	      here = p;
+	      if (strncmp (p, uu, sizeof (uu) - 1) == 0)
+		p += (sizeof (uu) - 1);
+	      else if (strncmp (p, ui, sizeof (ui) - 1) == 0)
+		p += (sizeof (ui) - 1);
+	      else
+		need_paren = 0;
+
+	      if (need_paren)
+		{
+		  /* Skip rest of macro name, open paren, whitespace. */
+		  while (*p && C_IDENTIFIER_CHAR_P (*p))
+		    p++;
+		  SKIPWHITE;
+		  if (*p++ == '(')
+		    SKIPWHITE;
+		  else
+		    {
+		      need_paren = 0;
+		      p = here;
+		    }
+		}
+	      c = *p;
+
+	      /* Do bookkeeping.  Maybe output lambda keywords. */
 	      in_ident = 1;
-	      ident_start = 1;
 #if 0
 	      /* XEmacs - This goes along with the change above. */
+	      ident_start = 1;
 	      if (need_space)
 		putc (' ', out);
 #endif
@@ -641,15 +634,26 @@ write_c_args (FILE *out, const char *UNUSED (func), char *buf,
 	      maxargs--;
 	    }
 	  else
-	    in_ident = 0;
+	    {
+	      /* Leaving identifier. */
+	      in_ident = 0;
+	      if (need_paren)
+		{
+		  SKIPWHITE;
+		  if (*p == ')')
+		    p++;
+		  c = *p;
+		  need_paren = 0;
+		}
+	    }
 	}
 
       /* Print the C argument list as it would appear in lisp:
-	 print underscores as hyphens, and print commas and newlines
+	 print underscores as hyphens, and print commas, tabs and newlines
 	 as spaces.  Collapse adjacent spaces into one.  */
       if (c == '_')
 	c = '-';
-      else if (c == ',' /* || c == '\n' */)
+      else if (c == ',' || c == '\n' || c == '\t')
 	c = ' ';
       /* XEmacs change: handle \n below for readability */
 
@@ -676,18 +680,28 @@ write_c_args (FILE *out, const char *UNUSED (func), char *buf,
 	  in_ident = 0;
 	  just_spaced = 0;
 	}
-      /* XEmacs change: if the character is carriage return or linefeed,
-	 escape it for the compiler */
+#if 0
+      /* [[ XEmacs change: if the character is carriage return or linefeed,
+	 escape it for the compiler ]] I doubt the clause with '\r' ever
+	 worked right, and outputting newlines now screws up the regexp
+	 in function-documentation-1, so don't do this; instead, we treat
+	 newlines like spaces. --ben */
       else if (c == '\n')
 	{
 	  putc('\\', out);
 	  putc('\n', out);
+	  c = ' ';
 	}
       else if (c == '\r')
 	{
 	  putc('\\', out);
 	  putc('\r', out);
 	}
+#else
+      else if (c == '\r') /* Just eat it, since we expect a newline to
+			     follow */
+	;
+#endif /* (not) 0 */
       else if (c != ' ' || !just_spaced)
 	{
 	  if (c >= 'a' && c <= 'z')
@@ -705,6 +719,8 @@ write_c_args (FILE *out, const char *UNUSED (func), char *buf,
   if (!ellcc)
     putc ('\n', out);
 }
+#undef SKIPWHITE
+
 
 /* Read through a c file.  If a .o or .obj file is named,
    the corresponding .c file is read instead.
@@ -1066,6 +1082,7 @@ scan_lisp_file (const char *filename, const char *mode)
     {
       char buffer[BUFSIZ];
       char type;
+      int no_docstring = 0;
 
       /* If not at end of line, skip till we get to one.  */
       if (c != '\n')
@@ -1169,7 +1186,7 @@ scan_lisp_file (const char *filename, const char *mode)
 	      fprintf (stderr, "## non-docstring in %s (%s)\n",
 		       buffer, filename);
 #endif
-	      continue;
+	      no_docstring = 1;
 	    }
 	}
 
@@ -1199,7 +1216,7 @@ scan_lisp_file (const char *filename, const char *mode)
 		  fprintf (stderr, "## non-docstring in %s (%s)\n",
 			   buffer, filename);
 #endif
-		  continue;
+                  no_docstring = 1;
 		}
 	    }
 	}
@@ -1258,7 +1275,7 @@ scan_lisp_file (const char *filename, const char *mode)
 		  fprintf (stderr, "## non-docstring in %s (%s)\n",
 			   buffer, filename);
 #endif
-		  continue;
+		  no_docstring = 1;
 		}
 	    }
 	}
@@ -1315,7 +1332,7 @@ scan_lisp_file (const char *filename, const char *mode)
 		  fprintf (stderr, "## non-docstring in %s (%s)\n",
 			   buffer, filename);
 #endif
-		  continue;
+                  no_docstring = 1;
 		}
 	    }
 	}
@@ -1373,7 +1390,7 @@ scan_lisp_file (const char *filename, const char *mode)
 		  fprintf (stderr, "## non-docstring in %s (%s)\n",
 			   buffer, filename);
 #endif
-		  continue;
+                  no_docstring = 1;
 		}
 	    }
 	}
@@ -1404,15 +1421,18 @@ scan_lisp_file (const char *filename, const char *mode)
       putc (037, outfile);
       putc (type, outfile);
       fprintf (outfile, "%s\n", buffer);
-      if (saved_string)
-	{
-	  fputs (saved_string, outfile);
-	  /* Don't use one dynamic doc string twice.  */
-	  free (saved_string);
-	  saved_string = 0;
-	}
-      else
-	read_c_string (infile, 1, 0);
+      if (!no_docstring)
+        {
+          if (saved_string)
+            {
+              fputs (saved_string, outfile);
+              /* Don't use one dynamic doc string twice.  */
+              free (saved_string);
+              saved_string = 0;
+            }
+          else
+            read_c_string (infile, 1, 0);
+        }
     }
   fclose (infile);
   return 0;
