@@ -1,7 +1,7 @@
 ;;; bytecomp-runtime.el --- byte-compiler support for inlining
 
 ;; Copyright (C) 1992, 1997 Free Software Foundation, Inc.
-;; Copyright (C) 2002 Ben Wing.
+;; Copyright (C) 2002, 2010 Ben Wing.
 
 ;; Author: Jamie Zawinski <jwz@jwz.org>
 ;; Author: Hallvard Furuseth <hbf@ulrik.uio.no>
@@ -10,20 +10,18 @@
 
 ;; This file is part of XEmacs.
 
-;; XEmacs is free software; you can redistribute it and/or modify it
-;; under the terms of the GNU General Public License as published by
-;; the Free Software Foundation; either version 2, or (at your option)
-;; any later version.
+;; XEmacs is free software: you can redistribute it and/or modify it
+;; under the terms of the GNU General Public License as published by the
+;; Free Software Foundation, either version 3 of the License, or (at your
+;; option) any later version.
 
-;; XEmacs is distributed in the hope that it will be useful, but
-;; WITHOUT ANY WARRANTY; without even the implied warranty of
-;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-;; General Public License for more details.
+;; XEmacs is distributed in the hope that it will be useful, but WITHOUT
+;; ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+;; FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
+;; for more details.
 
 ;; You should have received a copy of the GNU General Public License
-;; along with XEmacs; see the file COPYING.  If not, write to the 
-;; Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-;; Boston, MA 02111-1307, USA.
+;; along with XEmacs.  If not, see <http://www.gnu.org/licenses/>.
 
 ;;; Synched up with: FSF 19.30.
 
@@ -53,30 +51,26 @@
   "Cause the named functions to be open-coded when called from compiled code.
 They will only be compiled open-coded when `byte-optimize' is true."
   (cons 'eval-and-compile
-	(apply
-	 'nconc
-	 (mapcar
-	  #'(lambda (x)
-	      `((or (memq (get ',x 'byte-optimizer)
-			  '(nil byte-compile-inline-expand))
-		    (error
-		     "%s already has a byte-optimizer, can't make it inline"
-		     ',x))
-		(put ',x 'byte-optimizer 'byte-compile-inline-expand)))
-	  fns))))
+        (mapcan
+         #'(lambda (x)
+             `((or (memq (get ',x 'byte-optimizer)
+                         '(nil byte-compile-inline-expand))
+                   (error
+                    "%s already has a byte-optimizer, can't make it inline"
+                    ',x))
+               (put ',x 'byte-optimizer 'byte-compile-inline-expand)))
+         fns)))
 
 
 (defmacro proclaim-notinline (&rest fns)
   "Cause the named functions to no longer be open-coded."
   (cons 'eval-and-compile
-	(apply
-	 'nconc
-	 (mapcar
-	  #'(lambda (x)
-	      `((if (eq (get ',x 'byte-optimizer)
-			'byte-compile-inline-expand)
-		    (put ',x 'byte-optimizer nil))))
-	  fns))))
+        (mapcan
+         #'(lambda (x)
+             `((if (eq (get ',x 'byte-optimizer)
+                       'byte-compile-inline-expand)
+                   (put ',x 'byte-optimizer nil))))
+         fns)))
 
 ;; This has a special byte-hunk-handler in bytecomp.el.
 (defmacro defsubst (name arglist &rest body)
@@ -163,7 +157,7 @@ If you think you need this, you're probably making a mistake somewhere."
 
 (put 'eval-when-compile 'lisp-indent-hook 0)
 (defmacro eval-when-compile (&rest body)
-  "Like `progn', but evaluates the body at compile time.
+  "Like `progn', but evaluates BODY at compile time, and when interpeted.
 The result of the body appears to the compiler as a quoted constant."
   ;; Not necessary because we have it in b-c-initial-macro-environment
   ;; (list 'quote (eval (cons 'progn body)))
@@ -171,7 +165,8 @@ The result of the body appears to the compiler as a quoted constant."
 
 (put 'eval-and-compile 'lisp-indent-hook 0)
 (defmacro eval-and-compile (&rest body)
-  "Like `progn', but evaluates the body at compile time and at load time."
+  "Like `progn', but evaluates the body at compile time and at load time,
+and when interpreted."
   ;; Remember, it's magic.
   (cons 'progn body))
 
@@ -313,7 +308,7 @@ functionality at some entry point to the package.  See
   (let ((symbols (eval (car (cdr form)))))
     (unless (consp symbols)
       (setq symbols (list symbols)))
-    (setq symbols (mapcar #'(lambda (sym) (cons sym nil)) symbols))
+    (setq symbols (mapcar #'list symbols))
     (setq byte-compile-unresolved-functions
 	  (set-difference byte-compile-unresolved-functions symbols
 			  :key #'car))
@@ -430,7 +425,7 @@ throughout the code would be a major annoyance."
     ;; have an autoload later in the file for any functions in FUNCTIONS.
     ;; This is not something that code should ever do, though.)
     (setq byte-compile-autoload-environment
-	  (append (mapcar #'(lambda (sym) (cons sym nil)) functions)
+	  (append (mapcar #'list functions)
 		  byte-compile-autoload-environment)))
   nil)
 
@@ -492,6 +487,60 @@ SYMBOL must be quoted and can be a list of SYMBOLS.  See also
 `with-obsolete-variable'."
   `(with-byte-compiler-warnings-suppressed 'obsolete ,@body))
 
+
+
+(defmacro error-unless-tests-match (test &optional source)
+  "Signal an error unless TEST matches when being compiled and loaded.
+This is for use in a file that will be byte-compiled.  Unless TEST has the
+same nilness or non-nilness when the file is compiled and loaded, an error
+will be signalled.  SOURCE is the name of the source file."
+  (let ((source (eval source)))
+    `(unless (eq (not ,test) ,(not (eval test)))
+       (error ,(format "This file was compiled with `%s' %s,
+but it was %s when run.  This file needs to be compiled with
+the same value for the expression as when it is run.  Please delete
+%s and rebuild."
+		       test (if (eval test) "true" "false")
+		       (if (eval test) "false" "true")
+		       (cond
+			((null source) "the .elc for this file")
+			((string-match "\.elc$" source) source)
+			((string-match "\.el$" source) (concat source "c"))
+			(t (concat source ".elc"))))))))
+
+(defun byte-compile-file-being-compiled ()
+  "When byte-compiling a file, return the name of the file being compiled.
+Return nil otherwise."
+  (or
+   ;;The first of these, but not the second, seems to work noninteractively;
+   ;;vice-versa interactively.  This is because interactively a *Compile Log*
+   ;;buffer is created and byte-compile-log-1 inserts a "Compiling file ..."
+   ;;message into thelog buffer, and then sets byte-compile-current-file to 
+   ;;nil to indicate that the message shouldn't be printed again.
+   (and-boundp 'byte-compile-current-file byte-compile-current-file)
+   (and-boundp 'byte-compile-log-buffer
+     (with-current-buffer byte-compile-log-buffer
+       (save-excursion
+	 (and (re-search-backward "Compiling file \\(.*\\) at " nil t)
+	      (match-string 1)))))))
+
+
+(defmacro compiled-if (test if &rest else)
+  "Like a regular `if' statement but the TEST will be evalled at compile time.
+If TEST doesn't match at compile time and load time, an error will be
+signalled."
+  (let ((being-compiled (byte-compile-file-being-compiled)))
+    `(progn
+      (error-unless-tests-match ,test ,being-compiled)
+      ,(if (eval test)
+	   if
+	 `(progn ,else)))))
+
+(defmacro compiled-when (test &rest when)
+  "Like a regular `when' statement but the TEST will be evalled at compile time.
+See `compiled-if'."
+  `(compiled-if ,test (progn ,@when)))
+
 
 ;;; Interface to file-local byte-compiler parameters.
 ;;; Redefined in bytecomp.el.
