@@ -10,20 +10,20 @@
 ;; Updated:	05 Mar 2005 by Stephen J. Turnbull
 ;; Keywords: faces
 
-;; XEmacs is free software; you can redistribute it and/or modify it
-;; under the terms of the GNU General Public License as published by
-;; the Free Software Foundation; either version 2, or (at your option)
-;; any later version.
+;; This file is part of XEmacs.
 
-;; XEmacs is distributed in the hope that it will be useful, but
-;; WITHOUT ANY WARRANTY; without even the implied warranty of
-;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-;; General Public License for more details.
+;; XEmacs is free software: you can redistribute it and/or modify it
+;; under the terms of the GNU General Public License as published by the
+;; Free Software Foundation, either version 3 of the License, or (at your
+;; option) any later version.
+
+;; XEmacs is distributed in the hope that it will be useful, but WITHOUT
+;; ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+;; FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
+;; for more details.
 
 ;; You should have received a copy of the GNU General Public License
-;; along with XEmacs; see the file COPYING.  If not, write to the Free
-;; Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
-;; 02111-1307, USA.
+;; along with XEmacs.  If not, see <http://www.gnu.org/licenses/>.
 
 ;;; Synched up with: Not in GNU
 
@@ -352,7 +352,7 @@ corresponding Xft font slant constant."
 
 (defun fc-pattern-get-or-compute-size (pattern id)
   "Get the size from `pattern' associated with `id' or try to compute it.
-Returns 'fc-result-no-match if unsucessful."
+Returns 'fc-result-no-match if unsuccessful."
   ;;  Many font patterns don't have a "size" property, but do have a
   ;;  "dpi" and a "pixelsize" property".
   (let ((maybe (fc-pattern-get-size pattern id)))
