@@ -3,26 +3,25 @@
 ;; Copyright (C) 1993, 1994, 1996, 1997, 2000, 2001, 2003
 ;;   Free Software Foundation, Inc.
 ;; Copyright (C) 1995, 1996 Ben Wing.
+;; Copyright (C) 2010 Didier Verna
 
 ;; Maintainer: XEmacs Development Team
 ;; Keywords: internal, dumped
 
 ;; This file is part of XEmacs.
 
-;; XEmacs is free software; you can redistribute it and/or modify it
-;; under the terms of the GNU General Public License as published by
-;; the Free Software Foundation; either version 2, or (at your option)
-;; any later version.
+;; XEmacs is free software: you can redistribute it and/or modify it
+;; under the terms of the GNU General Public License as published by the
+;; Free Software Foundation, either version 3 of the License, or (at your
+;; option) any later version.
 
-;; XEmacs is distributed in the hope that it will be useful, but
-;; WITHOUT ANY WARRANTY; without even the implied warranty of
-;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-;; General Public License for more details.
+;; XEmacs is distributed in the hope that it will be useful, but WITHOUT
+;; ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+;; FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
+;; for more details.
 
 ;; You should have received a copy of the GNU General Public License
-;; along with XEmacs; see the file COPYING.  If not, write to the
-;; Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-;; Boston, MA 02111-1307, USA.
+;; along with XEmacs.  If not, see <http://www.gnu.org/licenses/>.
 
 ;;; Synched up with: FSF 21.3.
 
@@ -474,12 +473,13 @@ React to settings of `default-frame-plist', `initial-frame-plist' there."
 	      ;; onto a new frame.  The default-minibuffer-frame
 	      ;; variable must be handled similarly.
 	      (let ((users-of-initial
-		     (filtered-frame-list
+		     (remove-if-not
 		      #'(lambda (frame)
 				  (and (not (eq frame frame-initial-frame))
 				       (eq (window-frame
 					    (minibuffer-window frame))
-					   frame-initial-frame))))))
+					   frame-initial-frame)))
+                      (frame-list))))
 		(if (or users-of-initial
 			(eq default-minibuffer-frame frame-initial-frame))
 
@@ -487,10 +487,11 @@ React to settings of `default-frame-plist', `initial-frame-plist' there."
 		    ;; are only minibuffers.
 		    (let* ((new-surrogate
 			    (car
-			     (or (filtered-frame-list
+			     (or (remove-if-not
 				  #'(lambda (frame)
 				      (eq 'only
-					  (frame-property frame 'minibuffer))))
+					  (frame-property frame 'minibuffer)))
+                                  (frame-list))
 				 (minibuffer-frame-list))))
 			   (new-minibuffer (minibuffer-window new-surrogate)))
 
@@ -499,7 +500,7 @@ React to settings of `default-frame-plist', `initial-frame-plist' there."
 
 		      ;; Wean the frames using frame-initial-frame as
 		      ;; their minibuffer frame.
-		      (mapcar
+		      (mapc
 		       #'(lambda (frame)
 			   (set-frame-property frame 'minibuffer
 					       new-minibuffer))
@@ -642,8 +643,6 @@ tty	    A standard TTY connection or terminal.  CONNECTION should be
 	    and output (usually the TTY in which XEmacs started).  Only
 	    if support for TTY's was compiled into XEmacs.
 gtk	    A GTK device.
-ns	    A connection to a machine running the NeXTstep windowing
-	    system.  Not currently implemented.
 mswindows   A connection to a machine running Microsoft Windows NT or
 	    Windows 95/97.
 pc	    A direct-write MS-DOS frame.  Not currently implemented.
@@ -675,29 +674,22 @@ a new connection is opened."
 ;; XEmacs change: Emacs has make-frame here.  We have it in C, so no need for
 ;; frame-creation-function.
 
-;; XEmacs addition: support optional DEVICE argument.
+;; XEmacs addition: support optional DEVICE argument, use delete-if-not.
 (defun filtered-frame-list (predicate &optional device)
   "Return a list of all live frames which satisfy PREDICATE.
 If optional second arg DEVICE is non-nil, restrict the frames
  returned to that device."
-  (let ((frames (if device (device-frame-list device)
-		  (frame-list)))
-	good-frames)
-    (while (consp frames)
-      (if (funcall predicate (car frames))
-	  (setq good-frames (cons (car frames) good-frames)))
-      (setq frames (cdr frames)))
-    good-frames))
+  (delete-if-not predicate
+                 (if device (device-frame-list device) (frame-list))))
 
 ;; XEmacs addition: support optional DEVICE argument.
 (defun minibuffer-frame-list (&optional device)
   "Return a list of all frames with their own minibuffers.
 If optional second arg DEVICE is non-nil, restrict the frames
  returned to that device."
-  (filtered-frame-list
-   #'(lambda (frame)
-	       (eq frame (window-frame (minibuffer-window frame))))
-   device))
+  (delete-if-not 
+   #'(lambda (frame) (eq frame (window-frame (minibuffer-window frame))))
+   (if device (device-frame-list device) (frame-list))))
 
 ;; XEmacs omission: Emacs has frames-on-display-list here, but that is
 ;; essentially equivalent to supplying the optional DEVICE argument to
@@ -752,15 +744,15 @@ This deletes all bindings in PLIST for `top', `left', `width',
 Emacs uses this to avoid overriding explicit moves and resizings from
 the user during startup."
   (setq plist (canonicalize-lax-plist (copy-sequence plist)))
-  (mapcar #'(lambda (property)
-	      (if (lax-plist-member plist property)
-		  (progn
-		    (setq frame-initial-geometry-arguments
-			  (cons property
-				(cons (lax-plist-get plist property)
-				      frame-initial-geometry-arguments)))
-		    (setq plist (lax-plist-remprop plist property)))))
-	  '(height width top left user-size user-position))
+  (mapc #'(lambda (property)
+            (if (lax-plist-member plist property)
+                (progn
+                  (setq frame-initial-geometry-arguments
+                        (cons property
+                              (cons (lax-plist-get plist property)
+                                    frame-initial-geometry-arguments)))
+                  (setq plist (lax-plist-remprop plist property)))))
+        '(height width top left user-size user-position))
   plist)
 
 ;; XEmacs change: Emacs has focus-follows-mouse here, which lets them
@@ -862,14 +854,13 @@ The value returned is the value of the last form in BODY."
 (defun frame-list ()
   "Return a list of all frames on all devices/consoles."
   ;; Lists are copies, so nconc is safe here.
-  (apply 'nconc (mapcar 'device-frame-list (device-list))))
+  (mapcan #'device-frame-list (device-list)))
 
 (defun frame-type (&optional frame)
   "Return the type of the specified frame (e.g. `x' or `tty').
 This is equivalent to the type of the frame's device.
 Value is `tty' for a tty frame (a character-only terminal),
 `x' for a frame that is an X window,
-`ns' for a frame that is a NeXTstep window (not yet implemented),
 `mswindows' for a frame that is a MS Windows desktop window,
 `msprinter' for a frame that is a MS Windows print job,
 `stream' for a stream frame (which acts like a stdio stream), and
@@ -1018,6 +1009,28 @@ pixels) is kept by adjusting the numbers of the lines and columns."
   "Set property PROP of FRAME to VAL.  See `set-frame-properties'."
   (set-frame-properties frame (list prop val)))
 
+(defun set-frame-background-placement (placement)
+  "Set the background placement of the selected frame to PLACEMENT.
+When called interactively, prompt for the placement to use."
+  (interactive (list (intern (completing-read "Placement: "
+					      '(("absolute" absolute)
+						("relative" relative))
+					      nil t))))
+  (set-face-background-placement 'default placement (selected-frame)))
+
+(defun frame-background-placement ()
+  "Retrieve the selected frame's background placement."
+  (interactive)
+  (face-background-placement 'default (selected-frame)))
+
+(defun frame-background-placement-instance ()
+  "Retrieve the selected frame's background placement instance."
+  (interactive)
+  (face-background-placement-instance 'default (selected-frame)))
+
+;; #### FIXME: misnomers ! The functions below should be called
+;; set-frame-<blabla> -- dvl.
+
 ;; XEmacs change: this function differs significantly from Emacs.
 (defun set-background-color (color-name)
   "Set the background color of the selected frame to COLOR-NAME.
@@ -1186,8 +1199,7 @@ selected frame)."
      ((eq 'mswindows type)
       (> (declare-boundp mswindows-num-mouse-buttons) 0))
      ((device-on-window-system-p display)
-      ;; We assume X, NeXTstep, and GTK and the rest always have a pointing
-      ;; device. 
+      ;; We assume X, GTK and the rest always have a pointing device. 
       t)
     ((eq 'tty type)
      (and-fboundp 'gpm-is-supported-p
@@ -1726,9 +1738,10 @@ This is a subroutine of `get-frame-for-buffer' (which see)."
 	       (or (plist-get default-frame-plist 'name)
 		   default-frame-name))
 	     (frames
-	      (sort (filtered-frame-list #'(lambda (x)
-					     (or (frame-visible-p x)
-						 (frame-iconified-p x))))
+	      (sort (remove-if-not #'(lambda (x)
+                                       (or (frame-visible-p x)
+                                           (frame-iconified-p x)))
+                                   (frame-list))
 		    #'(lambda (s1 s2)
 			(cond ((and (frame-visible-p s1)
 				    (not (frame-visible-p s2))))
@@ -1889,7 +1902,6 @@ is normally set to `get-frame-for-buffer' (which see)."
       (set-window-point window 1)
       nil)))
 
-(setq pre-display-buffer-function 'get-frame-for-buffer)
 (setq temp-buffer-show-function 'show-temp-buffer-in-current-frame)
 
 
