@@ -11,19 +11,18 @@
 
 ;; This file is part of XEmacs.
 
-;; XEmacs is free software; you can redistribute it and/or modify
-;; it under the terms of the GNU General Public License as published by
-;; the Free Software Foundation; either version 2 of the License, or
-;; (at your option) any later version.
-;; 
-;; XEmacs is distributed in the hope that it will be useful,
-;; but WITHOUT ANY WARRANTY; without even the implied warranty of
-;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-;; GNU General Public License for more details.
-;; 
+;; XEmacs is free software: you can redistribute it and/or modify it
+;; under the terms of the GNU General Public License as published by the
+;; Free Software Foundation, either version 3 of the License, or (at your
+;; option) any later version.
+
+;; XEmacs is distributed in the hope that it will be useful, but WITHOUT
+;; ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+;; FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
+;; for more details.
+
 ;; You should have received a copy of the GNU General Public License
-;; along with XEmacs; if not, write to the Free Software
-;; Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+;; along with XEmacs.  If not, see <http://www.gnu.org/licenses/>.
 
 ;;; Synched up with: Not in FSF.
 
@@ -730,7 +729,7 @@ See also `hyper-apropos' and `hyper-describe-function'."
 	    (local mode-name)
 	    global local-str global-str
 	    font fore back undl
-	    aliases alias-desc desc)
+	    aliases alias-desc desc arglist)
 	(save-excursion
 	  (set-buffer (get-buffer-create hyper-apropos-help-buf))
 	  ;;(setq standard-output (current-buffer))
@@ -764,21 +763,22 @@ See also `hyper-apropos' and `hyper-describe-function'."
 					       (bytecode . "compiled Lisp ")
 					       (autoload . "autoloaded Lisp ")
 					       (lambda   . "Lisp "))))
-				  desc
-				  (case symtype
-				    ((autoload) (format ",\n(autoloaded from \"%s\")"
-							(nth 1 newsym)))
-				    ((bytecode) (format ",\n(loaded from \"%s\")"
-							(symbol-file symbol)))))
+				  desc ",\n(loaded from \""
+                                  (or (symbol-file symbol 'defun)
+                                      "[no file information available]")
+                                  "\")")
 		     local (current-local-map)
 		     global (current-global-map)
 		     obsolete (get symbol 'byte-obsolete-info)
-		     doc (or (condition-case nil
-				 (documentation symbol)
-			       (void-function
-				"(alias for undefined function)")
-			       (error "(unexpected error from `documention')"))
-			     "function not documented"))
+		     doc (function-documentation symbol t)
+                     arglist (let ((farglist (function-arglist symbol)))
+			       (if farglist
+				   (replace-in-string
+				    farglist
+				    (format "^(%s "
+					(regexp-quote (symbol-name symbol)))
+				    "(")
+				 "[not available]")))
 	       (save-excursion
 		 (set-buffer hyper-apropos-help-buf)
 		 (goto-char (point-max))
@@ -802,32 +802,7 @@ See also `hyper-apropos' and `hyper-describe-function'."
 		      'hyper-apropos-warning))
 		 (setq beg (point))
 		 (insert-face "arguments: " 'hyper-apropos-heading)
-		 (cond ((eq symtype 'lambda)
-			(princ (or (nth 1 newsym) "()")))
-		       ((eq symtype 'bytecode)
-			(princ (or (compiled-function-arglist newsym)
-				   "()")))
-		       ((and (eq symtype 'subr)
-			     (string-match
-			      "[\n\t ]*\narguments: ?\\((.*)\\)\n?\\'"
-			      doc))
-			(insert (substring doc
-					   (match-beginning 1)
-					   (match-end 1)))
-			(setq doc (substring doc 0 (match-beginning 0))))
-		       ((and (eq symtype 'subr)
-			     (string-match
-			      "\
-\[\n\t ]*([^\n\t )]+[\t ]*\\([^\n)]+\\)?)\\(:[\t ]*\\|\n?\\'\\)"
-			      doc))
-			(insert "("
-				(if (match-end 1)
-				    (substring doc
-					       (match-beginning 1)
-					       (match-end 1)))
-				")")
-			(setq doc (substring doc (match-end 0))))
-		       (t (princ "[not available]")))
+                 (princ arglist)
 		 (insert "\n\n")
 		 (hyper-apropos-insert-face doc)
 		 (insert "\n")
@@ -944,14 +919,14 @@ See also `hyper-apropos' and `hyper-describe-function'."
 	     (progn
 	       (setq ok t)
 	       (copy-face symbol 'hyper-apropos-temp-face 'global)
-	       (mapcar #'(lambda (property)
-			   (setq symtype (face-property-instance symbol
-								 property))
-			   (if symtype
-			       (set-face-property 'hyper-apropos-temp-face
-						  property
-						  symtype)))
-		       built-in-face-specifiers)
+	       (mapc #'(lambda (property)
+                         (setq symtype (face-property-instance symbol
+                                                               property))
+                         (if symtype
+                             (set-face-property 'hyper-apropos-temp-face
+                                                property
+                                                symtype)))
+                     built-in-face-specifiers)
 	       (setq font (cons (face-property-instance symbol 'font nil 0 t)
 				(face-property-instance symbol 'font))
 		     fore (cons (face-foreground-instance symbol nil 0 t)
