@@ -2,27 +2,25 @@
 
 ;; Copyright (C) 1985, 1996, 1997 Free Software Foundation, Inc.
 ;; Copyright (C) 1995 Tinker Systems.
-;; Copyright (C) 2002 Ben Wing.
+;; Copyright (C) 2002, 2010 Ben Wing.
 
 ;; Maintainer: FSF
 ;; Keywords: lisp, languages, dumped
 
 ;; This file is part of XEmacs.
 
-;; XEmacs is free software; you can redistribute it and/or modify it
-;; under the terms of the GNU General Public License as published by
-;; the Free Software Foundation; either version 2, or (at your option)
-;; any later version.
+;; XEmacs is free software: you can redistribute it and/or modify it
+;; under the terms of the GNU General Public License as published by the
+;; Free Software Foundation, either version 3 of the License, or (at your
+;; option) any later version.
 
-;; XEmacs is distributed in the hope that it will be useful, but
-;; WITHOUT ANY WARRANTY; without even the implied warranty of
-;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-;; General Public License for more details.
+;; XEmacs is distributed in the hope that it will be useful, but WITHOUT
+;; ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+;; FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
+;; for more details.
 
 ;; You should have received a copy of the GNU General Public License
-;; along with XEmacs; see the file COPYING.  If not, write to the Free
-;; Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
-;; 02111-1307, USA.
+;; along with XEmacs.  If not, see <http://www.gnu.org/licenses/>.
 
 ;;; Synched up with: FSF 19.34 (but starting to diverge).
 
@@ -424,36 +422,55 @@ using `\\[eval-last-sexp]' and `\\[eval-defun]' than it would have
 been treated noninteractively.
 
 The printed messages are \"defvar treated as defconst\" and \"defcustom
- evaluation forced\".  See `eval-interactive' for more details."
+evaluation forced\".  See `eval-interactive' for more details."
   :type 'boolean
   :group 'lisp)
 
 (defun eval-interactive (expr)
-  "Like `eval' except that it transforms defvars to defconsts.
-The evaluation of defcustom forms is forced."
+  "Evaluate EXPR; pass back multiple values, transform defvars to defconsts. 
+
+Always returns a list.  The length of this list will be something other than
+one if the form returned multiple values.  It will be zero if the form
+returned a single zero-length multiple value."
   (cond ((and (eq (car-safe expr) 'defvar)
 	      (> (length expr) 2))
-	 (eval (cons 'defconst (cdr expr)))
+	 (setq expr (multiple-value-list (eval (cons 'defconst (cdr expr)))))
 	 (when eval-interactive-verbose
 	   (message "defvar treated as defconst")
 	   (sit-for 1)
 	   (message ""))
-	 (nth 1 expr))
+         expr)
 	((and (eq (car-safe expr) 'defcustom)
 	      (> (length expr) 2)
 	      (default-boundp (nth 1 expr)))
 	 ;; Force variable to be bound
-	 ;; #### defcustom might specify a different :set method.
-	 (set-default (nth 1 expr) (eval (nth 2 expr)))
+         (funcall 
+          (or (plist-get expr :set) #'custom-set-default)
+          (nth 1 expr) (eval (nth 2 expr)))
 	 ;; And evaluate the defcustom
-	 (eval expr)
+	 (setq expr (multiple-value-list (eval expr)))
 	 (when eval-interactive-verbose
 	   (message "defcustom evaluation forced")
 	   (sit-for 1)
 	   (message ""))
-	 (nth 1 expr))
+         expr)
 	(t
-	 (eval expr))))
+	 (multiple-value-list (eval expr)))))
+
+(defun prin1-list-as-multiple-values (multiple-value-list &optional stream)
+  "Call `prin1' on each element of MULTIPLE-VALUE-LIST, separated by \" ;\\n\"
+
+If MULTIPLE-VALUE-LIST is zero-length, print the text
+\"#<zero length multiple value> ;\\n\".  Always returns nil."
+  (loop for value in multiple-value-list
+    with seen-first = nil
+    do
+    (if seen-first
+        (princ " ;\n" stream)
+      (setq seen-first t))
+    (prin1 value stream)
+    finally (unless seen-first
+	      (princ "#<zero length multiple value> ;" stream))))
 
 ;; XEmacs change, based on Bob Weiner suggestion
 (defun eval-last-sexp (eval-last-sexp-arg-internal) ;dynamic scoping wonderment
@@ -463,31 +480,32 @@ With argument, print output into current buffer."
   (let ((standard-output (if eval-last-sexp-arg-internal (current-buffer) t))
 	(opoint (point))
 	ignore-quotes)
-    (prin1 (eval-interactive
-	    (letf (((syntax-table) emacs-lisp-mode-syntax-table))
-	      (save-excursion
-		;; If this sexp appears to be enclosed in `...' then
-		;; ignore the surrounding quotes.
-		(setq ignore-quotes (or (eq (char-after) ?\')
-					(eq (char-before) ?\')))
-		(forward-sexp -1)
-		;; vladimir@cs.ualberta.ca 30-Jul-1997: skip ` in
-		;; `variable' so that the value is returned, not the
-		;; name.
-		(if (and ignore-quotes
-			 (eq (char-after) ?\`))
-		    (forward-char))
-		(save-restriction
-		  (narrow-to-region (point-min) opoint)
-		  (let ((expr (read (current-buffer))))
-		    (if (eq (car-safe expr) 'interactive)
-			;; If it's an (interactive ...) form, it's
-			;; more useful to show how an interactive call
-			;; would use it.
-			`(call-interactively
-			  (lambda (&rest args)
-			    ,expr args))
-		      expr)))))))))
+    (prin1-list-as-multiple-values
+     (eval-interactive
+      (letf (((syntax-table) emacs-lisp-mode-syntax-table))
+        (save-excursion
+          ;; If this sexp appears to be enclosed in `...' then
+          ;; ignore the surrounding quotes.
+          (setq ignore-quotes (or (eq (char-after) ?\')
+                                  (eq (char-before) ?\')))
+          (forward-sexp -1)
+          ;; vladimir@cs.ualberta.ca 30-Jul-1997: skip ` in
+          ;; `variable' so that the value is returned, not the
+          ;; name.
+          (if (and ignore-quotes
+                   (eq (char-after) ?\`))
+              (forward-char))
+          (save-restriction
+            (narrow-to-region (point-min) opoint)
+            (let ((expr (read (current-buffer))))
+              (if (eq (car-safe expr) 'interactive)
+                  ;; If it's an (interactive ...) form, it's
+                  ;; more useful to show how an interactive call
+                  ;; would use it.
+                  `(call-interactively
+                    (lambda (&rest args)
+                      ,expr args))
+                expr)))))))))
 
 (defun eval-defun (eval-defun-arg-internal)
   "Evaluate defun that point is in or before.
@@ -495,11 +513,12 @@ Print value in minibuffer.
 With argument, insert value in current buffer after the defun."
   (interactive "P")
   (let ((standard-output (if eval-defun-arg-internal (current-buffer) t)))
-    (prin1 (eval-interactive (save-excursion
-			       (end-of-defun)
-			       (beginning-of-defun)
-			       (read (current-buffer)))))))
-
+    (prin1-list-as-multiple-values
+     (eval-interactive
+      (save-excursion
+        (end-of-defun)
+        (beginning-of-defun)
+        (read (current-buffer)))))))
 
 (defun lisp-comment-indent ()
   (if (looking-at "\\s<\\s<\\s<")
@@ -678,7 +697,7 @@ of the start of the containing expression."
 (defvar lisp-function-and-type-regexp
   (concat "def\\("
 	  ;; def but not define-.
-	  "\\(un\\|advice\\|alias\\|macro\\*?\\|setf\\|subst\\*?\\|"
+	  "\\(un\\*?\\|advice\\|alias\\|macro\\*?\\|setf\\|subst\\*?\\|"
 	  "-edebug-spec\\|"
 	  ;; CLOS
 	  "method\\|generic\\|"
@@ -825,7 +844,11 @@ word-end; i.e. defunbbb won't match.")
 	       (lisp-indent-specform method state
 				     indent-point normal-indent))
 	      (method
-		(funcall method state indent-point)))))))
+		(funcall method state indent-point))
+	      ((and (> (length function) 5)
+		    (string-match "\\`with-" function))
+	       (lisp-indent-specform 0 state indent-point normal-indent
+				     t)))))))
 
 (defun lisp-indent-quoteform (state indent-point)
   (goto-char (car (cdr state)))
@@ -838,7 +861,36 @@ word-end; i.e. defunbbb won't match.")
 (defvar lisp-body-indent 2
   "Number of columns to indent the second line of a `(def...)' form.")
 
-(defun lisp-indent-specform (count state indent-point normal-indent)
+;; Calculate the appropriate indentation for a special form (e.g. defun) or
+;; a macro that behaves like a special form (e.g. with-temp-buffer).
+;; Return value is as for `calculate-lisp-indent' (q.v.).
+;;
+;; COUNT is the number of "distinguished" forms (e.g. arguments before the
+;; "body", in a macro taking a body as its last argument).
+;;
+;; INDENT-POINT is usually the value of (point) at the beginning of the
+;; line containing the form to be indented.
+;;
+;; STATE is the result of calling (parse-partial-sexp (point) INDENT-POINT)
+;; when (point) is sitting on (i.e. just before) the outermost containing
+;; left paren for the form to be indented.
+;;
+;; NORMAL-INDENT is the amount of "full" indentation used for arguments
+;; that aren't given some special indentation (usually less), and normally
+;; it lines up with the first word after the function name.  forms are
+;; indented as follows:
+;;
+;; -- The first and second distinguished forms are given 2 times body indent
+;;    (determined by `lisp-body-indent')
+;; -- Other distinguished forms are given normal indent
+;; -- The first nondistinguished form (the body, usually) is given body indent
+;; -- Normally, any further nondistinguished forms are given normal indent
+;; -- However, if INDENT-REMAINING-AS-BODY was specified, any further
+;;    nondistinguished forms are given body indent.  This is useful when the
+;;    number of distinguished forms isn't known or may vary.
+
+(defun lisp-indent-specform (count state indent-point normal-indent
+			     &optional indent-remaining-as-body)
   (let ((containing-form-start (elt state 1))
         (i count)
         body-indent containing-form-column)
@@ -879,7 +931,8 @@ word-end; i.e. defunbbb won't match.")
       ;; or if this is the first undistinguished form and the preceding
       ;; distinguished form has indentation at least as great as body-indent.
       (if (or (and (= i 0) (= count 0))
-              (and (= count 0) (<= body-indent normal-indent)))
+              (and (or (= count 0) indent-remaining-as-body)
+		   (<= body-indent normal-indent)))
           body-indent
           normal-indent))))
 
