@@ -11,20 +11,18 @@
 
 ;; This file is part of XEmacs.
 
-;; XEmacs is free software; you can redistribute it and/or modify it
-;; under the terms of the GNU General Public License as published by
-;; the Free Software Foundation; either version 2, or (at your option)
-;; any later version.
+;; XEmacs is free software: you can redistribute it and/or modify it
+;; under the terms of the GNU General Public License as published by the
+;; Free Software Foundation, either version 3 of the License, or (at your
+;; option) any later version.
 
-;; XEmacs is distributed in the hope that it will be useful, but
-;; WITHOUT ANY WARRANTY; without even the implied warranty of
-;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-;; General Public License for more details.
+;; XEmacs is distributed in the hope that it will be useful, but WITHOUT
+;; ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+;; FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
+;; for more details.
 
 ;; You should have received a copy of the GNU General Public License
-;; along with Xmacs; see the file COPYING.  If not, write to the
-;; Free Software Foundation, 59 Temple Place - Suite 330,
-;; Boston, MA 02111-1307, USA.
+;; along with XEmacs.  If not, see <http://www.gnu.org/licenses/>.
 
 ;;; Authorship:
 
@@ -247,7 +245,7 @@ Write your filter like this:
 		      (submenu-generate-accelerator-spec
 		       (mapcar #'(lambda (bmk)
 				   `[,bmk (bookmark-jump ',bmk)])
-			       (bookmark-all-names)))))
+			       (declare-fboundp (bookmark-all-names))))))
 		 (append menu '("---") items)))))
       "---"
       ["Insert %_Contents" bookmark-menu-insert
@@ -262,7 +260,7 @@ Write your filter like this:
 		    (submenu-generate-accelerator-spec
 		     (mapcar #'(lambda (bmk)
 				 `[,bmk (bookmark-delete ',bmk)])
-			     (bookmark-all-names)))))
+			     (declare-fboundp (bookmark-all-names))))))
       ["%_Edit Bookmark List" bookmark-bmenu-list
        :active (and-boundp 'bookmark-alist bookmark-alist)]
       "---"
