@@ -9,20 +9,18 @@
 
 ;; This file is part of XEmacs.
 
-;; XEmacs is free software; you can redistribute it and/or modify it
-;; under the terms of the GNU General Public License as published by
-;; the Free Software Foundation; either version 2, or (at your option)
-;; any later version.
+;; XEmacs is free software: you can redistribute it and/or modify it
+;; under the terms of the GNU General Public License as published by the
+;; Free Software Foundation, either version 3 of the License, or (at your
+;; option) any later version.
 
-;; XEmacs is distributed in the hope that it will be useful, but
-;; WITHOUT ANY WARRANTY; without even the implied warranty of
-;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-;; General Public License for more details.
+;; XEmacs is distributed in the hope that it will be useful, but WITHOUT
+;; ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+;; FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
+;; for more details.
 
 ;; You should have received a copy of the GNU General Public License
-;; along with XEmacs; see the file COPYING.  If not, write to the
-;; Free Software Foundation, 59 Temple Place - Suite 330,
-;; Boston, MA 02111-1307, USA.
+;; along with XEmacs.  If not, see <http://www.gnu.org/licenses/>.
 
 ;;; Synched up with: Not synched with FSF.  Almost completely divergent.
 
@@ -231,15 +229,7 @@ EVENT should be the button event that initiated the drag.
 Returns whether a drag was begun."
   ;; #### barely implemented.
   (when (click-inside-selection-p event)
-    (cond ((featurep 'offix)
-	   (declare-fboundp
-	    (offix-start-drag-region
-	     event
-	     (extent-start-position zmacs-region-extent)
-	     (extent-end-position zmacs-region-extent)))
-	   t)
-	  ((featurep 'cde)
-	   ;; should also work with CDE
+    (cond ((featurep 'cde)
 	   (declare-fboundp
 	    (cde-start-drag-region event
 				   (extent-start-position zmacs-region-extent)
@@ -278,18 +268,23 @@ It's also fantastic for debugging regular expressions."
 	     (message "Regex \"%s\" not found" exp)
 	     (ding nil 'quiet)))
 	  (t (setq val (if (fboundp 'eval-interactive)
-			   (eval-interactive exp)
-			 (eval exp)))))
-    (setq result-str (prin1-to-string val))
+                           (eval-interactive exp)
+			 (list (eval exp))))))
+    (setq result-str (mapconcat #'prin1-to-string val " ;\n"))
     ;; #### -- need better test
     (if (and (not force-window)
-	     (<= (length result-str) (window-width (selected-window))))
+	     (<= (length result-str) (window-width (selected-window)))
+             (not (string-match "\n" result-str)))
 	(message "%s" result-str)
       (with-output-to-temp-buffer "*Mouse-Eval*"
-	(if-fboundp 'pprint
-	    (pprint val)
-	  (prin1 val)))
-      )))
+        (loop
+          for value in val 
+          with seen-first = nil
+          do
+          (if seen-first 
+              (princ " ;\n")
+            (setq seen-first t))
+          (cl-prettyprint value))))))
 
 (defun mouse-line-length (event)
   "Print the length of the line indicated by the pointer."
@@ -998,7 +993,7 @@ also works, because the behavior is emulated."
       (let ((tmp start)) (setq start end end tmp)))
   (cond
    ((= start end)		; never delete the last remaining extent
-    (mapcar 'delete-extent (cdr extents))
+    (mapc 'delete-extent (cdr extents))
     (setcdr extents nil)
     (set-extent-endpoints (car extents) start start))
    (t
@@ -1044,7 +1039,7 @@ also works, because the behavior is emulated."
 	    (setq last rest
 		  rest (cdr rest)))
 	  (cond (rest
-		 (mapcar 'delete-extent rest)
+		 (mapc 'delete-extent rest)
 		 (setcdr last nil))
 		((not (eobp))
 		 (while (not (eobp))
@@ -1315,7 +1310,7 @@ also works, because the behavior is emulated."
 			 (set-extent-face e 'primary-selection)))))
       (add-hook 'pre-command-hook 'default-mouse-track-cleanup-extents-hook)
       (if (consp extent)		; rectangle-p
-	  (mapcar func extent)
+	  (mapc func extent)
 	(if extent
 	    (funcall func extent)))))
   t)
@@ -1329,10 +1324,10 @@ also works, because the behavior is emulated."
     (if (consp extent)
 	(if (funcall dead-func extent)
 	    (let (newval)
-	      (mapcar (function (lambda (x)
-				  (if (not (funcall dead-func x))
-				      (setq newval (cons x newval)))))
-		      extent)
+	      (mapc (function (lambda (x)
+                                (if (not (funcall dead-func x))
+                                    (setq newval (cons x newval)))))
+                    extent)
 	      (setq default-mouse-track-extent (nreverse newval))))
       (if (funcall dead-func extent)
 	  (setq default-mouse-track-extent nil)))))
@@ -1775,7 +1770,8 @@ other mouse buttons."
 		 ;; left window side has slipped (right side cannot be
 		 ;; moved any further to the right, so enlarge-window
 		 ;; plays bad games with the left edge.
-		 (if (or (/= (count-windows) (length old-edges-all-windows))
+		 (if (or (not (eql (count-windows)
+				   (length old-edges-all-windows)))
 			 (/= old-left (car (window-pixel-edges window)))
 			 ;; This check is very hairy. We allow any number
 			 ;; of left edges to change, but only to the same
@@ -1783,22 +1779,24 @@ other mouse buttons."
 			 (let ((all-that-bad nil)
 			       (new-left-ok nil)
 			       (new-right-ok nil))
-			   (mapcar* (lambda (window old-edges)
-				      (let ((new (car (window-pixel-edges window))))
-					(if (/= new (car old-edges))
-					    (if (and new-left-ok
-						     (/= new-left-ok new))
-						(setq all-that-bad t)
-					      (setq new-left-ok new)))))
-				    (window-list) old-edges-all-windows)
-			   (mapcar* (lambda (window old-edges)
-				      (let ((new (caddr (window-pixel-edges window))))
-					(if (/= new (caddr old-edges))
-					    (if (and new-right-ok
-						     (/= new-right-ok new))
-						(setq all-that-bad t)
-					      (setq new-right-ok new)))))
-				    (window-list) old-edges-all-windows)
+			   (mapc (lambda (window old-edges)
+                                   (let ((new
+                                          (car (window-pixel-edges window))))
+                                     (if (/= new (car old-edges))
+                                         (if (and new-left-ok
+                                                  (/= new-left-ok new))
+                                             (setq all-that-bad t)
+                                           (setq new-left-ok new)))))
+                                 (window-list) old-edges-all-windows)
+			   (mapc (lambda (window old-edges)
+                                   (let ((new
+                                          (caddr (window-pixel-edges window))))
+                                     (if (/= new (caddr old-edges))
+                                         (if (and new-right-ok
+                                                  (/= new-right-ok new))
+                                             (setq all-that-bad t)
+                                           (setq new-right-ok new)))))
+                                 (window-list) old-edges-all-windows)
 			   all-that-bad))
 		     (set-window-configuration backup-conf)))))))))
 
