@@ -6,20 +6,18 @@
 
 ;; This file is part of XEmacs.
 
-;; XEmacs is free software; you can redistribute it and/or modify it
-;; under the terms of the GNU General Public License as published by
-;; the Free Software Foundation; either version 2, or (at your option)
-;; any later version.
+;; XEmacs is free software: you can redistribute it and/or modify it
+;; under the terms of the GNU General Public License as published by the
+;; Free Software Foundation, either version 3 of the License, or (at your
+;; option) any later version.
 
-;; XEmacs is distributed in the hope that it will be useful, but
-;; WITHOUT ANY WARRANTY; without even the implied warranty of
-;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-;; General Public License for more details.
+;; XEmacs is distributed in the hope that it will be useful, but WITHOUT
+;; ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+;; FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
+;; for more details.
 
 ;; You should have received a copy of the GNU General Public License
-;; along with XEmacs; see the file COPYING.  If not, write to the
-;; Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-;; Boston, MA 02110-1301, USA.
+;; along with XEmacs.  If not, see <http://www.gnu.org/licenses/>.
 
 ;;; Commentary:
 
@@ -65,31 +63,41 @@
 			       Installation-file-coding-system)
 			    Installation-string)
 
-      ;; Convince the byte compiler that, really, this file can't be encoded
-      ;; as binary. Ugh.
-      system-type (symbol-value (intern "\u0073ystem-type"))
+      ;; This used to be here to convince the byte-compiler to encode the
+      ;; output file using escape-quoted. This is no longer necessary, but
+      ;; keeping it here avoids doing the eval-when-compile clause below
+      ;; twice, which is a significant improvement.
+      system-type (symbol-value (intern "\u0073ystem-type")))
 
-      unicode-query-coding-skip-chars-arg
-      (eval-when-compile 
-        (when-fboundp 'map-charset-chars 
-          (loop
-            for charset in (charset-list)
-            with skip-chars-string = ""
-            do
-            (block no-ucs-mapping
-              (map-charset-chars
-               #'(lambda (begin end)
-                   (loop
-                     while (and begin (>= end begin))
-                     do
-                     (when (= -1 (char-to-unicode begin))
-                       (return-from no-ucs-mapping))
-                     (setq begin (int-to-char (1+ begin)))))
-               charset)
-              (setq skip-chars-string
-                    (concat skip-chars-string
-                            (charset-skip-chars-string charset))))
-            finally return skip-chars-string))))
+;; When this file is being compiled, all the charsets have been loaded, so
+;; we can construct the query-skip-chars-arg string correctly. 
+(set-unicode-query-skip-chars-args
+ (eval-when-compile 
+   (when-fboundp 'map-charset-chars 
+     (loop
+       for charset in (charset-list)
+       with skip-chars-string = ""
+       do
+       (block no-ucs-mapping
+         (map-charset-chars
+          #'(lambda (begin end)
+              (loop
+                while (and begin (>= end begin))
+                do
+                (when (= -1 (char-to-unicode begin))
+                  (return-from no-ucs-mapping))
+                (setq begin (int-to-char (1+ begin)))))
+          charset)
+         (setq skip-chars-string
+               (concat skip-chars-string
+                       (charset-skip-chars-string charset))))
+       finally return skip-chars-string)))
+ unicode-invalid-sequence-regexp-range
+ (eval-when-compile
+   (concat (loop
+             for i from #x80 to #xFF
+             collect (aref (decode-coding-string (int-char i)
+                                                 'utf-8) 0)))))
 
 ;; At this point in the dump, all the charsets have been loaded. Now, load
 ;; their Unicode mappings.
