@@ -8,20 +8,18 @@
 
 ;; This file is part of XEmacs.
 
-;; XEmacs is free software; you can redistribute it and/or modify it
-;; under the terms of the GNU General Public License as published by
-;; the Free Software Foundation; either version 2, or (at your option)
-;; any later version.
+;; XEmacs is free software: you can redistribute it and/or modify it
+;; under the terms of the GNU General Public License as published by the
+;; Free Software Foundation, either version 3 of the License, or (at your
+;; option) any later version.
 
-;; XEmacs is distributed in the hope that it will be useful, but
-;; WITHOUT ANY WARRANTY; without even the implied warranty of
-;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-;; General Public License for more details.
+;; XEmacs is distributed in the hope that it will be useful, but WITHOUT
+;; ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+;; FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
+;; for more details.
 
 ;; You should have received a copy of the GNU General Public License
-;; along with XEmacs; see the file COPYING.  If not, write to the Free
-;; Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
-;; 02111-1307, USA.
+;; along with XEmacs.  If not, see <http://www.gnu.org/licenses/>.
 
 ;;; Synched up with: FSF 21.3.
 
@@ -50,7 +48,7 @@ only considered as a candidate to match `paragraph-start' or
 
 Prefix argument says to turn mode on if positive, off if negative.
 When the mode is turned on, if there are newlines in the buffer but no hard
-newlines, ask the user whether to mark as hard any newlines preceeding a
+newlines, ask the user whether to mark as hard any newlines preceding a
 `paragraph-start' line.  From a program, second arg INSERT specifies whether
 to do this; it can be `never' to change nothing, t or `always' to force
 marking, `guess' to try to do the right thing with no questions, nil
@@ -145,18 +143,17 @@ without a period."
   :group 'fill)
 
 (defcustom sentence-end
-  (purecopy
-   ;; This is a bit stupid since it's not auto-updated when the
-   ;; other variables are changed, but it's still useful info.
-   (concat (if sentence-end-without-period "\\w  \\|")
-	   "[.?!"
-	   (if (featurep 'mule)
-	       (decode-coding-string "\033$B!#!%!)!*\033$A!##.#?#!\033$(0!$!%!)!*\033$(G!$!%!)!*\033(B" 'iso-2022-7bit)
-	     "")
-	   "][]\"')}]*"
-	   (if sentence-end-double-space
-	       "\\($\\| $\\|\t\\|  \\)" "\\($\\|[\t ]\\)")
-	   "[ \t\n]*"))
+  ;; This is a bit stupid since it's not auto-updated when the
+  ;; other variables are changed, but it's still useful info.
+  (concat (if sentence-end-without-period "\\w  \\|")
+	  "[.?!"
+	  (if (featurep 'mule)
+	      (decode-coding-string "\033$B!#!%!)!*\033$A!##.#?#!\033$(0!$!%!)!*\033$(G!$!%!)!*\033(B" 'iso-2022-7bit)
+	    "")
+	  "][]\"')}]*"
+	  (if sentence-end-double-space
+	      "\\($\\| $\\|\t\\|  \\)" "\\($\\|[\t ]\\)")
+	  "[ \t\n]*")
   "*Regexp describing the end of a sentence.
 The value includes the whitespace following the sentence.
 All paragraph boundaries also end sentences, regardless.
@@ -319,7 +316,7 @@ Returns the count of paragraphs left to move."
 	  (forward-char 1))
 	(if (< (point) (point-max))
 	    (goto-char start))))
-    (if-fboundp #'constrain-to-field
+    (if-fboundp 'constrain-to-field
         (constrain-to-field nil opoint t)
       (error
        'void-function
@@ -438,7 +435,7 @@ sentences.  A paragraph boundary also terminates a sentence."
 	   (skip-chars-backward " \t\n")
 	 (goto-char par-end)))
       (setq arg (1- arg)))
-    (if-fboundp #'constrain-to-field
+    (if-fboundp 'constrain-to-field
         (constrain-to-field nil opoint t)
       (error
        'void-function
