@@ -10,20 +10,18 @@
 
 ;; This file is part of XEmacs.
 
-;; XEmacs is free software; you can redistribute it and/or modify it
-;; under the terms of the GNU General Public License as published by
-;; the Free Software Foundation; either version 2, or (at your option)
-;; any later version.
+;; XEmacs is free software: you can redistribute it and/or modify it
+;; under the terms of the GNU General Public License as published by the
+;; Free Software Foundation, either version 3 of the License, or (at your
+;; option) any later version.
 
-;; XEmacs is distributed in the hope that it will be useful, but
-;; WITHOUT ANY WARRANTY; without even the implied warranty of
-;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-;; General Public License for more details.
+;; XEmacs is distributed in the hope that it will be useful, but WITHOUT
+;; ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+;; FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
+;; for more details.
 
 ;; You should have received a copy of the GNU General Public License
-;; along with XEmacs; see the file COPYING.  If not, write to the
-;; Free Software Foundation, 59 Temple Place - Suite 330,
-;; Boston, MA 02111-1307, USA.
+;; along with XEmacs.  If not, see <http://www.gnu.org/licenses/>.
 
 ;;; Synched up with: FSF 19.34.
 
@@ -722,9 +720,7 @@ If this is nil, no message will be displayed.")
 	(declare-fboundp (init-mule-at-startup)))
 
       (if (featurep 'toolbar)
-	  (if (featurep 'infodock)
-	      (require 'id-x-toolbar)
-	    (init-toolbar)))
+          (init-toolbar))
 
       ;; Create the initial device (which may be the already-created stdio
       ;; device, if we're noninteractive).
@@ -773,7 +769,7 @@ If this is nil, no message will be displayed.")
       ;; various other places. We could make *scratch* honour the user's
       ;; choice of whether font-locking is in place by adding a call to
       ;; font-lock-set-defaults in `lisp-interaction-mode'; but that'll
-      ;; break if `intial-major-mode' is anything else. 
+      ;; break if `initial-major-mode' is anything else. 
       ;;
       ;; So, despite what `font-lock-set-defaults'' docstring says, this
       ;; *is* where we should call it to have the user's choice of font-lock
@@ -1046,9 +1042,15 @@ and replace it by (setq load-home-init-file t) if `val' is non-nil."
 	    (load-user-init-file))
 	(condition-case nil
 	    (call-with-condition-handler
-		#'(lambda (__load_init_file_arg__)
+	       ((macro
+		 . (lambda (lambda-expression)
+		     ;; Be serious about information hiding here:
+		     (nsublis
+		      '((load-init-handler-arg . #:load-init-gZK6A36gTed))
+		      lambda-expression)))
+		#'(lambda (load-init-handler-arg)
 		    (let ((errstr (error-message-string
-				   __load_init_file_arg__)))
+				   load-init-handler-arg)))
 		      (message "Error in init file: %s" errstr)
 		      (lwarn 'initialization 'error
 			"\
@@ -1066,8 +1068,8 @@ to XEmacs to enter the debugger when the error occurs and investigate the
 exact problem."
 			user-init-file errstr
 			(backtrace-in-condition-handler-eliminating-handler
-			 '__load_init_file_arg__)))
-		    (setq init-file-had-error t))
+			 'load-init-handler-arg)))
+		    (setq init-file-had-error t)))
 		#'(lambda ()
 		    (if load-user-init-file-p
 			(load-user-init-file))
@@ -1524,17 +1526,23 @@ This function is idempotent, so call this as often as you like!"
 " inhibit-packages inhibit-site-lisp called-early)
 	   'external-debugging-output)
     (princ (format
-"emacs-roots:
+"invocation-directory: %S
+invocation-name: %S
+configure-prefix-directory: %S
+configure-exec-prefix-directory: %S
+emacs-roots:
 %S
 emacs-data-roots:
 %S
 user-init-directory: %S
 configure-package-path: %S
-" emacs-roots emacs-data-roots user-init-directory configure-package-path)
+" invocation-directory invocation-name
+  configure-prefix-directory configure-exec-prefix-directory
+  emacs-roots emacs-data-roots user-init-directory configure-package-path)
 	   'external-debugging-output)
     )
 
-  (setq lisp-directory (paths-find-lisp-directory emacs-roots))
+  (setq lisp-directory (paths-find-lisp-directory emacs-data-roots))
 
   (if debug-paths
       (princ (format "configure-lisp-directory and lisp-directory:\n%S\n%S\n"
@@ -1544,7 +1552,7 @@ configure-package-path: %S
   (if (featurep 'mule)
       (progn
 	(setq mule-lisp-directory
-	      (paths-find-mule-lisp-directory emacs-roots
+	      (paths-find-mule-lisp-directory emacs-data-roots
 					      lisp-directory))
 	(if debug-paths
 	    (princ (format "configure-mule-lisp-directory and mule-lisp-directory:\n%S\n%S\n"
@@ -1553,7 +1561,7 @@ configure-package-path: %S
     (setq mule-lisp-directory '()))
 
   (setq site-directory (and (null inhibit-site-lisp)
-			    (paths-find-site-lisp-directory emacs-roots)))
+			    (paths-find-site-lisp-directory emacs-data-roots)))
 
   (if (and debug-paths (null inhibit-site-lisp))
       (princ (format "configure-site-directory and site-directory:\n%S\n%S\n"
@@ -1597,7 +1605,7 @@ configure-package-path: %S
   (unless called-early
     (setq Info-directory-list
 	  (paths-construct-info-path
-	   emacs-roots
+	   emacs-data-roots
 	   early-package-hierarchies late-package-hierarchies last-package-hierarchies))
 
     (if debug-paths
@@ -1628,7 +1636,7 @@ configure-package-path: %S
 		       configure-doc-directory doc-directory)
 	       'external-debugging-output))
     
-    (setq data-directory (paths-find-data-directory emacs-roots))
+    (setq data-directory (paths-find-data-directory emacs-data-roots))
     
     (if debug-paths
 	(princ (format "configure-data-directory and data-directory:\n%S\n%S\n"
