@@ -8,20 +8,18 @@
 
 ;; This file is part of XEmacs.
 
-;; XEmacs is free software; you can redistribute it and/or modify it
-;; under the terms of the GNU General Public License as published by
-;; the Free Software Foundation; either version 2, or (at your option)
-;; any later version.
+;; XEmacs is free software: you can redistribute it and/or modify it
+;; under the terms of the GNU General Public License as published by the
+;; Free Software Foundation, either version 3 of the License, or (at your
+;; option) any later version.
 
-;; XEmacs is distributed in the hope that it will be useful, but
-;; WITHOUT ANY WARRANTY; without even the implied warranty of
-;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-;; General Public License for more details.
+;; XEmacs is distributed in the hope that it will be useful, but WITHOUT
+;; ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+;; FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
+;; for more details.
 
 ;; You should have received a copy of the GNU General Public License
-;; along with XEmacs; see the file COPYING.  If not, write to the
-;; Free Software Foundation, 59 Temple Place - Suite 330,
-;; Boston, MA 02111-1307, USA.
+;; along with XEmacs.  If not, see <http://www.gnu.org/licenses/>.
 
 ;;; Synched up with: Not synched.
 
@@ -310,17 +308,20 @@ Does not restore the value of point in current buffer."
 	(setq ,mapping (set-window-configuration/mapping ,window-config)))
       ,mapping)))
 
-(defun set-window-configuration (configuration)
-  "Set the configuration of windows and buffers as specified by CONFIGURATION.
-CONFIGURATION must be a value previously returned
-by `current-window-configuration'."
-  (set-window-configuration/mapping configuration)
-  nil) ; make sure nobody relies on mapping return value
-
-(defun set-window-configuration/mapping (configuration)
+(defun set-window-configuration (configuration &optional set-frame-size-p)
   "Set the configuration of windows and buffers as specified by CONFIGURATION.
 CONFIGURATION must be a value previously returned
 by `current-window-configuration'.
+If SET-FRAME-SIZE-P is true, the frame size is also restored.
+"
+  (set-window-configuration/mapping configuration set-frame-size-p)
+  nil) ; make sure nobody relies on mapping return value
+
+(defun set-window-configuration/mapping (configuration &optional set-frame-size-p)
+  "Set the configuration of windows and buffers as specified by CONFIGURATION.
+CONFIGURATION must be a value previously returned
+by `current-window-configuration'.
+If SET-FRAME-SIZE-P is true, the frame size is also restored.
 Return alist mapping old windows to new windows.
 This alist maps the originally captured windows to the windows that correspond
 to them in the restored configuration.  It does not include entries for
@@ -329,10 +330,11 @@ windows that have not changed identity."
     (if (and (frame-live-p frame)
 	     (not (window-configuration-equal configuration
 					      (current-window-configuration))))
-	(really-set-window-configuration frame configuration))))
+	(really-set-window-configuration frame configuration set-frame-size-p))))
 
-(defun really-set-window-configuration (frame configuration)
+(defun really-set-window-configuration (frame configuration set-frame-size-p)
   "Set the window configuration CONFIGURATION on live frame FRAME.
+If SET-FRAME-SIZE-P is true, the frame size is also restored.
 Return alist mapping old windows to new windows."
   ;; avoid potential temporary problems
   (setq window-min-width 0)
@@ -340,50 +342,50 @@ Return alist mapping old windows to new windows."
   (setq minibuffer-scroll-window nil)
 
   (frame-reduce-to-one-window frame)
-  (set-window-configuration-frame-size configuration)
+  (if set-frame-size-p
+      (set-window-configuration-frame-size configuration))
 
-  ; avoid setting these if they're already up-to-date
-  ; This also avoids potential inaccuracies in these settings --Mike
+  ;; avoid setting these if they're already up-to-date
+  ;; This also avoids potential inaccuracies in these settings --Mike
   (when window-configuration-includes-position
     (let ((left (window-configuration-frame-left configuration))
-          (top (window-configuration-frame-top configuration)))
+	  (top (window-configuration-frame-top configuration)))
       (if (not (equal left (frame-property frame 'left)))
-          (set-frame-property frame 'left left))
+	  (set-frame-property frame 'left left))
       (if (not (equal top (frame-property frame 'top)))
-          (set-frame-property frame 'top top))))
+	  (set-frame-property frame 'top top))))
 
   ;; these may have changed because of the delete
-  (let ((root-window (frame-root-window frame)))
-    (enlarge-window-pixels 
-     (- (window-configuration-minibuffer-pixel-height configuration)
-	(window-pixel-height (minibuffer-window frame)))
-     nil
-     (minibuffer-window frame))
+  (set-window-pixel-height (minibuffer-window frame)
+			   (window-configuration-minibuffer-pixel-height configuration))
 
-    ;; avoid that `set-window-point' will set the buffer's point for
-    ;; the selected window
-    (select-window (minibuffer-window frame))
+  ;; avoid that `set-window-point' will set the buffer's point for
+  ;; the selected window
+  (select-window (minibuffer-window frame))
 
-    (let ((window-configuration-current-window nil)
-	  (mapping (list nil))) ; poor man's box
+  (let ((window-configuration-current-window nil)
+	(mapping (list nil)))		; poor man's box
       
-      (declare (special window-configuration-current-window))
-      (restore-saved-window configuration
-			    root-window
-			    (window-configuration-saved-root-window configuration)
-			    'vertical
-			    mapping) 
-      (if window-configuration-current-window
-	  (select-window window-configuration-current-window))
+    (declare (special window-configuration-current-window))
+    (restore-saved-window configuration
+			  (frame-root-window frame)
+			  (window-configuration-saved-root-window configuration)
+			  'vertical
+			  mapping) 
+    (if window-configuration-current-window
+	(select-window window-configuration-current-window))
     
-      (setq window-min-width (window-configuration-min-width configuration))
-      (setq window-min-height (window-configuration-min-height configuration))
+    (setq window-min-width (window-configuration-min-width configuration))
+    (setq window-min-height (window-configuration-min-height configuration))
       
-      (let ((buffer (window-configuration-current-buffer configuration)))
-	(if (buffer-live-p buffer)
-	    (set-buffer buffer)
-	  (set-buffer (car (buffer-list)))))
-      (car mapping))))
+    (let ((buffer (window-configuration-current-buffer configuration)))
+      (if (buffer-live-p buffer)
+	  (set-buffer buffer)
+	(set-buffer (car (buffer-list)))))
+    ; this resets the window configuration so that the frame is filled
+    (if (not set-frame-size-p)
+	(set-frame-pixel-size frame (frame-pixel-width frame) (frame-pixel-height frame)))
+    (car mapping)))
 
 (defun set-window-configuration-frame-size (configuration)
   "Restore the frame size of a window configuration."
@@ -485,14 +487,10 @@ mapping, which this function will extend."
 	;; window to what we want. --Mike
 	(if (not (eq window (frame-root-window (window-frame window))))
 	    (progn
-	      (enlarge-window-pixels (- (saved-window-pixel-width saved-window)
-					(window-pixel-width window))
-				     t
-				     window)
-	      (enlarge-window-pixels (- (saved-window-pixel-height saved-window)
-					(window-pixel-height window))
-				     nil
-				     window)))
+	      (set-window-pixel-width window 
+				      (saved-window-pixel-width saved-window))
+	      (set-window-pixel-height window
+				       (saved-window-pixel-height saved-window))))
 	(set-window-hscroll window (saved-window-hscroll saved-window))
 	(set-modeline-hscroll window
 			      (saved-window-modeline-hscroll saved-window))
@@ -512,6 +510,15 @@ mapping, which this function will extend."
   "Compute the pixel height of SAVED-WINDOW."
   (- (saved-window-pixel-bottom saved-window)
      (saved-window-pixel-top saved-window)))
+
+(defun set-window-pixel-width (window width)
+  "Set the pixel width of WINDOW."
+  (enlarge-window-pixels (- width (window-pixel-width window)) t window))
+
+(defun set-window-pixel-height (window height)
+  "Set the pixel height of WINDOW."
+  (enlarge-window-pixels (- height (window-pixel-height window)) nil window))
+
 
 ;; The window-config stack is stored as a list in frame property
 ;; 'window-config-stack, with the most recent element at the front.
@@ -542,7 +549,7 @@ Each frame has its own window-config stack."
   (interactive)
   (let ((wc (or config (current-window-configuration)))
 	(stack (window-config-stack)))
-    (if (or (= 0 (undoable-stack-a-length stack))
+    (if (or (eql 0 (undoable-stack-a-length stack))
 	    (not (equal (undoable-stack-a-top stack) wc)))
 	(undoable-stack-push stack wc))))
 
