@@ -11,20 +11,18 @@
 
 ;; This file is part of XEmacs.
 
-;; XEmacs is free software; you can redistribute it and/or modify it
-;; under the terms of the GNU General Public License as published by
-;; the Free Software Foundation; either version 2, or (at your option)
-;; any later version.
+;; XEmacs is free software: you can redistribute it and/or modify it
+;; under the terms of the GNU General Public License as published by the
+;; Free Software Foundation, either version 3 of the License, or (at your
+;; option) any later version.
 
-;; XEmacs is distributed in the hope that it will be useful, but
-;; WITHOUT ANY WARRANTY; without even the implied warranty of
-;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-;; General Public License for more details.
+;; XEmacs is distributed in the hope that it will be useful, but WITHOUT
+;; ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+;; FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
+;; for more details.
 
 ;; You should have received a copy of the GNU General Public License
-;; along with XEmacs; see the file COPYING.  If not, write to the 
-;; Free Software Foundation, 59 Temple Place - Suite 330,
-;; Boston, MA 02111-1307, USA.
+;; along with XEmacs.  If not, see <http://www.gnu.org/licenses/>.
 ;;; Code:
 
 ;; #### - implement these...
@@ -119,14 +117,14 @@ It must be set at run-time.")
     (vector
      family
      (mapcar 
-      '(lambda (weight-symbol) 
-	 (let ((pair (assoc weight-symbol
-			    '((:light "Light")
-			      (:medium "Medium")
-			      (:demibold "Demibold")
-			      (:bold "Bold")
-			      (:black "Black")))))
-	   (if pair (cadr pair))))
+      (lambda (weight-symbol) 
+        (let ((pair (assoc weight-symbol
+                           '((:light "Light")
+                             (:medium "Medium")
+                             (:demibold "Demibold")
+                             (:bold "Bold")
+                             (:black "Black")))))
+          (if pair (cadr pair))))
       weights)
      '(0)
      nil)))
@@ -140,24 +138,24 @@ It must be set at run-time.")
 	  (vector
 	   (mapcar 'fc-make-font-menu-entry families)
 	   (mapcar 
-	    '(lambda (family)
-	       (vector family `(font-menu-set-font ,family nil nil)
-		       :style 'radio :active nil :selected nil))
+            (lambda (family)
+              (vector family `(font-menu-set-font ,family nil nil)
+                      :style 'radio :active nil :selected nil))
 	    families)
 	   (mapcar
-	    '(lambda (size)
-	       (vector
-		(number-to-string size)
-		`(font-menu-set-font nil nil ,size)
-		:style 'radio :active nil :selected nil))
+	    (lambda (size)
+              (vector
+               (number-to-string size)
+               `(font-menu-set-font nil nil ,size)
+               :style 'radio :active nil :selected nil))
 	    ;; common size list in decipoints, fontconfig wants points
 	    (mapcar (lambda (x) (/ x 10)) font-menu-common-sizes))
 	   (mapcar
-	    '(lambda (weight)
-	       (vector
-		weight
-		`(font-menu-set-font nil ,weight nil)
-		:style 'radio :active nil :selected nil))
+	    (lambda (weight)
+              (vector
+               weight
+               `(font-menu-set-font nil ,weight nil)
+               :style 'radio :active nil :selected nil))
 	    '("Light" "Medium" "Demibold" "Bold" "Black"))))
 	 ;; get or initialize the entry for device
 	 (dev-cache (or (assq device device-fonts-cache)
@@ -253,21 +251,21 @@ or if you change your font path, you can call this to re-initialize the menus."
      (vector
       cache
       (mapcar (lambda (x)
-		(vector x
+                (vector x
 			(list 'font-menu-set-font x nil nil)
-			':style 'radio ':active nil ':selected nil))
+			:style 'radio :active nil :selected nil))
 	      families)
       (mapcar (lambda (x)
 		(vector (if (/= 0 (% x 10))
 			    (number-to-string (/ x 10.0))
 			  (number-to-string (/ x 10)))
 			(list 'font-menu-set-font nil nil x)
-			':style 'radio ':active nil ':selected nil))
+			:style 'radio :active nil :selected nil))
 	      sizes)
       (mapcar (lambda (x)
 		(vector x
 			(list 'font-menu-set-font nil x nil)
-			':style 'radio ':active nil ':selected nil))
+			:style 'radio :active nil :selected nil))
 	      weights)))
     (cdr dev-cache)))
 
