@@ -9,20 +9,18 @@
 
 ;; This file is part of XEmacs.
 
-;; XEmacs is free software; you can redistribute it and/or modify it
-;; under the terms of the GNU General Public License as published by
-;; the Free Software Foundation; either version 2, or (at your option)
-;; any later version.
+;; XEmacs is free software: you can redistribute it and/or modify it
+;; under the terms of the GNU General Public License as published by the
+;; Free Software Foundation, either version 3 of the License, or (at your
+;; option) any later version.
 
-;; XEmacs is distributed in the hope that it will be useful, but
-;; WITHOUT ANY WARRANTY; without even the implied warranty of
-;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-;; General Public License for more details.
+;; XEmacs is distributed in the hope that it will be useful, but WITHOUT
+;; ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+;; FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
+;; for more details.
 
 ;; You should have received a copy of the GNU General Public License
-;; along with XEmacs; see the file COPYING.  If not, write to the
-;; Free Software Foundation, 59 Temple Place - Suite 330,
-;; Boston, MA 02111-1307, USA.
+;; along with XEmacs.  If not, see <http://www.gnu.org/licenses/>.
 
 ;;; Synched up with: Not synched.
 
@@ -92,7 +90,7 @@
                      compose-ring-map compose-caron-map compose-macron-map
                      compose-breve-map compose-dot-map
                      compose-doubleacute-map compose-ogonek-map
-                     compose-hook-map compose-horn-map)
+                     compose-hook-map compose-horn-map compose-stroke-map)
     do (autoload map "x-compose" nil t 'keymap))
 
   (loop 
@@ -208,7 +206,8 @@
          (dead-doubleacute      compose-doubleacute-map)
          (dead-ogonek           compose-ogonek-map)
          (dead-hook             compose-hook-map)
-         (dead-horn             compose-horn-map))
+         (dead-horn             compose-horn-map)
+	 (dead-stroke		compose-stroke-map))
     
     ;; Get the correct value for function-key-map
     with function-key-map = (symbol-value-in-console 'function-key-map
