@@ -3,20 +3,18 @@
 
  This file is part of XEmacs.
 
- XEmacs is free software; you can redistribute it and/or modify it
- under the terms of the GNU General Public License as published by the
- Free Software Foundation; either version 2, or (at your option) any
- later version.
+XEmacs is free software: you can redistribute it and/or modify it
+under the terms of the GNU General Public License as published by the
+Free Software Foundation, either version 3 of the License, or (at your
+option) any later version.
 
- XEmacs is distributed in the hope that it will be useful, but WITHOUT
- ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
- FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
- for more details.
+XEmacs is distributed in the hope that it will be useful, but WITHOUT
+ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
+for more details.
 
- You should have received a copy of the GNU General Public License
- along with XEmacs; see the file COPYING.  If not, write to
- the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- Boston, MA 02111-1307, USA.  */
+You should have received a copy of the GNU General Public License
+along with XEmacs.  If not, see <http://www.gnu.org/licenses/>. */
 
  /* Synched up with: Gcs 1.7 */
 
@@ -116,11 +114,6 @@ extern	Pixel	AllocGreyPixelC(Widget, XColor *fg, XColor *bg, int scale) ;
 extern	void	Draw3dBox(Widget w, int  x, int y, int wid, int hgt, int s,
 			GC topgc, GC botgc) ;
 
-#if XtSpecificationRelease < 5
-extern	GC	XtAllocateGC(Widget, int depth, u_long mask,
-			XGCValues *, u_long dynamic, u_long dontcare) ;
-#endif
-
 #else
 
 extern	GC	AllocFgGC() ;
@@ -135,10 +128,6 @@ extern	Pixel	AllocGreyPixel() ;
 extern	Pixel	AllocGreyPixelC() ;
 extern	void	Draw3dBox() ;
 
-#if XtSpecificationRelease < 5
-extern	GC	XtAllocateGC() ;
-#endif
-
 #endif
 
 #endif	/* GCS_H */
