@@ -1,15 +1,15 @@
-/* CANNA interface -*- coding: euc-jp -*-
+/* CANNA interface -*- coding: utf-8 -*-
 
    Copyright (C) 1995 Free Software Foundation, Inc.
    Copyright (C) 1995 Sun Microsystems, Inc.
-   Copyright (C) 2005 Ben Wing.
+   Copyright (C) 2005, 2010 Ben Wing.
 
 This file is part of XEmacs.
 
-XEmacs is free software; you can redistribute it and/or modify it
+XEmacs is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
-Free Software Foundation; either version 2, or (at your option) any
-later version.
+Free Software Foundation, either version 3 of the License, or (at your
+option) any later version.
 
 XEmacs is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
@@ -17,9 +17,7 @@ FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 for more details.
 
 You should have received a copy of the GNU General Public License
-along with XEmacs; see the file COPYING.  If not, write to
-the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-Boston, MA 02111-1307, USA.  */
+along with XEmacs.  If not, see <http://www.gnu.org/licenses/>. */
 
 /* Synched up with: Mule 2.3.  Not in FSF. */
 
@@ -199,8 +197,8 @@ static Lisp_Object kanjiYomiList (int, int);
 static Lisp_Object CANNA_mode_keys (void);
 static Lisp_Object Qeuc_jp;
 
-#define make_euc_string(p, len) make_ext_string ((Extbyte *) p, len, Qeuc_jp)
-#define build_euc_string(p) build_ext_string ((Extbyte *) p, Qeuc_jp)
+#define make_euc_string(p, len) make_extstring ((Extbyte *) p, len, Qeuc_jp)
+#define build_euc_string(p) build_extstring ((Extbyte *) p, Qeuc_jp)
 
 #ifdef CANNA_MULE
 static int euc_jp_num_chars (unsigned char *, int);
@@ -237,10 +235,10 @@ storeResults (char *buf, int len, jrKanjiStatus *ks)
     }
   else
     {
-      /* ꤷʸ (the confirmed string) */
+      /* 確定した文字列 (the confirmed string) */
       Vcanna_kakutei_string = make_euc_string (buf, len);
       val = make_int (len);
-      /* ꤷʸɤߤξ...
+      /* 確定した文字列の読みの情報...
 	 (info about the reading of the confirmed string) */
       Vcanna_kakutei_yomi = Vcanna_kakutei_romaji = Qnil;
       if (ks->info & KanjiYomiInfo)
@@ -253,20 +251,20 @@ storeResults (char *buf, int len, jrKanjiStatus *ks)
 	      int yomilen2;
 
 	      Vcanna_kakutei_yomi =
-		make_euc_string (p, yomilen); /* ɤ (reading) */
+		make_euc_string (p, yomilen); /* 読み (reading) */
 	      p += yomilen + 1;
 	      yomilen2 = strlen (p);
 	      if (len + yomilen + yomilen2 + 2 < KEYTOSTRSIZE)
 		{
 		  Vcanna_kakutei_romaji =
 		    make_euc_string (p, yomilen2);
-				/* ޻ (romanization) */
+				/* ローマ字 (romanization) */
 		}
 	    }
 	}
 
 
-      /* ɽʸǤ
+      /* 候補表示の文字列です。
 	 (string for displaying candidate translations) */
       Vcanna_henkan_string = Qnil;
       if (ks->length >= 0)
@@ -293,7 +291,7 @@ storeResults (char *buf, int len, jrKanjiStatus *ks)
 #endif /* CANNA_MULE */
 	}
 
-      /* ξ (information about the echo area menu) */
+      /* 一覧の情報 (information about the echo area menu) */
       Vcanna_ichiran_string = Qnil;
       if (ks->info & KanjiGLineInfo && ks->gline.length >= 0)
 	{
@@ -311,14 +309,14 @@ storeResults (char *buf, int len, jrKanjiStatus *ks)
 #endif /* CANNA_MULE */
 	}
 
-      /* ⡼ɤξ (mode information) */
+      /* モードの情報 (mode information) */
       Vcanna_mode_string = Qnil;
       if (ks->info & KanjiModeInfo)
 	{
 	  Vcanna_mode_string = build_euc_string ((Extbyte *) ks->mode);
 	}
 
-      /* ¾ξ (other information) */
+      /* その他の情報 (other information) */
       canna_empty_info = (ks->info & KanjiEmptyInfo) ? 1 : 0;
       canna_through_info = (ks->info & KanjiThroughInfo) ? 1 : 0;
     }
@@ -334,7 +332,7 @@ No separator will be used otherwise.
        (num))
 {
   /* This is actually a Boolean! */
-  char *kugiri; /* ʸڤ򤹤뤫 (display clause separator?) */
+  char *kugiri; /* 文節区切りをするか？ (display clause separator?) */
 
   kugiri = NILP (num) ? (char *) 0 : (char *) 1;
 
@@ -367,7 +365,7 @@ If nil is specified for each arg, the default value will be used.
   char **p, **q;
 
   /* This is actually a Boolean! */
-  char *kugiri; /* ʸڤ򤹤뤫 (display clause separator?) */
+  char *kugiri; /* 文節区切りをするか？ (display clause separator?) */
 
   IRCP_context = -1;
 
@@ -390,7 +388,7 @@ If nil is specified for each arg, the default value will be used.
       CHECK_STRING (server);
       jrKanjiControl (0, KC_SETSERVERNAME,
 		      /* !!#### Check encoding */
-		      NEW_LISP_STRING_TO_EXTERNAL (server, Qnative));
+		      LISP_STRING_TO_EXTERNAL (server, Qnative));
     }
 
   if (NILP (rcfile))
@@ -401,18 +399,12 @@ If nil is specified for each arg, the default value will be used.
     {
       CHECK_STRING (rcfile);
       jrKanjiControl (0, KC_SETINITFILENAME,
-		      NEW_LISP_STRING_TO_EXTERNAL (rcfile, Qfile_name));
+		      LISP_STRING_TO_EXTERNAL (rcfile, Qfile_name));
     }
 
   {
     char **warning = (char **) 0;
-#ifdef nec_ews_svr4
-    stop_polling ();
-#endif /* nec_ews_svr4 */
     res = jrKanjiControl (0, KC_INITIALIZE, (char *) &warning);
-#ifdef nec_ews_svr4
-    start_polling ();
-#endif /* nec_ews_svr4 */
     val = Qnil;
     if (warning)
       {
@@ -430,7 +422,7 @@ If nil is specified for each arg, the default value will be used.
   if (res == -1)
     {
       val = Fcons (build_euc_string (jrKanjiError), val);
-      /* ˥饤ǼԤ硣 (on initialization failure) */
+      /* イニシャライズで失敗した場合。 (on initialization failure) */
       return Fcons (Qnil, val);
     }
   else
@@ -450,12 +442,12 @@ If nil is specified for each arg, the default value will be used.
 #ifndef CANNA_MULE
       jrKanjiControl (0, KC_INHIBITHANKAKUKANA, (char *) 1);
 #else
-      /* mule äȾѥʤȤ
+      /* mule だったら半角カタカナも使える
 	 (Mule can use half-width katakana) */
       if (canna_inhibit_hankakukana)
 	jrKanjiControl (0, KC_INHIBITHANKAKUKANA, (char *) 1);
 #endif
-      jrKanjiControl (0, KC_YOMIINFO, (char *) 2); /* : ޻ޤ֤
+      jrKanjiControl (0, KC_YOMIINFO, (char *) 2); /* ※２: ローマ字まで返す
 						      (*2: return to
 						      romanized form) */
       val = Fcons (Qnil, val);
@@ -567,7 +559,7 @@ Store yomi characters as a YOMI of kana-to-kanji conversion.
   Extbyte *ext;
 
   CHECK_STRING (yomi);
-  LISP_STRING_TO_EXTERNAL (yomi, ext, Qeuc_jp);
+  ext = LISP_STRING_TO_EXTERNAL (yomi, Qeuc_jp);
   strncpy (key_buffer, ext, sizeof (key_buffer));
   key_buffer[sizeof (key_buffer) - 1] = '\0';
   ks.length = strlen (key_buffer);
@@ -579,7 +571,7 @@ Store yomi characters as a YOMI of kana-to-kanji conversion.
   else
     {
       CHECK_STRING (roma);
-      LISP_STRING_TO_EXTERNAL (roma, ext, Qeuc_jp);
+      ext = LISP_STRING_TO_EXTERNAL (roma, Qeuc_jp);
       ks.mode = (unsigned char *) (key_buffer + ks.length + 1);
       strncpy (key_buffer + ks.length + 1, ext,
 	       sizeof (key_buffer) - ks.length - 1);
@@ -587,7 +579,7 @@ Store yomi characters as a YOMI of kana-to-kanji conversion.
     }
 
   ks.echoStr = (unsigned char *) key_buffer;
-  ksv.buffer = (unsigned char *) key_buffer; /*  (return value) */
+  ksv.buffer = (unsigned char *) key_buffer; /* 返値用 (return value) */
   ksv.bytes_buffer = KEYTOSTRSIZE;
   ksv.ks = &ks;
 
@@ -635,7 +627,7 @@ Parse customize string.
   Extbyte *ext;
 
   CHECK_STRING (str);
-  LISP_STRING_TO_EXTERNAL (str, ext, Qeuc_jp);
+  ext = LISP_STRING_TO_EXTERNAL (str, Qeuc_jp);
   strncpy (key_buffer, ext, sizeof (key_buffer));
   key_buffer[sizeof (key_buffer) - 1] = '\0';
   p = (Extbyte **) key_buffer;
@@ -667,7 +659,7 @@ Get current mode string.
 
 #define RKBUFSIZE 1024
 
-static unsigned char yomibuf[RKBUFSIZE];
+static UExtbyte yomibuf[RKBUFSIZE];
 static short kugiri[RKBUFSIZE / 2];
 
 static int
@@ -722,11 +714,11 @@ Clause separator is set.
       return Qnil;
     }
 
-  LISP_STRING_TO_EXTERNAL (yomi, ext, Qeuc_jp);
+  ext = LISP_STRING_TO_EXTERNAL (yomi, Qeuc_jp);
   strncpy ((char *) yomibuf, ext, sizeof (yomibuf));
   yomibuf[sizeof (yomibuf) - 1] = '\0';
 
-  nbun = RkBgnBun (IRCP_context, yomibuf, strlen ((char *) yomibuf),
+  nbun = RkBgnBun (IRCP_context, (char *) yomibuf, strlen ((char *) yomibuf),
 		   (RK_XFER << RK_XFERBITS) | RK_KFER);
 
   return kanjiYomiList (IRCP_context, nbun);
@@ -842,7 +834,7 @@ End conversion.
     {
       return Qnil;
     }
-  RkEndBun (IRCP_context, 1); /* ؽϤĤǤԤäɤΤʤΤ
+  RkEndBun (IRCP_context, 1); /* 学習はいつでも行って良いものなのか？
 				 (is it OK to invoke learning function
 				 at arbitrary times?) */
   return Qt;
