@@ -1,14 +1,14 @@
 /* LDAP client interface for XEmacs.
    Copyright (C) 1998 Free Software Foundation, Inc.
-   Copyright (C) 2004 Ben Wing.
+   Copyright (C) 2004, 2005, 2010 Ben Wing.
    
 
 This file is part of XEmacs.
 
-XEmacs is free software; you can redistribute it and/or modify it
+XEmacs is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
-Free Software Foundation; either version 2, or (at your option) any
-later version.
+Free Software Foundation, either version 3 of the License, or (at your
+option) any later version.
 
 XEmacs is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
@@ -16,9 +16,7 @@ FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 for more details.
 
 You should have received a copy of the GNU General Public License
-along with XEmacs; see the file COPYING.  If not, write to
-the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-Boston, MA 02111-1307, USA.  */
+along with XEmacs.  If not, see <http://www.gnu.org/licenses/>. */
 
 /* Synched up with: Not in FSF. */
 
@@ -99,7 +97,7 @@ signal_ldap_error (LDAP *ld,
 #endif
     }
   invalid_operation ("LDAP error",
-		     build_ext_string (ldap_err2string (ldap_err), Qnative));
+		     build_extstring (ldap_err2string (ldap_err), Qnative));
 }
 
 
@@ -130,18 +128,18 @@ print_ldap (Lisp_Object obj, Lisp_Object printcharfun, int UNUSED (escapeflag))
   Lisp_LDAP *ldap = XLDAP (obj);
 
   if (print_readably)
-    printing_unreadable_object ("#<ldap %s>", XSTRING_DATA (ldap->host));
+    printing_unreadable_object_fmt ("#<ldap %s>", XSTRING_DATA (ldap->host));
 
   write_fmt_string_lisp (printcharfun, "#<ldap %S", 1, ldap->host);
   if (!ldap->ld)
-    write_c_string (printcharfun,"(dead) ");
+    write_ascstring (printcharfun,"(dead) ");
   write_fmt_string (printcharfun, " 0x%lx>", (long)ldap);
 }
 
 static Lisp_LDAP *
 allocate_ldap (void)
 {
-  Lisp_LDAP *ldap = ALLOC_LCRECORD_TYPE (Lisp_LDAP, &lrecord_ldap);
+  Lisp_LDAP *ldap = XLDAP (ALLOC_NORMAL_LISP_OBJECT (ldap));
 
   ldap->ld = NULL;
   ldap->host = Qnil;
@@ -149,23 +147,19 @@ allocate_ldap (void)
 }
 
 static void
-finalize_ldap (void *header, int for_disksave)
+finalize_ldap (Lisp_Object obj)
 {
-  Lisp_LDAP *ldap = (Lisp_LDAP *) header;
-
-  if (for_disksave)
-    invalid_operation ("Can't dump an emacs containing LDAP objects",
-			 make_ldap (ldap));
+  Lisp_LDAP *ldap = XLDAP (obj);
 
   if (ldap->ld)
     ldap_unbind (ldap->ld);
   ldap->ld = NULL;
 }
 
-DEFINE_LRECORD_IMPLEMENTATION ("ldap", ldap, 0,
-                               mark_ldap, print_ldap, finalize_ldap,
-                               NULL, NULL, ldap_description, Lisp_LDAP);
-
+DEFINE_NODUMP_LISP_OBJECT ("ldap", ldap, mark_ldap,
+			   print_ldap, finalize_ldap,
+			   NULL, NULL, ldap_description,
+			   Lisp_LDAP);
 
 /************************************************************************/
 /*                        Basic ldap accessors                          */
@@ -263,13 +257,13 @@ the LDAP library XEmacs was compiled with: `simple', `krbv41' and `krbv42'.
 	else if (EQ (keyword, Qbinddn))
 	  {
 	    CHECK_STRING (value);
-	    LISP_STRING_TO_EXTERNAL (value, ldap_binddn, Qnative);
+	    ldap_binddn = LISP_STRING_TO_EXTERNAL (value, Qnative);
 	  }
 	/* Password */
 	else if (EQ (keyword, Qpasswd))
 	  {
 	    CHECK_STRING (value);
-	    LISP_STRING_TO_EXTERNAL (value, ldap_password, Qnative);
+	    ldap_password = LISP_STRING_TO_EXTERNAL (value, Qnative);
 	  }
 	/* Deref */
 	else if (EQ (keyword, Qderef))
@@ -307,7 +301,7 @@ the LDAP library XEmacs was compiled with: `simple', `krbv41' and `krbv42'.
 
   /* Connect to the server and bind */
   slow_down_interrupts ();
-  ld = ldap_open (NEW_LISP_STRING_TO_EXTERNAL (host, Qnative), ldap_port);
+  ld = ldap_open (LISP_STRING_TO_EXTERNAL (host, Qnative), ldap_port);
   speed_up_interrupts ();
 
   if (ld == NULL )
@@ -346,7 +340,7 @@ the LDAP library XEmacs was compiled with: `simple', `krbv41' and `krbv42'.
   if (err != LDAP_SUCCESS)
     {
       signal_error (Qprocess_error, "Failed binding to the server",
-		    build_ext_string (ldap_err2string (err), Qnative));
+		    build_extstring (ldap_err2string (err), Qnative));
     }
 
   ldap = allocate_ldap ();
@@ -425,7 +419,7 @@ entry according to the value of WITHDN.
   LDAP *ld;
   LDAPMessage *e;
   BerElement *ptr;
-  Extbyte *a, *dn;
+  Extbyte *a, *dn, *bs, *filt;
   int i, rc;
   int  matches;
   struct ldap_unwind_struct unwind;
@@ -486,7 +480,7 @@ entry according to the value of WITHDN.
 	EXTERNAL_LIST_LOOP_2 (current, attrs)
 	  {
 	    CHECK_STRING (current);
-	    LISP_STRING_TO_EXTERNAL (current, ldap_attributes[i], Qnative);
+	    ldap_attributes[i] = LISP_STRING_TO_EXTERNAL (current, Qnative);
 	    ++i;
 	  }
       }
@@ -497,14 +491,12 @@ entry according to the value of WITHDN.
   CHECK_SYMBOL (attrsonly);
 
   /* Perform the search */
-  if (ldap_search (ld,
-                   NILP (base) ? "" :
-		   NEW_LISP_STRING_TO_EXTERNAL (base, Qnative),
-                   ldap_scope,
-                   NILP (filter) ? "" :
-		   NEW_LISP_STRING_TO_EXTERNAL (filter, Qnative),
-                   ldap_attributes,
-                   NILP (attrsonly) ? 0 : 1)
+  bs = NILP (base) ? (Extbyte *) "" :
+    LISP_STRING_TO_EXTERNAL (base, Qnative);
+  filt = NILP (filter) ? (Extbyte *) "" :
+    LISP_STRING_TO_EXTERNAL (filter, Qnative);
+  if (ldap_search (ld, bs, ldap_scope, filt, ldap_attributes,
+		   NILP (attrsonly) ? 0 : 1)
       == -1)
     {
       signal_ldap_error (ld, NULL, 0);
@@ -537,19 +529,19 @@ entry according to the value of WITHDN.
           dn = ldap_get_dn (ld, e);
           if (dn == NULL)
             signal_ldap_error (ld, e, 0);
-          entry = Fcons (build_ext_string (dn, Qnative), Qnil);
+          entry = Fcons (build_extstring (dn, Qnative), Qnil);
         }
       for (a = ldap_first_attribute (ld, e, &ptr);
            a != NULL;
            a = ldap_next_attribute (ld, e, ptr))
         {
-          list = Fcons (build_ext_string (a, Qnative), Qnil);
+          list = Fcons (build_extstring (a, Qnative), Qnil);
           unwind.vals = ldap_get_values_len (ld, e, a);
           if (unwind.vals != NULL)
             {
               for (i = 0; unwind.vals[i] != NULL; i++)
                 {
-                  list = Fcons (make_ext_string ((Extbyte *) unwind.vals[i]->bv_val,
+                  list = Fcons (make_extstring ((Extbyte *) unwind.vals[i]->bv_val,
                                                  unwind.vals[i]->bv_len,
                                                  Qnative),
                                 list);
@@ -618,7 +610,6 @@ containing attribute/value string pairs.
   int rc;
   int i, j;
   Elemcount len;
-
   Lisp_Object values  = Qnil;
   struct gcpro gcpro1;
 
@@ -648,8 +639,8 @@ containing attribute/value string pairs.
 	CHECK_CONS (current);
 	CHECK_STRING (XCAR (current));
 	ldap_mods_ptrs[i] = &(ldap_mods[i]);
-	LISP_STRING_TO_EXTERNAL (XCAR (current), ldap_mods[i].mod_type,
-				 Qnative);
+	ldap_mods[i].mod_type =
+	  LISP_STRING_TO_EXTERNAL (XCAR (current), Qnative);
 	ldap_mods[i].mod_op = LDAP_MOD_ADD | LDAP_MOD_BVALUES;
 	values = XCDR (current);
 	if (CONSP (values))
@@ -690,7 +681,7 @@ containing attribute/value string pairs.
       }
   }
   ldap_mods_ptrs[i] = NULL;
-  rc = ldap_add_s (ld, NEW_LISP_STRING_TO_EXTERNAL (dn, Qnative),
+  rc = ldap_add_s (ld, LISP_STRING_TO_EXTERNAL (dn, Qnative),
 		   ldap_mods_ptrs);
   if (rc != LDAP_SUCCESS)
     signal_ldap_error (ld, NULL, rc);
@@ -717,7 +708,6 @@ or `replace'. ATTR is the LDAP attribute type to modify.
   int i, j, rc;
   Lisp_Object mod_op;
   Elemcount len;
-
   Lisp_Object values  = Qnil;
   struct gcpro gcpro1;
 
@@ -758,8 +748,8 @@ or `replace'. ATTR is the LDAP attribute type to modify.
 	  invalid_constant ("Invalid LDAP modification type", mod_op);
 	current = XCDR (current);
 	CHECK_STRING (XCAR (current));
-	LISP_STRING_TO_EXTERNAL (XCAR (current), ldap_mods[i].mod_type,
-				 Qnative);
+	ldap_mods[i].mod_type =
+	  LISP_STRING_TO_EXTERNAL (XCAR (current), Qnative);
 	values = XCDR (current);
 	len = (Elemcount) XINT (Flength (values));
 	bervals = alloca_array (struct berval, len);
@@ -783,7 +773,7 @@ or `replace'. ATTR is the LDAP attribute type to modify.
       }
   }
   ldap_mods_ptrs[i] = NULL;
-  rc = ldap_modify_s (ld, NEW_LISP_STRING_TO_EXTERNAL (dn, Qnative),
+  rc = ldap_modify_s (ld, LISP_STRING_TO_EXTERNAL (dn, Qnative),
 		      ldap_mods_ptrs);
   if (rc != LDAP_SUCCESS)
     signal_ldap_error (ld, NULL, rc);
@@ -808,7 +798,7 @@ DN is the distinguished name of the entry to delete.
   ld = XLDAP (ldap)->ld;
   CHECK_STRING (dn);
 
-  rc = ldap_delete_s (ld, NEW_LISP_STRING_TO_EXTERNAL (dn, Qnative));
+  rc = ldap_delete_s (ld, LISP_STRING_TO_EXTERNAL (dn, Qnative));
   if (rc != LDAP_SUCCESS)
     signal_ldap_error (ld, NULL, rc);
 
@@ -818,7 +808,7 @@ DN is the distinguished name of the entry to delete.
 void
 syms_of_eldap (void)
 {
-  INIT_LRECORD_IMPLEMENTATION (ldap);
+  INIT_LISP_OBJECT (ldap);
 
   DEFSYMBOL (Qeldap);
   DEFSYMBOL (Qldapp);
@@ -880,7 +870,7 @@ void
 unload_eldap (void)
 {
   /* Remove defined types */
-  UNDEF_LRECORD_IMPLEMENTATION (ldap);
+  UNDEF_LISP_OBJECT (ldap);
 
   /* Remove staticpro'ing of symbols */
   unstaticpro_nodump (&Qeldap);
