@@ -6,6 +6,21 @@
   Author:  SL Baur <steve@xemacs.org>
   Maintainer:  SL Baur <steve@xemacs.org>
 
+This file is part of XEmacs.
+
+XEmacs is free software: you can redistribute it and/or modify it
+under the terms of the GNU General Public License as published by the
+Free Software Foundation, either version 3 of the License, or (at your
+option) any later version.
+
+XEmacs is distributed in the hope that it will be useful, but WITHOUT
+ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
+for more details.
+
+You should have received a copy of the GNU General Public License
+along with XEmacs.  If not, see <http://www.gnu.org/licenses/>.
+
 Please send patches to this file to me first before submitting them to
 xemacs-patches.
 
@@ -90,14 +105,15 @@ TODO (in rough order of priority):
    interface to lcrecord handling has changed with 21.2, so unfortunately
    we will need a few snippets of backwards compatibility code.
 */
-#if (EMACS_MAJOR_VERSION == 21) && (EMACS_MINOR_VERSION < 2)
+#if (EMACS_MAJOR_VERSION == 21) && (EMACS_MINOR_VERSION <= 1)
 #define RUNNING_XEMACS_21_1 1
+#elif (EMACS_MAJOR_VERSION == 21) && (EMACS_MINOR_VERSION <= 4)
+#define RUNNING_XEMACS_21_4 1
 #endif
 
 /* #define POSTGRES_LO_IMPORT_IS_VOID 1 */
 
 #include "lisp.h"
-#include "sysdep.h"
 
 #include "buffer.h"
 #include "postgresql.h"
@@ -105,6 +121,8 @@ TODO (in rough order of priority):
 #ifdef HAVE_SHLIB
 # include "emodules.h"
 #endif
+#include "sysdep.h"
+#include "sysfile.h"
 
 #ifdef RUNNING_XEMACS_21_1 /* handle interface changes */
 #define PG_OS_CODING FORMAT_FILENAME
@@ -118,15 +136,36 @@ TODO (in rough order of priority):
 Lisp_Object Vpg_coding_system;
 #endif
 
-#define CHECK_LIVE_CONNECTION(P) do {					\
-	if (!P || (PQstatus (P) != CONNECTION_OK)) {			\
-		char *e = "bad value";					\
-		if (P) e = PQerrorMessage (P);				\
-	 signal_ferror (Qprocess_error, "dead connection [%s]", e);	\
-	} } while (0)
-#define PUKE_IF_NULL(p) do {						 \
-	if (!p) signal_error (Qinvalid_argument, "bad value", Qunbound); \
-	} while (0)
+#define CHECK_LIVE_CONNECTION(P)					\
+do									\
+{									\
+  if (!P || (PQstatus (P) != CONNECTION_OK))				\
+    {									\
+      Lisp_Object err;							\
+									\
+      if (P)								\
+	err = build_extstring (PQerrorMessage (P), PG_OS_CODING);	\
+      else								\
+	err = build_msg_string ("Bad value");				\
+      signal_error (Qprocess_error, "Dead connection", err);		\
+    }									\
+}									\
+while (0)
+
+#define PUKE_IF_NULL(p)							\
+do									\
+{									\
+  if (!p) signal_error (Qinvalid_argument, "Bad value", Qunbound);	\
+}									\
+while (0)
+
+#define SIGNAL_ERROR(p, reason)						\
+do									\
+{									\
+  signal_error (Qprocess_error, reason,					\
+		build_extstring (PQerrorMessage (p), PG_OS_CODING));	\
+}									\
+while (0)  
 
 static Lisp_Object VXPGHOST;
 static Lisp_Object VXPGUSER;
@@ -203,7 +242,7 @@ print_pgconn (Lisp_Object obj, Lisp_Object printcharfun,
   char buf[256];
   PGconn *P;
   ConnStatusType cst;
-  char *host="", *db="", *user="", *port="";
+  const char *host="", *db="", *user="", *port="";
 
   P = (XPGCONN (obj))->pgconn;
 
@@ -229,9 +268,9 @@ print_pgconn (Lisp_Object obj, Lisp_Object printcharfun,
     strcpy (buf, "#<PGconn connecting>"); /* evil! */
 
   if (print_readably)
-    printing_unreadable_object ("%s", buf);
+    printing_unreadable_object_fmt ("%s", buf);
   else
-    write_c_string (printcharfun, buf);
+    write_cistring (printcharfun, buf);
 }
 
 static Lisp_PGconn *
@@ -240,14 +279,18 @@ allocate_pgconn (void)
 #ifdef RUNNING_XEMACS_21_1
   Lisp_PGconn *pgconn = ALLOC_LCRECORD_TYPE (Lisp_PGconn,
 					     lrecord_pgconn);
-#else
+#elif defined (RUNNING_XEMACS_21_4)
   Lisp_PGconn *pgconn = ALLOC_LCRECORD_TYPE (Lisp_PGconn,
 					     &lrecord_pgconn);
+#else
+  Lisp_PGconn *pgconn = XPGCONN (ALLOC_NORMAL_LISP_OBJECT (pgconn));
 #endif
   pgconn->pgconn = (PGconn *)NULL;
   return pgconn;
 }
 
+#ifdef RUNNING_XEMACS_21_4
+
 static void
 finalize_pgconn (void *header, int for_disksave)
 {
@@ -264,18 +307,41 @@ finalize_pgconn (void *header, int for_disksave)
     }
 }
 
+#else /* not RUNNING_XEMACS_21_4 */
+
+static void
+finalize_pgconn (Lisp_Object obj)
+{
+  Lisp_PGconn *pgconn = XPGCONN (obj);
+
+  if (pgconn->pgconn)
+    {
+      PQfinish (pgconn->pgconn);
+      pgconn->pgconn = (PGconn *)NULL;
+    }
+}
+
+#endif /* (not) RUNNING_XEMACS_21_4 */
+
 #ifdef RUNNING_XEMACS_21_1
 DEFINE_LRECORD_IMPLEMENTATION ("pgconn", pgconn,
 			       mark_pgconn, print_pgconn, finalize_pgconn,
 			       NULL, NULL,
 			       Lisp_PGconn);
-#else
+#elif defined (RUNNING_XEMACS_21_4)
 DEFINE_LRECORD_IMPLEMENTATION ("pgconn", pgconn,
 			       0, /*dumpable-flag*/
 			       mark_pgconn, print_pgconn, finalize_pgconn,
 			       NULL, NULL,
 			       pgconn_description,
 			       Lisp_PGconn);
+#else
+DEFINE_NODUMP_LISP_OBJECT ("pgconn", pgconn,
+			   mark_pgconn, print_pgconn,
+			   finalize_pgconn,
+			   NULL, NULL,
+			   pgconn_description,
+			   Lisp_PGconn);
 #endif
 /****/
 
@@ -338,7 +404,7 @@ print_pgresult (Lisp_Object obj, Lisp_Object printcharfun,
 		   PQcmdStatus (res));
 	  break;
 	default:
-notuples:
+	notuples:
 	  /* No counts to print */
 	  sprintf (buf, RESULT_DEFAULT_FMT, /* evil! */
 		   PQresStatus (PQresultStatus (res)),
@@ -350,9 +416,9 @@ notuples:
     strcpy (buf, "#<PGresult DEAD>"); /* evil! */
 
   if (print_readably)
-    printing_unreadable_object ("%s", buf);
+    printing_unreadable_object_fmt ("%s", buf);
   else
-    write_c_string (printcharfun, buf);
+    write_cistring (printcharfun, buf);
 }
 
 #undef RESULT_TUPLES_FMT
@@ -365,14 +431,18 @@ allocate_pgresult (void)
 #ifdef RUNNING_XEMACS_21_1
   Lisp_PGresult *pgresult = ALLOC_LCRECORD_TYPE (Lisp_PGresult,
 						 lrecord_pgresult);
-#else
+#elif defined (RUNNING_XEMACS_21_4)
   Lisp_PGresult *pgresult = ALLOC_LCRECORD_TYPE (Lisp_PGresult,
 						 &lrecord_pgresult);
+#else
+  Lisp_PGresult *pgresult = XPGRESULT (ALLOC_NORMAL_LISP_OBJECT (pgresult));
 #endif
   pgresult->pgresult = (PGresult *)NULL;
   return pgresult;
 }
 
+#ifdef RUNNING_XEMACS_21_4
+
 static void
 finalize_pgresult (void *header, int for_disksave)
 {
@@ -389,18 +459,40 @@ finalize_pgresult (void *header, int for_disksave)
     }
 }
 
+#else /* not RUNNING_XEMACS_21_4 */
+
+static void
+finalize_pgresult (Lisp_Object obj)
+{
+  Lisp_PGresult *pgresult = XPGRESULT (obj);
+
+  if (pgresult->pgresult)
+    {
+      PQclear (pgresult->pgresult);
+      pgresult->pgresult = (PGresult *)NULL;
+    }
+}
+
+#endif /* (not) RUNNING_XEMACS_21_4 */
+
 #ifdef RUNNING_XEMACS_21_1
 DEFINE_LRECORD_IMPLEMENTATION ("pgresult", pgresult,
 			       mark_pgresult, print_pgresult, finalize_pgresult,
 			       NULL, NULL,
 			       Lisp_PGresult);
-#else
+#elif defined (RUNNING_XEMACS_21_4)
 DEFINE_LRECORD_IMPLEMENTATION ("pgresult", pgresult,
 			       0, /*dumpable-flag*/
 			       mark_pgresult, print_pgresult, finalize_pgresult,
 			       NULL, NULL,
 			       pgresult_description,
 			       Lisp_PGresult);
+#else
+DEFINE_NODUMP_LISP_OBJECT ("pgresult", pgresult,
+			   mark_pgresult, print_pgresult, finalize_pgresult,
+			   NULL, NULL,
+			   pgresult_description,
+			   Lisp_PGresult);
 #endif
 
 /***********************/
@@ -431,23 +523,25 @@ Return a connection default structure.
 
   pcio = PQconndefaults();
   if (!pcio) return Qnil; /* can never happen in libpq-7.0 */
-  temp = list1 (Fcons (build_ext_string (pcio[0].keyword, PG_OS_CODING),
-		       Fcons (build_ext_string (pcio[0].envvar, PG_OS_CODING),
-			      Fcons (build_ext_string (pcio[0].compiled, PG_OS_CODING),
-				     Fcons (build_ext_string (pcio[0].val, PG_OS_CODING),
-					    Fcons (build_ext_string (pcio[0].label, PG_OS_CODING),
-						   Fcons (build_ext_string (pcio[0].dispchar, PG_OS_CODING),
-							  Fcons (make_int (pcio[0].dispsize), Qnil))))))));
+  temp =
+    list1 (nconc2 (list4 (build_extstring (pcio[0].keyword, PG_OS_CODING),
+			  build_extstring (pcio[0].envvar, PG_OS_CODING),
+			  build_extstring (pcio[0].compiled, PG_OS_CODING),
+			  build_extstring (pcio[0].val, PG_OS_CODING)),
+		   list3 (build_extstring (pcio[0].label, PG_OS_CODING),
+			  build_extstring (pcio[0].dispchar, PG_OS_CODING),
+			  make_int (pcio[0].dispsize))));
 
   for (i = 1; pcio[i].keyword; i++)
     {
-      temp1 = list1 (Fcons (build_ext_string (pcio[i].keyword, PG_OS_CODING),
-			    Fcons (build_ext_string (pcio[i].envvar, PG_OS_CODING),
-				   Fcons (build_ext_string (pcio[i].compiled, PG_OS_CODING),
-					  Fcons (build_ext_string (pcio[i].val, PG_OS_CODING),
-						 Fcons (build_ext_string (pcio[i].label, PG_OS_CODING),
-							Fcons (build_ext_string (pcio[i].dispchar, PG_OS_CODING),
-							       Fcons (make_int (pcio[i].dispsize), Qnil))))))));
+      temp1 =
+	list1 (nconc2 (list4 (build_extstring (pcio[i].keyword, PG_OS_CODING),
+			      build_extstring (pcio[i].envvar, PG_OS_CODING),
+			      build_extstring (pcio[i].compiled, PG_OS_CODING),
+			      build_extstring (pcio[i].val, PG_OS_CODING)),
+		       list3 (build_extstring (pcio[i].label, PG_OS_CODING),
+			      build_extstring (pcio[i].dispchar, PG_OS_CODING),
+			      make_int (pcio[i].dispsize))));
       {
 	Lisp_Object args[2];
 	args[0] = temp;
@@ -464,46 +558,59 @@ Return a connection default structure.
 PGconn *PQconnectdb(const char *conninfo)
 */
 
-/* ###autoload */
-DEFUN ("pq-connectdb", Fpq_connectdb, 1, 1, 0, /*
-Make a new connection to a PostgreSQL backend.
-*/
-	(conninfo))
+#ifdef HAVE_POSTGRESQLV7
+#define USED_IF_V7(x) x
+#else
+#define USED_IF_V7(x) UNUSED (x)
+#endif
+
+static Lisp_Object
+postgresql_connect (Lisp_Object conninfo, int USED_IF_V7 (async))
 {
   PGconn *P;
   Lisp_PGconn *lisp_pgconn;
-  char *error_message = "Out of Memory?";
-  char *c_conninfo;
 
   CHECK_STRING (conninfo);
 
-  TO_EXTERNAL_FORMAT(LISP_STRING, conninfo,
-		     C_STRING_ALLOCA, c_conninfo, Qnative);
-  P = PQconnectdb (c_conninfo);
+  P = (
+#ifdef HAVE_POSTGRESQLV7
+       async ? PQconnectStart : 
+#endif
+       PQconnectdb)
+    (LISP_STRING_TO_EXTERNAL (conninfo, PG_OS_CODING));
   if (P && (PQstatus (P) == CONNECTION_OK))
     {
-      (void)PQsetNoticeProcessor (P, xemacs_notice_processor, NULL);
-      lisp_pgconn = allocate_pgconn();
+      (void) PQsetNoticeProcessor (P, xemacs_notice_processor, NULL);
+      lisp_pgconn = allocate_pgconn ();
       lisp_pgconn->pgconn = P;
       return make_pgconn (lisp_pgconn);
     }
   else
     {
       /* Connection failed.  Destroy the connection and signal an error. */
-      char buf[BLCKSZ];
-      strcpy (buf, error_message);
+
+      Lisp_Object errmsg;
       if (P)
 	{
-	  /* storage for the error message gets erased when call PQfinish */
-	  /* so we must temporarily stash it somewhere */
-	  strncpy (buf, PQerrorMessage (P), sizeof (buf));
-	  buf[sizeof (buf) - 1] = '\0';
+	  errmsg = build_extstring (PQerrorMessage (P), PG_OS_CODING);
 	  PQfinish (P);
 	}
-      signal_ferror (Qprocess_error, "libpq: %s", buf);
+      else
+	errmsg = build_msg_string ("Out of Memory?");
+      signal_error (Qprocess_error, "Connecting to PostGreSQL backend",
+		    errmsg);
     }
 }
 
+/* ###autoload */
+DEFUN ("pq-connectdb", Fpq_connectdb, 1, 1, 0, /*
+Make a new connection to a PostgreSQL backend.
+*/
+	(conninfo))
+{
+  return postgresql_connect (conninfo, 0);
+}
+
 /* PQconnectStart Makes a new asynchronous connection to a backend.
 PGconn *PQconnectStart(const char *conninfo)
 */
@@ -515,37 +622,7 @@ Make a new asynchronous connection to a PostgreSQL backend.
 */
 	(conninfo))
 {
-  PGconn *P;
-  Lisp_PGconn *lisp_pgconn;
-  char *error_message = "Out of Memory?";
-  char *c_conninfo;
-
-  CHECK_STRING (conninfo);
-  TO_EXTERNAL_FORMAT (LISP_STRING, conninfo,
-		      C_STRING_ALLOCA, c_conninfo, Qnative);
-  P = PQconnectStart (c_conninfo);
-
-  if (P && (PQstatus (P) != CONNECTION_BAD))
-    {
-      (void)PQsetNoticeProcessor (P, xemacs_notice_processor, NULL);
-      lisp_pgconn = allocate_pgconn();
-      lisp_pgconn->pgconn = P;
-
-      return make_pgconn (lisp_pgconn);
-    }
-  else
-    {
-      /* capture the error message before destroying the object */
-      char buf[BLCKSZ];
-      strcpy (buf, error_message);
-      if (P)
-	{
-	  strncpy (buf, PQerrorMessage (P), sizeof (buf));
-	  buf[sizeof (buf) - 1] = '\0';
-	  PQfinish (P);
-	}
-      signal_ferror (Qprocess_error, "libpq: %s", buf);
-    }
+  return postgresql_connect (conninfo, 1);
 }
 
 DEFUN ("pq-connect-poll", Fpq_connect_poll, 1, 1, 0, /*
@@ -566,10 +643,7 @@ Poll an asynchronous connection for completion
     {
     case PGRES_POLLING_FAILED:
       /* Something Bad has happened */
-      {
-	char *e = PQerrorMessage (P);
-	signal_ferror (Qprocess_error, "libpq: %s", e);
-      }
+      SIGNAL_ERROR (P, "Polling asynchronous connection");
     case PGRES_POLLING_OK:
       return Qpgres_polling_ok;
     case PGRES_POLLING_READING:
@@ -732,10 +806,7 @@ Reset connection to the backend asynchronously.
   CHECK_LIVE_CONNECTION (P);
 
   if (PQresetStart (P)) return Qt;
-  {
-    char *e = PQerrorMessage (P);
-    signal_ferror (Qprocess_error, "libpq: %s", e);
-  }
+  SIGNAL_ERROR (P, "Resetting connection");
 }
 
 DEFUN ("pq-reset-poll", Fpq_reset_poll, 1, 1, 0, /*
@@ -755,11 +826,7 @@ Poll an asynchronous reset for completion.
   switch (polling_status)
     {
     case PGRES_POLLING_FAILED:
-      /* Something Bad has happened */
-      {
-	char *e = PQerrorMessage (P);
-	signal_ferror (Qprocess_error, "libpq: %s", e);
-      }
+      SIGNAL_ERROR (P, "Polling asynchronous reset");
     case PGRES_POLLING_OK:
       return Qpgres_polling_ok;
     case PGRES_POLLING_READING:
@@ -821,22 +888,22 @@ pq::backend-pid   Process ID of backend process
     /* PQdb Returns the database name of the connection.
        char *PQdb(PGconn *conn)
      */
-    return build_ext_string (PQdb(P), PG_OS_CODING);
+    return build_extstring (PQdb(P), PG_OS_CODING);
   else if (EQ (field, Qpquser))
     /* PQuser Returns the user name of the connection.
        char *PQuser(PGconn *conn)
      */
-    return build_ext_string (PQuser(P), PG_OS_CODING);
+    return build_extstring (PQuser(P), PG_OS_CODING);
   else if (EQ (field, Qpqpass))
     /* PQpass Returns the password of the connection.
        char *PQpass(PGconn *conn)
      */
-    return build_ext_string (PQpass(P), PG_OS_CODING);
+    return build_extstring (PQpass(P), PG_OS_CODING);
   else if (EQ (field, Qpqhost))
     /* PQhost Returns the server host name of the connection.
        char *PQhost(PGconn *conn)
      */
-    return build_ext_string (PQhost(P), PG_OS_CODING);
+    return build_extstring (PQhost(P), PG_OS_CODING);
   else if (EQ (field, Qpqport))
     {
       char *p;
@@ -852,12 +919,12 @@ pq::backend-pid   Process ID of backend process
     /* PQtty Returns the debug tty of the connection.
        char *PQtty(PGconn *conn)
      */
-    return build_ext_string (PQtty(P), PG_OS_CODING);
+    return build_extstring (PQtty(P), PG_OS_CODING);
   else if (EQ (field, Qpqoptions))
   /* PQoptions Returns the backend options used in the connection.
      char *PQoptions(PGconn *conn)
    */
-    return build_ext_string (PQoptions(P), PG_OS_CODING);
+    return build_extstring (PQoptions(P), PG_OS_CODING);
   else if (EQ (field, Qpqstatus))
     {
       ConnStatusType cst;
@@ -886,7 +953,7 @@ pq::backend-pid   Process ID of backend process
        by an operation on the connection.
        char *PQerrorMessage(PGconn* conn);
      */
-    return build_ext_string (PQerrorMessage(P), PG_OS_CODING);
+    return build_extstring (PQerrorMessage(P), PG_OS_CODING);
   else if (EQ (field, Qpqbackendpid))
     /* PQbackendPID Returns the process ID of the backend server handling
        this connection.
@@ -919,7 +986,8 @@ Submit a query to Postgres and wait for the result.
 
   R = PQexec (P, c_query);
   {
-    char *tag, buf[BLCKSZ];
+    const Ascbyte *tag;
+    char buf[BLCKSZ];
 
     if (!R) out_of_memory ("query: out of memory", Qunbound);
     else
@@ -970,7 +1038,7 @@ Returns: t if successfully submitted
 		      C_STRING_ALLOCA, c_query, Qnative);
 
   if (PQsendQuery (P, c_query)) return Qt;
-  else signal_ferror (Qprocess_error, "async query: %s", PQerrorMessage (P));
+  else SIGNAL_ERROR (P, "Sending asynchronous query");
 }
 
 DEFUN ("pq-get-result", Fpq_get_result, 1, 1, 0, /*
@@ -992,7 +1060,8 @@ NIL is returned when no more query work remains.
   if (!R) return Qnil; /* not an error, there's no more data to get */
 
   {
-    char *tag, buf[BLCKSZ];
+    const Ascbyte *tag;
+    char buf[BLCKSZ];
 
     switch (PQresultStatus (R))
       {
@@ -1044,7 +1113,9 @@ Return result status of the query.
   case PGRES_FATAL_ERROR: return Qpgres_fatal_error;
   default:
     /* they've added a new field we don't know about */
-    signal_ferror (Qprocess_error, "Help!  Unknown exec status code %08x from backend!", est);
+    signal_ferror (Qprocess_error,
+		   "Help!  Unknown exec status code %08x from backend!",
+		   est);
   }
 }
 
@@ -1059,7 +1130,7 @@ Return stringified result status of the query.
   R = (XPGRESULT (result))->pgresult;
   PUKE_IF_NULL (R);
 
-  return build_ext_string (PQresStatus (PQresultStatus (R)), PG_OS_CODING);
+  return build_extstring (PQresStatus (PQresultStatus (R)), PG_OS_CODING);
 }
 
 /* Sundry PGresult accessor functions */
@@ -1074,7 +1145,7 @@ Return last message associated with the query.
   R = (XPGRESULT (result))->pgresult;
   PUKE_IF_NULL (R);
 
-  return build_ext_string (PQresultErrorMessage (R), PG_OS_CODING);
+  return build_extstring (PQresultErrorMessage (R), PG_OS_CODING);
 }
 
 DEFUN ("pq-ntuples", Fpq_ntuples, 1, 1, 0, /*
@@ -1132,7 +1203,7 @@ Field indices start at 0.
   R = (XPGRESULT (result))->pgresult;
   PUKE_IF_NULL (R);
 
-  return build_ext_string (PQfname (R, XINT (field_index)), PG_OS_CODING);
+  return build_extstring (PQfname (R, XINT (field_index)), PG_OS_CODING);
 }
 
 DEFUN ("pq-fnumber", Fpq_fnumber, 2, 2, 0, /*
@@ -1217,7 +1288,7 @@ Tuple and field indices start at 0.
   R = (XPGRESULT (result))->pgresult;
   PUKE_IF_NULL (R);
 
-  return build_ext_string (PQgetvalue (R, XINT (tup_num), XINT (field_num)),
+  return build_extstring (PQgetvalue (R, XINT (tup_num), XINT (field_num)),
 			   PG_OS_CODING);
 }
 
@@ -1267,7 +1338,7 @@ Returns the command status string from the SQL command that generated the result
   R = (XPGRESULT (result))->pgresult;
   PUKE_IF_NULL (R);
 
-  return build_ext_string (PQcmdStatus (R), PG_OS_CODING);
+  return build_extstring (PQcmdStatus (R), PG_OS_CODING);
 }
 
 DEFUN ("pq-cmd-tuples", Fpq_cmd_tuples, 1, 1, 0, /*
@@ -1281,7 +1352,7 @@ Returns the number of rows affected by the SQL command.
   R = (XPGRESULT (result))->pgresult;
   PUKE_IF_NULL (R);
 
-  return build_ext_string (PQcmdTuples (R), PG_OS_CODING);
+  return build_extstring (PQcmdTuples (R), PG_OS_CODING);
 }
 
 DEFUN ("pq-oid-value", Fpq_oid_value, 1, 1, 0, /*
@@ -1375,7 +1446,7 @@ aren't any notifications to process.
   {
     Lisp_Object temp;
 
-    temp = list2 (build_ext_string (PGN->relname, PG_OS_CODING), make_int (PGN->be_pid));
+    temp = list2 (build_extstring (PGN->relname, PG_OS_CODING), make_int (PGN->be_pid));
     free ((void *)PGN);
     return temp;
   }
@@ -1405,9 +1476,7 @@ DEFUN ("pq-lo-import", Fpq_lo_import, 2, 2, 0, /*
   P = (XPGCONN (conn))->pgconn;
   CHECK_LIVE_CONNECTION (P);
 
-  TO_EXTERNAL_FORMAT (LISP_STRING, filename,
-		      C_STRING_ALLOCA, c_filename,
-		      Qfile_name);
+  LISP_PATHNAME_CONVERT_OUT (filename, c_filename);
 
   return make_int ((int)lo_import (P, c_filename));
 }
@@ -1426,8 +1495,7 @@ DEFUN ("pq-lo-export", Fpq_lo_export, 3, 3, 0, /*
   P = (XPGCONN (conn))->pgconn;
   CHECK_LIVE_CONNECTION (P);
 
-  TO_EXTERNAL_FORMAT (LISP_STRING, filename,
-		      C_STRING_ALLOCA, c_filename, Qfile_name);
+  LISP_PATHNAME_CONVERT_OUT (filename, c_filename);
 
   return make_int ((int)lo_export (P, XINT (oid), c_filename));
 }
@@ -1489,7 +1557,7 @@ returned.
 
   ret = PQgetline (P, buffer, sizeof (buffer));
 
-  return Fcons (make_int (ret), build_ext_string (buffer, PG_OS_CODING));
+  return Fcons (make_int (ret), build_extstring (buffer, PG_OS_CODING));
 }
 
 DEFUN ("pq-put-line", Fpq_put_line, 2, 2, 0, /*
@@ -1558,7 +1626,7 @@ The returned string is *not* null-terminated.
   if (ret == -1) return Qt; /* done! */
   else if (!ret) return Qnil; /* no data yet */
   else return Fcons (make_int (ret),
-		     make_ext_string ((Extbyte *) buffer, ret, PG_OS_CODING));
+		     make_extstring ((Extbyte *) buffer, ret, PG_OS_CODING));
 }
 
 DEFUN ("pq-put-nbytes", Fpq_put_nbytes, 2, 2, 0, /*
@@ -1599,8 +1667,8 @@ void
 syms_of_postgresql(void)
 {
 #ifndef RUNNING_XEMACS_21_1
-  INIT_LRECORD_IMPLEMENTATION (pgconn);
-  INIT_LRECORD_IMPLEMENTATION (pgresult);
+  INIT_LISP_OBJECT (pgconn);
+  INIT_LISP_OBJECT (pgresult);
 #endif
   DEFSYMBOL (Qpostgresql);
 
@@ -1828,7 +1896,7 @@ init_postgresql_from_environment (void)
 
 #define FROB(envvar, var)			\
   if ((p = egetenv (envvar)))			\
-    var = build_intstring (p);	\
+    var = build_istring (p);	\
   else						\
     var = Qnil
 
@@ -1872,8 +1940,8 @@ unload_postgresql (void)
 {
 #ifndef RUNNING_XEMACS_21_1
   /* Remove defined types */
-  UNDEF_LRECORD_IMPLEMENTATION (pgconn);
-  UNDEF_LRECORD_IMPLEMENTATION (pgresult);
+  UNDEF_LISP_OBJECT (pgconn);
+  UNDEF_LISP_OBJECT (pgresult);
 #endif
 
   /* Remove staticpro'ing of symbols */
