@@ -4,10 +4,10 @@
 
 This file is part of XEmacs.
 
-XEmacs is free software; you can redistribute it and/or modify it
+XEmacs is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
-Free Software Foundation; either version 2, or (at your option) any
-later version.
+Free Software Foundation, either version 3 of the License, or (at your
+option) any later version.
 
 XEmacs is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
@@ -15,9 +15,7 @@ FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 for more details.
 
 You should have received a copy of the GNU General Public License
-along with XEmacs; see the file COPYING.  If not, write to
-the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-Boston, MA 02111-1307, USA.  */
+along with XEmacs.  If not, see <http://www.gnu.org/licenses/>. */
 
 /* Synched up with: FSF 19.30, Mule 2.0. */
 
@@ -86,6 +84,9 @@ to make ARG be the prefix argument when `foo' is called as a command.
 The "call" to `interactive' is actually a declaration rather than a function;
  it tells `call-interactively' how to read arguments
  to pass to the function.
+The interactive form must appear at the top level of the function body.  If
+ it is wrapped in a `let' or `progn' or similar, Lisp will not even realize
+ the function is an interactive command!
 When actually called, `interactive' just returns nil.
 
 The argument of `interactive' is usually a string containing a code letter
@@ -203,35 +204,13 @@ Z -- (and current-prefix-arg (fboundp 'read-coding-system)
                                (eq window (active-minibuffer-window)))))
               (error "Attempt to select inactive minibuffer window"))
             (select window)))))
-`_' (setq zmacs-region-stays t)
-
-*/
+`_' (setq zmacs-region-stays t) *//* FIXME: moving end of previous comment
+to a separate line causes docstring lossage! */
        (UNUSED (args)))
 {
   return Qnil;
 }
 
-/* Originally, this was just a function -- but `custom' used a
-   garden-variety version, so why not make it a subr?  */
-/* #### Move it to another file! */
-DEFUN ("quote-maybe", Fquote_maybe, 1, 1, 0, /*
-Quote EXPR if it is not self quoting.
-*/
-       (expr))
-{
-  return ((NILP (expr)
-	   || EQ (expr, Qt)
-	   || INTP (expr)
-	   || FLOATP (expr)
-	   || CHARP (expr)
-	   || STRINGP (expr)
-	   || VECTORP (expr)
-	   || KEYWORDP (expr)
-	   || BIT_VECTORP (expr)
-	   || (CONSP (expr) && EQ (XCAR (expr), Qlambda)))
-	  ? expr : list2 (Qquote, expr));
-}
-
 /* Modify EXPR by quotifying each element (except the first).  */
 static Lisp_Object
 quotify_args (Lisp_Object expr)
@@ -398,7 +377,7 @@ when reading the arguments.
 
       GCPRO3 (function, specs, input);
       /* Compute the arg values using the user's expression.  */
-      specs = Feval (specs);
+      specs = IGNORE_MULTIPLE_VALUES (Feval (specs));
       if (EQ (record_flag, Qlambda)) /* XEmacs addition */
 	{
 	  UNGCPRO;
@@ -914,7 +893,7 @@ when reading the arguments.
 	    {
 	      Lisp_Object tem = call1 (Qread_expression, PROMPT ());
 	      /* visargs[argnum] = Fprin1_to_string (tem, Qnil); */
-	      args[argnum] = Feval (tem);
+              args[argnum] = IGNORE_MULTIPLE_VALUES (Feval (tem));
 	      arg_from_tty = 1;
 	      break;
 	    }
@@ -1046,7 +1025,6 @@ syms_of_callint (void)
 #endif
 
   DEFSUBR (Finteractive);
-  DEFSUBR (Fquote_maybe);
   DEFSUBR (Fcall_interactively);
   DEFSUBR (Fprefix_numeric_value);
 }
