@@ -3,10 +3,10 @@
 
 This file is part of XEmacs.
 
-XEmacs is free software; you can redistribute it and/or modify it
+XEmacs is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
-Free Software Foundation; either version 2, or (at your option) any
-later version.
+Free Software Foundation, either version 3 of the License, or (at your
+option) any later version.
 
 XEmacs is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
@@ -14,9 +14,7 @@ FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 for more details.
 
 You should have received a copy of the GNU General Public License
-along with XEmacs; see the file COPYING.  If not, write to
-the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-Boston, MA 02111-1307, USA.  */
+along with XEmacs.  If not, see <http://www.gnu.org/licenses/>. */
 
 /* Synched up with: Not in FSF. */
 
@@ -212,7 +210,7 @@ mswindows_perhaps_init_unseen_key_defaults (struct console *UNUSED(con),
 	 can use eq as the test without worrying. */
       Vmswindows_seen_characters = make_lisp_hash_table (128,
 							 HASH_TABLE_NON_WEAK,
-							 HASH_TABLE_EQ);
+							 Qeq);
     }
   /* Might give the user an opaque error if make_lisp_hash_table fails,
      but it shouldn't crash. */
@@ -347,8 +345,7 @@ write_string_to_mswindows_debugging_output (const Ibyte *str, Bytecount len)
   if (initialized && !inhibit_non_essential_conversion_operations)
     {
       TO_EXTERNAL_FORMAT (DATA, (str, len),
-			  C_STRING_ALLOCA, extptr,
-			  Qmswindows_tstr);
+			  C_STRING_ALLOCA, extptr, Qmswindows_tstr);
       qxeOutputDebugString (extptr);
     }
   else
@@ -578,12 +575,12 @@ no effect.  */
     }
 
   CHECK_STRING (message_);
-  LISP_STRING_TO_TSTR (message_, msgout);
+  msgout = LISP_STRING_TO_TSTR (message_);
   
   if (!NILP (title))
     {
       CHECK_STRING (title);
-      LISP_STRING_TO_TSTR (title, titleout);
+      titleout = LISP_STRING_TO_TSTR (title);
     }
 
   {
