@@ -3,10 +3,10 @@
 
 This file is part of XEmacs.
 
-XEmacs is free software; you can redistribute it and/or modify it
+XEmacs is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
-Free Software Foundation; either version 2, or (at your option) any
-later version.
+Free Software Foundation, either version 3 of the License, or (at your
+option) any later version.
 
 XEmacs is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
@@ -14,9 +14,7 @@ FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 for more details.
 
 You should have received a copy of the GNU General Public License
-along with XEmacs; see the file COPYING.  If not, write to
-the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-Boston, MA 02111-1307, USA.  */
+along with XEmacs.  If not, see <http://www.gnu.org/licenses/>. */
 
 /* Synched up with: Not in FSF. */
 
@@ -43,8 +41,6 @@ DEFINE_CONSOLE_TYPE (x);
 
 int wedge_metacity;	/* nonzero means update WM_HINTS always */
 
-extern void x_has_keysym (KeySym, Lisp_Object, int);
-
 static int
 x_initially_selected_for_input (struct console *UNUSED (con))
 {
@@ -162,12 +158,12 @@ get_display_arg_connection (void)
       /* assert: display_arg is only set if we found the display
 	 arg earlier so we can't fail to find it now. */
       assert (disp_name != NULL);
-      conn = build_ext_string (disp_name, Qcommand_argument_encoding);
+      conn = build_extstring (disp_name, Qcommand_argument_encoding);
       free_argc_argv (argv);
       return conn;
     }
   else
-    return build_ext_string (XDisplayName (0), Qx_display_name_encoding);
+    return build_extstring (XDisplayName (0), Qx_display_name_encoding);
 }
 
 /* "semi-canonicalize" means convert to a nicer form for printing, but
@@ -202,7 +198,7 @@ x_semi_canonicalize_console_connection (Lisp_Object connection,
 
   /* Check for a couple of standard special cases */
   if (string_ichar (connection, 0) == ':')
-    connection = concat2 (build_string ("localhost"), connection);
+    connection = concat2 (build_ascstring ("localhost"), connection);
   else
     {
       /* connection =~ s/^unix:/localhost:/; */
@@ -217,7 +213,7 @@ x_semi_canonicalize_console_connection (Lisp_Object connection,
 	  INC_IBYTEPTR (p);
 	}
 
-      connection = concat2 (build_string ("localhost:"),
+      connection = concat2 (build_ascstring ("localhost:"),
 			    make_string (p, end - p));
     }
  ok:
@@ -242,7 +238,7 @@ x_canonicalize_console_connection (Lisp_Object connection, Error_Behavior errb)
 
     split_up_display_spec (connection, &hostname_length, &display_length,
 			   &screen_length);
-    hostname = Fsubstring (connection, Qzero, make_int (hostname_length));
+    hostname = Fsubseq (connection, Qzero, make_int (hostname_length));
     hostname = canonicalize_host_name (hostname);
     connection = concat2 (hostname,
 			  make_string (XSTRING_DATA (connection)
@@ -277,7 +273,7 @@ x_semi_canonicalize_device_connection (Lisp_Object connection,
 			 &screen_length);
 
   if (!screen_length)
-    connection = concat2 (connection, build_string (".0"));
+    connection = concat2 (connection, build_ascstring (".0"));
   RETURN_UNGCPRO (connection);
 }
 
@@ -335,7 +331,7 @@ x_perhaps_init_unseen_key_defaults (struct console *con, Lisp_Object key)
       CHECK_CHAR(key);
 
       buf[set_itext_ichar(buf, XCHAR(key))] = '\0';
-      key_name = build_intstring (buf);
+      key_name = build_istring (buf);
 
       /* We need to do the lookup and compare later, because we can't check
 	 the Qcharacter_of_keysym property belonging to an actual character. */
@@ -347,7 +343,7 @@ x_perhaps_init_unseen_key_defaults (struct console *con, Lisp_Object key)
       return Qnil;
     }
 
-  LISP_STRING_TO_EXTERNAL (key_name, keysym_ext, Qctext);
+  keysym_ext = LISP_STRING_TO_EXTERNAL (key_name, Qctext);
   xkeysym = XStringToKeysym(keysym_ext);
   if (NoSymbol == xkeysym) 
     {
