@@ -1,14 +1,14 @@
 /* device functions for mswindows.
    Copyright (C) 1994, 1995 Board of Trustees, University of Illinois.
    Copyright (C) 1994, 1995 Free Software Foundation, Inc.
-   Copyright (C) 2000, 2001, 2002 Ben Wing.
+   Copyright (C) 2000, 2001, 2002, 2010 Ben Wing.
 
 This file is part of XEmacs.
 
-XEmacs is free software; you can redistribute it and/or modify it
+XEmacs is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
-Free Software Foundation; either version 2, or (at your option) any
-later version.
+Free Software Foundation, either version 3 of the License, or (at your
+option) any later version.
 
 XEmacs is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
@@ -16,9 +16,7 @@ FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 for more details.
 
 You should have received a copy of the GNU General Public License
-along with XEmacs; see the file COPYING.  If not, write to
-the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-Boston, MA 02111-1307, USA.  */
+along with XEmacs.  If not, see <http://www.gnu.org/licenses/>. */
 
 /* Synched up with: Not in FSF. */
 
@@ -45,7 +43,7 @@ Boston, MA 02111-1307, USA.  */
 
 #include "console-msw-impl.h"
 #include "console-stream.h"
-#include "objects-msw.h"
+#include "fontcolor-msw.h"
 
 #include "sysdep.h"
 
@@ -75,11 +73,9 @@ static const struct memory_description mswindows_device_data_description_1 [] =
 };
 
 #ifdef NEW_GC
-DEFINE_LRECORD_IMPLEMENTATION ("mswindows-device", mswindows_device,
-			       1, /*dumpable-flag*/
-                               0, 0, 0, 0, 0,
-			       mswindows_device_data_description_1,
-			       Lisp_Mswindows_Device);
+DEFINE_DUMPABLE_INTERNAL_LISP_OBJECT ("mswindows-device", mswindows_device,
+				      0, mswindows_device_data_description_1,
+				      Lisp_Mswindows_Device);
 #else /* not NEW_GC */
 extern const struct sized_memory_description mswindows_device_data_description;
 
@@ -96,11 +92,9 @@ static const struct memory_description msprinter_device_data_description_1 [] =
 };
 
 #ifdef NEW_GC
-DEFINE_LRECORD_IMPLEMENTATION ("msprinter-device", msprinter_device,
-			       1, /*dumpable-flag*/
-                               0, 0, 0, 0, 0,
-			       msprinter_device_data_description_1,
-			       Lisp_Msprinter_Device);
+DEFINE_DUMPABLE_INTERNAL_LISP_OBJECT ("msprinter-device", msprinter_device,
+				      0, msprinter_device_data_description_1,
+				      Lisp_Msprinter_Device);
 #else /* not NEW_GC */
 extern const struct sized_memory_description msprinter_device_data_description;
 
@@ -166,8 +160,7 @@ mswindows_init_device (struct device *d, Lisp_Object UNUSED (props))
   init_one_device (d);
 
 #ifdef NEW_GC
-  d->device_data = alloc_lrecord_type (struct mswindows_device,
-				       &lrecord_mswindows_device);
+  d->device_data = XMSWINDOWS_DEVICE (ALLOC_NORMAL_LISP_OBJECT (mswindows_device));
 #else /* not NEW_GC */
   d->device_data = xnew_and_zero (struct mswindows_device);
 #endif /* not NEW_GC */
@@ -305,7 +298,7 @@ mswindows_delete_device (struct device *d)
 
   DeleteDC (DEVICE_MSWINDOWS_HCDC (d));
 #ifndef NEW_GC
-  xfree (d->device_data, void *);
+  xfree (d->device_data);
 #endif /* not NEW_GC */
 }
 
@@ -446,7 +439,7 @@ msprinter_init_device_internal (struct device *d, Lisp_Object printer_name)
 
   DEVICE_MSPRINTER_NAME (d) = printer_name;
 
-  LISP_STRING_TO_TSTR (printer_name, printer_ext);
+  printer_ext = LISP_STRING_TO_TSTR (printer_name);
 
   if (!qxeOpenPrinter (printer_ext, &DEVICE_MSPRINTER_HPRINTER (d), NULL))
     {
@@ -498,7 +491,7 @@ msprinter_default_printer (void)
   if (qxeGetProfileString (XETEXT ("windows"), XETEXT ("device"), NULL, name,
 			   sizeof (name) / XETCHAR_SIZE) <= 0)
     return Qnil;
-  TSTR_TO_C_STRING (name, nameint);
+  nameint = TSTR_TO_ITEXT (name);
 
   if (nameint[0] == '\0')
     return Qnil;
@@ -507,7 +500,7 @@ msprinter_default_printer (void)
      name[] or ALLOCA ()ed */
   qxestrtok (nameint, ",");
 
-  return build_intstring (nameint);
+  return build_istring (nameint);
 }
 
 
@@ -523,8 +516,7 @@ msprinter_init_device (struct device *d, Lisp_Object UNUSED (props))
   Extbyte *printer_name;
 
 #ifdef NEW_GC
-  d->device_data = alloc_lrecord_type (struct msprinter_device,
-				       &lrecord_msprinter_device);
+  d->device_data = XMSPRINTER_DEVICE (ALLOC_NORMAL_LISP_OBJECT (msprinter_device));
 #else /* not NEW_GC */
   d->device_data = xnew_and_zero (struct msprinter_device);
 #endif /* not NEW_GC */
@@ -545,7 +537,7 @@ msprinter_init_device (struct device *d, Lisp_Object UNUSED (props))
   if (!msprinter_init_device_internal (d, DEVICE_CONNECTION (d)))
     signal_open_printer_error (d);
 
-  LISP_STRING_TO_TSTR (DEVICE_CONNECTION (d), printer_name);
+  printer_name = LISP_STRING_TO_TSTR (DEVICE_CONNECTION (d));
   /* Determine DEVMODE size and store the default DEVMODE */
   dm_size = qxeDocumentProperties (NULL, DEVICE_MSPRINTER_HPRINTER (d),
 				   printer_name, NULL, NULL, 0);
@@ -579,7 +571,8 @@ msprinter_delete_device (struct device *d)
 	}
 
 #ifndef NEW_GC
-      xfree (d->device_data, void *);
+      xfree (d->device_data);
+      d->device_data = 0;
 #endif /* not NEW_GC */
     }
 }
@@ -658,7 +651,7 @@ sync_printer_with_devmode (struct device* d, DEVMODEW* devmode_in,
 {
   /* Change connection if the device changed */
   if (!NILP (devname)
-      && lisp_strcasecmp (devname, DEVICE_MSPRINTER_NAME (d)) != 0)
+      && lisp_strcasecmp_i18n (devname, DEVICE_MSPRINTER_NAME (d)) != 0)
     {
       Lisp_Object new_connection = devname;
 
@@ -671,8 +664,8 @@ sync_printer_with_devmode (struct device* d, DEVMODEW* devmode_in,
 	     suffix. */
 	  Ibyte new_connext[20];
 
-	  qxesprintf (new_connext, ":%X", d->header.uid);
-	  new_connection = concat2 (devname, build_intstring (new_connext));
+	  qxesprintf (new_connext, ":%X", LISP_OBJECT_UID (wrap_device (d)));
+	  new_connection = concat2 (devname, build_istring (new_connext));
 	}
       DEVICE_CONNECTION (d) = new_connection;
 
@@ -687,7 +680,7 @@ sync_printer_with_devmode (struct device* d, DEVMODEW* devmode_in,
   {
     Extbyte *nameext;
 
-    LISP_STRING_TO_TSTR (DEVICE_MSPRINTER_NAME (d), nameext);
+    nameext = LISP_STRING_TO_TSTR (DEVICE_MSPRINTER_NAME (d));
 
     /* Apply the new devmode to the printer */
     qxeDocumentProperties (NULL, DEVICE_MSPRINTER_HPRINTER (d),
@@ -1043,7 +1036,7 @@ Return value is the previously selected settings object.
       Extbyte *nameext;
       LONG dm_size;
 
-      LISP_STRING_TO_TSTR (DEVICE_MSPRINTER_NAME (d), nameext);
+      nameext = LISP_STRING_TO_TSTR (DEVICE_MSPRINTER_NAME (d));
       dm_size = qxeDocumentProperties (NULL, DEVICE_MSPRINTER_HPRINTER (d),
 				       nameext, NULL, NULL, 0);
       if (dm_size <= 0)
@@ -1154,35 +1147,26 @@ print_devmode (Lisp_Object obj, Lisp_Object printcharfun,
 {
   Lisp_Devmode *dm = XDEVMODE (obj);
   if (print_readably)
-    printing_unreadable_object ("#<msprinter-settings 0x%x>",
-				dm->header.uid);
-  write_c_string (printcharfun, "#<msprinter-settings");
+    printing_unreadable_lisp_object (obj, 0);
+  write_ascstring (printcharfun, "#<msprinter-settings");
   if (!NILP (dm->printer_name))
     write_fmt_string_lisp (printcharfun, " for %S", 1, dm->printer_name);
   if (!NILP (dm->device))
     write_fmt_string_lisp (printcharfun, " (currently on %s)", 1, dm->device);
-  write_fmt_string (printcharfun, " 0x%x>", dm->header.uid);
+  write_fmt_string (printcharfun, " 0x%x>", LISP_OBJECT_UID (obj));
 }
 
 static void
-finalize_devmode (void *header, int for_disksave)
+finalize_devmode (Lisp_Object obj)
 {
-  Lisp_Devmode *dm = (Lisp_Devmode *) header;
-
-  if (for_disksave)
-    {
-      Lisp_Object devmode = wrap_devmode (dm);
-
-      invalid_operation
-	("Cannot dump XEmacs containing an msprinter-settings object",
-	 devmode);
-    }
+  Lisp_Devmode *dm = XDEVMODE (obj);
 
   assert (NILP (dm->device));
 }
 
 static int
-equal_devmode (Lisp_Object obj1, Lisp_Object obj2, int UNUSED (depth))
+equal_devmode (Lisp_Object obj1, Lisp_Object obj2, int UNUSED (depth),
+	       int UNUSED (foldcase))
 {
   Lisp_Devmode *dm1 = XDEVMODE (obj1);
   Lisp_Devmode *dm2 = XDEVMODE (obj2);
@@ -1195,34 +1179,33 @@ equal_devmode (Lisp_Object obj1, Lisp_Object obj2, int UNUSED (depth))
     return 0;
   if (NILP (dm1->printer_name) || NILP (dm2->printer_name))
     return 1;
-  return lisp_strcasecmp (dm1->printer_name, dm2->printer_name) == 0;
+  return lisp_strcasecmp_i18n (dm1->printer_name, dm2->printer_name) == 0;
 }
 
 static Hashcode
-hash_devmode (Lisp_Object obj, int depth)
+hash_devmode (Lisp_Object obj, int depth, Boolint UNUSED (equalp))
 {
   Lisp_Devmode *dm = XDEVMODE (obj);
 
   return HASH3 (XDEVMODE_SIZE (dm),
 		dm->devmode ? memory_hash (dm->devmode, XDEVMODE_SIZE (dm))
 		: 0,
-		internal_hash (dm->printer_name, depth + 1));
+		internal_hash (dm->printer_name, depth + 1, 0));
 }
 
-DEFINE_LRECORD_IMPLEMENTATION ("msprinter-settings", devmode,
-			       0, /*dumpable-flag*/
-			       mark_devmode, print_devmode, finalize_devmode,
-			       equal_devmode, hash_devmode, 
-			       devmode_description,
-			       Lisp_Devmode);
+DEFINE_NODUMP_LISP_OBJECT ("msprinter-settings", devmode,
+			   mark_devmode, print_devmode,
+			   finalize_devmode,
+			   equal_devmode, hash_devmode, 
+			   devmode_description,
+			   Lisp_Devmode);
 
 static Lisp_Object
 allocate_devmode (DEVMODEW* src_devmode, int do_copy,
 		  Lisp_Object src_name, struct device *d)
 {
-  Lisp_Devmode *dm;
-
-  dm = ALLOC_LCRECORD_TYPE (Lisp_Devmode, &lrecord_devmode);
+  Lisp_Object obj = ALLOC_NORMAL_LISP_OBJECT (devmode);
+  Lisp_Devmode *dm = XDEVMODE (obj);
 
   if (d)
     dm->device = wrap_device (d);
@@ -1241,7 +1224,7 @@ allocate_devmode (DEVMODEW* src_devmode, int do_copy,
       dm->devmode = src_devmode;
     }
 
-  return wrap_devmode (dm);
+  return obj;
 }
 
 DEFUN ("msprinter-settings-copy", Fmsprinter_settings_copy, 1, 1, 0, /*
@@ -1344,9 +1327,12 @@ values.  Return value is nil if there are no printers installed.
 
   GCPRO2 (result, def_printer);
 
+  def_printer = msprinter_default_printer ();
+
   while (num_printers--)
     {
       Extbyte *printer_name;
+      Lisp_Object printer_name_lisp;
       if (have_nt)
 	{
 	  PRINTER_INFO_4 *info = (PRINTER_INFO_4 *) data_buf;
@@ -1358,12 +1344,15 @@ values.  Return value is nil if there are no printers installed.
 	  printer_name = (Extbyte *) info->pPrinterName;
 	}
       data_buf += enum_entry_size;
-
-      result = Fcons (build_tstr_string (printer_name), result);
+      
+      printer_name_lisp = build_tstr_string (printer_name);
+      if (0 != qxestrcasecmp (XSTRING_DATA (def_printer),
+			      XSTRING_DATA (printer_name_lisp)))
+	{
+	  result = Fcons (printer_name_lisp, result);
+	}
     }
 
-  def_printer = msprinter_default_printer ();
-  result = Fdelete (def_printer, result);
   result = Fcons (def_printer, result);
 
   RETURN_UNGCPRO (result);
@@ -1377,11 +1366,11 @@ values.  Return value is nil if there are no printers installed.
 void
 syms_of_device_mswindows (void)
 {
-  INIT_LRECORD_IMPLEMENTATION (devmode);
+  INIT_LISP_OBJECT (devmode);
 
 #ifdef NEW_GC
-  INIT_LRECORD_IMPLEMENTATION (mswindows_device);
-  INIT_LRECORD_IMPLEMENTATION (msprinter_device);
+  INIT_LISP_OBJECT (mswindows_device);
+  INIT_LISP_OBJECT (msprinter_device);
 #endif /* NEW_GC */
 
   DEFSUBR (Fmsprinter_get_settings);
