@@ -10,10 +10,10 @@
 
 This file is part of XEmacs.
 
-XEmacs is free software; you can redistribute it and/or modify it
+XEmacs is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
-Free Software Foundation; either version 2, or (at your option) any
-later version.
+Free Software Foundation, either version 3 of the License, or (at your
+option) any later version.
 
 XEmacs is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
@@ -21,9 +21,7 @@ FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 for more details.
 
 You should have received a copy of the GNU General Public License
-along with XEmacs; see the file COPYING.  If not, write to
-the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-Boston, MA 02111-1307, USA.  */
+along with XEmacs.  If not, see <http://www.gnu.org/licenses/>. */
 
 /* Synched up with: Rewritten by Ben Wing.  Not in FSF. */
 
@@ -591,24 +589,21 @@ emacs_doprnt_1 (Lisp_Object stream, const Ibyte *format_nonreloc,
 	      Lisp_Object obj = largs[spec->argnum - 1];
 	      if (CHARP (obj))
 		obj = make_int (XCHAR (obj));
-#ifdef WITH_NUMBER_TYPES
 	      if (!NUMBERP (obj))
-#else
-	      if (!INT_OR_FLOATP (obj))
-#endif
 		{
 		  /* WARNING!  This MUST be big enough for the sprintf below */
 		  CIbyte msg[48];
 		  sprintf (msg,
 			   "format specifier %%%c doesn't match argument type",
 			   ch);
-		  syntax_error (msg, Qnil);
+		  syntax_error (msg, Qunbound);
 		}
 	      else if (strchr (double_converters, ch))
 		{
-#ifdef WITH_NUMBER_TYPES
-		  if (INTP (obj) || FLOATP (obj))
-		    arg.d = XFLOATINT (obj);
+		  if (INTP (obj))
+		    arg.d = XINT (obj);
+		  else if (FLOATP (obj))
+		    arg.d = XFLOAT_DATA (obj);
 #ifdef HAVE_BIGNUM
 		  else if (BIGNUMP (obj))
 		    arg.d = bignum_to_double (XBIGNUM_DATA (obj));
@@ -631,14 +626,11 @@ emacs_doprnt_1 (Lisp_Object stream, const Ibyte *format_nonreloc,
 			}
 		    }
 #endif
-#else /* !WITH_NUMBER_TYPES */
-		  arg.d = XFLOATINT (obj);
-#endif /* WITH_NUMBER_TYPES */
 		}
 	      else
 		{
 		  if (FLOATP (obj))
-		    obj = Ftruncate (obj);
+		    obj = Ftruncate (obj, Qnil);
 #ifdef HAVE_BIGFLOAT
 		  else if (BIGFLOATP (obj))
 		    {
@@ -725,7 +717,7 @@ emacs_doprnt_1 (Lisp_Object stream, const Ibyte *format_nonreloc,
 		  CIbyte msg[60];
 		  sprintf (msg, "invalid character value %d to %%c spec",
 			   a);
-		  syntax_error (msg, Qnil);
+		  syntax_error (msg, Qunbound);
 		}
 
 	      charlen = set_itext_ichar (charbuf, a);
@@ -754,7 +746,7 @@ emacs_doprnt_1 (Lisp_Object stream, const Ibyte *format_nonreloc,
 			    strlen ((const char *) text_to_print),
 			    spec->minwidth, -1, spec->minus_flag,
 			    spec->zero_flag);
-		  xfree (text_to_print, Ibyte *);
+		  xfree (text_to_print);
 		}
 #endif
 #ifdef HAVE_RATIO
@@ -766,7 +758,7 @@ emacs_doprnt_1 (Lisp_Object stream, const Ibyte *format_nonreloc,
 			    strlen ((const char *) text_to_print),
 			    spec->minwidth, -1, spec->minus_flag,
 			    spec->zero_flag);
-		  xfree (text_to_print, Ibyte *);
+		  xfree (text_to_print);
 		}
 #endif
 	    }
@@ -779,7 +771,7 @@ emacs_doprnt_1 (Lisp_Object stream, const Ibyte *format_nonreloc,
 	      doprnt_2 (stream, text_to_print,
 			strlen ((const char *) text_to_print),
 			spec->minwidth, -1, spec->minus_flag, spec->zero_flag);
-	      xfree (text_to_print, Ibyte *);
+	      xfree (text_to_print);
 	    }
 #endif /* HAVE_BIGFLOAT */
           else if (ch == 'b')
