@@ -5,10 +5,10 @@
 
 This file is part of XEmacs.
 
-XEmacs is free software; you can redistribute it and/or modify it
+XEmacs is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
-Free Software Foundation; either version 2, or (at your option) any
-later version.
+Free Software Foundation, either version 3 of the License, or (at your
+option) any later version.
 
 XEmacs is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
@@ -16,9 +16,7 @@ FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 for more details.
 
 You should have received a copy of the GNU General Public License
-along with XEmacs; see the file COPYING.  If not, write to
-the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-Boston, MA 02111-1307, USA.  */
+along with XEmacs.  If not, see <http://www.gnu.org/licenses/>. */
 
 /* Synched up with: Mule 2.0, FSF 19.30. */
 
@@ -95,7 +93,7 @@ init_editfns (void)
   if ((p = egetenv ("NAME")))
     /* I don't think it's the right thing to do the ampersand
        modification on NAME.  Not that it matters anymore...  -hniksic */
-    Vuser_full_name = build_intstring (p);
+    Vuser_full_name = build_istring (p);
   else
     Vuser_full_name = Fuser_full_name (Qnil);
 }
@@ -370,6 +368,8 @@ Save point, mark, and current buffer; execute BODY; restore those things.
 Executes BODY just like `progn'.
 The values of point, mark and the current buffer are restored
 even in case of abnormal exit (throw or error).
+
+arguments: (&rest BODY)
 */
        (args))
 {
@@ -395,6 +395,8 @@ save_current_buffer_restore (Lisp_Object buffer)
 DEFUN ("save-current-buffer", Fsave_current_buffer, 0, UNEVALLED, 0, /*
 Save the current buffer; execute BODY; restore the current buffer.
 Executes BODY just like `progn'.
+
+arguments: (&rest BODY)
 */
        (args))
 {
@@ -645,7 +647,7 @@ On Unix it is obtained from TMPDIR, with /tmp as the default.
     }
 #endif
 
-  return build_intstring (tmpdir);
+  return build_istring (tmpdir);
 }
 
 DEFUN ("user-login-name", Fuser_login_name, 0, 1, 0, /*
@@ -674,7 +676,7 @@ ignored and this function returns the login name for that UID, or nil.
   /* #### - I believe this should return nil instead of "unknown" when pw==0
      pw=0 is indicated by a null return from user_login_name
   */
-  return returned_name ? build_intstring (returned_name) : Qnil;
+  return returned_name ? build_istring (returned_name) : Qnil;
 }
 
 /* This function may be called from other C routines when a
@@ -740,7 +742,7 @@ This ignores the environment variables LOGNAME and USER, so it differs from
   struct passwd *pw = qxe_getpwuid (getuid ());
   /* #### - I believe this should return nil instead of "unknown" when pw==0 */
 
-  return build_string (pw ? pw->pw_name : "unknown");
+  return build_extstring (pw ? pw->pw_name : "unknown", Quser_name_encoding);
 }
 
 DEFUN ("user-uid", Fuser_uid, 0, 0, 0, /*
@@ -827,7 +829,7 @@ void
 uncache_home_directory (void)
 {
   if (cached_home_directory)
-    xfree (cached_home_directory, Ibyte *);
+    xfree (cached_home_directory);
   cached_home_directory = NULL;
 }
 
@@ -858,20 +860,16 @@ get_home_directory (void)
 			  homepath);
 	    }
 	  else
+#endif	/* !WIN32_NATIVE */
 	    {
-	      cached_home_directory = qxestrdup ((Ibyte *) "C:\\");
+              /* Unix, typically.
+                 Using "/" isn't quite right, but what should we do?
+                 We probably should try to extract pw_dir from /etc/passwd,
+                 before falling back to this. */
+	      cached_home_directory
+                = qxestrdup ((const Ibyte *) DEFAULT_DIRECTORY_FALLBACK);
 	      output_home_warning = 1;
 	    }
-#else	/* !WIN32_NATIVE */
-	  /*
-	   * Unix, typically.
-	   * Using "/" isn't quite right, but what should we do?
-	   * We probably should try to extract pw_dir from /etc/passwd,
-	   * before falling back to this.
-	   */
-	  cached_home_directory = qxestrdup ((Ibyte *) "/");
-	  output_home_warning = 1;
-#endif	/* !WIN32_NATIVE */
 	}
       if (initialized && output_home_warning)
 	{
@@ -895,7 +893,7 @@ Return the user's home directory, as a string.
   Ibyte *path = get_home_directory ();
 
   return !path ? Qnil :
-    Fexpand_file_name (Fsubstitute_in_file_name (build_intstring (path)),
+    Fexpand_file_name (Fsubstitute_in_file_name (build_istring (path)),
 		       Qnil);
 }
 
@@ -1044,11 +1042,10 @@ FORMAT-STRING may contain %-sequences to substitute parts of the time.
 %Y is replaced by the year with century.
 %z is replaced by the time zone as a numeric offset (e.g +0530, -0800 etc.)
 %Z is replaced by the time zone abbreviation.
+%\\xe6 is replaced by the month as a lowercase Roman number (i-xii)
+%\\xc6 is replaced by the month as an uppercase Roman number (I-XII)
 
 The number of options reflects the `strftime' function.
-
-BUG: If the charset used by the current locale is not ISO 8859-1, the
-characters appearing in the day and month names may be incorrect.
 */
        (format_string, time_))
 {
@@ -1068,16 +1065,17 @@ characters appearing in the day and month names may be incorrect.
       Extbyte *buf = alloca_extbytes (size);
       Extbyte *formext;
       /* make a copy of the static buffer returned by localtime() */
-      struct tm tm = * localtime(&value); 
+      struct tm tm = *localtime (&value); 
       
       *buf = 1;
 
       /* !!#### this use of external here is not totally safe, and
 	 potentially data lossy. */
-      LISP_STRING_TO_EXTERNAL (format_string, formext, Qnative);
+      formext = LISP_STRING_TO_EXTERNAL (format_string,
+					 Qtime_function_encoding);
       if (emacs_strftime (buf, size, formext, &tm)
 	  || !*buf)
-	return build_ext_string (buf, Qnative);
+	return build_extstring (buf, Qtime_function_encoding);
       /* If buffer was too small, make it bigger.  */
       size *= 2;
     }
@@ -1102,29 +1100,28 @@ ZONE is an integer indicating the number of seconds east of Greenwich.
   time_t time_spec;
   struct tm save_tm;
   struct tm *decoded_time;
-  Lisp_Object list_args[9];
 
   if (! lisp_to_time (specified_time, &time_spec))
     invalid_argument ("Invalid time specification", Qunbound);
 
   decoded_time = localtime (&time_spec);
-  list_args[0] = make_int (decoded_time->tm_sec);
-  list_args[1] = make_int (decoded_time->tm_min);
-  list_args[2] = make_int (decoded_time->tm_hour);
-  list_args[3] = make_int (decoded_time->tm_mday);
-  list_args[4] = make_int (decoded_time->tm_mon + 1);
-  list_args[5] = make_int (decoded_time->tm_year + 1900);
-  list_args[6] = make_int (decoded_time->tm_wday);
-  list_args[7] = (decoded_time->tm_isdst)? Qt : Qnil;
 
   /* Make a copy, in case gmtime modifies the struct.  */
   save_tm = *decoded_time;
   decoded_time = gmtime (&time_spec);
-  if (decoded_time == 0)
-    list_args[8] = Qnil;
-  else
-    list_args[8] = make_int (difftm (&save_tm, decoded_time));
-  return Flist (9, list_args);
+
+  return listn(9,
+	       make_int (save_tm.tm_sec),
+	       make_int (save_tm.tm_min),
+	       make_int (save_tm.tm_hour),
+	       make_int (save_tm.tm_mday),
+	       make_int (save_tm.tm_mon + 1),
+	       make_int (save_tm.tm_year + 1900),
+	       make_int (save_tm.tm_wday),
+	       save_tm.tm_isdst ? Qt : Qnil,
+	       (decoded_time == NULL)
+	       ? Qnil
+	       : make_int (difftm (&save_tm, decoded_time)));
 }
 
 static void set_time_zone_rule (Extbyte *tzstring);
@@ -1139,7 +1136,7 @@ make_time (time_t tiempo)
 }
 
 DEFUN ("encode-time", Fencode_time, 6, MANY, 0, /*
-  Convert SECOND, MINUTE, HOUR, DAY, MONTH, YEAR and ZONE to internal time.
+Convert SECOND, MINUTE, HOUR, DAY, MONTH, YEAR and ZONE to internal time.
 This is the reverse operation of `decode-time', which see.
 ZONE defaults to the current time zone rule.  This can
 be a string (as from `set-time-zone-rule'), or it can be a list
@@ -1155,6 +1152,8 @@ Out-of-range values for SEC, MINUTE, HOUR, DAY, or MONTH are allowed;
 for example, a DAY of 0 means the day preceding the given month.
 Year numbers less than 100 are treated just like other year numbers.
 If you want them to stand for years in this century, you must do that yourself.
+
+arguments: (SECOND MINUTE HOUR DAY MONTH YEAR &optional ZONE &rest REST)
 */
        (int nargs, Lisp_Object *args))
 {
@@ -1185,7 +1184,7 @@ If you want them to stand for years in this century, you must do that yourself.
       Extbyte **oldenv = environ, **newenv;
 
       if (STRINGP (zone))
-	LISP_STRING_TO_EXTERNAL (zone, tzstring, Qnative);
+	tzstring = LISP_STRING_TO_EXTERNAL (zone, Qtime_zone_encoding);
       else if (INTP (zone))
 	{
 	  int abszone = abs (XINT (zone));
@@ -1234,7 +1233,7 @@ and from `file-attributes'.
 {
   time_t value;
   Ibyte *the_ctime;
-  EMACS_INT len; /* this is what make_ext_string() accepts; ####
+  EMACS_INT len; /* this is what make_extstring() accepts; ####
 		    should it be an Bytecount? */
 
   if (! lisp_to_time (specified_time, &value))
@@ -1317,7 +1316,7 @@ the data it can't find.
 #endif
 #endif /* not HAVE_TM_ZONE */
       if (s)
-	tem = build_ext_string (s, Qnative);
+	tem = build_extstring (s, Qtime_zone_encoding);
       else
 	{
 	  Ibyte buf[6];
@@ -1326,7 +1325,7 @@ the data it can't find.
 	  int am = (offset < 0 ? -offset : offset) / 60;
 	  qxesprintf (buf, "%c%02d%02d", (offset < 0 ? '-' : '+'), am/60,
 		      am%60);
-	  tem = build_intstring (buf);
+	  tem = build_istring (buf);
 	}
       return list2 (make_int (offset), tem);
     }
@@ -1431,12 +1430,12 @@ If TZ is nil, use implementation-defined default time zone information.
   else
     {
       CHECK_STRING (tz);
-      LISP_STRING_TO_EXTERNAL (tz, tzstring, Qnative);
+      tzstring = LISP_STRING_TO_EXTERNAL (tz, Qtime_zone_encoding);
     }
 
   set_time_zone_rule (tzstring);
   if (environbuf)
-    xfree (environbuf, Extbyte **);
+    xfree (environbuf);
   environbuf = environ;
 
   return Qnil;
@@ -1473,10 +1472,12 @@ buffer_insert1 (struct buffer *buf, Lisp_Object arg)
    so we don't care if it gets trashed.  */
 
 DEFUN ("insert", Finsert, 0, MANY, 0, /*
-Insert the arguments, either strings or characters, at point.
+Insert ARGS, either strings or characters, at point.
 Point moves forward so that it ends up after the inserted text.
 Any other markers at the point of insertion remain before the text.
 If a string has non-null string-extent-data, new extents will be created.
+
+arguments: (&rest ARGS)
 */
        (int nargs, Lisp_Object *args))
 {
@@ -1495,6 +1496,8 @@ DEFUN ("insert-before-markers", Finsert_before_markers, 0, MANY, 0, /*
 Insert strings or characters at point, relocating markers after the text.
 Point moves forward so that it ends up after the inserted text.
 Any other markers at the point of insertion also end up after the text.
+
+arguments: (&rest ARGS)
 */
        (int nargs, Lisp_Object *args))
 {
@@ -2124,8 +2127,8 @@ DEFUN ("save-restriction", Fsave_restriction, 0, UNEVALLED, 0, /*
 Execute BODY, saving and restoring current buffer's restrictions.
 The buffer's restrictions make parts of the beginning and end invisible.
 \(They are set up with `narrow-to-region' and eliminated with `widen'.)
-This special form, `save-restriction', saves the current buffer's restrictions
-when it is entered, and restores them when it is exited.
+This special operator, `save-restriction', saves the current buffer's
+restrictions when it is entered, and restores them when it is exited.
 So any `narrow-to-region' within BODY lasts only until the end of the form.
 The old restrictions settings are restored
 even in case of abnormal exit (throw or error).
@@ -2139,6 +2142,8 @@ widened and then made changes outside the old restricted area.)
 Note: if you are using both `save-excursion' and `save-restriction',
 use `save-excursion' outermost:
     (save-excursion (save-restriction ...))
+
+arguments: (&rest BODY)
 */
        (body))
 {
@@ -2219,6 +2224,8 @@ The `#' flag means print numbers in an alternate, more verbose format:
    %g and %G conversions.
 
 Use %% to put a single % into the output.
+
+arguments: (CONTROL-STRING &rest ARGS)
 */
        (int nargs, Lisp_Object *args))
 {
@@ -2247,22 +2254,10 @@ If BUFFER is nil, the current buffer is assumed.
   x2 = XCHAR (character2);
 
   return (!NILP (b->case_fold_search)
-	  ? DOWNCASE (b, x1) == DOWNCASE (b, x2)
+	  ? CANONCASE (b, x1) == CANONCASE (b, x2)
 	  : x1 == x2)
     ? Qt : Qnil;
 }
-
-DEFUN ("char=", Fchar_Equal, 2, 2, 0, /*
-Return t if two characters match, case is significant.
-Both arguments must be characters (i.e. NOT integers).
-*/
-       (character1, character2))
-{
-  CHECK_CHAR_COERCE_INT (character1);
-  CHECK_CHAR_COERCE_INT (character2);
-
-  return EQ (character1, character2) ? Qt : Qnil;
-}
 
 #if 0 /* Undebugged FSFmacs code */
 /* Transpose the markers in two regions of the current buffer, and
@@ -2388,7 +2383,6 @@ syms_of_editfns (void)
   DEFSYMBOL (Quser_files_and_directories);
 
   DEFSUBR (Fchar_equal);
-  DEFSUBR (Fchar_Equal);
   DEFSUBR (Fgoto_char);
   DEFSUBR (Fstring_to_char);
   DEFSUBR (Fchar_to_string);
