@@ -1,9 +1,10 @@
-/* This file is part of XEmacs.
+/* Copyright (C) 2010 Ben Wing.
+This file is part of XEmacs.
 
-XEmacs is free software; you can redistribute it and/or modify it
+XEmacs is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
-Free Software Foundation; either version 2, or (at your option) any
-later version.
+Free Software Foundation, either version 3 of the License, or (at your
+option) any later version.
 
 XEmacs is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
@@ -11,9 +12,7 @@ FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 for more details.
 
 You should have received a copy of the GNU General Public License
-along with XEmacs; see the file COPYING.  If not, write to
-the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-Boston, MA 02111-1307, USA.  */
+along with XEmacs.  If not, see <http://www.gnu.org/licenses/>. */
 
 /* Synched up with: Not in FSF. */
 
@@ -26,7 +25,7 @@ Boston, MA 02111-1307, USA.  */
    * Trying to free a pointer not returned by malloc.
    * Trying to realloc a pointer not returned by malloc.
 
-   In addition, every word of every block freed is set to 0xdeadbeef
+   In addition, every word of every block freed is set to 0xDEADBEEF
    (-559038737).  This causes many uses of freed storage to be trapped or
    recognized.
 
@@ -43,10 +42,10 @@ Boston, MA 02111-1307, USA.  */
    return addresses.
 
    If UNMAPPED_FREE is defined, instead of setting every word of freed
-   storage to 0xdeadbeef, every call to malloc goes on its own page(s).
+   storage to 0xDEADBEEF, every call to malloc goes on its own page(s).
    When free() is called, the block is read and write protected.  This
    is very useful when debugging, since it usually generates a bus error
-   when the deadbeef hack might only cause some garbage to be printed.
+   when the DEADBEEF hack might only cause some garbage to be printed.
    However, this is too slow for everyday use, since it takes an enormous
    number of pages.
 
@@ -141,8 +140,7 @@ check_free (void *ptr)
 #if !defined(__linux__)
 	  /* I originally wrote:  "There's really no need to drop core."
 	     I have seen the error of my ways. -slb */
-	  if (strict_free_check)
-	    ABORT ();
+	  assert (!strict_free_check);
 #endif
 	  printf("Freeing unmalloc'ed memory at %p\n", ptr);
 	  __free_hook = check_free;
@@ -155,8 +153,7 @@ check_free (void *ptr)
 	  /* This happens when you free twice */
 #if !defined(__linux__)
 	  /* See above comment. */
-	  if (strict_free_check)
-	    ABORT ();
+	  assert (!strict_free_check);
 #endif
 	  printf("Freeing %p twice\n", ptr);
 	  __free_hook = check_free;
@@ -172,7 +169,7 @@ check_free (void *ptr)
       if (strict_free_check)
 	mprotect (ptr, rounded_up_size, PROT_NONE);
 #else
-      /* Set every word in the block to 0xdeadbeef */
+      /* Set every word in the block to 0xDEADBEEF */
       if (strict_free_check)
 	{
 	  unsigned long long_length = (size + (sizeof (long) - 1))
@@ -182,7 +179,7 @@ check_free (void *ptr)
           /* Not using the DEADBEEF_CONSTANT #define, since we don't know
            * that allocation sizes will be multiples of eight. */
 	  for (i = 0; i < long_length; i++)
-	    ((unsigned long *) ptr)[i] = 0xdeadbeef;
+	    ((unsigned long *) ptr)[i] = 0xDEADBEEF;
 	}
 #endif
       free_queue[current_free].address = ptr;
@@ -448,7 +445,7 @@ block_input_history blhist[BLHISTLIMIT];
 note_block_input (char *file, int line)
 {
   note_block (file, line, block_type);
-  if (interrupt_input_blocked > 2) ABORT();
+  assert (interrupt_input_blocked <= 2);
 }
 
 note_unblock_input (char* file, int line)
@@ -488,13 +485,13 @@ log_gcpro (char *file, int line, struct gcpro *value, blocktype type)
   if (type == ungcpro_type)
     {
       if (value == gcprolist) goto OK;
-      if (! gcprolist) ABORT ();
+      assert (gcprolist);
       if (value == gcprolist->next) goto OK;
-      if (! gcprolist->next) ABORT ();
+      assert (gcprolist->next);
       if (value == gcprolist->next->next) goto OK;
-      if (! gcprolist->next->next) ABORT ();
+      assert (gcprolist->next->next);
       if (value == gcprolist->next->next->next) goto OK;
-      if (! gcprolist->next->next->next) ABORT ();
+      assert (gcprolist->next->next->next);
       if (value == gcprolist->next->next->next->next) goto OK;
       ABORT ();
     OK:;
