@@ -1,13 +1,13 @@
 /* Commonly-used symbols -- include file
    Copyright (C) 1995 Sun Microsystems.
-   Copyright (C) 1995, 1996, 2000, 2001, 2002, 2003 Ben Wing.
+   Copyright (C) 1995, 1996, 2000, 2001, 2002, 2003, 2010 Ben Wing.
 
 This file is part of XEmacs.
 
-XEmacs is free software; you can redistribute it and/or modify it
+XEmacs is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
-Free Software Foundation; either version 2, or (at your option) any
-later version.
+Free Software Foundation, either version 3 of the License, or (at your
+option) any later version.
 
 XEmacs is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
@@ -15,9 +15,7 @@ FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 for more details.
 
 You should have received a copy of the GNU General Public License
-along with XEmacs; see the file COPYING.  If not, write to
-the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-Boston, MA 02111-1307, USA.  */
+along with XEmacs.  If not, see <http://www.gnu.org/licenses/>. */
 
 /* Synched up with: Not in FSF. */
 
@@ -32,6 +30,8 @@ Boston, MA 02111-1307, USA.  */
    SYMBOL_KEYWORD (Q_foo); declares a keyword symbol ":foo"
    SYMBOL_GENERAL (Qfoo, "bar"); declares a symbol named "bar" but stored in
      the variable Qfoo
+   SYMBOL_KEYWORD_GENERAL (Q_foo_, ":bar"); declares a keyword named ":bar"
+      but stored in the variable Q_foo_. 
 
 To sort the crap in this file, use the following:
 
@@ -46,9 +46,9 @@ To sort the crap in this file, use the following:
 SYMBOL (Qabort);
 SYMBOL_KEYWORD (Q_accelerator);
 SYMBOL_KEYWORD (Q_active);
-SYMBOL (Qactually_requested);
 SYMBOL (Qafter);
 SYMBOL (Qall);
+SYMBOL_KEYWORD (Q_allow_other_keys);
 SYMBOL (Qand);
 SYMBOL (Qappend);
 SYMBOL (Qascii);
@@ -72,6 +72,7 @@ SYMBOL_KEYWORD (Q_buttons);
 SYMBOL_KEYWORD (Q_callback);
 SYMBOL_KEYWORD (Q_callback_ex);
 SYMBOL (Qcancel);
+SYMBOL (Qcar);
 SYMBOL (Qcategory);
 SYMBOL (Qccl_program);
 SYMBOL (Qcenter);
@@ -83,6 +84,7 @@ SYMBOL (Qcharacter);
 SYMBOL (Qchars);
 SYMBOL (Qcode_page);
 SYMBOL (Qcoding_system);
+SYMBOL (Qcoerce);
 SYMBOL (Qcolor);
 SYMBOL (Qcolumns);
 SYMBOL (Qcommand);
@@ -90,15 +92,20 @@ SYMBOL_KEYWORD (Q_config);
 SYMBOL (Qconsole);
 SYMBOL (Qcontrol_1);
 SYMBOL (Qcopies);
+SYMBOL (Qcount);
 SYMBOL_MODULE_API (Qcritical);
 SYMBOL (Qctext);
 SYMBOL (Qcurrent);
 SYMBOL (Qcursor);
 SYMBOL (Qdata);
+SYMBOL_KEYWORD (Q_data);
 SYMBOL (Qdde);
 SYMBOL (Qdead);
 SYMBOL (Qdebug);
 SYMBOL (Qdefault);
+/* We name the C variable corresponding to the keyword Q_default_, not
+   Q_default, to allow it to be useful with PARSE_KEYWORDS (). */
+SYMBOL_KEYWORD_GENERAL (Q_default_, ":default");
 SYMBOL_MODULE_API (Qdelete);
 SYMBOL (Qdelq);
 SYMBOL (Qdescription);
@@ -113,13 +120,10 @@ SYMBOL (Qdisplay);
 SYMBOL (Qdoc_string);
 SYMBOL (Qdocumentation);
 SYMBOL (Qduplex);
-SYMBOL (Qdynarr_overhead);
 SYMBOL (Qemergency);
 SYMBOL (Qempty);
+SYMBOL_KEYWORD (Q_end);
 SYMBOL (Qencode_as_utf_8);
-SYMBOL (Qeq);
-SYMBOL (Qeql);
-SYMBOL (Qequal);
 SYMBOL (Qeval);
 SYMBOL (Qevent);
 SYMBOL (Qextents);
@@ -132,15 +136,16 @@ SYMBOL_MODULE_API (Qfile_name);
 SYMBOL_KEYWORD (Q_filter);
 SYMBOL (Qfinal);
 SYMBOL (Qfixnum);
+SYMBOL_MODULE_API (Qfixnump);
 SYMBOL (Qfloat);
 SYMBOL (Qfont);
 SYMBOL (Qframe);
 SYMBOL (Qframes);
 SYMBOL (Qfrom_page);
+SYMBOL (Qfrom_unicode);
 SYMBOL (Qfull_assoc);
 SYMBOL (Qfuncall);
 SYMBOL (Qfunction);
-SYMBOL (Qgap_overhead);
 SYMBOL (Qgarbage_collection);
 SYMBOL (Qgeneric);
 SYMBOL (Qgeometry);
@@ -171,6 +176,7 @@ SYMBOL_KEYWORD (Q_items);
 SYMBOL_KEYWORD (Q_justify);
 SYMBOL_KEYWORD (Q_vertically_justify);
 SYMBOL_KEYWORD (Q_horizontally_justify);
+SYMBOL_KEYWORD (Q_key);
 SYMBOL (Qkey);
 SYMBOL (Qkey_assoc);
 SYMBOL (Qkey_mapping);
@@ -189,8 +195,8 @@ SYMBOL (Qlist);
 SYMBOL (Qlittle_endian);
 SYMBOL (Qlocale);
 SYMBOL (Qlow);
+SYMBOL_GENERAL (Qlss, "<");
 SYMBOL (Qmagic);
-SYMBOL (Qmalloc_overhead);
 SYMBOL_KEYWORD (Q_margin_width);
 SYMBOL (Qmarkers);
 SYMBOL (Qmax);
@@ -235,8 +241,11 @@ SYMBOL_KEYWORD (Q_printer_settings);
 SYMBOL (Qprocess);
 SYMBOL_KEYWORD (Q_properties);
 SYMBOL (Qprovide);
+SYMBOL (Qquery_coding_clear_highlights);
+SYMBOL (Qquery_coding_warning_face);
 SYMBOL (Qquestion);
 SYMBOL_KEYWORD (Q_question);
+SYMBOL (Qquote);
 SYMBOL (Qradio);
 SYMBOL (Qrassoc);
 SYMBOL (Qrassq);
@@ -262,16 +271,20 @@ SYMBOL (Qsound);
 SYMBOL (Qspace);
 SYMBOL (Qspecifier);
 SYMBOL (Qstandard);
+SYMBOL_KEYWORD (Q_start);
 SYMBOL (Qstream);
 SYMBOL (Qstring);
+SYMBOL (Qstring_match);
 SYMBOL_KEYWORD (Q_style);
 SYMBOL_KEYWORD (Q_suffix);
 SYMBOL (Qsubtype);
+SYMBOL (Qsucceeded);
 SYMBOL (Qsymbol);
 SYMBOL (Qsyntax);
 SYMBOL (Qsystem_default);
 SYMBOL (Qterminal);
 SYMBOL (Qtest);
+SYMBOL_KEYWORD (Q_test);
 SYMBOL (Qtext);
 SYMBOL_KEYWORD (Q_text);
 SYMBOL (Qthis_command);
@@ -285,8 +298,11 @@ SYMBOL (Qtop);
 SYMBOL (Qtop_margin);
 SYMBOL (Qtty);
 SYMBOL (Qtype);
+SYMBOL_KEYWORD (Q_type);
 SYMBOL (Qundecided);
 SYMBOL (Qundefined);
+SYMBOL (Qunencodable);
+SYMBOL (Qinvalid_sequence);
 SYMBOL (Qunimplemented);
 SYMBOL (Qunicode_registries);
 SYMBOL (Qunicode_type);
