@@ -2,15 +2,15 @@
    Copyright (C) 1993, 1994, 1998 Free Software Foundation, Inc.
    Copyright (C) 1995 Board of Trustees, University of Illinois.
    Copyright (C) 1995 Tinker Systems
-   Copyright (C) 1995, 1996, 2001, 2002, 2004, 2005 Ben Wing
+   Copyright (C) 1995, 1996, 2001, 2002, 2004, 2005, 2010 Ben Wing
    Copyright (C) 1995 Sun Microsystems
 
 This file is part of XEmacs.
 
-XEmacs is free software; you can redistribute it and/or modify it
+XEmacs is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
-Free Software Foundation; either version 2, or (at your option) any
-later version.
+Free Software Foundation, either version 3 of the License, or (at your
+option) any later version.
 
 XEmacs is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
@@ -18,9 +18,7 @@ FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 for more details.
 
 You should have received a copy of the GNU General Public License
-along with XEmacs; see the file COPYING.  If not, write to
-the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-Boston, MA 02111-1307, USA.  */
+along with XEmacs.  If not, see <http://www.gnu.org/licenses/>. */
 
 /* Synched up with: Not in FSF. */
 
@@ -34,6 +32,7 @@ Boston, MA 02111-1307, USA.  */
    Many changes for color work and optimizations by Jareth Hein for 21.0
    Switch of GIF/JPEG/PNG to new EImage intermediate code by Jareth Hein for 21.0
    TIFF code by Jareth Hein for 21.0
+   GIF support changed to external giflib by Jerry James for 21.5
    TODO:
    Convert images.el to C and stick it in here?
    This file is really repetitious; can we refactor?
@@ -46,7 +45,7 @@ Boston, MA 02111-1307, USA.  */
 #include "device-impl.h"
 #include "faces.h"
 #include "glyphs.h"
-#include "objects-impl.h"
+#include "fontcolor-impl.h"
 
 #include "buffer.h"
 #include "frame.h"
@@ -99,13 +98,13 @@ Lisp_Object Qpng;
 
 BEGIN_C_DECLS
 
-#ifdef WIN32_NATIVE
-/* #### Yuck!  More horrifitude.  tiffio.h, below, includes <windows.h>,
-   which defines INT32 and INT16, the former differently and incompatibly
-   from jmorecfg.h, included by jpeglib.h.  We can disable the stuff in
-   jmorecfg.h by defining XMD_H (clever, huh?); then we define these
-   typedefs the way that <windows.h> wants them (which is more correct,
-   anyway; jmorecfg.h defines INT32 as `long'). */
+#ifdef WIN32_ANY
+/* #### Yuck!  More horrifitude.  tiffio.h, below, and sysfile.h above,
+   include <windows.h>, which defines INT32 and INT16, the former
+   differently and incompatibly from jmorecfg.h, included by jpeglib.h.  We
+   can disable the stuff in jmorecfg.h by defining XMD_H (clever, huh?);
+   then we define these typedefs the way that <windows.h> wants them (which
+   is more correct, anyway; jmorecfg.h defines INT32 as `long'). */
 #define XMD_H
 typedef signed int INT32;
 typedef signed short INT16;
@@ -119,8 +118,15 @@ typedef unsigned char boolean;
 #define HAVE_BOOLEAN		/* prevent jmorecfg.h from redefining it */
 #endif
 
+/* Yet more breakage... jmorecfg.h unconditionally defines FAR either as
+   "far" or as blank.  Windef.h unconditionally defines FAR as "far".
+   We'll avoid the compile warning by redefing FAR the way windows defines it,
+   after loading the JPEG headers. */
+#undef FAR
 #include <jpeglib.h>
 #include <jerror.h>
+#undef FAR
+#define FAR far
 
 END_C_DECLS
 
@@ -169,10 +175,16 @@ jpeg_instantiate_unwind (Lisp_Object unwind_obj)
     jpeg_destroy_decompress (data->cinfo_ptr);
 
   if (data->instream)
-    retry_fclose (data->instream);
+    {
+      retry_fclose (data->instream);
+      data->instream = 0;
+    }
 
   if (data->eimage)
-    xfree (data->eimage, Binbyte *);
+    {
+      xfree (data->eimage);
+      data->eimage = 0;
+    }
 
   return Qnil;
 }
@@ -320,7 +332,7 @@ my_jpeg_output_message (j_common_ptr cinfo)
 
   /* Create the message */
   (*cinfo->err->format_message) (cinfo, buffer);
-  EXTERNAL_TO_C_STRING (buffer, intbuf, Qnative);
+  intbuf = EXTERNAL_TO_ITEXT (buffer, Qjpeg_error_message_encoding);
   warn_when_safe (Qjpeg, Qinfo, "%s", intbuf);
 }
 
@@ -374,7 +386,7 @@ jpeg_instantiate (Lisp_Object image_instance, Lisp_Object instantiator,
 
 	/* Create the message */
 	(*cinfo.err->format_message) ((j_common_ptr) &cinfo, buffer);
-	errstring = build_ext_string (buffer, Qnative);
+	errstring = build_extstring (buffer, Qjpeg_error_message_encoding);
 
 	signal_image_error_2 ("JPEG decoding error",
 			      errstring, instantiator);
@@ -395,7 +407,7 @@ jpeg_instantiate (Lisp_Object image_instance, Lisp_Object instantiator,
     /* #### This is a definite problem under Mule due to the amount of
        stack data it might allocate.  Need to be able to convert and
        write out to a file. */
-    TO_EXTERNAL_FORMAT (LISP_STRING, data, ALLOCA, (bytes, len), Qbinary);
+    LISP_STRING_TO_SIZED_EXTERNAL (data, bytes, len, Qbinary);
     jpeg_memory_src (&cinfo, (JOCTET *) bytes, len);
   }
 
@@ -409,6 +421,7 @@ jpeg_instantiate (Lisp_Object image_instance, Lisp_Object instantiator,
    */
 
   {
+    UINT_64_BIT pixels_sq;
     int jpeg_gray = 0;		/* if we're dealing with a grayscale */
     /* Step 4: set parameters for decompression.   */
 
@@ -431,7 +444,10 @@ jpeg_instantiate (Lisp_Object image_instance, Lisp_Object instantiator,
     jpeg_start_decompress (&cinfo);
 
     /* Step 6: Read in the data and put into EImage format (8bit RGB triples)*/
-
+    pixels_sq =
+      (UINT_64_BIT) cinfo.output_width * (UINT_64_BIT) cinfo.output_height;
+    if (pixels_sq > ((size_t) -1) / 3)
+      signal_image_error ("JPEG image too large to instantiate", instantiator);
     unwind.eimage =
       xnew_binbytes (cinfo.output_width * cinfo.output_height * 3);
     if (!unwind.eimage)
@@ -523,7 +539,7 @@ jpeg_instantiate (Lisp_Object image_instance, Lisp_Object instantiator,
  *                               GIF                                  *
  **********************************************************************/
 
-#include "gifrlib.h"
+#include <gif_lib.h>
 
 static void
 gif_validate (Lisp_Object instantiator)
@@ -565,10 +581,14 @@ gif_instantiate_unwind (Lisp_Object unwind_obj)
   if (data->giffile)
     {
       DGifCloseFile (data->giffile);
-      GifFree(data->giffile);
+      FreeSavedImages (data->giffile);
+      data->giffile = 0;
     }
   if (data->eimage)
-    xfree (data->eimage, Binbyte *);
+    {
+      xfree (data->eimage);
+      data->eimage = 0;
+    }
 
   return Qnil;
 }
@@ -580,10 +600,10 @@ typedef struct gif_memory_storage
   Bytecount index;		/* Where are we?  */
 } gif_memory_storage;
 
-static Bytecount
-gif_read_from_memory (GifByteType *buf, Bytecount size, VoidPtr data)
+static int
+gif_read_from_memory (GifFileType *gif, GifByteType *buf, int size)
 {
-  gif_memory_storage *mem = (gif_memory_storage *) data;
+  gif_memory_storage *mem = (gif_memory_storage *) gif->UserData;
 
   if (size > (mem->len - mem->index))
     return -1;
@@ -592,26 +612,40 @@ gif_read_from_memory (GifByteType *buf, Bytecount size, VoidPtr data)
   return size;
 }
 
-static int
-gif_memory_close (VoidPtr UNUSED (data))
+static const char *
+gif_decode_error_string (void)
 {
-  return 0;
-}
-
-struct gif_error_struct
-{
-  const Extbyte *err_str;	/* return the error string */
-  jmp_buf setjmp_buffer;	/* for return to caller */
-};
-
-static void
-gif_error_func (const Extbyte *err_str, VoidPtr error_ptr)
-{
-  struct gif_error_struct *error_data = (struct gif_error_struct *) error_ptr;
-
-  /* return to setjmp point */
-  error_data->err_str = err_str;
-  longjmp (error_data->setjmp_buffer, 1);
+  switch (GifLastError ())
+    {
+    case D_GIF_ERR_OPEN_FAILED:
+      return "GIF error: unable to open";
+    case D_GIF_ERR_READ_FAILED:
+      return "GIF error: read failed";
+    case D_GIF_ERR_NOT_GIF_FILE:
+      return "GIF error: not a GIF file";
+    case D_GIF_ERR_NO_SCRN_DSCR:
+      return "GIF error: no Screen Descriptor detected";
+    case D_GIF_ERR_NO_IMAG_DSCR:
+      return "GIF error: no Image Descriptor detected";
+    case D_GIF_ERR_NO_COLOR_MAP:
+      return "GIF error: no global or local color map";
+    case D_GIF_ERR_WRONG_RECORD:
+      return "GIF error: wrong record type";
+    case D_GIF_ERR_DATA_TOO_BIG:
+      return "GIF error: image is larger than indicated by header";
+    case D_GIF_ERR_NOT_ENOUGH_MEM:
+      return "GIF error: out of memory";
+    case D_GIF_ERR_CLOSE_FAILED:
+      return "GIF error: failed to close file";
+    case D_GIF_ERR_NOT_READABLE:
+      return "GIF error: file is not readable";
+    case D_GIF_ERR_IMAGE_DEFECT:
+      return "GIF error: image is defective";
+    case D_GIF_ERR_EOF_TOO_SOON:
+      return "GIF error: image EOF detected before image complete";
+    default:
+      return "GIF error: unknown error";
+    }
 }
 
 static void
@@ -626,7 +660,6 @@ gif_instantiate (Lisp_Object image_instance, Lisp_Object instantiator,
   struct gif_unwind_data unwind;
   int speccount = specpdl_depth ();
   gif_memory_storage mem_struct;
-  struct gif_error_struct gif_err;
   Binbyte *bytes;
   Bytecount len;
   int height = 0;
@@ -642,49 +675,40 @@ gif_instantiate (Lisp_Object image_instance, Lisp_Object instantiator,
 
     assert (!NILP (data));
 
-    if (!(unwind.giffile = GifSetup()))
-      signal_image_error ("Insufficient memory to instantiate GIF image", instantiator);
-
-    /* set up error facilities */
-    if (setjmp(gif_err.setjmp_buffer))
-      {
-	/* An error was signaled. No clean up is needed, as unwind handles that
-	   for us.  Just pass the error along. */
-	Ibyte *interr;
-	Lisp_Object errstring;
-	EXTERNAL_TO_C_STRING (gif_err.err_str, interr, Qnative);
-	errstring = build_msg_intstring (interr);
-	signal_image_error_2 ("GIF decoding error", errstring, instantiator);
-      }
-    GifSetErrorFunc(unwind.giffile, (Gif_error_func)gif_error_func, (VoidPtr)&gif_err);
-
-    TO_EXTERNAL_FORMAT (LISP_STRING, data, ALLOCA, (bytes, len), Qbinary);
+    LISP_STRING_TO_SIZED_EXTERNAL (data, bytes, len, Qbinary); 
     mem_struct.bytes = bytes;
     mem_struct.len = len;
     mem_struct.index = 0;
-    GifSetReadFunc(unwind.giffile, gif_read_from_memory, (VoidPtr)&mem_struct);
-    GifSetCloseFunc(unwind.giffile, gif_memory_close, (VoidPtr)&mem_struct);
-    DGifInitRead(unwind.giffile);
+    unwind.giffile = DGifOpen (&mem_struct, gif_read_from_memory);
+    if (unwind.giffile == NULL)
+      signal_image_error (gif_decode_error_string (), instantiator);
 
     /* Then slurp the image into memory, decoding along the way.
        The result is the image in a simple one-byte-per-pixel
-       format (#### the GIF routines only support 8-bit GIFs,
-       it appears). */
-    DGifSlurp (unwind.giffile);
+       format. */
+    if (DGifSlurp (unwind.giffile) == GIF_ERROR)
+      signal_image_error (gif_decode_error_string (), instantiator);
   }
 
   /* 3. Now create the EImage(s) */
   {
-    ColorMapObject *cmo = unwind.giffile->SColorMap;
+    ColorMapObject *cmo = (unwind.giffile->Image.ColorMap ? unwind.giffile->Image.ColorMap : unwind.giffile->SColorMap);
     int i, j, row, pass, interlace, slice;
+    UINT_64_BIT pixels_sq;
     Binbyte *eip;
     /* interlaced gifs have rows in this order:
        0, 8, 16, ..., 4, 12, 20, ..., 2, 6, 10, ..., 1, 3, 5, ...  */
     static int InterlacedOffset[] = { 0, 4, 2, 1 };
     static int InterlacedJumps[] = { 8, 8, 4, 2 };
 
+    if (cmo == NULL)
+      signal_image_error ("GIF image has no color map", instantiator);
+
     height = unwind.giffile->SHeight;
     width = unwind.giffile->SWidth;
+    pixels_sq = (UINT_64_BIT) width * (UINT_64_BIT) height;
+    if (pixels_sq > ((size_t) -1) / (3 * unwind.giffile->ImageCount))
+      signal_image_error ("GIF image too large to instantiate", instantiator);
     unwind.eimage =
       xnew_binbytes (width * height * 3 * unwind.giffile->ImageCount);
     if (!unwind.eimage)
@@ -740,7 +764,7 @@ gif_instantiate (Lisp_Object image_instance, Lisp_Object instantiator,
   if (unwind.giffile->ImageCount > 1)
     {
     /* See if there is a timeout value. In theory there could be one
-       for every image - but that makes the implementation way to
+       for every image - but that makes the implementation way too
        complicated for now so we just take the first. */
       unsigned short timeout = 0;
       Lisp_Object tid;
@@ -865,10 +889,16 @@ png_instantiate_unwind (Lisp_Object unwind_obj)
     }
 
   if (data->instream)
-    retry_fclose (data->instream);
+    {
+      retry_fclose (data->instream);
+      data->instream = 0;
+    }
 
   if (data->eimage)
-    xfree(data->eimage, Binbyte *);
+    {
+      xfree (data->eimage);
+      data->eimage = 0;
+    }
 
   return Qnil;
 }
@@ -896,7 +926,8 @@ png_instantiate (Lisp_Object image_instance, Lisp_Object instantiator,
       /* Something blew up:
 	 just display the error (cleanup happens in the unwind) */
       signal_image_error_2 ("Error decoding PNG",
-			     build_string(png_err_stct.err_str),
+			     build_extstring (png_err_stct.err_str,
+					       Qerror_message_encoding),
 			     instantiator);
     }
 
@@ -936,7 +967,7 @@ png_instantiate (Lisp_Object image_instance, Lisp_Object instantiator,
 
     /* #### This is a definite problem under Mule due to the amount of
        stack data it might allocate.  Need to think about using Lstreams */
-    TO_EXTERNAL_FORMAT (LISP_STRING, data, ALLOCA, (bytes, len), Qbinary);
+    LISP_STRING_TO_SIZED_EXTERNAL (data, bytes, len, Qbinary); 
     tbr.bytes = bytes;
     tbr.len = len;
     tbr.index = 0;
@@ -946,19 +977,31 @@ png_instantiate (Lisp_Object image_instance, Lisp_Object instantiator,
   png_read_info (png_ptr, info_ptr);
 
   {
-    int y;
+    int y, padding;
     Binbyte **row_pointers;
-    height = info_ptr->height;
-    width = info_ptr->width;
+    UINT_64_BIT pixels_sq;
+    height = png_get_image_height (png_ptr, info_ptr);
+    width = png_get_image_width (png_ptr, info_ptr);
+    pixels_sq = (UINT_64_BIT) width * (UINT_64_BIT) height;
+    if (pixels_sq > ((size_t) -1) / 3)
+      signal_image_error ("PNG image too large to instantiate", instantiator);
+
+    /* Wow, allocate all the memory.  Truly, exciting.
+       Well, yes, there's excitement to be had.  It turns out that libpng
+       strips in place, so the last row overruns the buffer if depth is 16
+       or there's an alpha channel.  This is a crash on Linux.  So we need
+       to add padding.
+       The worst case is reducing 8 bytes (16-bit RGBA) to 3 (8-bit RGB). */
+
+    padding = 5 * width;
+    unwind.eimage = xnew_array_and_zero (Binbyte,
+					 (size_t) (pixels_sq * 3 + padding));
 
-    /* Wow, allocate all the memory.  Truly, exciting. */
-    unwind.eimage = xnew_array_and_zero (Binbyte, width * height * 3);
     /* libpng expects that the image buffer passed in contains a
        picture to draw on top of if the png has any transparencies.
        This could be a good place to pass that in... */
 
     row_pointers  = xnew_array (png_byte *, height);
-
     for (y = 0; y < height; y++)
       row_pointers[y] = unwind.eimage + (width * 3 * y);
 
@@ -978,16 +1021,15 @@ png_instantiate (Lisp_Object image_instance, Lisp_Object instantiator,
 	}
       else
 	{
-	  Lisp_Color_Instance *c;
-	  Lisp_Object rgblist;
-
-	  c = XCOLOR_INSTANCE (bkgd);
-	  rgblist = MAYBE_LISP_DEVMETH (XDEVICE (c->device),
-					color_instance_rgb_components,
-					(c));
-	  my_background.red = (unsigned short) XINT (XCAR (rgblist));
-	  my_background.green = (unsigned short) XINT (XCAR (XCDR (rgblist)));
-	  my_background.blue = (unsigned short) XINT (XCAR (XCDR (XCDR (rgblist))));
+	  Lisp_Color_Instance *c = XCOLOR_INSTANCE (bkgd);
+	  Lisp_Object rgb = MAYBE_LISP_DEVMETH (XDEVICE (c->device),
+						color_instance_rgb_components,
+						(c));
+#define GETCOLOR(col) my_background.col = (unsigned short) XINT (XCAR (rgb))
+	  GETCOLOR(red); rgb = XCDR (rgb);
+	  GETCOLOR(green); rgb = XCDR (rgb);
+	  GETCOLOR(blue);
+#undef GETCOLOR
 	}
 
       if (png_get_bKGD (png_ptr, info_ptr, &image_background))
@@ -1000,63 +1042,68 @@ png_instantiate (Lisp_Object image_instance, Lisp_Object instantiator,
 
     /* Now that we're using EImage, ask for 8bit RGB triples for any type
        of image*/
-    /* convert palette images to full RGB */
-    if (info_ptr->color_type == PNG_COLOR_TYPE_PALETTE)
-      png_set_expand (png_ptr);
-    /* send grayscale images to RGB too */
-    if (info_ptr->color_type == PNG_COLOR_TYPE_GRAY ||
-        info_ptr->color_type == PNG_COLOR_TYPE_GRAY_ALPHA)
-      png_set_gray_to_rgb (png_ptr);
-    /* we can't handle alpha values */
-    if (info_ptr->color_type & PNG_COLOR_MASK_ALPHA)
-      png_set_strip_alpha (png_ptr);
-    /* tell libpng to strip 16 bit depth files down to 8 bits */
-    if (info_ptr->bit_depth == 16)
-      png_set_strip_16 (png_ptr);
-    /* if the image is < 8 bits, pad it out */
-    if (info_ptr->bit_depth < 8)
+    switch (png_get_color_type (png_ptr, info_ptr))
       {
-	if (info_ptr->color_type == PNG_COLOR_TYPE_GRAY)
-	  png_set_expand (png_ptr);
-	else
-	  png_set_packing (png_ptr);
+      case PNG_COLOR_TYPE_PALETTE:
+        /* convert palette images to RGB */
+        png_set_palette_to_rgb (png_ptr);
+        break;
+
+      case PNG_COLOR_TYPE_GRAY:
+      case PNG_COLOR_TYPE_GRAY_ALPHA:
+        /* convert grayscale images to RGB */
+        png_set_gray_to_rgb (png_ptr);
+        break;
+
+      default:
+        /* pad images with depth < 8 bits */
+        if (png_get_bit_depth (png_ptr, info_ptr) < 8)
+          {
+            png_set_packing (png_ptr);
+          }
+        break;
       }
 
+    /* strip 16-bit depth files down to 8 bits */
+    if (png_get_bit_depth (png_ptr, info_ptr) == 16)
+      png_set_strip_16 (png_ptr);
+    /* strip alpha channel
+       #### shouldn't we handle this?
+       first call png_read_update_info in case above transformations
+       have generated an alpha channel */
+    png_read_update_info(png_ptr, info_ptr);
+    if (png_get_color_type (png_ptr, info_ptr) & PNG_COLOR_MASK_ALPHA)
+      png_set_strip_alpha (png_ptr);
+
     png_read_image (png_ptr, row_pointers);
     png_read_end (png_ptr, info_ptr);
 
-#if 1 /* def PNG_SHOW_COMMENTS */
-    /* ####
-     * I turn this off by default now, because the !%^@#!% comments
-     * show up every time the image is instantiated, which can get
-     * really really annoying.  There should be some way to pass this
-     * type of data down into the glyph code, where you can get to it
-     * from lisp anyway. - WMP
-     */
-    /* #### I've turned this on, since these warnings are now
-       unobtrusive. */
+    /* #### There should be some way to pass this type of data down
+     * into the glyph code, where you can get to it from lisp
+     * anyway. - WMP */
     {
-      int i;
+      int ii, num_text = 0;
+      png_textp text_ptr = NULL;
       DECLARE_EISTRING (key);
       DECLARE_EISTRING (text);
 
-      for (i = 0 ; i < info_ptr->num_text ; i++)
-	{
-	  /* How paranoid do I have to be about no trailing NULLs, and
-	     using (int)info_ptr->text[i].text_length, and strncpy and a temp
-	     string somewhere? */
-          eireset(key);
-          eireset(text);
-          eicpy_ext(key, info_ptr->text[i].key, Qbinary);
-          eicpy_ext(text, info_ptr->text[i].text, Qbinary);
+      if (png_get_text (png_ptr, info_ptr, &text_ptr, &num_text) > 0)
+        {
+          for (ii = 0 ; ii < num_text; ii++)
+            {
+              eireset (key);
+              eireset (text);
 
-	  warn_when_safe (Qpng, Qinfo, "%s - %s",
-			  eidata(key), eidata(text));
-	}
+              eicpy_ext (key, text_ptr[ii].key, Qbinary);
+              eicpy_ext (text, text_ptr[ii].text, Qbinary);
+
+              warn_when_safe (Qpng, Qinfo, "%s - %s", eidata (key),
+                              eidata (text));
+            }
+        }
     }
-#endif
 
-    xfree (row_pointers, Binbyte **);
+    xfree (row_pointers);
   }
 
   /* now instantiate */
@@ -1113,10 +1160,14 @@ tiff_instantiate_unwind (Lisp_Object unwind_obj)
   free_opaque_ptr (unwind_obj);
   if (data->tiff)
     {
-      TIFFClose(data->tiff);
+      TIFFClose (data->tiff);
+      data->tiff = 0;
     }
   if (data->eimage)
-    xfree (data->eimage, Binbyte *);
+    {
+      xfree (data->eimage);
+      data->eimage = 0;
+    }
 
   return Qnil;
 }
@@ -1287,7 +1338,8 @@ tiff_instantiate (Lisp_Object image_instance, Lisp_Object instantiator,
       /* An error was signaled. No clean up is needed, as unwind handles that
 	 for us.  Just pass the error along. */
       signal_image_error_2 ("TIFF decoding error",
-			    build_string(tiff_err_data.err_str),
+			    build_extstring (tiff_err_data.err_str,
+					      Qerror_message_encoding),
 			    instantiator);
     }
   TIFFSetErrorHandler ((TIFFErrorHandler)tiff_error_func);
@@ -1299,14 +1351,13 @@ tiff_instantiate (Lisp_Object image_instance, Lisp_Object instantiator,
 
     uint32 *raster;
     Binbyte *ep;
+    UINT_64_BIT pixels_sq;
 
     assert (!NILP (data));
 
     /* #### This is a definite problem under Mule due to the amount of
        stack data it might allocate.  Think about Lstreams... */
-    TO_EXTERNAL_FORMAT (LISP_STRING, data,
-			ALLOCA, (bytes, len),
-			Qbinary);
+    LISP_STRING_TO_SIZED_EXTERNAL (data, bytes, len, Qbinary);
     mem_struct.bytes = bytes;
     mem_struct.len = len;
     mem_struct.index = 0;
@@ -1321,12 +1372,15 @@ tiff_instantiate (Lisp_Object image_instance, Lisp_Object instantiator,
 
     TIFFGetField (unwind.tiff, TIFFTAG_IMAGEWIDTH, &width);
     TIFFGetField (unwind.tiff, TIFFTAG_IMAGELENGTH, &height);
-    unwind.eimage = xnew_binbytes (width * height * 3);
+    pixels_sq = (UINT_64_BIT) width * (UINT_64_BIT) height;
+    if (pixels_sq >= 1 << 29)
+      signal_image_error ("TIFF image too large to instantiate", instantiator);
+    unwind.eimage = xnew_binbytes ((size_t) pixels_sq * 3);
 
     /* #### This is little more than proof-of-concept/function testing.
        It needs to be reimplemented via scanline reads for both memory
        compactness. */
-    raster = (uint32*) _TIFFmalloc (width * height * sizeof (uint32));
+    raster = (uint32*) _TIFFmalloc ((tsize_t) (pixels_sq * sizeof (uint32)));
     if (raster != NULL)
       {
 	int i, j;
