@@ -2,15 +2,15 @@
    Copyright (C) 1993, 1994 Free Software Foundation, Inc.
    Copyright (C) 1995 Board of Trustees, University of Illinois.
    Copyright (C) 1995 Tinker Systems
-   Copyright (C) 1995, 1996, 2001, 2002, 2004, 2005 Ben Wing
+   Copyright (C) 1995, 1996, 2001, 2002, 2004, 2005, 2010 Ben Wing
    Copyright (C) 1995 Sun Microsystems
 
 This file is part of XEmacs.
 
-XEmacs is free software; you can redistribute it and/or modify it
+XEmacs is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
-Free Software Foundation; either version 2, or (at your option) any
-later version.
+Free Software Foundation, either version 3 of the License, or (at your
+option) any later version.
 
 XEmacs is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
@@ -18,9 +18,7 @@ FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 for more details.
 
 You should have received a copy of the GNU General Public License
-along with XEmacs; see the file COPYING.  If not, write to
-the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-Boston, MA 02111-1307, USA.  */
+along with XEmacs.  If not, see <http://www.gnu.org/licenses/>. */
 
 /* Synched up with: Not in FSF. */
 
@@ -67,7 +65,7 @@ Boston, MA 02111-1307, USA.  */
 
 #include "console-gtk-impl.h"
 #include "glyphs-gtk.h"
-#include "objects-gtk-impl.h"
+#include "fontcolor-gtk-impl.h"
 #include "ui-gtk.h"
 
 #include "sysfile.h"
@@ -268,7 +266,7 @@ convert_EImage_to_GDKImage (Lisp_Object device, int width, int height,
 #endif
 	    }
 	}
-      xfree(qtable, quant_table *);
+      xfree (qtable);
     } else {
       unsigned long rshift,gshift,bshift,rbits,gbits,bbits,junk;
       junk = vis->red_mask;
@@ -362,7 +360,7 @@ gtk_print_image_instance (struct Lisp_Image_Instance *p,
       if (IMAGE_INSTANCE_GTK_MASK (p))
 	write_fmt_string (printcharfun, "/0x%lx",
 			  (unsigned long) IMAGE_INSTANCE_GTK_MASK (p));
-      write_c_string (printcharfun, ")");
+      write_ascstring (printcharfun, ")");
       break;
 #ifdef HAVE_SUBWINDOWS
     case IMAGE_SUBWINDOW:
@@ -422,7 +420,7 @@ gtk_finalize_image_instance (struct Lisp_Image_Instance *p)
 		    gdk_pixmap_unref (IMAGE_INSTANCE_GTK_PIXMAP_SLICE (p,i));
 		    IMAGE_INSTANCE_GTK_PIXMAP_SLICE (p, i) = 0;
 		  }
-	      xfree (IMAGE_INSTANCE_GTK_PIXMAP_SLICES (p), GdkPixmap **);
+	      xfree (IMAGE_INSTANCE_GTK_PIXMAP_SLICES (p));
 	      IMAGE_INSTANCE_GTK_PIXMAP_SLICES (p) = 0;
 	    }
 
@@ -450,11 +448,11 @@ gtk_finalize_image_instance (struct Lisp_Image_Instance *p)
       && IMAGE_INSTANCE_TYPE (p) != IMAGE_SUBWINDOW
       && IMAGE_INSTANCE_GTK_PIXELS (p))
     {
-      xfree (IMAGE_INSTANCE_GTK_PIXELS (p), unsigned long *);
+      xfree (IMAGE_INSTANCE_GTK_PIXELS (p));
       IMAGE_INSTANCE_GTK_PIXELS (p) = 0;
     }
 
-  xfree (p->data, void *);
+  xfree (p->data);
   p->data = 0;
 }
 
@@ -483,7 +481,8 @@ gtk_image_instance_equal (struct Lisp_Image_Instance *p1,
 }
 
 static Hashcode
-gtk_image_instance_hash (struct Lisp_Image_Instance *p, int UNUSED (depth))
+gtk_image_instance_hash (struct Lisp_Image_Instance *p, int UNUSED (depth),
+                         Boolint UNUSED (equalp))
 {
   switch (IMAGE_INSTANCE_TYPE (p))
     {
@@ -819,8 +818,7 @@ void init_image_instance_from_gdk_pixmap (struct Lisp_Image_Instance *ii,
   GdkWindow *d;
   gint width, height, depth;
 
-  if (!DEVICE_GTK_P (device))
-    ABORT ();
+  assert (DEVICE_GTK_P (device));
 
   IMAGE_INSTANCE_DEVICE (ii) = device;
   IMAGE_INSTANCE_TYPE (ii) = IMAGE_COLOR_PIXMAP;
@@ -907,7 +905,7 @@ gtk_init_image_instance_from_eimage (struct Lisp_Image_Instance *ii,
       if (!gdk_image)
 	{
 	  if (pixtbl)
-	    xfree (pixtbl, unsigned long *);
+	    xfree (pixtbl);
 	  signal_image_error("EImage to GdkImage conversion failed", instantiator);
 	}
 
@@ -1089,9 +1087,9 @@ xbm_instantiate_1 (Lisp_Object image_instance, Lisp_Object instantiator,
 
   if (!NILP (mask_data))
     {
-      TO_EXTERNAL_FORMAT (LISP_STRING, XCAR (XCDR (XCDR (mask_data))),
-			  C_STRING_ALLOCA, gcc_may_you_rot_in_hell,
-			  Qfile_name);
+      gcc_may_you_rot_in_hell =
+	LISP_STRING_TO_EXTERNAL (XCAR (XCDR (XCDR (mask_data))),
+				 Qfile_name);
       mask =
 	pixmap_from_xbm_inline (IMAGE_INSTANCE_DEVICE (ii),
 				XINT (XCAR (mask_data)),
@@ -1116,9 +1114,7 @@ gtk_xbm_instantiate (Lisp_Object image_instance, Lisp_Object instantiator,
 
   assert (!NILP (data));
 
-  TO_EXTERNAL_FORMAT (LISP_STRING, XCAR (XCDR (XCDR (data))),
-		      C_STRING_ALLOCA, gcc_go_home,
-		      Qbinary);
+  gcc_go_home = LISP_STRING_TO_EXTERNAL (XCAR (XCDR (XCDR (data))), Qbinary);
 
   xbm_instantiate_1 (image_instance, instantiator, pointer_fg,
 		     pointer_bg, dest_mask, XINT (XCAR (data)),
@@ -1256,7 +1252,7 @@ gtk_xpm_instantiate (Lisp_Object image_instance, Lisp_Object instantiator,
 					   &nsymbols);
   assert (!NILP (data));
 
-  LISP_STRING_TO_EXTERNAL(data, dstring, Qbinary);
+  dstring = LISP_STRING_TO_EXTERNAL (data, Qbinary);
 
   /*
    * GTK only uses the 'c' color entry of an XPM and doesn't use the symbolic
@@ -1302,7 +1298,7 @@ gtk_xpm_instantiate (Lisp_Object image_instance, Lisp_Object instantiator,
   }
 
   if (color_symbols)
-    xfree (color_symbols, struct color_symbol *);
+    xfree (color_symbols);
 
   if (!pixmap)
     signal_image_error ("Error reading pixmap", data);
@@ -1329,10 +1325,10 @@ gtk_xpm_instantiate (Lisp_Object image_instance, Lisp_Object instantiator,
       break;
 
     case IMAGE_POINTER:
-	/* #### Gtk does not give us access to the hotspots of a pixmap */
+      /* #### Gtk does not give us access to the hotspots of a pixmap */
 
-	IMAGE_INSTANCE_PIXMAP_HOTSPOT_X (ii) = 1;
-	IMAGE_INSTANCE_PIXMAP_HOTSPOT_Y (ii) = 1;
+      IMAGE_INSTANCE_PIXMAP_HOTSPOT_X (ii) = make_int(1);
+      IMAGE_INSTANCE_PIXMAP_HOTSPOT_Y (ii) = make_int(1);
 
 
       image_instance_convert_to_pointer (ii, instantiator, pointer_fg,
@@ -1383,7 +1379,7 @@ gtk_xface_instantiate (Lisp_Object image_instance, Lisp_Object instantiator,
 
   assert (!NILP (data));
 
-  LISP_STRING_TO_EXTERNAL (data, dstring, Qbinary);
+  dstring = LISP_STRING_TO_EXTERNAL (data, Qbinary);
 
   if ((p = strchr (dstring, ':')))
     {
@@ -1775,7 +1771,7 @@ font_instantiate (Lisp_Object image_instance, Lisp_Object instantiator,
   source = gdk_font_load (source_name);
   if (! source)
     gui_error_2 ("couldn't load font",
-			   build_string (source_name),
+			   build_cistring (source_name),
 			   data);
   if (count == 2)
     mask = 0;
@@ -1787,7 +1783,7 @@ font_instantiate (Lisp_Object image_instance, Lisp_Object instantiator,
       if (!mask)
 	/* continuable */
 	Fsignal (Qgui_error, list3 (build_msg_string ("couldn't load font"),
-				    build_string (mask_name), data));
+				    build_cistring (mask_name), data));
     }
   if (!mask)
     mask_char = 0;
@@ -1930,9 +1926,7 @@ cursor_font_instantiate (Lisp_Object image_instance, Lisp_Object instantiator,
   if (!(dest_mask & IMAGE_POINTER_MASK))
     incompatible_image_types (instantiator, dest_mask, IMAGE_POINTER_MASK);
 
-  TO_EXTERNAL_FORMAT (LISP_STRING, data,
-		      C_STRING_ALLOCA, name_ext,
-		      Qfile_name);
+  name_ext = LISP_STRING_TO_EXTERNAL (data, Qfile_name);
 
   if ((i = cursor_name_to_index (name_ext)) == -1)
     invalid_argument ("Unrecognized cursor-font name", data);
@@ -2131,7 +2125,7 @@ gtk_redisplay_widget (Lisp_Image_Instance *p)
     {
       char* str;
       Lisp_Object val = IMAGE_INSTANCE_WIDGET_TEXT (p);
-      LISP_STRING_TO_EXTERNAL (val, str, Qnative);
+      str = LISP_STRING_TO_EXTERNAL (val, Qnative);
 
       /* #### Need to special case each type of GtkWidget here! */
     }
@@ -2279,7 +2273,7 @@ gtk_widget_instantiate_1 (Lisp_Object image_instance, Lisp_Object instantiator,
 
   if (!NILP (IMAGE_INSTANCE_WIDGET_TEXT (ii)))
     {
-      LISP_STRING_TO_EXTERNAL (IMAGE_INSTANCE_WIDGET_TEXT (ii), nm, Qnative);
+      nm = LISP_STRING_TO_EXTERNAL (IMAGE_INSTANCE_WIDGET_TEXT (ii), Qnative);
     }
 
   ii->data = xnew_and_zero (struct gtk_subwindow_data);
@@ -2534,9 +2528,7 @@ gtk_add_tab_item(Lisp_Object image_instance,
       name = item;
     }
 
-  TO_EXTERNAL_FORMAT (LISP_STRING, name,
-		      C_STRING_ALLOCA, c_name,
-		      Qctext);
+  c_name = LISP_STRING_TO_EXTERNAL (name, Qctext);
 
   /* Dummy widget that the notbook wants to display when a tab is selected. */
   box = gtk_vbox_new (FALSE, 3);
@@ -2941,7 +2933,7 @@ complex_vars_of_glyphs_gtk (void)
      vector3 (Qxbm, Q_data,					\
 	      list3 (make_int (name##_width),			\
 		     make_int (name##_height),			\
-		     make_ext_string ((Extbyte*) name##_bits,	\
+		     make_extstring ((Extbyte*) name##_bits,	\
 				      sizeof (name##_bits),	\
 				      Qbinary))),		\
      Qglobal, Qgtk, Qnil)
@@ -2955,7 +2947,7 @@ complex_vars_of_glyphs_gtk (void)
 }
 
 /* X specific crap */
-#include <gdk/gdkx.h>
+#include "sysgdkx.h"
 /* #### Should remove all this X specific stuff when GTK/GDK matures a
    bit more and provides an abstraction for it. */
 static int
