@@ -4,10 +4,10 @@
 
 This file is part of XEmacs.
 
-XEmacs is free software; you can redistribute it and/or modify it
+XEmacs is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
-Free Software Foundation; either version 2, or (at your option) any
-later version.
+Free Software Foundation, either version 3 of the License, or (at your
+option) any later version.
 
 XEmacs is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
@@ -15,9 +15,7 @@ FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 for more details.
 
 You should have received a copy of the GNU General Public License
-along with XEmacs; see the file COPYING.  If not, write to
-the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-Boston, MA 02111-1307, USA.  */
+along with XEmacs.  If not, see <http://www.gnu.org/licenses/>. */
 
 /* Synched up with: Not in FSF. */
 
@@ -47,7 +45,7 @@ Boston, MA 02111-1307, USA.  */
 
 #include "console-msw-impl.h"
 #include "glyphs-msw.h"
-#include "objects-msw-impl.h"
+#include "fontcolor-msw-impl.h"
 
 #define WIDGET_GLYPH_SLOT 0
 
@@ -202,7 +200,7 @@ static BITMAPINFO *convert_EImage_to_DIBitmap (Lisp_Object device,
 
       if (!bmp_data)
 	{
-	  xfree (bmp_info, BITMAPINFO *);
+	  xfree (bmp_info);
 	  return NULL;
 	}
 
@@ -233,7 +231,7 @@ static BITMAPINFO *convert_EImage_to_DIBitmap (Lisp_Object device,
 					     sizeof(RGBQUAD) * ncolors);
       if (!bmp_info)
 	{
-	  xfree (qtable, quant_table *);
+	  xfree (qtable);
 	  return NULL;
 	}
 
@@ -251,8 +249,8 @@ static BITMAPINFO *convert_EImage_to_DIBitmap (Lisp_Object device,
 
       if (!*bmp_data)
 	{
-	  xfree (qtable, quant_table *);
-	  xfree (bmp_info, BITMAPINFO *);
+	  xfree (qtable);
+	  xfree (bmp_info);
 	  return NULL;
 	}
 
@@ -279,7 +277,7 @@ static BITMAPINFO *convert_EImage_to_DIBitmap (Lisp_Object device,
 	      *dp++ = QUANT_GET_COLOR (qtable,rd,gr,bl);
 	    }
 	}
-      xfree (qtable, quant_table *);
+      xfree (qtable);
     }
   /* fix up the standard stuff */
   bmp_info->bmiHeader.biWidth = width;
@@ -474,8 +472,8 @@ mswindows_init_image_instance_from_eimage (Lisp_Image_Instance *ii,
 	image_instance_add_dibitmap (ii, bmp_info, bmp_data, bmp_bits, slice,
 				     instantiator);
 
-      xfree (bmp_info, BITMAPINFO *);
-      xfree (bmp_data, Binbyte *);
+      xfree (bmp_info);
+      xfree (bmp_data);
     }
 }
 
@@ -543,7 +541,7 @@ mswindows_initialize_image_instance_mask (Lisp_Image_Instance *image,
 				 (void **) (void *) &and_bits,
 				 0,0)))
     {
-      xfree (bmp_info, BITMAPINFO *);
+      xfree (bmp_info);
       return;
     }
 
@@ -570,7 +568,7 @@ mswindows_initialize_image_instance_mask (Lisp_Image_Instance *image,
 		 bmp_info,
 		 DIB_RGB_COLORS) <= 0)
     {
-      xfree (bmp_info, BITMAPINFO *);
+      xfree (bmp_info);
       return;
     }
 
@@ -602,8 +600,8 @@ mswindows_initialize_image_instance_mask (Lisp_Image_Instance *image,
 	     bmp_info,
 	     DIB_RGB_COLORS);
 
-  xfree (bmp_info, BITMAPINFO *);
-  xfree (dibits, Binbyte *);
+  xfree (bmp_info);
+  xfree (dibits);
 
   SelectObject(hcdc, old);
 
@@ -854,7 +852,7 @@ static int xpm_to_eimage (Lisp_Object image, const Extbyte *buffer,
   colortbl = xnew_array_and_zero (COLORREF, xpmimage.ncolors);
   if (!colortbl)
     {
-      xfree (*data, Binbyte *);
+      xfree (*data);
       XpmFreeXpmImage (&xpmimage);
       XpmFreeXpmInfo (&xpminfo);
       return 0;
@@ -910,8 +908,8 @@ static int xpm_to_eimage (Lisp_Object image, const Extbyte *buffer,
 	}
 
     label_no_color:
-      xfree (*data, Binbyte *);
-      xfree (colortbl, COLORREF *);
+      xfree (*data);
+      xfree (colortbl);
       XpmFreeXpmImage (&xpmimage);
       XpmFreeXpmInfo (&xpminfo);
       return 0;
@@ -937,7 +935,7 @@ static int xpm_to_eimage (Lisp_Object image, const Extbyte *buffer,
 
   XpmFreeXpmImage (&xpmimage);
   XpmFreeXpmInfo (&xpminfo);
-  xfree (colortbl, COLORREF *);
+  xfree (colortbl);
   return TRUE;
 }
 
@@ -968,9 +966,7 @@ mswindows_xpm_instantiate (Lisp_Object image_instance,
 
   assert (!NILP (data));
 
-  TO_EXTERNAL_FORMAT (LISP_STRING, data,
-		      ALLOCA, (bytes, len),
-		      Qbinary);
+  LISP_STRING_TO_SIZED_EXTERNAL (data, bytes, len, Qbinary);
 
   /* in case we have color symbols */
   color_symbols = extract_xpm_color_names (device, domain,
@@ -988,9 +984,9 @@ mswindows_xpm_instantiate (Lisp_Object image_instance,
     {
       while (nsymbols--)
 	{
-	  xfree (color_symbols[nsymbols].name, Ibyte *);
+	  xfree (color_symbols[nsymbols].name);
 	}
-      xfree(color_symbols, struct color_symbol *);
+      xfree (color_symbols);
     }
 
   /* build a bitmap from the eimage */
@@ -1000,15 +996,15 @@ mswindows_xpm_instantiate (Lisp_Object image_instance,
       signal_image_error ("XPM to EImage conversion failed",
 			  image_instance);
     }
-  xfree (eimage, Binbyte *);
+  xfree (eimage);
 
   /* Now create the pixmap and set up the image instance */
   init_image_instance_from_dibitmap (ii, bmp_info, dest_mask,
 				     bmp_data, bmp_bits, 1, instantiator,
 				     x_hot, y_hot, transp);
 
-  xfree (bmp_info, BITMAPINFO *);
-  xfree (bmp_data, Binbyte *);
+  xfree (bmp_info);
+  xfree (bmp_data);
 }
 #endif /* HAVE_XPM */
 
@@ -1055,9 +1051,7 @@ bmp_instantiate (Lisp_Object image_instance, Lisp_Object instantiator,
 
   assert (!NILP (data));
 
-  TO_EXTERNAL_FORMAT (LISP_STRING, data,
-		      ALLOCA, (bytes, len),
-		      Qbinary);
+  LISP_STRING_TO_SIZED_EXTERNAL (data, bytes, len, Qbinary);
 
   /* Then slurp the image into memory, decoding along the way.
      The result is the image in a simple one-byte-per-pixel
@@ -1101,7 +1095,7 @@ mswindows_resource_possible_dest_types (void)
 
 typedef struct
 {
-  CIbyte *name;
+  const Ascbyte *name;
   int	resource_id;
 } resource_t;
 
@@ -1243,7 +1237,7 @@ mswindows_resource_instantiate (Lisp_Object image_instance,
     {
       Extbyte *fname;
 
-      LOCAL_FILE_FORMAT_TO_TSTR (file, fname);
+      LISP_LOCAL_FILE_FORMAT_TO_TSTR (file, fname);
 
       if (NILP (resource_id))
 	resid = fname;
@@ -1254,7 +1248,7 @@ mswindows_resource_instantiate (Lisp_Object image_instance,
 							      type));
 
 	  if (!resid)
-	    LISP_STRING_TO_TSTR (resource_id, resid);
+	    resid = LISP_STRING_TO_TSTR (resource_id);
 	}
     }
   else if (!(resid = MAKEINTRESOURCE (resource_name_to_resource (resource_id,
@@ -1361,7 +1355,7 @@ xbm_create_bitmap_from_data (HDC hdc, const Binbyte *data,
 
   if (!new_data)
     {
-      xfree (bmp_info, BITMAPINFO *);
+      xfree (bmp_info);
       return NULL;
     }
 
@@ -1412,17 +1406,17 @@ xbm_create_bitmap_from_data (HDC hdc, const Binbyte *data,
 			     &bmp_buf,
 			     0,0);
 
-  xfree (bmp_info, BITMAPINFO *);
+  xfree (bmp_info);
 
   if (!bitmap || !bmp_buf)
     {
-      xfree (new_data, Binbyte *);
+      xfree (new_data);
       return NULL;
     }
 
   /* copy in the actual bitmap */
   memcpy (bmp_buf, new_data, height * new_width);
-  xfree (new_data, Binbyte *);
+  xfree (new_data);
 
   return bitmap;
 }
@@ -1573,9 +1567,9 @@ xbm_instantiate_1 (Lisp_Object image_instance, Lisp_Object instantiator,
     {
       Binbyte *ext_data;
 
-      TO_EXTERNAL_FORMAT (LISP_STRING, XCAR (XCDR (XCDR (mask_data))),
-			  C_STRING_ALLOCA, ext_data,
-			  Qbinary);
+      ext_data =
+	(Binbyte *) LISP_STRING_TO_EXTERNAL (XCAR (XCDR (XCDR (mask_data))),
+					     Qbinary);
       mask = xbm_create_bitmap_from_data (hdc,
 					  ext_data,
 					  XINT (XCAR (mask_data)),
@@ -1603,9 +1597,9 @@ mswindows_xbm_instantiate (Lisp_Object image_instance,
 
   assert (!NILP (data));
 
-  TO_EXTERNAL_FORMAT (LISP_STRING, XCAR (XCDR (XCDR (data))),
-		      C_STRING_ALLOCA, ext_data,
-		      Qbinary);
+  ext_data =
+    (const Binbyte *) LISP_STRING_TO_EXTERNAL (XCAR (XCDR (XCDR (data))),
+					       Qbinary);
 
   xbm_instantiate_1 (image_instance, instantiator, pointer_fg,
 		     pointer_bg, dest_mask, XINT (XCAR (data)),
@@ -1650,9 +1644,7 @@ mswindows_xface_instantiate (Lisp_Object image_instance,
 
   assert (!NILP (data));
 
-  TO_EXTERNAL_FORMAT (LISP_STRING, data,
-		      C_STRING_ALLOCA, dstring,
-		      Qbinary);
+  dstring = (const Binbyte *) LISP_STRING_TO_EXTERNAL (data, Qbinary);
 
   if ((p = (Binbyte *) strchr ((char *) dstring, ':')))
     {
@@ -1724,7 +1716,7 @@ mswindows_print_image_instance (Lisp_Image_Instance *p,
 	  write_fmt_string (printcharfun, "/0x%lx",
 			    (unsigned long) IMAGE_INSTANCE_MSWINDOWS_MASK (p));
 	}
-      write_c_string (printcharfun, ")");
+      write_ascstring (printcharfun, ")");
       break;
 
     default:
@@ -1784,7 +1776,7 @@ mswindows_finalize_image_instance (Lisp_Image_Instance *p)
 		    DeleteObject (IMAGE_INSTANCE_MSWINDOWS_BITMAP_SLICE (p, i));
 		  IMAGE_INSTANCE_MSWINDOWS_BITMAP_SLICE (p, i) = 0;
 		}
-	      xfree (IMAGE_INSTANCE_MSWINDOWS_BITMAP_SLICES (p), HBITMAP *);
+	      xfree (IMAGE_INSTANCE_MSWINDOWS_BITMAP_SLICES (p));
 	      IMAGE_INSTANCE_MSWINDOWS_BITMAP_SLICES (p) = 0;
 	    }
 	  if (IMAGE_INSTANCE_MSWINDOWS_MASK (p))
@@ -1798,7 +1790,7 @@ mswindows_finalize_image_instance (Lisp_Image_Instance *p)
 
   if (p->data)
     {
-      xfree (p->data, void *);
+      xfree (p->data);
       p->data = 0;
     }
 }
@@ -1968,7 +1960,7 @@ mswindows_map_subwindow (Lisp_Image_Instance *p, int x, int y,
       /* Doing this once does not seem to be enough, for instance when
 	 mapping the search dialog this gets called four times. If we
 	 only set on the first time through then the subwindow never
-	 gets focus as intended. However, doing this everytime doesn't
+	 gets focus as intended. However, doing this every time doesn't
 	 seem so bad, after all we only need to redo this after the
 	 focus changes - and if that happens resetting the initial
 	 focus doesn't seem so bad. */
@@ -2032,7 +2024,7 @@ mswindows_redisplay_widget (Lisp_Image_Instance *p)
       && !NILP (IMAGE_INSTANCE_WIDGET_TEXT (p)))
     {
       Extbyte *lparam = 0;
-      LISP_STRING_TO_TSTR (IMAGE_INSTANCE_WIDGET_TEXT (p), lparam);
+      lparam = LISP_STRING_TO_TSTR (IMAGE_INSTANCE_WIDGET_TEXT (p));
       qxeSendMessage (WIDGET_INSTANCE_MSWINDOWS_HANDLE (p),
 		      WM_SETTEXT, 0, (LPARAM) lparam);
     }
@@ -2146,7 +2138,7 @@ mswindows_subwindow_instantiate (Lisp_Object image_instance,
 			  GWL_HINSTANCE),
 			 NULL);
 
-  qxeSetWindowLong (wnd, GWL_USERDATA, (LONG)LISP_TO_VOID(image_instance));
+  qxeSetWindowLong (wnd, GWL_USERDATA, (LONG)STORE_LISP_IN_VOID(image_instance));
   IMAGE_INSTANCE_SUBWINDOW_ID (ii) = wnd;
 }
 
@@ -2250,7 +2242,7 @@ mswindows_widget_instantiate (Lisp_Object image_instance,
     }
 
   if (!NILP (IMAGE_INSTANCE_WIDGET_TEXT (ii)))
-    LISP_STRING_TO_TSTR (IMAGE_INSTANCE_WIDGET_TEXT (ii), nm);
+    nm = LISP_STRING_TO_TSTR (IMAGE_INSTANCE_WIDGET_TEXT (ii));
 
   /* allocate space for the clip window and then allocate the clip window */
   ii->data = xnew_and_zero (struct mswindows_subwindow_data);
@@ -2272,7 +2264,7 @@ mswindows_widget_instantiate (Lisp_Object image_instance,
     gui_error ("window creation failed with code",
 	       make_int (GetLastError()));
 
-  C_STRING_TO_TSTR (class_, classext);
+  classext = ITEXT_TO_TSTR (class_);
 
   if ((wnd = qxeCreateWindowEx (exflags /* | WS_EX_NOPARENTNOTIFY*/,
 				classext,
@@ -2294,7 +2286,7 @@ mswindows_widget_instantiate (Lisp_Object image_instance,
 	       make_int (GetLastError()));
 
   IMAGE_INSTANCE_SUBWINDOW_ID (ii) = wnd;
-  qxeSetWindowLong (wnd, GWL_USERDATA, (LONG)LISP_TO_VOID(image_instance));
+  qxeSetWindowLong (wnd, GWL_USERDATA, (LONG)STORE_LISP_IN_VOID(image_instance));
   /* set the widget font from the widget face */
   if (!NILP (IMAGE_INSTANCE_WIDGET_TEXT (ii)))
     qxeSendMessage (wnd, WM_SETFONT,
@@ -2486,10 +2478,11 @@ static HTREEITEM add_tree_item (Lisp_Object image_instance,
       tvitem.item.lParam =
 	mswindows_register_gui_item (image_instance, item, domain);
       tvitem.item.mask |= TVIF_PARAM;
-      LISP_STRING_TO_TSTR (XGUI_ITEM (item)->name, tvitem.item.pszText);
+      tvitem.item.pszText =
+	(LPWSTR) LISP_STRING_TO_TSTR (XGUI_ITEM (item)->name);
     }
   else
-    LISP_STRING_TO_TSTR (item, tvitem.item.pszText);
+    tvitem.item.pszText = (LPWSTR) LISP_STRING_TO_TSTR (item);
       
   tvitem.item.cchTextMax = qxetcslen ((Extbyte *) tvitem.item.pszText);
       
@@ -2602,12 +2595,12 @@ add_tab_item (Lisp_Object image_instance,
       tcitem.lParam =
 	mswindows_register_gui_item (image_instance, item, domain);
       tcitem.mask |= TCIF_PARAM;
-      LISP_STRING_TO_TSTR (XGUI_ITEM (item)->name, tcitem.pszText);
+      tcitem.pszText = (XELPTSTR) LISP_STRING_TO_TSTR (XGUI_ITEM (item)->name);
     }
   else
     {
       CHECK_STRING (item);
-      LISP_STRING_TO_TSTR (item, tcitem.pszText);
+      tcitem.pszText = (XELPTSTR) LISP_STRING_TO_TSTR (item);
     }
 
   tcitem.cchTextMax = qxetcslen ((Extbyte *) tcitem.pszText);
@@ -2785,7 +2778,7 @@ mswindows_combo_box_instantiate (Lisp_Object image_instance,
 		      pointer_bg, dest_mask, domain);
 
   /* We now have everything right apart from the height. */
-  default_face_font_info (domain, 0, 0, &height, 0, 0);
+  default_face_font_info (domain, 0, 0, 0, &height, 0);
   GET_LIST_LENGTH (items, len);
 
   height = (height + DEFAULT_WIDGET_BORDER_WIDTH * 2 ) * len;
@@ -2813,7 +2806,7 @@ mswindows_combo_box_instantiate (Lisp_Object image_instance,
   LIST_LOOP (rest, items)
     {
       Extbyte *lparam;
-      LISP_STRING_TO_TSTR (XCAR (rest), lparam);
+      lparam = LISP_STRING_TO_TSTR (XCAR (rest));
       if (qxeSendMessage (wnd, CB_ADDSTRING, 0, (LPARAM)lparam) == CB_ERR)
 	gui_error ("error adding combo entries", instantiator);
     }
@@ -2930,8 +2923,7 @@ mswindows_widget_query_string_geometry (Lisp_Object string, Lisp_Object face,
       SIZE size;
 
       SelectObject (hdc, mswindows_widget_hfont (face, domain, string));
-      TO_EXTERNAL_FORMAT (LISP_STRING, string, ALLOCA, (str, len),
-			  Qmswindows_tstr);
+      LISP_STRING_TO_SIZED_EXTERNAL (string, str, len, Qmswindows_tstr);
       qxeGetTextExtentPoint32 (hdc, str, len / XETCHAR_SIZE, &size);
       *width = size.cx;
     }
