@@ -5,20 +5,18 @@
    
    This file is part of XEmacs.
    
-   XEmacs is free software; you can redistribute it and/or modify it
+   XEmacs is free software: you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
-   Free Software Foundation; either version 2, or (at your option) any
-   later version.
-   
+   Free Software Foundation, either version 3 of the License, or (at your
+   option) any later version.
+
    XEmacs is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.
-   
+
    You should have received a copy of the GNU General Public License
-   along with XEmacs; see the file COPYING.  If not, write to
-   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.  */
+   along with XEmacs.  If not, see <http://www.gnu.org/licenses/>. */
 
 /* Synched up with: Not in FSF. */
 
@@ -186,7 +184,7 @@ This function is the process handler for the GPM connection.
 
 static void turn_off_gpm (char *process_name)
 {
-  Lisp_Object process = Fget_process (build_string (process_name));
+  Lisp_Object process = Fget_process (build_cistring (process_name));
   int fd = -1;
 
   if (NILP (process))
@@ -201,7 +199,7 @@ static void turn_off_gpm (char *process_name)
 
   clear_gpm_state (fd);
 
-  Fdelete_process (build_string (process_name));
+  Fdelete_process (build_cistring (process_name));
 }
 
 #ifdef TIOCLINUX
@@ -493,7 +491,7 @@ Return non-nil if GPM mouse support is currently enabled on DEVICE.
   snprintf (process_name, sizeof(process_name) - 1, "gpm for %s",
 	    console_name);
 
-  proc = Fget_process (build_string (process_name));
+  proc = Fget_process (build_cistring (process_name));
 
   if (NILP (proc))
     return (Qnil);
@@ -585,7 +583,7 @@ Toggle accepting of GPM mouse events.
       set_descriptor_non_blocking (gpm_fd);
       store_gpm_state (gpm_fd);
       gpm_process =
-	connect_to_file_descriptor (build_string (process_name), Qnil,
+	connect_to_file_descriptor (build_cistring (process_name), Qnil,
 				    make_int (gpm_fd),
 				    make_int (gpm_fd));
 
