@@ -6,10 +6,10 @@
 
 This file is part of XEmacs.
 
-XEmacs is free software; you can redistribute it and/or modify it
+XEmacs is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
-Free Software Foundation; either version 2, or (at your option) any
-later version.
+Free Software Foundation, either version 3 of the License, or (at your
+option) any later version.
 
 XEmacs is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
@@ -17,9 +17,7 @@ FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 for more details.
 
 You should have received a copy of the GNU General Public License
-along with XEmacs; see the file COPYING.  If not, write to
-the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-Boston, MA 02111-1307, USA.  */
+along with XEmacs.  If not, see <http://www.gnu.org/licenses/>. */
 
 /* Synched up with: Not in FSF. */
 
@@ -81,9 +79,9 @@ snarf_widget_value_mapper (widget_value *val, void *closure)
   struct widget_value_mapper *z = (struct widget_value_mapper *) closure;
 
   if (val->call_data)
-    z->protect_me = Fcons (VOID_TO_LISP (val->call_data), z->protect_me);
+    z->protect_me = Fcons (GET_LISP_FROM_VOID (val->call_data), z->protect_me);
   if (val->accel)
-    z->protect_me = Fcons (VOID_TO_LISP (val->accel), z->protect_me);
+    z->protect_me = Fcons (GET_LISP_FROM_VOID (val->accel), z->protect_me);
 
   return 0;
 }
@@ -198,9 +196,9 @@ void
 free_popup_widget_value_tree (widget_value *wv)
 {
   if (! wv) return;
-  if (wv->key) xfree (wv->key, char *);
-  if (wv->value) xfree (wv->value, char *);
-  if (wv->name) xfree (wv->name, char *);
+  if (wv->key) xfree (wv->key);
+  if (wv->value) xfree (wv->value);
+  if (wv->name) xfree (wv->name);
 
   wv->name = wv->value = wv->key = (char *) 0xDEADBEEF; /* -559038737 base 10*/
 
@@ -243,7 +241,7 @@ popup_selection_callback (Widget widget, LWLIB_ID UNUSED (id),
     return;
   if (((EMACS_INT) client_data) == 0)
     return;
-  data = VOID_TO_LISP (client_data);
+  data = GET_LISP_FROM_VOID (client_data);
   frame = wrap_frame (f);
 
 #if 0
@@ -325,8 +323,9 @@ popup_selection_callback (Widget widget, LWLIB_ID UNUSED (id),
   Lisp_Object wses_form = (form);					\
   (slot) = (NILP (wses_form) ? 0 :					\
 	    EQ (wses_form, Qt) ? 1 :					\
-	    !NILP (in_display ? eval_within_redisplay (wses_form)	\
-		   : Feval (wses_form)));				\
+	    !NILP (in_display ?                                         \
+                   IGNORE_MULTIPLE_VALUES (eval_within_redisplay (wses_form)) \
+		   : IGNORE_MULTIPLE_VALUES (Feval (wses_form))));      \
 } while (0)
 #else
   /* Treat the activep slot of the menu item as a boolean */
@@ -359,10 +358,7 @@ menu_separator_style_and_to_external (const Ibyte *s)
 	    : xstrdup ("shadowDoubleEtchedIn"));
   else if (*p == ':')
     {
-      Extbyte *retval;
-
-      C_STRING_TO_EXTERNAL_MALLOC (p + 1, retval, Qlwlib_encoding);
-      return retval;
+      return ITEXT_TO_EXTERNAL_MALLOC (p + 1, Qlwlib_encoding);
     }
 
   return NULL;
@@ -384,7 +380,7 @@ add_accel_and_to_external (Lisp_Object string)
       }
 
   if (found_accel)
-    LISP_STRING_TO_EXTERNAL_MALLOC (string, retval, Qlwlib_encoding);
+    retval = LISP_STRING_TO_EXTERNAL_MALLOC (string, Qlwlib_encoding);
   else
     {
       Bytecount namelen = XSTRING_LENGTH (string);
@@ -392,7 +388,7 @@ add_accel_and_to_external (Lisp_Object string)
       chars[0] = '%';
       chars[1] = '_';
       memcpy (chars + 2, name, namelen + 1);
-      C_STRING_TO_EXTERNAL_MALLOC (chars, retval, Qlwlib_encoding);
+      retval = ITEXT_TO_EXTERNAL_MALLOC (chars, Qlwlib_encoding);
     }
 
   return retval;
@@ -416,7 +412,7 @@ button_item_to_widget_value (Lisp_Object gui_object_instance,
       if (accel_p)
 	wv->name = add_accel_and_to_external (gui_item);
       else
-	LISP_STRING_TO_EXTERNAL_MALLOC (gui_item, wv->name, Qlwlib_encoding);
+	wv->name = LISP_STRING_TO_EXTERNAL_MALLOC (gui_item, Qlwlib_encoding);
       return 1;
     }
   else if (!GUI_ITEMP (gui_item))
@@ -436,18 +432,18 @@ button_item_to_widget_value (Lisp_Object gui_object_instance,
 #endif /* HAVE_MENUBARS */
 
   if (!STRINGP (pgui->name))
-    pgui->name = Feval (pgui->name);
+    pgui->name = IGNORE_MULTIPLE_VALUES (Feval (pgui->name));
 
   CHECK_STRING (pgui->name);
   if (accel_p)
     {
       wv->name = add_accel_and_to_external (pgui->name);
-      wv->accel = LISP_TO_VOID (gui_item_accelerator (gui_item));
+      wv->accel = STORE_LISP_IN_VOID (gui_item_accelerator (gui_item));
     }
   else
     {
-      LISP_STRING_TO_EXTERNAL_MALLOC (pgui->name, wv->name, Qlwlib_encoding);
-      wv->accel = LISP_TO_VOID (Qnil);
+      wv->name = LISP_STRING_TO_EXTERNAL_MALLOC (pgui->name, Qlwlib_encoding);
+      wv->accel = STORE_LISP_IN_VOID (Qnil);
     }
 
   if (!NILP (pgui->suffix))
@@ -459,18 +455,18 @@ button_item_to_widget_value (Lisp_Object gui_object_instance,
 	suffix2 = pgui->suffix;
       else
 	{
-	  suffix2 = Feval (pgui->suffix);
+	  suffix2 = IGNORE_MULTIPLE_VALUES (Feval (pgui->suffix));
 	  CHECK_STRING (suffix2);
 	}
 
-      LISP_STRING_TO_EXTERNAL_MALLOC (suffix2, wv->value, Qlwlib_encoding);
+      wv->value = LISP_STRING_TO_EXTERNAL_MALLOC (suffix2, Qlwlib_encoding);
     }
 
   wv_set_evalable_slot (wv->enabled, pgui->active);
   wv_set_evalable_slot (wv->selected, pgui->selected);
 
   if (!NILP (pgui->callback) || !NILP (pgui->callback_ex))
-    wv->call_data = LISP_TO_VOID (cons3 (gui_object_instance,
+    wv->call_data = STORE_LISP_IN_VOID (cons3 (gui_object_instance,
 					 pgui->callback,
 					 pgui->callback_ex));
 
@@ -485,7 +481,7 @@ button_item_to_widget_value (Lisp_Object gui_object_instance,
       CHECK_STRING (pgui->keys);
       pgui->keys = Fsubstitute_command_keys (pgui->keys);
       if (XSTRING_LENGTH (pgui->keys) > 0)
-	LISP_STRING_TO_EXTERNAL_MALLOC (pgui->keys, wv->key, Qlwlib_encoding);
+	wv->key = LISP_STRING_TO_EXTERNAL_MALLOC (pgui->keys, Qlwlib_encoding);
       else
 	wv->key = 0;
     }
@@ -495,7 +491,7 @@ button_item_to_widget_value (Lisp_Object gui_object_instance,
       /* #### Warning, dependency here on current_buffer and point */
       where_is_to_char (pgui->callback, buf);
       if (eilen (buf) > 0)
-	C_STRING_TO_EXTERNAL_MALLOC (eidata (buf), wv->key, Qlwlib_encoding);
+	wv->key = ITEXT_TO_EXTERNAL_MALLOC (eidata (buf), Qlwlib_encoding);
       else
 	wv->key = 0;
       eifree (buf);
