@@ -1,12 +1,13 @@
 /* Define general gutter support.
    Copyright (C) 1999 Andy Piper.
+   Copyright (C) 2010 Ben Wing.
 
 This file is part of XEmacs.
 
-XEmacs is free software; you can redistribute it and/or modify it
+XEmacs is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
-Free Software Foundation; either version 2, or (at your option) any
-later version.
+Free Software Foundation, either version 3 of the License, or (at your
+option) any later version.
 
 XEmacs is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
@@ -14,9 +15,7 @@ FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 for more details.
 
 You should have received a copy of the GNU General Public License
-along with XEmacs; see the file COPYING.  If not, write to
-the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-Boston, MA 02111-1307, USA.  */
+along with XEmacs.  If not, see <http://www.gnu.org/licenses/>. */
 
 /* Synched up with: Not in FSF. */
 
@@ -36,18 +35,6 @@ DECLARE_SPECIFIER_TYPE (gutter);
 #define DEFAULT_GUTTER_WIDTH		40
 #define DEFAULT_GUTTER_BORDER_WIDTH	2
 
-enum gutter_pos
-{
-  TOP_GUTTER     = 0,
-  BOTTOM_GUTTER  = 1,
-  LEFT_GUTTER    = 2,
-  RIGHT_GUTTER   = 3
-};
-
-/* Iterate over all possible gutter positions */
-#define GUTTER_POS_LOOP(var) \
-  for (var = (enum gutter_pos) 0; var < 4; var = (enum gutter_pos) (var + 1))
-
 extern Lisp_Object Qgutter;
 
 extern Lisp_Object Vgutter_size[4];
@@ -97,13 +84,13 @@ int display_boxes_in_gutter_p (struct frame *f, struct display_box* db,
 
 /* these macros predicate size on position and type of window */
 #define WINDOW_REAL_TOP_GUTTER_BOUNDS(w)	\
-   WINDOW_REAL_GUTTER_BOUNDS (w,TOP_GUTTER)
+   WINDOW_REAL_GUTTER_BOUNDS (w, TOP_EDGE)
 #define WINDOW_REAL_BOTTOM_GUTTER_BOUNDS(w)	\
-   WINDOW_REAL_GUTTER_BOUNDS (w,BOTTOM_GUTTER)
+   WINDOW_REAL_GUTTER_BOUNDS (w, BOTTOM_EDGE)
 #define WINDOW_REAL_LEFT_GUTTER_BOUNDS(w)	\
-   WINDOW_REAL_GUTTER_BOUNDS (w,LEFT_GUTTER)
+   WINDOW_REAL_GUTTER_BOUNDS (w, LEFT_EDGE)
 #define WINDOW_REAL_RIGHT_GUTTER_BOUNDS(w)	\
-   WINDOW_REAL_GUTTER_BOUNDS (w,RIGHT_GUTTER)
+   WINDOW_REAL_GUTTER_BOUNDS (w, RIGHT_EDGE)
 
 #define FRAME_GUTTER_VISIBLE(f, pos) \
    WINDOW_REAL_GUTTER_VISIBLE (XWINDOW (FRAME_LAST_NONMINIBUF_WINDOW (f)), pos)
@@ -118,13 +105,9 @@ int display_boxes_in_gutter_p (struct frame *f, struct display_box* db,
 WINDOW_GUTTER (XWINDOW (FRAME_LAST_NONMINIBUF_WINDOW (f)), pos)
 
 /* these macros predicate size on position and type of window */
-#define FRAME_TOP_GUTTER_BOUNDS(f) \
-   WINDOW_REAL_GUTTER_BOUNDS (XWINDOW (FRAME_LAST_NONMINIBUF_WINDOW (f)), TOP_GUTTER)
-#define FRAME_BOTTOM_GUTTER_BOUNDS(f) \
-   WINDOW_REAL_GUTTER_BOUNDS (XWINDOW (FRAME_LAST_NONMINIBUF_WINDOW (f)), BOTTOM_GUTTER)
-#define FRAME_LEFT_GUTTER_BOUNDS(f) \
-   WINDOW_REAL_GUTTER_BOUNDS (XWINDOW (FRAME_LAST_NONMINIBUF_WINDOW (f)), LEFT_GUTTER)
-#define FRAME_RIGHT_GUTTER_BOUNDS(f) \
-   WINDOW_REAL_GUTTER_BOUNDS (XWINDOW (FRAME_LAST_NONMINIBUF_WINDOW (f)), RIGHT_GUTTER)
+#define FRAME_TOP_GUTTER_BOUNDS(f) FRAME_GUTTER_BOUNDS (f, TOP_EDGE)
+#define FRAME_BOTTOM_GUTTER_BOUNDS(f) FRAME_GUTTER_BOUNDS (f, BOTTOM_EDGE)
+#define FRAME_LEFT_GUTTER_BOUNDS(f) FRAME_GUTTER_BOUNDS (f, LEFT_EDGE)
+#define FRAME_RIGHT_GUTTER_BOUNDS(f) FRAME_GUTTER_BOUNDS (f, RIGHT_EDGE)
 
 #endif /* INCLUDED_gutter_h_ */
