@@ -1,13 +1,13 @@
 /* Hash tables.
    Copyright (C) 1992, 1993, 1994 Free Software Foundation, Inc.
-   Copyright (C) 2003, 2004 Ben Wing.
+   Copyright (C) 2003, 2004, 2010 Ben Wing.
 
 This file is part of XEmacs.
 
-XEmacs is free software; you can redistribute it and/or modify it
+XEmacs is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
-Free Software Foundation; either version 2, or (at your option) any
-later version.
+Free Software Foundation, either version 3 of the License, or (at your
+option) any later version.
 
 XEmacs is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
@@ -15,9 +15,7 @@ FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 for more details.
 
 You should have received a copy of the GNU General Public License
-along with XEmacs; see the file COPYING.  If not, write to
-the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-Boston, MA 02111-1307, USA.  */
+along with XEmacs.  If not, see <http://www.gnu.org/licenses/>. */
 
 /* Synched up with: Not in FSF. */
 
@@ -28,7 +26,7 @@ Boston, MA 02111-1307, USA.  */
 #include "lisp.h"
 #include "hash.h"
 
-#define NULL_ENTRY ((void *) 0xdeadbeef) /* -559038737 base 10 */
+#define NULL_ENTRY ((void *) 0xDEADBEEF) /* -559038737 base 10 */
 
 #define COMFORTABLE_SIZE(size) (21 * (size) / 16)
 
@@ -174,8 +172,8 @@ clrhash (struct hash_table *hash_table)
 void
 free_hash_table (struct hash_table *hash_table)
 {
-  xfree (hash_table->harray, hentry *);
-  xfree (hash_table, struct hash_table *);
+  xfree (hash_table->harray);
+  xfree (hash_table);
 }
 
 struct hash_table *
@@ -224,7 +222,7 @@ grow_hash_table (struct hash_table *hash_table, Elemcount new_size)
     rehash (old_harray, hash_table, old_size);
   }
 
-  xfree (old_harray, hentry *);
+  xfree (old_harray);
 }
 
 void
