@@ -3,10 +3,10 @@
 
 This file is part of XEmacs.
 
-XEmacs is free software; you can redistribute it and/or modify it
+XEmacs is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
-Free Software Foundation; either version 2, or (at your option) any
-later version.
+Free Software Foundation, either version 3 of the License, or (at your
+option) any later version.
 
 XEmacs is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
@@ -14,9 +14,7 @@ FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 for more details.
 
 You should have received a copy of the GNU General Public License
-along with XEmacs; see the file COPYING.  If not, write to
-the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-Boston, MA 02111-1307, USA.  */
+along with XEmacs.  If not, see <http://www.gnu.org/licenses/>. */
 
 /* Synched up with: Not in FSF. */
 
@@ -46,8 +44,8 @@ Charcount buffer_insert_raw_string_1 (struct buffer *buf, Charbpos pos,
 				      Bytecount length, int flags);
 Charcount buffer_insert_lisp_string_1 (struct buffer *buf, Charbpos pos,
 				       Lisp_Object str, int flags);
-Charcount buffer_insert_c_string_1 (struct buffer *buf, Charbpos pos,
-				    const char *s, int flags);
+Charcount buffer_insert_ascstring_1 (struct buffer *buf, Charbpos pos,
+				    const Ascbyte *s, int flags);
 Charcount buffer_insert_emacs_char_1 (struct buffer *buf, Charbpos pos,
 				      Ichar ch, int flags);
 Charcount buffer_insert_c_char_1 (struct buffer *buf, Charbpos pos, char c,
@@ -63,8 +61,8 @@ Charcount buffer_insert_from_buffer_1 (struct buffer *buf, Charbpos pos,
   buffer_insert_string_1 (buf, -1, nonreloc, reloc, offset, length, 0)
 #define buffer_insert_raw_string(buf, string, length) \
   buffer_insert_raw_string_1 (buf, -1, string, length, 0)
-#define buffer_insert_c_string(buf, s) \
-  buffer_insert_c_string_1 (buf, -1, s, 0)
+#define buffer_insert_ascstring(buf, s) \
+  buffer_insert_ascstring_1 (buf, -1, s, 0)
 #define buffer_insert_lisp_string(buf, str) \
   buffer_insert_lisp_string_1 (buf, -1, str, 0)
 #define buffer_insert_c_char(buf, c) \
