@@ -1,13 +1,13 @@
 /* Win32 internationalization functions.
-   Copyright (C) 2000, 2001, 2002, 2004 Ben Wing.
+   Copyright (C) 2000, 2001, 2002, 2004, 2010 Ben Wing.
    Copyright (C) 2000 IKEYAMA Tomonori.
 
 This file is part of XEmacs.
 
-XEmacs is free software; you can redistribute it and/or modify it
+XEmacs is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
-Free Software Foundation; either version 2, or (at your option) any
-later version.
+Free Software Foundation, either version 3 of the License, or (at your
+option) any later version.
 
 XEmacs is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
@@ -15,9 +15,7 @@ FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 for more details.
 
 You should have received a copy of the GNU General Public License
-along with XEmacs; see the file COPYING.  If not, write to
-the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-Boston, MA 02111-1307, USA.  */
+along with XEmacs.  If not, see <http://www.gnu.org/licenses/>. */
 
 /* Synched up with: Not in FSF. */
 
@@ -42,7 +40,7 @@ Boston, MA 02111-1307, USA.  */
 #include "window-impl.h"
 
 #include "console-msw-impl.h"
-#include "objects-msw-impl.h"
+#include "fontcolor-msw-impl.h"
 
 #ifndef CYGWIN_HEADERS
 # include <mbctype.h>
@@ -85,7 +83,7 @@ LCID current_locale;
 struct lang_to_string
 {
   int code;
-  char *string;
+  const Ascbyte *string;
 };
 
 struct lang_to_string lang_to_string_table[] =
@@ -650,7 +648,7 @@ langcode_to_lang (int code, struct lang_to_string *table,
 
   for (i = 0; i < table_size; i++)
     if (code == table[i].code)
-      return build_string (table[i].string);
+      return build_ascstring (table[i].string);
   return Qnil;
 }
 
@@ -668,7 +666,7 @@ sublangcode_to_lang (int lang, int sublang, struct lang_to_string *table,
 	  if (!table[i].string)
 	    break;
 	  if (sublang == table[i].code)
-	    return build_string (table[i].string);
+	    return build_ascstring (table[i].string);
 	}
       else if (!table[i].string && lang == table[i].code)
 	found_lang = 1;
@@ -677,11 +675,11 @@ sublangcode_to_lang (int lang, int sublang, struct lang_to_string *table,
   switch (sublang)
     {
     case SUBLANG_NEUTRAL:
-      return build_string ("NEUTRAL");
+      return build_ascstring ("NEUTRAL");
     case SUBLANG_DEFAULT:
-      return build_string ("DEFAULT");
+      return build_ascstring ("DEFAULT");
     case SUBLANG_SYS_DEFAULT:
-      return build_string ("SYS_DEFAULT");
+      return build_ascstring ("SYS_DEFAULT");
     }
 
   return Qnil;
@@ -1288,8 +1286,8 @@ If LCID (a 16-bit number) is not a valid locale, the result is nil.
 {
   int got_abbrev;
   int got_full;
-  char abbrev_name[32] = { 0 };
-  char full_name[256] = { 0 };
+  Extbyte abbrev_name[32] = { 0 };
+  Extbyte full_name[256] = { 0 };
 
   CHECK_INT (lcid);
 
@@ -1298,27 +1296,28 @@ If LCID (a 16-bit number) is not a valid locale, the result is nil.
 
   if (NILP (longform))
     {
-      got_abbrev = GetLocaleInfo (XINT (lcid),
-				  LOCALE_SABBREVLANGNAME | LOCALE_USE_CP_ACP,
-				  abbrev_name, sizeof (abbrev_name));
+      got_abbrev = qxeGetLocaleInfo (XINT (lcid),
+				     LOCALE_SABBREVLANGNAME |
+				     LOCALE_USE_CP_ACP,
+				     abbrev_name, sizeof (abbrev_name));
       if (got_abbrev)
-	return build_string (abbrev_name);
+	return build_tstr_string (abbrev_name);
     }
   else if (EQ (longform, Qt))
     {
-      got_full = GetLocaleInfo (XINT (lcid),
-				LOCALE_SLANGUAGE | LOCALE_USE_CP_ACP,
-				full_name, sizeof (full_name));
+      got_full = qxeGetLocaleInfo (XINT (lcid),
+				   LOCALE_SLANGUAGE | LOCALE_USE_CP_ACP,
+				   full_name, sizeof (full_name));
       if (got_full)
-	return build_string (full_name);
+	return build_tstr_string (full_name);
     }
   else if (NUMBERP (longform))
     {
-      got_full = GetLocaleInfo (XINT (lcid),
-				XINT (longform),
-				full_name, sizeof (full_name));
+      got_full = qxeGetLocaleInfo (XINT (lcid),
+				   XINT (longform),
+				   full_name, sizeof (full_name));
       if (got_full)
-	return make_unibyte_string (full_name, got_full);
+	return build_tstr_string (full_name);
     }
 
   return Qnil;
@@ -1600,10 +1599,10 @@ wcslen (const wchar_t *str)
 wchar_t *
 wcsncpy (wchar_t *dst0, const wchar_t *src0, size_t count)
 {
-  if (dst0 == NULL || src0 == NULL) return NULL;
   wchar_t *dscan;
   const wchar_t *sscan;
 
+  if (dst0 == NULL || src0 == NULL) return NULL;
   dscan = dst0;
   sscan = src0;
   while (count > 0)
@@ -1703,7 +1702,7 @@ lcid_to_locale_mule_or_no (LCID USED_IF_MULE (lcid))
 #ifdef MULE
   return lcid_to_locale (lcid);
 #else
-  return Fcons (build_string ("NEUTRAL"), build_string ("DEFAULT"));
+  return Fcons (build_ascstring ("NEUTRAL"), build_ascstring ("DEFAULT"));
 #endif
 }
 
@@ -1791,9 +1790,14 @@ mswindows_multibyte_to_unicode_putprop (Lisp_Object codesys,
 	data->cp_type = MULTIBYTE_MAC;
       else
 	{
-	  CHECK_NATNUM (value);
 	  data->locale_type = MULTIBYTE_SPECIFIED_CODE_PAGE;
-	  data->cp = XINT (value);
+#ifdef HAVE_BIGNUM
+          check_integer_range (value, Qzero, make_integer (INT_MAX));
+	  data->cp = BIGNUMP (value) ? bignum_to_int (XBIGNUM_DATA (value)) : XINT (value);
+#else
+          CHECK_NATNUM (value);
+          data->cp = XINT (value);
+#endif
 	}
     }
   else if (EQ (key, Qlocale))
@@ -1866,7 +1870,7 @@ mswindows_multibyte_to_unicode_print (Lisp_Object cs, Lisp_Object printcharfun,
   struct mswindows_multibyte_to_unicode_coding_system *data =
     XCODING_SYSTEM_TYPE_DATA (cs, mswindows_multibyte_to_unicode);
 
-  write_c_string (printcharfun, "(");
+  write_ascstring (printcharfun, "(");
   if (data->locale_type == MULTIBYTE_SPECIFIED_CODE_PAGE)
     print_internal (make_int (data->cp), printcharfun, 1);
   else
@@ -1874,7 +1878,7 @@ mswindows_multibyte_to_unicode_print (Lisp_Object cs, Lisp_Object printcharfun,
       write_fmt_string_lisp (printcharfun, "%s, ", 1, mswindows_multibyte_to_unicode_getprop (cs, Qlocale));
       print_internal (mswindows_multibyte_to_unicode_getprop (cs, Qcode_page), printcharfun, 0);
     }
-  write_c_string (printcharfun, ")");
+  write_ascstring (printcharfun, ")");
 }
 
 /* ----------------------------------------------------------------------- */
@@ -1940,7 +1944,7 @@ convert_multibyte_to_internal_malloc (const Extbyte *src, Bytecount n,
   TO_INTERNAL_FORMAT (DATA, (unidata, size), MALLOC, (intdata, size),
 		      Qmswindows_unicode);
 
-  xfree (unidata, Extbyte *);
+  xfree (unidata);
 
   if (size_out)
     *size_out = size;
@@ -2328,10 +2332,10 @@ vars_of_intl_win32 (void)
 {
 #ifdef MULE
   Vmswindows_charset_code_page_table =
-    make_lisp_hash_table (50, HASH_TABLE_NON_WEAK, HASH_TABLE_EQ);
+    make_lisp_hash_table (50, HASH_TABLE_NON_WEAK, Qeq);
   staticpro (&Vmswindows_charset_code_page_table);
   Vmswindows_charset_registry_table =
-    make_lisp_hash_table (50, HASH_TABLE_NON_WEAK, HASH_TABLE_EQ);
+    make_lisp_hash_table (50, HASH_TABLE_NON_WEAK, Qeq);
   staticpro (&Vmswindows_charset_registry_table);
 #endif /* MULE */
 }
@@ -2349,17 +2353,18 @@ determine_if_using_unicode (void)
 void
 complex_vars_of_intl_win32 (void)
 {
-  Fmake_coding_system
+  Fmake_coding_system_internal
     (Qmswindows_unicode, Qunicode,
-     build_msg_string ("MS Windows Unicode"),
-     nconc2 (list4 (Qdocumentation,
-		    build_msg_string (
+     build_defer_string ("MS Windows Unicode"),
+     listu (Qdocumentation,
+            build_defer_string (
 "Converts to the Unicode encoding for Windows API calls.\n"
 "This encoding is equivalent to standard UTF16, little-endian."
 ),
-		    Qmnemonic, build_string ("MSW-U")),
-	     list4 (Qunicode_type, Qutf_16,
-		    Qlittle_endian, Qt)));
+            Qmnemonic, build_ascstring ("MSW-U"),
+            Qunicode_type, Qutf_16,
+            Qlittle_endian, Qt,
+            Qunbound));
 
 #ifdef MULE
   /* Just temporarily.  This will get fixed in mule-msw-init.el. */
@@ -2393,6 +2398,10 @@ init_intl_win32 (void)
   set_current_lcid (GetUserDefaultLCID ());
 #endif /* MULE */
 
+#ifdef HAVE_CYGWIN_CONV_PATH
+  Fprovide (intern ("cygwin-use-utf-8"));
+#endif
+
   if (initialized)
     /* If not initialized, we also call this, but early -- see the
        previous function. */
