@@ -1,14 +1,14 @@
 /* Lisp parsing and input streams.
    Copyright (C) 1985-1989, 1992-1995 Free Software Foundation, Inc.
    Copyright (C) 1995 Tinker Systems.
-   Copyright (C) 1996, 2001, 2002, 2003 Ben Wing.
+   Copyright (C) 1996, 2001, 2002, 2003, 2010 Ben Wing.
 
 This file is part of XEmacs.
 
-XEmacs is free software; you can redistribute it and/or modify it
+XEmacs is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
-Free Software Foundation; either version 2, or (at your option) any
-later version.
+Free Software Foundation, either version 3 of the License, or (at your
+option) any later version.
 
 XEmacs is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
@@ -16,9 +16,7 @@ FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 for more details.
 
 You should have received a copy of the GNU General Public License
-along with XEmacs; see the file COPYING.  If not, write to
-the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-Boston, MA 02111-1307, USA.  */
+along with XEmacs.  If not, see <http://www.gnu.org/licenses/>. */
 
 /* Synched up with: Mule 2.0, FSF 19.30. */
 
@@ -148,10 +146,6 @@ static Lisp_Object Vload_force_doc_string_list;
 /* A resizing-buffer stream used to temporarily hold data while reading */
 static Lisp_Object Vread_buffer_stream;
 
-#ifdef COMPILED_FUNCTION_ANNOTATION_HACK
-Lisp_Object Vcurrent_compiled_function_annotation;
-#endif
-
 static int load_byte_code_version;
 
 /* An array describing all known built-in structure types */
@@ -360,23 +354,23 @@ check_if_suppressed (Ibyte *nonreloc, Lisp_Object reloc)
     len -= 3;
 
   {
-      EXTERNAL_LIST_LOOP_2 (acons, Vload_suppress_alist)
+    EXTERNAL_LIST_LOOP_2 (cons, Vload_suppress_alist)
       {
-	  if (CONSP (acons) && STRINGP (XCAR (acons)))
+	if (CONSP (cons) && STRINGP (XCAR (cons)))
 	  {
-	      Lisp_Object name = XCAR (acons);
-	      if (XSTRING_LENGTH (name) == len &&
-		  !memcmp (XSTRING_DATA (name), nonreloc, len))
+	    Lisp_Object name = XCAR (cons);
+	    if (XSTRING_LENGTH (name) == len &&
+		!memcmp (XSTRING_DATA (name), nonreloc, len))
 	      {
-		  struct gcpro gcpro1;
-		  Lisp_Object val;
+		struct gcpro gcpro1;
+		Lisp_Object val;
 
-		  GCPRO1 (reloc);
-		  val = Feval (XCDR (acons));
-		  UNGCPRO;
+		GCPRO1 (reloc);
+		val = IGNORE_MULTIPLE_VALUES (Feval (XCDR (cons)));
+		UNGCPRO;
 
-		  if (!NILP (val))
-		      return 1;
+		if (!NILP (val))
+		  return 1;
 	      }
 	  }
       }
@@ -593,7 +587,7 @@ encoding detection or end-of-line detection.
 
       fd = locate_file (Vload_path, file,
                         ((!NILP (nosuffix)) ? Qnil :
-			 build_string (load_ignore_elc_files ? ".el:" :
+			 build_ascstring (load_ignore_elc_files ? ".el:" :
 				       ".elc:.el:")),
                         &found,
                         -1);
@@ -848,9 +842,9 @@ decode_mode_1 (Lisp_Object mode)
     return W_OK;
   else if (EQ (mode, Qreadable))
     return R_OK;
-  else if (INTP (mode))
+  else if (INTEGERP (mode))
     {
-      check_int_range (XINT (mode), 0, 7);
+      check_integer_range (mode, Qzero, make_int (7));
       return XINT (mode);
     }
   else
@@ -919,8 +913,7 @@ for details.
 static Lisp_Object
 locate_file_refresh_hashing (Lisp_Object directory)
 {
-  Lisp_Object hash =
-    make_directory_hash_table (XSTRING_DATA (directory));
+  Lisp_Object hash = make_directory_hash_table (directory);
 
   if (!NILP (hash))
     Fputhash (directory, hash, Vlocate_file_hash_table);
@@ -1097,7 +1090,7 @@ locate_file_in_directory_mapper (Ibyte *fn, void *arg)
 	    {
 	      /* We succeeded; return this descriptor and filename.  */
 	      if (closure->storeptr)
-		*closure->storeptr = build_intstring (fn);
+		*closure->storeptr = build_istring (fn);
 
 	      return 1;
 	    }
@@ -1181,7 +1174,7 @@ static int
 locate_file_construct_suffixed_files_mapper (Ibyte *fn, void *arg)
 {
   Lisp_Object *tail = (Lisp_Object *) arg;
-  *tail = Fcons (build_intstring (fn), *tail);
+  *tail = Fcons (build_istring (fn), *tail);
   return 0;
 }
 
@@ -1271,7 +1264,7 @@ locate_file (Lisp_Object path, Lisp_Object str, Lisp_Object suffixes,
 
   /* Is it really necessary to gcpro path and str?  It shouldn't be
      unless some caller has fucked up.  There are known instances that
-     call us with build_string("foo:bar") as SUFFIXES, though. */
+     call us with build_ascstring("foo:bar") as SUFFIXES, though. */
   GCPRO4 (path, str, suffixes, suffixtab);
 
   /* if this filename has directory components, it's too complicated
@@ -1451,9 +1444,6 @@ readevalloop (Lisp_Object readcharfun,
 
   internal_bind_lisp_object (&Vcurrent_load_list, Qnil);
 
-#ifdef COMPILED_FUNCTION_ANNOTATION_HACK
-  Vcurrent_compiled_function_annotation = Qnil;
-#endif
   GCPRO2 (val, sourcename);
 
   LOADHIST_ATTACH (sourcename);
@@ -1619,9 +1609,6 @@ STREAM or the value of `standard-input' may be:
 
   Vread_objects = Qnil;
 
-#ifdef COMPILED_FUNCTION_ANNOTATION_HACK
-  Vcurrent_compiled_function_annotation = Qnil;
-#endif
   if (EQ (stream, Qread_char))
     {
       Lisp_Object val = call1 (Qread_from_minibuffer,
@@ -1648,9 +1635,6 @@ START and END optionally delimit a substring of STRING from which to read;
   Lisp_Object lispstream = Qnil;
   struct gcpro gcpro1;
 
-#ifdef COMPILED_FUNCTION_ANNOTATION_HACK
-  Vcurrent_compiled_function_annotation = Qnil;
-#endif
   GCPRO1 (lispstream);
   CHECK_STRING (string);
   get_string_range_byte (string, start, end, &startval, &endval,
@@ -1832,8 +1816,12 @@ read_escape (Lisp_Object readcharfun)
 	      }
 	  }
 	if (i >= 0400)
-	  syntax_error ("Non-ISO-8859-1 character specified with octal escape",
-			make_int (i));
+	  {
+	    read_syntax_error ((Ascbyte *) emacs_sprintf_malloc
+			       (NULL,
+				"Non-ISO-8859-1 octal character escape, "
+				"?\\%.3o", i));
+	  }
 	return i;
       }
 
@@ -1841,13 +1829,23 @@ read_escape (Lisp_Object readcharfun)
       /* A hex escape, as in ANSI C, except that we only allow latin-1
 	 characters to be read this way.  What is "\x4e03" supposed to
 	 mean, anyways, if the internal representation is hidden?
-         This is also consistent with the treatment of octal escapes. */
+         This is also consistent with the treatment of octal escapes.
+
+         Note that we don't accept ?\XAB as specifying the character with
+         numeric value 171; it must be ?\xAB. */
       {
+#define OVERLONG_INFO "Overlong hex character escape, ?\\x"
+
 	REGISTER Ichar i = 0;
 	REGISTER int count = 0;
+	Ascbyte seen[] = OVERLONG_INFO "\0\0\0\0\0";
+	REGISTER Ascbyte *seenp = seen + sizeof (OVERLONG_INFO) - 1;
+
+#undef OVERLONG_INFO
+
 	while (++count <= 2)
 	  {
-	    c = readchar (readcharfun);
+	    c = readchar (readcharfun), *seenp = c, ++seenp;
 	    /* Remember, can't use isdigit(), isalpha() etc. on Ichars */
 	    if      (c >= '0' && c <= '9')  i = (i << 4) + (c - '0');
 	    else if (c >= 'a' && c <= 'f')  i = (i << 4) + (c - 'a') + 10;
@@ -1861,21 +1859,12 @@ read_escape (Lisp_Object readcharfun)
 
         if (count == 3)
           {
-            c = readchar (readcharfun);
+            c = readchar (readcharfun), *seenp = c, ++seenp;
             if ((c >= '0' && c <= '9') ||
                 (c >= 'a' && c <= 'f') ||
                 (c >= 'A' && c <= 'F'))
               {
-                Lisp_Object args[2];
-
-                if      (c >= '0' && c <= '9')  i = (i << 4) + (c - '0');
-                else if (c >= 'a' && c <= 'f')  i = (i << 4) + (c - 'a') + 10;
-                else if (c >= 'A' && c <= 'F')  i = (i << 4) + (c - 'A') + 10;
-
-                args[0] = build_string ("?\\x%x");
-                args[1] = make_int (i);
-                syntax_error ("Overlong hex character escape",
-                              Fformat (2, args));
+		read_syntax_error (seen);
               }
             unreadchar (readcharfun, c);
           }
@@ -1995,8 +1984,14 @@ read_atom (Lisp_Object readcharfun,
 	  if (*read_ptr == '+')
 	    read_ptr++;
 	  ratio_set_string (scratch_ratio, read_ptr, 0);
-	  ratio_canonicalize (scratch_ratio);
-	  return Fcanonicalize_number (make_ratio_rt (scratch_ratio));
+	  if (bignum_sign (ratio_denominator (scratch_ratio)) != 0) {
+	    ratio_canonicalize (scratch_ratio);
+	    return Fcanonicalize_number (make_ratio_rt (scratch_ratio));
+	  }
+	  return Fsignal (Qinvalid_read_syntax,
+			  list2 (build_msg_string
+				 ("Invalid ratio constant in reader"),
+				 make_string ((Ibyte *) read_ptr, len)));
 	}
 #endif
       if (isfloat_string (read_ptr))
@@ -2129,7 +2124,7 @@ read_bit_vector (Lisp_Object readcharfun)
       Dynarr_add (dyn, bit);
     }
 
-  val = make_bit_vector_from_byte_vector (Dynarr_atp (dyn, 0),
+  val = make_bit_vector_from_byte_vector (Dynarr_begin (dyn),
 					  Dynarr_length (dyn));
   Dynarr_free (dyn);
 
@@ -2154,8 +2149,7 @@ define_structure_type (Lisp_Object type,
   st.instantiate = instantiate;
   Dynarr_add (the_structure_type_dynarr, st);
 
-  return Dynarr_atp (the_structure_type_dynarr,
-		     Dynarr_length (the_structure_type_dynarr) - 1);
+  return Dynarr_lastp (the_structure_type_dynarr);
 }
 
 void
@@ -2395,34 +2389,36 @@ read_raw_string (Lisp_Object readcharfun)
   Ichar c;
   Ichar permit_unicode = 0; 
 
-  do {
-    c = reader_nextchar(readcharfun);
-    switch (c) {
-      /* #r:engine"my sexy raw string" -- raw string w/ flags*/
-      /* case ':': */
-      /* #ru"Hi there\u20AC \U000020AC" -- raw string, honouring Unicode. */
-    case 'u':
-    case 'U':
-      permit_unicode = c; 
-      continue;
+  do
+    {
+      c = reader_nextchar (readcharfun);
+      switch (c)
+	{
+	  /* #r:engine"my sexy raw string" -- raw string w/ flags*/
+	  /* case ':': */
+	  /* #ru"Hi there\u20AC \U000020AC" -- raw string, honouring Unicode. */
+	case 'u':
+	case 'U':
+	  permit_unicode = c; 
+	  continue;
 
-      /* #r"my raw string" -- raw string */
-    case '\"':
-      return read_string(readcharfun, '\"', 1, permit_unicode);
-      /* invalid syntax */
-    default:
-      {
-	if (permit_unicode)
+	  /* #r"my raw string" -- raw string */
+	case '\"':
+	  return read_string (readcharfun, '\"', 1, permit_unicode);
+	  /* invalid syntax */
+	default:
 	  {
-	    unreadchar(readcharfun, permit_unicode);
+	    if (permit_unicode)
+	      {
+		unreadchar (readcharfun, permit_unicode);
+	      }
+	    unreadchar (readcharfun, c);
+	    return Fsignal (Qinvalid_read_syntax,
+			    list1 (build_msg_string
+				   ("unrecognized raw string syntax")));
 	  }
-	unreadchar(readcharfun, c);
-	return Fsignal(Qinvalid_read_syntax,
-		       list1(build_string
-			     ("unrecognized raw string syntax")));
-      }
-    }
-  } while (1);
+	}
+    } while (1);
 }
 
 /* Read the next Lisp object from the stream READCHARFUN and return it.
@@ -2581,7 +2577,7 @@ retry:
 		  if (CONSP (tmp) && UNBOUNDP (XCAR (tmp)))
 		    free_cons (tmp);
 		  return Fsignal (Qinvalid_read_syntax,
-				   list1 (build_string ("#")));
+				   list1 (build_ascstring ("#")));
 		}
 	      GCPRO1 (tmp);
 	      /* Read the intervals and their properties.  */
@@ -2659,11 +2655,11 @@ retry:
             /* bit vectors */
 	  case '*': return read_bit_vector (readcharfun);
             /* #o10 => 8 -- octal constant syntax */
-	  case 'o': return read_integer (readcharfun, 8);
+	  case 'o': case 'O': return read_integer (readcharfun, 8);
             /* #xdead => 57005 -- hex constant syntax */
-	  case 'x': return read_integer (readcharfun, 16);
+	  case 'x': case 'X': return read_integer (readcharfun, 16);
             /* #b010 => 2 -- binary constant syntax */
-	  case 'b': return read_integer (readcharfun, 2);
+	  case 'b': case 'B': return read_integer (readcharfun, 2);
 	    /* #r"raw\stringt" -- raw string syntax */
 	  case 'r': return read_raw_string(readcharfun);
             /* #s(foobar key1 val1 key2 val2) -- structure syntax */
@@ -2739,13 +2735,13 @@ retry:
 					   make_int (n)));
 		}
 	      return Fsignal (Qinvalid_read_syntax,
-			      list1 (build_string ("#")));
+			      list1 (build_ascstring ("#")));
 	    }
 	  default:
 	    {
 	      unreadchar (readcharfun, c);
 	      return Fsignal (Qinvalid_read_syntax,
-			      list1 (build_string ("#")));
+			      list1 (build_ascstring ("#")));
 	    }
 	  }
       }
@@ -2883,7 +2879,6 @@ isfloat_string (const char *cp)
 	      || state == (DOT_CHAR|TRAIL_INT|E_CHAR|EXP_INT)));
 }
 
-#ifdef HAVE_RATIO
 int
 isratio_string (const char *cp)
 {
@@ -2914,7 +2909,7 @@ isratio_string (const char *cp)
   return *cp == '\0' || *cp == ' ' || *cp =='\t' || *cp == '\n' ||
     *cp == '\r' || *cp == '\f';
 }
-#endif
+
 
 static void *
 sequence_reader (Lisp_Object readcharfun,
@@ -3010,16 +3005,6 @@ read_list_conser (Lisp_Object readcharfun, void *state, Charcount UNUSED (len))
 	}
     }
 
-#ifdef COMPILED_FUNCTION_ANNOTATION_HACK
-  if (s->length == 1 && s->allow_dotted_lists && EQ (XCAR (s->head), Qfset))
-    {
-      if (CONSP (elt) && EQ (XCAR (elt), Qquote) && CONSP (XCDR (elt)))
-	Vcurrent_compiled_function_annotation = XCAR (XCDR (elt));
-      else
-	Vcurrent_compiled_function_annotation = elt;
-    }
-#endif
-
   elt = Fcons (elt, Qnil);
   if (!NILP (s->tail))
     XCDR (s->tail) = elt;
@@ -3055,10 +3040,6 @@ read_list (Lisp_Object readcharfun,
 {
   struct read_list_state s;
   struct gcpro gcpro1, gcpro2;
-#ifdef COMPILED_FUNCTION_ANNOTATION_HACK
-  Lisp_Object old_compiled_function_annotation =
-    Vcurrent_compiled_function_annotation;
-#endif
 
   s.head = Qnil;
   s.tail = Qnil;
@@ -3068,9 +3049,6 @@ read_list (Lisp_Object readcharfun,
   GCPRO2 (s.head, s.tail);
 
   sequence_reader (readcharfun, terminator, &s, read_list_conser);
-#ifdef COMPILED_FUNCTION_ANNOTATION_HACK
-  Vcurrent_compiled_function_annotation = old_compiled_function_annotation;
-#endif
 
   if ((purify_flag || load_force_doc_strings) && check_for_doc_references)
     {
@@ -3121,7 +3099,7 @@ read_list (Lisp_Object readcharfun,
 		       because that would make the directory absolute
 		       now.  */
 		    XCAR (XCAR (holding_cons)) =
-		      concat2 (build_string ("../lisp/"),
+		      concat2 (build_ascstring ("../lisp/"),
 			       Ffile_name_nondirectory
 			       (Vload_file_name_internal));
 		}
@@ -3215,7 +3193,7 @@ read_compiled_function (Lisp_Object readcharfun, Ichar terminator)
 		make_byte_code_args[iii] = Qzero;
 	      else
 		XCAR (make_byte_code_args[iii]) =
-		  concat2 (build_string ("../lisp/"),
+		  concat2 (build_ascstring ("../lisp/"),
 			   Ffile_name_nondirectory
 			   (Vload_file_name_internal));
 	    }
@@ -3256,7 +3234,7 @@ init_lread (void)
   /* kludge: locate-file does not work for a null load-path, even if
      the file name is absolute. */
 
-  Vload_path = Fcons (build_string (""), Qnil);
+  Vload_path = Fcons (build_ascstring (""), Qnil);
 
   /* This used to get initialized in init_lread because all streams
      got closed when dumping occurs.  This is no longer true --
@@ -3406,7 +3384,7 @@ This is normally used when compiling packages of elisp files that may have
 complex dependencies.  Ignoring all elc files with `load-ignore-elc-files'
 would also be safe, but much slower.
 */ );
-  load_ignore_out_of_date_elc_files = 0;
+  load_ignore_out_of_date_elc_files = 1;
 
   DEFVAR_BOOL ("load-always-display-messages",
 	       &load_always_display_messages /*
@@ -3478,20 +3456,12 @@ character escape syntaxes or just read them incorrectly.
   Vload_file_name_internal = Qnil;
   staticpro (&Vload_file_name_internal);
 
-#ifdef COMPILED_FUNCTION_ANNOTATION_HACK
-  Vcurrent_compiled_function_annotation = Qnil;
-  staticpro (&Vcurrent_compiled_function_annotation);
-#endif
-
   /* So that early-early stuff will work */
   Ffset (Qload,	Qload_internal);
 
 #ifdef FEATUREP_SYNTAX
   DEFSYMBOL (Qfeaturep);
   Fprovide (intern ("xemacs"));
-#ifdef INFODOCK
-  Fprovide (intern ("infodock"));
-#endif /* INFODOCK */
 #endif /* FEATUREP_SYNTAX */
 
 #ifdef LISP_BACKQUOTES
@@ -3500,6 +3470,7 @@ character escape syntaxes or just read them incorrectly.
 
 #ifdef I18N3
   Vfile_domain = Qnil;
+  staticpro (&Vfile_domain);
 #endif
 
   Vread_objects = Qnil;
@@ -3507,7 +3478,12 @@ character escape syntaxes or just read them incorrectly.
 
   Vlocate_file_hash_table = make_lisp_hash_table (200,
 						  HASH_TABLE_NON_WEAK,
-						  HASH_TABLE_EQUAL);
+#ifdef DEFAULT_FILE_SYSTEM_IGNORE_CASE
+						  Qequalp
+#else
+						  Qequal
+#endif
+						  );
   staticpro (&Vlocate_file_hash_table);
 #ifdef DEBUG_XEMACS
   symbol_value (XSYMBOL (intern ("Vlocate-file-hash-table")))
