@@ -1,22 +1,20 @@
 /* Machine description file for intel 386.
    Copyright (C) 1987 Free Software Foundation, Inc.
 
-This file is part of GNU Emacs.
+This file is part of XEmacs.
 
-GNU Emacs is free software; you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 2, or (at your option)
-any later version.
+XEmacs is free software: you can redistribute it and/or modify it
+under the terms of the GNU General Public License as published by the
+Free Software Foundation, either version 3 of the License, or (at your
+option) any later version.
 
-GNU Emacs is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
+XEmacs is distributed in the hope that it will be useful, but WITHOUT
+ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
+for more details.
 
 You should have received a copy of the GNU General Public License
-along with XEmacs; see the file COPYING.  If not, write to
-the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-Boston, MA 02111-1307, USA.  */
+along with XEmacs.  If not, see <http://www.gnu.org/licenses/>. */
 
 /* Synched up with: FSF 19.31. */
 
@@ -27,27 +25,16 @@ Boston, MA 02111-1307, USA.  */
 NOTE-START
 Intel 386 (-machine=intel386 or -machine=is386.h)
 
-  The possibilities for -opsystem are: bsd4-2, usg5-2-2, usg5-3,
-  isc2-2, 386-ix, esix, linux, sco3.2v4, and xenix.
+  The possibilities for -opsystem are: bsd4-2, usg5-4, usg5-4-2, and linux.
 
   18.58 should support a wide variety of operating systems.
-  Use isc2-2 for Interactive 386/ix version 2.2.
-  Use 386ix for prior versions.
-  Use esix for Esix.
   Use linux for Linux.
   It isn't clear what to do on an SCO system.
 
-  -machine=is386 is used for an Integrated Solutions 386 machine.
-  It may also be correct for Microport systems.
-
 Cubix QBx/386 (-machine=intel386 -opsystem=usg5-3)
 
   Changes merged in 19.1.  Systems before 2/A/0 may fail to compile etags.c
   due to a compiler bug.
-
-Prime EXL (-machine=intel386 -opsystem=usg5-3)
-
-  Minor changes merged in 19.1.
 NOTE-END */
 
 /* Now define a symbol for the cpu type, if your compiler
@@ -58,8 +45,7 @@ NOTE-END */
 #define INTEL386
 
 /* crt0.c, if it is used, should use the i386-bsd style of entry.
-   with no extra dummy args.  On USG and XENIX,
-   NO_REMAP says this isn't used. */
+   with no extra dummy args.  On USG, NO_REMAP says this isn't used. */
 
 /* Mly 16-Jan-96 16:38:32: this is part of a prototype -- same bug present in 
    other m*.h files */
@@ -69,17 +55,6 @@ NOTE-END */
 
 #define DOT_GLOBAL_START
 
-#ifdef XENIX
-/* Data type of load average, as read out of kmem.  */
-#define LOAD_AVE_TYPE short
-
-/* Convert that into an integer that is 100 for a load average of 1.0  */
-#define LOAD_AVE_CVT(x) (((double) (x)) * 100.0 / FSCALE)
-
-#define FSCALE 256.0         /* determined by experimentation...  */
-#endif
-
-
 #ifdef SOLARIS2
 /* Data type of load average, as read out of kmem.  */
 #define LOAD_AVE_TYPE long
@@ -106,34 +81,10 @@ NOTE-END */
 #endif
 #endif /* not SOLARIS2 */
 
-#ifdef XENIX
-
-/* Define NO_REMAP if memory segmentation makes it not work well
-   to change the boundary between the text section and data section
-   when Emacs is dumped.  If you define this, the preloaded Lisp
-   code will not be sharable; but that's better than failing completely.  */
-
-#define NO_REMAP
-
-/* Since cannot purify, use standard Xenix 386 startup code. */
-
-#define START_FILES "/lib/386/Sseg.o pre-crt0.o /lib/386/Scrt0.o"
-
-/* These really use terminfo.  */
-
-#define LIBS_TERMCAP "/lib/386/Slibcurses.a /lib/386/Slibtinfo.a /lib/386/Slibx.a"
-
-/* Standard libraries for this machine.  Since `-l' doesn't work in `ld'.  */
-/* '__fltused' is unresolved w/o Slibcfp.a */
-#define LIB_STANDARD "/lib/386/Slibcfp.a /lib/386/Slibc.a"
-#else /* not XENIX */
-
 #ifdef USG
-
 #define NO_REMAP 
 #define TEXT_START 0
 #endif /* USG */
-#endif /* not XENIX */
 
 #ifdef linux
 /* libc-linux/sysdeps/linux/i386/ulimit.c says that due to shared library, */
