@@ -1,22 +1,20 @@
 /* m- file for Mips machines.
    Copyright (C) 1987, 1992 Free Software Foundation, Inc.
 
-This file is part of GNU Emacs.
+This file is part of XEmacs.
 
-GNU Emacs is free software; you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 2, or (at your option)
-any later version.
+XEmacs is free software: you can redistribute it and/or modify it
+under the terms of the GNU General Public License as published by the
+Free Software Foundation, either version 3 of the License, or (at your
+option) any later version.
 
-GNU Emacs is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
+XEmacs is distributed in the hope that it will be useful, but WITHOUT
+ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
+for more details.
 
 You should have received a copy of the GNU General Public License
-along with XEmacs; see the file COPYING.  If not, write to
-the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-Boston, MA 02111-1307, USA.  */
+along with XEmacs.  If not, see <http://www.gnu.org/licenses/>. */
 
 /* Synched up with: FSF 19.31. */
 
@@ -77,7 +75,7 @@ NOTE-END  */
 
 /* Alter some of the options used when linking.  */
 
-#if !defined(NEWSOS5) && !defined(linux)
+#if !defined(linux)
 #ifdef BSD
 
 /* DECstations don't have this library. */
@@ -100,9 +98,9 @@ NOTE-END  */
 #define C_DEBUG_SWITCH "-O -g3"
 
 #endif /* not BSD */
-#endif /* !NEWSOS5 && !linux */
+#endif /* !linux */
 
-#if !defined(NEWSOS5) && !defined(linux)
+#if !defined(linux)
 #ifdef USG
 
 /* Don't try to use SIGIO even though it is defined.  */
@@ -127,4 +125,4 @@ NOTE-END  */
 #define TERMINFO
 #undef MAIL_USE_FLOCK  /* Someone should check this.  */
 #endif /* BSD */
-#endif /* !NEWSOS5 && !linux */
+#endif /* !linux */
