@@ -1,12 +1,13 @@
 /* Keyboard macros.
    Copyright (C) 1985, 1986, 1992, 1993, 1994 Free Software Foundation, Inc.
+   Copyright (C) 2010 Ben Wing.
 
 This file is part of XEmacs.
 
-XEmacs is free software; you can redistribute it and/or modify it
+XEmacs is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
-Free Software Foundation; either version 2, or (at your option) any
-later version.
+Free Software Foundation, either version 3 of the License, or (at your
+option) any later version.
 
 XEmacs is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
@@ -14,9 +15,7 @@ FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 for more details.
 
 You should have received a copy of the GNU General Public License
-along with XEmacs; see the file COPYING.  If not, write to
-the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-Boston, MA 02111-1307, USA.  */
+along with XEmacs.  If not, see <http://www.gnu.org/licenses/>. */
 
 /* Synched up with: FSF 19.30. */
 
@@ -181,7 +180,7 @@ store_kbd_macro_event (Lisp_Object event)
 void
 pop_kbd_macro_event (Lisp_Object event)
 {
-  if (NILP (Vexecuting_macro)) ABORT ();
+  assert (!NILP (Vexecuting_macro));
 
   if (STRINGP (Vexecuting_macro) || VECTORP (Vexecuting_macro))
     {
@@ -197,7 +196,7 @@ pop_kbd_macro_event (Lisp_Object event)
 					  with Qt to force an early exit. */
     signal_error (Qinvalid_state, "junk in executing-macro", Qunbound);
 
-  Fthrow (Qexecute_kbd_macro, Qt);
+  throw_or_bomb_out (Qexecute_kbd_macro, Qt, 0, Qnil, Qnil);
 }
 
 
