@@ -4,20 +4,18 @@
 
 This file is part of XEmacs.
 
-XEmacs is free software; you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 2, or (at your option)
-any later version.
+XEmacs is free software: you can redistribute it and/or modify it
+under the terms of the GNU General Public License as published by the
+Free Software Foundation, either version 3 of the License, or (at your
+option) any later version.
 
-XEmacs is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
+XEmacs is distributed in the hope that it will be useful, but WITHOUT
+ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
+for more details.
 
 You should have received a copy of the GNU General Public License
-along with XEmacs; see the file COPYING.  If not, write to
-the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-Boston, MA 02111-1307, USA.  */
+along with XEmacs.  If not, see <http://www.gnu.org/licenses/>. */
 
 #ifndef INCLUDED_mule_ccl_h_
 #define INCLUDED_mule_ccl_h_
@@ -69,12 +67,24 @@ struct ccl_program {
 					   line-feed.  */
 #define CCL_CODING_EOL_CR	2	/* Carriage-return only.  */
 
-/* Alist of fontname patterns vs corresponding CCL program.  */
-extern Lisp_Object Vfont_ccl_encoder_alist;
+/* If OBJECT is symbol designating a registered CCL program, return it.
+   Else if OBJECT is a vector CCL program with no unresolved symbols, return
+   it.
+   Else, if OBJECT is a vector CCL program with unresolved symbols, return a
+   newly-created vector reflecting the CCL program with all symbols
+   resolved, if that is currently possible in this XEmacs.
 
-/* Setup fields of the structure pointed by CCL appropriately for the
-   execution of ccl program CCL_PROG (symbol or vector).  */
-extern int setup_ccl_program (struct ccl_program *, Lisp_Object);
+   Otherwise, signal `invalid-argument'. */
+extern Lisp_Object get_ccl_program (Lisp_Object object);
+
+/* Set up fields of the structure pointed by CCL appropriately for the
+   execution of ccl program CCL_PROG (a symbol or a vector).
+
+   If CCL_PROG is a vector and contains unresolved symbols, this function
+   will throw an assertion failure. To avoid this, call get_ccl_program at
+   the point that you receive the CCL program from Lisp, and use and store
+   its (resolved) result instead. */
+extern int setup_ccl_program (struct ccl_program *, Lisp_Object ccl_prog);
 
 extern int ccl_driver (struct ccl_program *, const unsigned char *,
 		       unsigned_char_dynarr *, int, int *, int);
