@@ -6,10 +6,10 @@
 
 This file is part of XEmacs.
 
-XEmacs is free software; you can redistribute it and/or modify it
+XEmacs is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
-Free Software Foundation; either version 2, or (at your option) any
-later version.
+Free Software Foundation, either version 3 of the License, or (at your
+option) any later version.
 
 XEmacs is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
@@ -17,9 +17,7 @@ FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 for more details.
 
 You should have received a copy of the GNU General Public License
-along with XEmacs; see the file COPYING.  If not, write to
-the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-Boston, MA 02111-1307, USA.  */
+along with XEmacs.  If not, see <http://www.gnu.org/licenses/>. */
 
 /* Written by Kirill M. Katsnelson <kkm@kis.ru>, April 1998 */
 
@@ -656,9 +654,8 @@ nt_alloc_process_data (Lisp_Process *p)
 }
 
 static void
-nt_finalize_process_data (Lisp_Process *p, int for_disksave)
+nt_finalize_process_data (Lisp_Process *p)
 {
-  assert (!for_disksave);
   /* If it's still in the list of processes we are waiting on delete
      it.  This can happen if we forcibly delete a process and are unable
      to kill it. */
@@ -693,11 +690,10 @@ nt_init_process (void)
  */
 
 static DOESNT_RETURN
-mswindows_report_winsock_error (const char *string, Lisp_Object data,
+mswindows_report_winsock_error (const Ascbyte *reason, Lisp_Object data,
 				int errnum)
 {
-  report_file_type_error (Qnetwork_error, mswindows_lisp_error (errnum),
-			  string, data);
+  signal_error_2 (Qnetwork_error, reason, mswindows_lisp_error (errnum), data);
 }
 
 static void
@@ -861,7 +857,7 @@ nt_create_process (Lisp_Process *p,
 	 args_or_ret);
 
     /* #### What about path names, which may be links? */
-    LISP_STRING_TO_TSTR (args_or_ret, command_line);
+    command_line = LISP_STRING_TO_TSTR (args_or_ret);
 
     UNGCPRO; /* args_or_ret */
   }
@@ -1160,7 +1156,7 @@ nt_deactivate_process (Lisp_Process *p,
      of handles when lots of processes are run. (The handle gets closed
      anyway upon GC, but that might be a ways away, esp. if
      deleted-exited-processes is set to nil.) */
-  nt_finalize_process_data (p, 0);
+  nt_finalize_process_data (p);
 }
 
 /*
@@ -1237,7 +1233,7 @@ get_internet_address (Lisp_Object host, struct sockaddr_in *address)
     Extbyte *hostext;
     unsigned long inaddr;
 
-    LISP_STRING_TO_EXTERNAL (host, hostext, Qmswindows_host_name_encoding);
+    hostext = LISP_STRING_TO_EXTERNAL (host, Qmswindows_host_name_encoding);
     inaddr = inet_addr (hostext);
     if (inaddr != INADDR_NONE)
       {
@@ -1255,7 +1251,7 @@ get_internet_address (Lisp_Object host, struct sockaddr_in *address)
   {
     Extbyte *hostext;
 
-    LISP_STRING_TO_EXTERNAL (host, hostext, Qmswindows_host_name_encoding);
+    hostext = LISP_STRING_TO_EXTERNAL (host, Qmswindows_host_name_encoding);
 
     hasync = WSAAsyncGetHostByName (hwnd, XM_SOCKREPLY, hostext,
 				    buf, sizeof (buf));
@@ -1313,7 +1309,8 @@ nt_canonicalize_host_name (Lisp_Object host)
     return host;
 
   if (address.sin_family == AF_INET)
-    return build_string (inet_ntoa (address.sin_addr));
+    return build_extstring (inet_ntoa (address.sin_addr),
+			     Qunix_host_name_encoding);
   else
     return host;
 }
@@ -1348,8 +1345,8 @@ nt_open_network_stream (Lisp_Object name, Lisp_Object host,
       Extbyte *servext;
 
       CHECK_STRING (service);
-      LISP_STRING_TO_EXTERNAL (service, servext,
-			       Qmswindows_service_name_encoding);
+      servext = LISP_STRING_TO_EXTERNAL (service,
+					 Qmswindows_service_name_encoding);
 
       svc_info = getservbyname (servext, "tcp");
       if (svc_info == 0)
