@@ -6,10 +6,10 @@
 
 This file is part of XEmacs.
 
-XEmacs is free software; you can redistribute it and/or modify it
+XEmacs is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
-Free Software Foundation; either version 2, or (at your option) any
-later version.
+Free Software Foundation, either version 3 of the License, or (at your
+option) any later version.
 
 XEmacs is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
@@ -17,9 +17,7 @@ FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 for more details.
 
 You should have received a copy of the GNU General Public License
-along with XEmacs; see the file COPYING.  If not, write to
-the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-Boston, MA 02111-1307, USA.  */
+along with XEmacs.  If not, see <http://www.gnu.org/licenses/>. */
 
 /* Mule-ized as of 6-14-00 */
 
@@ -32,10 +30,6 @@ Boston, MA 02111-1307, USA.  */
 
 #include <config.h>
 
-#if !defined (NO_SUBPROCESSES)
-
-/* The entire file is within this conditional */
-
 #include "lisp.h"
 
 #include "buffer.h"
@@ -130,7 +124,7 @@ static int
 close_process_descs_mapfun (const void *UNUSED (key), void *contents,
 			    void *UNUSED (arg))
 {
-  Lisp_Object proc = VOID_TO_LISP (contents);
+  Lisp_Object proc = GET_LISP_FROM_VOID (contents);
   USID vaffan, culo;
 
   event_stream_delete_io_streams (XPROCESS (proc)->pipe_instream,
@@ -226,8 +220,8 @@ allocate_pty (void)
      grovelling code in allocate_pty_the_old_fashioned_way(). */
   int master_fd = -1;
   const Extbyte *slave_name = NULL;
-  const CIbyte *clone = NULL;
-  static const CIbyte * const clones[] =
+  const Ascbyte *clone = NULL;
+  static const Ascbyte * const clones[] =
     /* Different pty master clone devices */
     {
       "/dev/ptmx",      /* Various systems */
@@ -311,7 +305,7 @@ allocate_pty (void)
   {
     Ibyte *slaveint;
 
-    EXTERNAL_TO_C_STRING (slave_name, slaveint, Qfile_name);
+    slaveint = EXTERNAL_TO_ITEXT (slave_name, Qfile_name);
     qxestrncpy (pty_name, slaveint, sizeof (pty_name));
   }
 
@@ -338,9 +332,7 @@ allocate_pty (void)
     struct group *tty_group = getgrnam ("tty");
     if (tty_group != NULL)
       {
-	Extbyte *ptyout;
-
-	C_STRING_TO_EXTERNAL (pty_name, ptyout, Qfile_name);
+	Extbyte *ptyout = ITEXT_TO_EXTERNAL (pty_name, Qfile_name);
 	chown (ptyout, (uid_t) -1, tty_group->gr_gid);
       }
   }
@@ -429,22 +421,12 @@ create_bidirectional_pipe (EMACS_INT *inchannel, EMACS_INT *outchannel,
 {
   int sv[2];
 
-#ifdef SKTPAIR
-  if (socketpair (AF_UNIX, SOCK_STREAM, 0, sv) < 0)
-    return -1;
-  *outchannel = *inchannel = sv[0];
-  *forkout = *forkin = sv[1];
-#else /* not SKTPAIR */
-  int temp;
-  temp = pipe (sv);
-  if (temp < 0) return -1;
+  if (pipe (sv) < 0) return -1;
   *inchannel = sv[0];
   *forkout = sv[1];
-  temp = pipe (sv);
-  if (temp < 0) return -1;
+  if (pipe (sv) < 0) return -1;
   *outchannel = sv[1];
   *forkin = sv[0];
-#endif /* not SKTPAIR */
   return 0;
 }
 
@@ -472,7 +454,7 @@ get_internet_address (Lisp_Object host, struct sockaddr_in *address,
       h_errno = 0;
 #endif
 
-      LISP_STRING_TO_EXTERNAL (host, hostext, Qunix_host_name_encoding);
+      hostext = LISP_STRING_TO_EXTERNAL (host, Qunix_host_name_encoding);
 
       /* Some systems can't handle SIGIO/SIGALARM in gethostbyname. */
       slow_down_interrupts ();
@@ -496,7 +478,7 @@ get_internet_address (Lisp_Object host, struct sockaddr_in *address,
       Extbyte *hostext;
 
       /* Attempt to interpret host as numeric inet address */
-      LISP_STRING_TO_EXTERNAL (host, hostext, Qunix_host_name_encoding);
+      hostext = LISP_STRING_TO_EXTERNAL (host, Qunix_host_name_encoding);
       numeric_addr = inet_addr (hostext);
       if (NUMERIC_ADDR_ERROR)
 	{
@@ -538,10 +520,9 @@ set_socket_nonblocking_maybe (int fd,
 	  Extbyte *tailportext;
 
 	  CHECK_STRING (tail_port);
-	  TO_EXTERNAL_FORMAT (LISP_STRING, tail_port, C_STRING_ALLOCA,
-			      tailportext, Qnative);
-
-	  svc_info = getservbyname (tailportext, proto);
+	  svc_info = getservbyname (LISP_STRING_TO_EXTERNAL
+				    (tail_port, Qunix_service_name_encoding),
+				    proto);
 	  if ((svc_info != 0) && (svc_info->s_port == port))
 	    break;
 	  else
@@ -902,10 +883,8 @@ child_setup (int in, int out, int err, Ibyte **new_argv,
     nice (- emacs_priority);
 #endif
 
-#if !defined (NO_SUBPROCESSES)
   /* Close Emacs's descriptors that this process should not have.  */
   close_process_descs ();
-#endif /* not NO_SUBPROCESSES */
   close_load_descs ();
 
   /* [[Note that use of alloca is always safe here.  It's obvious for systems
@@ -1172,10 +1151,6 @@ unix_create_process (Lisp_Process *p,
 	       process, and I don't feel like messing with
 	       this now. */
 
-	    /* There was some weirdo, probably wrong,
-	       conditionalization on RTU and UNIPLUS here.
-	       I deleted it.  So sue me. */
-
 	    /* SunOS has TIOCSCTTY but the close/open method
 	       also works. */
 
@@ -1320,7 +1295,7 @@ unix_create_process (Lisp_Process *p,
   if (separate_err)
     retry_close (forkerr);
 
-  p->tty_name = pty_flag ? build_intstring (pty_name) : Qnil;
+  p->tty_name = pty_flag ? build_istring (pty_name) : Qnil;
 
   /* Notice that SIGCHLD was not blocked. (This is not possible on
      some systems.) No biggie if SIGCHLD occurs right around the
@@ -1836,16 +1811,14 @@ unix_canonicalize_host_name (Lisp_Object host)
 #endif
   hints.ai_socktype = SOCK_STREAM;
   hints.ai_protocol = 0;
-  LISP_STRING_TO_EXTERNAL (host, ext_host, Qnative);
+  ext_host = LISP_STRING_TO_EXTERNAL (host, Qunix_host_name_encoding);
   retval = getaddrinfo (ext_host, NULL, &hints, &res);
   if (retval != 0)
     {
-      CIbyte *gai_err;
-
-      EXTERNAL_TO_C_STRING (gai_strerror (retval), gai_err,
-			    Qstrerror_encoding);
-      maybe_signal_error (Qio_error, gai_err, host,
-			  Qprocess, ERROR_ME_DEBUG_WARN);
+      maybe_signal_error_2 (Qio_error, "Canonicalizing host name",
+			    build_extstring (gai_strerror (retval),
+					     Qstrerror_encoding),
+			    host, Qprocess, ERROR_ME_DEBUG_WARN);
       canonname = host;
     }
   else
@@ -1853,7 +1826,8 @@ unix_canonicalize_host_name (Lisp_Object host)
       int gni = getnameinfo (res->ai_addr, res->ai_addrlen,
 			     addrbuf, sizeof(addrbuf),
 			     NULL, 0, NI_NUMERICHOST);
-      canonname = gni ? host : build_ext_string (addrbuf, Qnative);
+      canonname = gni ? host : build_extstring (addrbuf,
+						 Qunix_host_name_encoding);
 
       freeaddrinfo (res);
     }
@@ -1866,7 +1840,8 @@ unix_canonicalize_host_name (Lisp_Object host)
     return host;
 
   if (address.sin_family == AF_INET)
-    return build_string (inet_ntoa (address.sin_addr));
+    return build_extstring (inet_ntoa (address.sin_addr),
+			     Qunix_host_name_encoding);
   else
     /* #### any clue what to do here? */
     return host;
@@ -1919,8 +1894,8 @@ unix_open_network_stream (Lisp_Object name, Lisp_Object host,
     else
       {
 	CHECK_STRING (service);
-	LISP_STRING_TO_EXTERNAL (service, portstring,
-				 Qunix_service_name_encoding);
+	portstring = LISP_STRING_TO_EXTERNAL (service,
+					      Qunix_service_name_encoding);
 	port = 0;
       }
 
@@ -1932,15 +1907,14 @@ unix_open_network_stream (Lisp_Object name, Lisp_Object host,
     else /* EQ (protocol, Qudp) */
       hints.ai_socktype = SOCK_DGRAM;
     hints.ai_protocol = 0;
-    LISP_STRING_TO_EXTERNAL (host, ext_host, Qunix_host_name_encoding);
+    ext_host = LISP_STRING_TO_EXTERNAL (host, Qunix_host_name_encoding);
     retval = getaddrinfo (ext_host, portstring, &hints, &res);
     if (retval != 0)
       {
-	CIbyte *gai_err;
-
-	EXTERNAL_TO_C_STRING (gai_strerror (retval), gai_err,
-			      Qstrerror_encoding);
-	signal_error (Qio_error, gai_err, list2 (host, service));
+	signal_error_2 (Qio_error, "Converting host name to IP address",
+			build_extstring (gai_strerror (retval),
+					 Qstrerror_encoding),
+			list2 (host, service));
       }
 
     /* address loop */
@@ -1959,8 +1933,8 @@ unix_open_network_stream (Lisp_Object name, Lisp_Object host,
 	Extbyte *servext;
 
 	CHECK_STRING (service);
-	LISP_STRING_TO_EXTERNAL (service, servext,
-				 Qunix_service_name_encoding);
+	servext = LISP_STRING_TO_EXTERNAL (service,
+					   Qunix_service_name_encoding);
 
 	if (EQ (protocol, Qtcp))
 	  svc_info = getservbyname (servext, "tcp");
@@ -1998,18 +1972,6 @@ unix_open_network_stream (Lisp_Object name, Lisp_Object host,
 	    continue;
 	  }
 
-#ifdef CONNECT_NEEDS_SLOWED_INTERRUPTS
-	/* Slow down polling.  Some kernels have a bug which causes retrying
-	   connect to fail after a connect. (Note that the entire purpose
-	   for this code is a very old comment concerning an Ultrix bug that
-	   requires this code.  We used to do this ALWAYS despite this!
-	   This messes up C-g out of connect() in a big way.  So instead we
-	   just assume that anyone who sees such a kernel bug will define
-	   this constant, which for now is only defined under Ultrix.) --ben
-	*/
-	slow_down_interrupts ();
-#endif
-
       loop:
 
 	/* A system call interrupted with a SIGALRM or SIGIO comes back
@@ -2017,14 +1979,8 @@ unix_open_network_stream (Lisp_Object name, Lisp_Object host,
 	SETJMP (break_system_call_jump);
 	if (QUITP)
 	  {
-#ifdef CONNECT_NEEDS_SLOWED_INTERRUPTS
-	    speed_up_interrupts ();
-#endif
 	    QUIT;
 	    /* In case something really weird happens ... */
-#ifdef CONNECT_NEEDS_SLOWED_INTERRUPTS
-	    slow_down_interrupts ();
-#endif
 	  }
 
 	/* Break out of connect with a signal (it isn't otherwise possible).
@@ -2066,11 +2022,6 @@ unix_open_network_stream (Lisp_Object name, Lisp_Object host,
 	    failed_connect = 1;
 	    retry_close (s);
             s = -1;
-
-#ifdef CONNECT_NEEDS_SLOWED_INTERRUPTS
-	    speed_up_interrupts ();
-#endif
-
 	    continue;
 	  }
 
@@ -2097,10 +2048,6 @@ unix_open_network_stream (Lisp_Object name, Lisp_Object host,
 #endif /* USE_GETADDRINFO */
       } /* address loop */
 
-#ifdef CONNECT_NEEDS_SLOWED_INTERRUPTS
-    speed_up_interrupts ();
-#endif
-
 #ifdef USE_GETADDRINFO
     freeaddrinfo (res);
 #endif
@@ -2171,10 +2118,10 @@ unix_open_multicast_group (Lisp_Object name, Lisp_Object dest,
 
   CHECK_STRING (dest);
 
-  CHECK_NATNUM (port);
+  check_integer_range (port, Qzero, make_integer (USHRT_MAX));
   theport = htons ((unsigned short) XINT (port));
 
-  CHECK_NATNUM (ttl);
+  check_integer_range (ttl, Qzero, make_integer (UCHAR_MAX));
   thettl = (unsigned char) XINT (ttl);
 
   if ((udp = getprotobyname ("udp")) == NULL)
@@ -2239,10 +2186,6 @@ unix_open_multicast_group (Lisp_Object name, Lisp_Object dest,
      instead of 'sendto'. Consequently, we 'connect' this socket. */
 
   /* See open-network-stream-internal for comments on this part of the code */
-#ifdef CONNECT_NEEDS_SLOWED_INTERRUPTS
-  slow_down_interrupts ();
-#endif
-
  loop:
 
   /* A system call interrupted with a SIGALRM or SIGIO comes back
@@ -2250,14 +2193,8 @@ unix_open_multicast_group (Lisp_Object name, Lisp_Object dest,
   SETJMP (break_system_call_jump);
   if (QUITP)
     {
-#ifdef CONNECT_NEEDS_SLOWED_INTERRUPTS
-      speed_up_interrupts ();
-#endif
       QUIT;
       /* In case something really weird happens ... */
-#ifdef CONNECT_NEEDS_SLOWED_INTERRUPTS
-      slow_down_interrupts ();
-#endif
     }
 
   /* Break out of connect with a signal (it isn't otherwise possible).
@@ -2288,19 +2225,12 @@ unix_open_multicast_group (Lisp_Object name, Lisp_Object dest,
 
       retry_close (rs);
       retry_close (ws);
-#ifdef CONNECT_NEEDS_SLOWED_INTERRUPTS
-      speed_up_interrupts ();
-#endif
 
       errno = xerrno;
       report_network_error ("error connecting socket", list3 (Qunbound, name,
 							      port));
     }
 
-#ifdef CONNECT_NEEDS_SLOWED_INTERRUPTS
-  speed_up_interrupts ();
-#endif
-
   /* scope */
   if (setsockopt (ws, IPPROTO_IP, IP_MULTICAST_TTL,
 		  &thettl, sizeof (thettl)) < 0)
@@ -2361,5 +2291,3 @@ vars_of_process_unix (void)
 {
   Fprovide (intern ("unix-processes"));
 }
-
-#endif /* !defined (NO_SUBPROCESSES) */
