@@ -1,13 +1,13 @@
 /* Why the hell is XEmacs so fucking slow?
-   Copyright (C) 1996, 2002, 2003, 2004 Ben Wing.
+   Copyright (C) 1996, 2002, 2003, 2004, 2010 Ben Wing.
    Copyright (C) 1998 Free Software Foundation, Inc.
 
 This file is part of XEmacs.
 
-XEmacs is free software; you can redistribute it and/or modify it
+XEmacs is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
-Free Software Foundation; either version 2, or (at your option) any
-later version.
+Free Software Foundation, either version 3 of the License, or (at your
+option) any later version.
 
 XEmacs is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
@@ -15,9 +15,7 @@ FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 for more details.
 
 You should have received a copy of the GNU General Public License
-along with XEmacs; see the file COPYING.  If not, write to
-the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-Boston, MA 02111-1307, USA.  */
+along with XEmacs.  If not, see <http://www.gnu.org/licenses/>. */
 
 #include <config.h>
 #include "lisp.h"
@@ -25,6 +23,7 @@ Boston, MA 02111-1307, USA.  */
 #include "backtrace.h"
 #include "bytecode.h"
 #include "elhash.h"
+#include "gc.h"
 #include "hash.h"
 #include "profile.h"
 
@@ -125,7 +124,7 @@ create_timing_profile_table (void)
 {
   /* The hash code can safely be called from a signal handler except when
      it has to grow the hash table.  In this case, it calls realloc(),
-     which is not (in general) re-entrant.  The way we deal with this is
+     which is not (in general) reentrant.  The way we deal with this is
      documented at the top of this file. */
   if (!big_profile_table)
     big_profile_table = make_hash_table (2000);
@@ -137,16 +136,16 @@ create_profile_tables (void)
   create_timing_profile_table ();
   if (NILP (Vtotal_timing_profile_table))
     Vtotal_timing_profile_table =
-      make_lisp_hash_table (1000, HASH_TABLE_NON_WEAK, HASH_TABLE_EQ);
+      make_lisp_hash_table (1000, HASH_TABLE_NON_WEAK, Qeq);
   if (NILP (Vcall_count_profile_table))
     Vcall_count_profile_table =
-      make_lisp_hash_table (1000, HASH_TABLE_NON_WEAK, HASH_TABLE_EQ);
+      make_lisp_hash_table (1000, HASH_TABLE_NON_WEAK, Qeq);
   if (NILP (Vgc_usage_profile_table))
     Vgc_usage_profile_table =
-      make_lisp_hash_table (1000, HASH_TABLE_NON_WEAK, HASH_TABLE_EQ);
+      make_lisp_hash_table (1000, HASH_TABLE_NON_WEAK, Qeq);
   if (NILP (Vtotal_gc_usage_profile_table))
     Vtotal_gc_usage_profile_table =
-      make_lisp_hash_table (1000, HASH_TABLE_NON_WEAK, HASH_TABLE_EQ);
+      make_lisp_hash_table (1000, HASH_TABLE_NON_WEAK, Qeq);
 }
 
 static Lisp_Object
@@ -315,13 +314,13 @@ sigprof_handler (int UNUSED (signo))
 	long count;
 	const void *vval;
 
-	if (gethash (LISP_TO_VOID (fun), big_profile_table, &vval))
+	if (gethash (STORE_LISP_IN_VOID (fun), big_profile_table, &vval))
 	  count = (long) vval;
 	else
 	  count = 0;
 	count++;
 	vval = (const void *) count;
-	puthash (LISP_TO_VOID (fun), (void *) vval, big_profile_table);
+	puthash (STORE_LISP_IN_VOID (fun), (void *) vval, big_profile_table);
       }
 
       profiling_lock = 0;
@@ -364,8 +363,16 @@ will be properly accumulated. (To clear, use `clear-profiling-info'.)
     msecs = default_profiling_interval;
   else
     {
-      CHECK_NATNUM (microsecs);
+#ifdef HAVE_BIGNUM
+      check_integer_range (microsecs, make_int (1000), make_integer (INT_MAX));
+      msecs =
+        BIGNUMP (microsecs) ? bignum_to_int (XBIGNUM_DATA (microsecs)) :
+                                             XINT (microsecs);
+#else
+      check_integer_range (microsecs, make_int (1000),
+                           make_integer (EMACS_INT_MAX));
       msecs = XINT (microsecs);
+#endif
     }
   if (msecs <= 0)
     msecs = 1000;
@@ -463,7 +470,7 @@ get_profiling_info_timing_maphash (const void *void_key,
     = (struct get_profiling_info_closure *) void_closure;
   EMACS_INT val;
 
-  key = VOID_TO_LISP (void_key);
+  key = GET_LISP_FROM_VOID (void_key);
   val = (EMACS_INT) void_val;
 
   Fputhash (key, make_int (val), closure->timing);
@@ -475,7 +482,7 @@ copy_hash_table_or_blank (Lisp_Object table)
 {
   return !NILP (table) ? Fcopy_hash_table (table) :
     make_lisp_hash_table (100, HASH_TABLE_NON_WEAK,
-			  HASH_TABLE_EQ);
+			  Qeq);
 }
 
 DEFUN ("get-profiling-info", Fget_profiling_info, 0, 0, 0, /*
@@ -514,7 +521,7 @@ are recorded
   const void *overhead;
 
   closure.timing =
-    make_lisp_hash_table (100, HASH_TABLE_NON_WEAK, HASH_TABLE_EQUAL);
+    make_lisp_hash_table (100, HASH_TABLE_NON_WEAK, Qequal);
 
   if (big_profile_table)
     {
@@ -524,7 +531,7 @@ are recorded
       /* OK, OK ...  the total-timing table is not going to have an entry
 	 for profile overhead, and it looks strange for it to come out 0,
 	 so make sure it looks reasonable. */
-      if (!gethash (LISP_TO_VOID (QSprofile_overhead), big_profile_table,
+      if (!gethash (STORE_LISP_IN_VOID (QSprofile_overhead), big_profile_table,
 		    &overhead))
 	overhead = 0;
       Fputhash (QSprofile_overhead, make_int ((EMACS_INT) overhead),
@@ -533,15 +540,16 @@ are recorded
       unbind_to (count);
     }
 
-  retv = nconc2 (list6 (Qtiming, closure.timing, Qtotal_timing,
-			copy_hash_table_or_blank (Vtotal_timing_profile_table),
-			Qcall_count,
-			copy_hash_table_or_blank (Vcall_count_profile_table)),
-		 list4 (Qgc_usage,
-			copy_hash_table_or_blank (Vgc_usage_profile_table),
-			Qtotal_gc_usage,
-			copy_hash_table_or_blank (Vtotal_gc_usage_profile_table
-						  )));
+  retv = listu (Qtiming, closure.timing,
+                Qtotal_timing,
+                copy_hash_table_or_blank (Vtotal_timing_profile_table),
+                Qcall_count,
+                copy_hash_table_or_blank (Vcall_count_profile_table),
+                Qgc_usage,
+                copy_hash_table_or_blank (Vgc_usage_profile_table),
+                Qtotal_gc_usage,
+                copy_hash_table_or_blank (Vtotal_gc_usage_profile_table),
+                Qunbound);
   unbind_to (depth);
   return retv;
 }
@@ -557,7 +565,7 @@ set_profiling_info_timing_maphash (Lisp_Object key,
       ("Function timing count is not an integer in given entry",
        key, val);
 
-  puthash (LISP_TO_VOID (key), (void *) XINT (val), big_profile_table);
+  puthash (STORE_LISP_IN_VOID (key), (void *) XINT (val), big_profile_table);
 
   return 0;
 }
@@ -609,9 +617,9 @@ mark_profiling_info_maphash (const void *void_key,
 			     void *UNUSED (void_closure))
 {
 #ifdef USE_KKCC
-  kkcc_gc_stack_push_lisp_object (VOID_TO_LISP (void_key), 0, -1);
+  kkcc_gc_stack_push_lisp_object_0 (GET_LISP_FROM_VOID (void_key));
 #else /* NOT USE_KKCC */
-  mark_object (VOID_TO_LISP (void_key));
+  mark_object (GET_LISP_FROM_VOID (void_key));
 #endif /* NOT USE_KKCC */
   return 0;
 }
@@ -683,28 +691,28 @@ value can be.
   profiling_lock = 0;
 
 #ifdef DEBUG_XEMACS
-  QSin_temp_spot_1 = build_msg_string ("(in temp spot 1)");
+  QSin_temp_spot_1 = build_defer_string ("(in temp spot 1)");
   staticpro (&QSin_temp_spot_1);
 
-  QSin_temp_spot_2 = build_msg_string ("(in temp spot 2)");
+  QSin_temp_spot_2 = build_defer_string ("(in temp spot 2)");
   staticpro (&QSin_temp_spot_2);
 
-  QSin_temp_spot_3 = build_msg_string ("(in temp spot 3)");
+  QSin_temp_spot_3 = build_defer_string ("(in temp spot 3)");
   staticpro (&QSin_temp_spot_3);
 
-  QSin_temp_spot_4 = build_msg_string ("(in temp spot 4)");
+  QSin_temp_spot_4 = build_defer_string ("(in temp spot 4)");
   staticpro (&QSin_temp_spot_4);
 
-  QSin_temp_spot_5 = build_msg_string ("(in temp spot 5)");
+  QSin_temp_spot_5 = build_defer_string ("(in temp spot 5)");
   staticpro (&QSin_temp_spot_5);
 #endif /* DEBUG_XEMACS */
 
-  QSunknown = build_msg_string ("(unknown)");
+  QSunknown = build_defer_string ("(unknown)");
   staticpro (&QSunknown);
   QSprocessing_events_at_top_level =
-    build_msg_string ("(processing events at top level)");
+    build_defer_string ("(processing events at top level)");
   staticpro (&QSprocessing_events_at_top_level);
-  QSprofile_overhead = build_msg_string ("(profile overhead)");
+  QSprofile_overhead = build_defer_string ("(profile overhead)");
   staticpro (&QSprofile_overhead);
 
   DEFSYMBOL (Qtiming);
