@@ -6,10 +6,10 @@
 
 This file is part of XEmacs.
 
-XEmacs is free software; you can redistribute it and/or modify it
+XEmacs is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
-Free Software Foundation; either version 2, or (at your option) any
-later version.
+Free Software Foundation, either version 3 of the License, or (at your
+option) any later version.
 
 XEmacs is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
@@ -17,9 +17,7 @@ FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 for more details.
 
 You should have received a copy of the GNU General Public License
-along with XEmacs; see the file COPYING.  If not, write to
-the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-Boston, MA 02111-1307, USA.  */
+along with XEmacs.  If not, see <http://www.gnu.org/licenses/>. */
 
 /* Synched up with: Not completely synched with FSF.  Mostly divergent
    from FSF. */
@@ -44,7 +42,7 @@ Boston, MA 02111-1307, USA.  */
 #include "window.h"
 
 #include "console-tty-impl.h"
-#include "objects-tty-impl.h"
+#include "fontcolor-tty-impl.h"
 
 #include "syssignal.h"
 
@@ -106,10 +104,10 @@ static void term_get_fkeys (Lisp_Object keymap, char **address);
  column, so we use ichar_string_displayed_columns().
  ****************************************************************************/
 static int
-tty_text_width (struct frame *f, struct face_cachel *UNUSED (cachel),
+tty_text_width (struct window *w, struct face_cachel *UNUSED (cachel),
 		const Ichar *str, Charcount len)
 {
-  struct console *c = XCONSOLE(FRAME_CONSOLE (f));
+  struct console *c = WINDOW_XCONSOLE (w);
 
   if (CONSOLE_TTY_MULTIPLE_WIDTH (c))
     {
@@ -428,7 +426,8 @@ tty_clear_region (Lisp_Object window, struct device* UNUSED (d),
 		  struct frame * f, face_index findex, int x, int y,
 		  int width, int height, Lisp_Object UNUSED (fcolor),
 		  Lisp_Object UNUSED (bcolor),
-		  Lisp_Object UNUSED (background_pixmap))
+		  Lisp_Object UNUSED (background_pixmap),
+		  Lisp_Object UNUSED (background_placement))
 {
   struct console *c = XCONSOLE (FRAME_CONSOLE (f));
   int line;
@@ -575,12 +574,12 @@ tty_output_ichar_dynarr (struct window *w, struct display_line *dl,
   else
     Dynarr_reset (tty_output_ichar_dynarr_dynarr);
 
-  convert_ichar_string_into_ibyte_dynarr (Dynarr_atp (buf, 0),
+  convert_ichar_string_into_ibyte_dynarr (Dynarr_begin (buf),
 					    Dynarr_length (buf),
 					    tty_output_ichar_dynarr_dynarr);
 
   tty_output_ibyte_string (w, dl,
-			     Dynarr_atp (tty_output_ichar_dynarr_dynarr, 0),
+			     Dynarr_begin (tty_output_ichar_dynarr_dynarr),
 			     Dynarr_length (tty_output_ichar_dynarr_dynarr),
 			     xpos, findex, cursor);
 }
@@ -641,7 +640,7 @@ set_foreground_to (struct console *c, Lisp_Object sym)
     {
       substitute_in_dynamic_color_string (Vtty_dynamic_color_fg,
 					  Fsymbol_name (sym));
-      escseq = Dynarr_atp (sidcs_dynarr, 0);
+      escseq = Dynarr_begin (sidcs_dynarr);
       escseqlen = Dynarr_length (sidcs_dynarr);
     }
 #endif
@@ -671,7 +670,7 @@ set_background_to (struct console *c, Lisp_Object sym)
     {
       substitute_in_dynamic_color_string (Vtty_dynamic_color_bg,
 					  Fsymbol_name (sym));
-      escseq = Dynarr_atp (sidcs_dynarr, 0);
+      escseq = Dynarr_begin (sidcs_dynarr);
       escseqlen = Dynarr_length (sidcs_dynarr);
     }
 #endif
@@ -1115,6 +1114,13 @@ init_tty_for_redisplay (struct device *d, char *terminal_type)
   if (CONSOLE_TTY_DATA (c)->width <= 0 || CONSOLE_TTY_DATA (c)->height <= 0)
     return TTY_SIZE_UNSPECIFIED;
 
+  CONSOLE_TTY_DATA (c)->colors = tgetnum("Co");
+  if (CONSOLE_TTY_DATA (c)->colors == 0)
+    CONSOLE_TTY_DATA (c)->colors = tgetnum("colors");
+  if (CONSOLE_TTY_DATA (c)->colors == 0)
+    /* There is always foreground and background. */
+    CONSOLE_TTY_DATA (c)->colors = 2;
+
   /*
    * Initialize cursor motion information.
    */
@@ -1412,7 +1418,7 @@ term_get_fkeys_1 (Lisp_Object function_key_map)
       char *sequence = tgetstr (keys[i].cap, address);
       if (sequence)
 	Fdefine_key (function_key_map,
-		     build_ext_string (sequence, Qbinary),
+		     build_extstring (sequence, Qbinary),
 		     vector1 (intern (keys[i].name)));
     }
 
@@ -1426,11 +1432,11 @@ term_get_fkeys_1 (Lisp_Object function_key_map)
     const char *k0      = tgetstr ("k0", address);
 
     if (k_semi)
-      Fdefine_key (function_key_map, build_ext_string (k_semi, Qbinary),
+      Fdefine_key (function_key_map, build_extstring (k_semi, Qbinary),
 		   vector1 (intern ("f10")));
 
     if (k0)
-      Fdefine_key (function_key_map, build_ext_string (k0, Qbinary),
+      Fdefine_key (function_key_map, build_extstring (k0, Qbinary),
 		   vector1 (intern (k_semi ? "f0" : "f10")));
   }
 
@@ -1454,7 +1460,7 @@ term_get_fkeys_1 (Lisp_Object function_key_map)
 	    {
 	      sprintf (fkey, "f%d", i);
 	      Fdefine_key (function_key_map,
-			   build_ext_string (sequence, Qbinary),
+			   build_extstring (sequence, Qbinary),
 			   vector1 (intern (fkey)));
 	    }
 	}
@@ -1470,7 +1476,7 @@ term_get_fkeys_1 (Lisp_Object function_key_map)
 	char *sequence = tgetstr (cap2, address);		\
 	if (sequence)						\
 	  Fdefine_key (function_key_map,			\
-		       build_ext_string (sequence, Qbinary),	\
+		       build_extstring (sequence, Qbinary),	\
 		       vector1 (intern (keyname)));		\
       }								\
   } while (0)
