@@ -5,10 +5,10 @@
 
 This file is part of XEmacs.
 
-XEmacs is free software; you can redistribute it and/or modify it
+XEmacs is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
-Free Software Foundation; either version 2, or (at your option) any
-later version.
+Free Software Foundation, either version 3 of the License, or (at your
+option) any later version.
 
 XEmacs is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
@@ -16,9 +16,7 @@ FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 for more details.
 
 You should have received a copy of the GNU General Public License
-along with XEmacs; see the file COPYING.  If not, write to
-the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-Boston, MA 02111-1307, USA.  */
+along with XEmacs.  If not, see <http://www.gnu.org/licenses/>. */
 
 /* Synched up with: Not in FSF. */
 
@@ -719,14 +717,9 @@ extern Lisp_Object Qtop_bottom;
 /*************************************************************************/
 EXFUN (Fredraw_frame, 2);
 
-int redisplay_text_width_string (struct window *w, int findex,
+int redisplay_text_width_string (Lisp_Object domain, Lisp_Object face,
 				 Ibyte *nonreloc, Lisp_Object reloc,
 				 Bytecount offset, Bytecount len);
-int redisplay_frame_text_width_string (struct frame *f,
-				       Lisp_Object face,
-				       Ibyte *nonreloc,
-				       Lisp_Object reloc,
-				       Bytecount offset, Bytecount len);
 int redisplay_frame (struct frame *f, int preemption_check);
 void redisplay_no_pre_idle_hook (void);
 void redisplay (void);
@@ -782,9 +775,9 @@ void redisplay_glyph_changed (Lisp_Object glyph, Lisp_Object property,
 
 #ifdef MEMORY_USAGE_STATS
 int compute_display_line_dynarr_usage (display_line_dynarr *dyn,
-				       struct overhead_stats *ovstats);
+				       struct usage_stats *ustats);
 int compute_line_start_cache_dynarr_usage (line_start_cache_dynarr *dyn,
-					   struct overhead_stats *ovstats);
+					   struct usage_stats *ustats);
 #endif
 
 
@@ -853,4 +846,6 @@ void exit_redisplay_critical_section_maybe (int depth);
 int enter_redisplay_critical_section_if (Boolint from_outside);
 void exit_redisplay_critical_section_if (Boolint from_outside, int depth);
 
+void redisplay_cancel_ritual_suicide(void);
+
 #endif /* INCLUDED_redisplay_h_ */
