@@ -1,3 +1,21 @@
+/* Definitions file for XEmacs running on Mach (BSD 4.3)
+   Copyright (C) 1985, 1986, 1993 Free Software Foundation, Inc.
+
+This file is part of XEmacs.
+
+XEmacs is free software: you can redistribute it and/or modify it
+under the terms of the GNU General Public License as published by the
+Free Software Foundation, either version 3 of the License, or (at your
+option) any later version.
+
+XEmacs is distributed in the hope that it will be useful, but WITHOUT
+ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
+for more details.
+
+You should have received a copy of the GNU General Public License
+along with XEmacs.  If not, see <http://www.gnu.org/licenses/>. */
+
 /* Synched up with: FSF 19.31. */
 
 /* I don't care if this doesn't do more than including bsd4-3.h;
@@ -14,10 +32,6 @@
 #undef SYSTEM_TYPE
 #define SYSTEM_TYPE "mach"
 
-/* Define this macro if system defines a type `union wait'.  */
-
-#define HAVE_UNION_WAIT
-
 /* Don't send signals to subprocesses by "typing" special chars at them. */
 #undef SIGNALS_VIA_CHARACTERS
 
