@@ -4,10 +4,10 @@
 
 This file is part of XEmacs.
 
-XEmacs is free software; you can redistribute it and/or modify it
+XEmacs is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
-Free Software Foundation; either version 2, or (at your option) any
-later version.
+Free Software Foundation, either version 3 of the License, or (at your
+option) any later version.
 
 XEmacs is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
@@ -15,9 +15,7 @@ FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 for more details.
 
 You should have received a copy of the GNU General Public License
-along with XEmacs; see the file COPYING.  If not, write to
-the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-Boston, MA 02111-1307, USA.  */
+along with XEmacs.  If not, see <http://www.gnu.org/licenses/>. */
 
 /* Synched up with: Not synched with FSF. */
 
@@ -109,7 +107,7 @@ symbol_to_ms_cf (Lisp_Object value)
   if (STRINGP (value))
     {
       Extbyte *valext;
-      LISP_STRING_TO_TSTR (value, valext);
+      valext = LISP_STRING_TO_TSTR (value);
       return qxeRegisterClipboardFormat (valext);
     }
 
@@ -324,7 +322,7 @@ mswindows_own_selection (Lisp_Object selection_name,
     {
       CloseClipboard ();
 
-      xfree (src, void *);
+      xfree (src);
       return Qnil;
     }
 
@@ -335,12 +333,12 @@ mswindows_own_selection (Lisp_Object selection_name,
       GlobalFree (hValue);
       CloseClipboard ();
 
-      xfree (src, void *);
+      xfree (src);
       return Qnil;
     }
 
   memcpy (dst, src, size);
-  xfree (src, void *);
+  xfree (src);
 
   GlobalUnlock (hValue);
 
@@ -438,7 +436,7 @@ mswindows_register_selection_data_type (Lisp_Object type_name)
   Extbyte *nameext;
   UINT format;
 
-  LISP_STRING_TO_TSTR (type_name, nameext);
+  nameext = LISP_STRING_TO_TSTR (type_name);
   format = qxeRegisterClipboardFormat (nameext);
 
   if (format)
@@ -561,7 +559,7 @@ mswindows_get_foreign_selection (Lisp_Object selection_symbol,
     }
 
   /* Place it in a Lisp string */
-  ret = make_ext_string ((Extbyte *) data, size, Qbinary);
+  ret = make_extstring ((Extbyte *) data, size, Qbinary);
 
   GlobalUnlock (data);
   CloseClipboard ();
