@@ -5,10 +5,10 @@
 
 This file is part of XEmacs.
 
-XEmacs is free software; you can redistribute it and/or modify it
+XEmacs is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
-Free Software Foundation; either version 2, or (at your option) any
-later version.
+Free Software Foundation, either version 3 of the License, or (at your
+option) any later version.
 
 XEmacs is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
@@ -16,9 +16,7 @@ FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 for more details.
 
 You should have received a copy of the GNU General Public License
-along with XEmacs; see the file COPYING.  If not, write to
-the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-Boston, MA 02111-1307, USA.  */
+along with XEmacs.  If not, see <http://www.gnu.org/licenses/>. */
 
 /* Synched up with: Not in FSF. */
 
@@ -98,9 +96,9 @@ Lisp_Object Q_volume, Q_pitch, Q_duration, Q_sound;
 Lisp_Object Qsound_error;
 
 DOESNT_RETURN
-report_sound_error (const Ascbyte *string, Lisp_Object data)
+report_sound_error (const Ascbyte *reason, Lisp_Object data)
 {
-  report_error_with_errno (Qsound_error, string, data);
+  report_error_with_errno (Qsound_error, reason, data);
 }
 
 DEFUN ("play-sound-file", Fplay_sound_file, 1, 3, "fSound file name: ", /*
@@ -158,7 +156,7 @@ Windows the sound file must be in WAV format.
     {
       Extbyte *fileext;
 
-      LISP_STRING_TO_EXTERNAL (file, fileext, Qfile_name);
+      LISP_PATHNAME_CONVERT_OUT (file, fileext);
       /* #### ALSA code should allow specification of a device. */
       if (alsa_play_sound_file (fileext, vol))
 	return Qnil;
@@ -170,7 +168,7 @@ Windows the sound file must be in WAV format.
     {
       Extbyte *fileext;
 
-      LISP_STRING_TO_EXTERNAL (file, fileext, Qfile_name);
+      LISP_PATHNAME_CONVERT_OUT (file, fileext);
       /* #### NAS code should allow specification of a device. */
       if (nas_play_sound_file (fileext, vol))
 	return Qnil;
@@ -183,7 +181,7 @@ Windows the sound file must be in WAV format.
       Extbyte *fileext;
       int result;
 
-      LISP_STRING_TO_EXTERNAL (file, fileext, Qfile_name);
+      LISP_PATHNAME_CONVERT_OUT (file, fileext);
 
       /* #### ESD uses alarm(). But why should we also stop SIGIO? */
       stop_interrupts ();
@@ -382,9 +380,7 @@ If the sound cannot be played in any other way, the standard "bell" will sound.
       Binbyte *soundext;
       Bytecount soundextlen;
 
-      TO_EXTERNAL_FORMAT (LISP_STRING, sound,
-			  ALLOCA, (soundext, soundextlen),
-			  Qbinary);
+      LISP_STRING_TO_SIZED_EXTERNAL (sound, soundext, soundextlen, Qbinary);
       if (alsa_play_sound_data (soundext, soundextlen, vol))
 	return Qnil;
     }
@@ -396,9 +392,7 @@ If the sound cannot be played in any other way, the standard "bell" will sound.
       Binbyte *soundext;
       Bytecount soundextlen;
 
-      TO_EXTERNAL_FORMAT (LISP_STRING, sound,
-			  ALLOCA, (soundext, soundextlen),
-			  Qbinary);
+      LISP_STRING_TO_SIZED_EXTERNAL (sound, soundext, soundextlen, Qbinary);
       if (nas_play_sound_data (soundext, soundextlen, vol))
 	return Qnil;
     }
@@ -411,8 +405,7 @@ If the sound cannot be played in any other way, the standard "bell" will sound.
       Bytecount soundextlen;
       int succes;
 
-      TO_EXTERNAL_FORMAT (LISP_STRING, sound, ALLOCA, (soundext, soundextlen),
-			  Qbinary);
+      LISP_STRING_TO_SIZED_EXTERNAL (sound, soundext, soundextlen, Qbinary);
       
       /* #### ESD uses alarm(). But why should we also stop SIGIO? */
       stop_interrupts ();
@@ -432,9 +425,7 @@ If the sound cannot be played in any other way, the standard "bell" will sound.
       Bytecount soundextlen;
       int succes;
 
-      TO_EXTERNAL_FORMAT (LISP_STRING, sound,
-			  ALLOCA, (soundext, soundextlen),
-			  Qbinary);
+      LISP_STRING_TO_SIZED_EXTERNAL (sound, soundext, soundextlen, Qbinary);
       /* The sound code doesn't like getting SIGIO interrupts. Unix sucks! */
       stop_interrupts ();
       succes = play_sound_data (soundext, soundextlen, vol);
