@@ -45,16 +45,12 @@ static int
 init_device (int volume, Binbyte *data, int fd,
 	     unsigned int *header_length)
 {
-#ifdef SUNOS4_0_3
-  if (header_length) *header_length = 0;
-  return 0;
-#else
   Audio_hdr file_hdr;
 
   reset_volume_p = 0;
   reset_device_p = 0;
 
-  if (data && fd) ABORT (); /* one or the other */
+  assert (!(data && fd)); /* one or the other */
 
   if (AUDIO_SUCCESS != audio_get_play_config (audio_fd, &dev_hdr))
     {
@@ -108,7 +104,6 @@ init_device (int volume, Binbyte *data, int fd,
   }
 
   return 0;
-#endif
 }
 
 
