@@ -4,10 +4,10 @@
 
 This file is part of XEmacs.
 
-XEmacs is free software; you can redistribute it and/or modify it
+XEmacs is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
-Free Software Foundation; either version 2, or (at your option) any
-later version.
+Free Software Foundation, either version 3 of the License, or (at your
+option) any later version.
 
 XEmacs is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
@@ -15,9 +15,7 @@ FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 for more details.
 
 You should have received a copy of the GNU General Public License
-along with XEmacs; see the file COPYING.  If not, write to
-the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-Boston, MA 02111-1307, USA.  */
+along with XEmacs.  If not, see <http://www.gnu.org/licenses/>. */
 
 /* Synched up with: Not really in FSF. */
 
@@ -30,8 +28,6 @@ Boston, MA 02111-1307, USA.  */
 #include <sys/resource.h>
 #endif
 
-#if !defined (NO_SUBPROCESSES)
-
 #ifdef MINGW
 #include <../mingw/process.h>
 #elif defined (CYGWIN)
@@ -53,11 +49,6 @@ Boston, MA 02111-1307, USA.  */
 #  include <netinet/in.h>
 #  include <arpa/inet.h>
 # endif
-# ifdef NEED_NET_ERRNO_H
-#  include <net/errno.h>
-# endif /* NEED_NET_ERRNO_H */
-#elif defined (SKTPAIR)
-# include <sys/socket.h>
 #endif /* HAVE_SOCKETS */
 
 #ifdef WIN32_NATIVE
@@ -118,12 +109,6 @@ Boston, MA 02111-1307, USA.  */
 /* Define first descriptor number available for subprocesses.  */
 #define FIRST_PROC_DESC 3
 
-#ifdef IRIS
-# include <sys/sysmacros.h>	/* for "minor" */
-#endif /* not IRIS */
-
-#endif /* !NO_SUBPROCESSES */
-
 #ifdef AIX
 #include <sys/select.h>
 #endif
@@ -203,7 +188,7 @@ pid_t qxe_getpid (void);
 
 #include "syssignal.h" /* needed for SIGTSTP */
 
-#if !defined (WIN32_NATIVE) && ((!defined (SIGTSTP) && !defined (USG_JOBCTRL)) || !defined (HAVE_MKDIR) || !defined (HAVE_RMDIR))
+#if !defined (WIN32_NATIVE) && (!defined (SIGTSTP) || !defined (HAVE_MKDIR) || !defined (HAVE_RMDIR))
 
 #define NEED_SYNC_PROCESS_CODE
 
