@@ -1,14 +1,14 @@
 /* C support for testing XEmacs - see tests/automated/c-tests.el
    Copyright (C) 2000 Martin Buchholz
-   Copyright (C) 2001, 2002 Ben Wing.
+   Copyright (C) 2001, 2002, 2010 Ben Wing.
    Copyright (C) 2006 The Free Software Foundation, Inc.
 
 This file is part of XEmacs.
 
-XEmacs is free software; you can redistribute it and/or modify it
+XEmacs is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
-Free Software Foundation; either version 2, or (at your option) any
-later version.
+Free Software Foundation, either version 3 of the License, or (at your
+option) any later version.
 
 XEmacs is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
@@ -16,9 +16,7 @@ FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 for more details.
 
 You should have received a copy of the GNU General Public License
-along with XEmacs; see the file COPYING.  If not, write to
-the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-Boston, MA 02111-1307, USA.  */
+along with XEmacs.  If not, see <http://www.gnu.org/licenses/>. */
 
 /* Author: Martin Buchholz
 
@@ -38,12 +36,18 @@ Boston, MA 02111-1307, USA.  */
 static Lisp_Object Vtest_function_list;
 
 DEFUN ("test-data-format-conversion", Ftest_data_format_conversion, 0, 0, "", /*
-Test TO_EXTERNAL_FORMAT() and TO_INTERNAL_FORMAT()
+  Return list of results of test TO_EXTERNAL_FORMAT() and TO_INTERNAL_FORMAT().
+For use by the automated test suite.  See tests/automated/c-tests.
+
+Each element is a list (DESCRIPTION, STATUS, REASON).
+DESCRIPTION is a string describing the test.
+STATUS is a symbol, either t (pass) or nil (fail).
+REASON is nil or a string describing the failure (not required).
 */
        ())
 {
   void *ptr; Bytecount len;
-  Lisp_Object string, opaque;
+  Lisp_Object string = Qnil, opaque = Qnil, conversion_result = Qnil;
 
   Ibyte int_foo[] = "\n\nfoo\nbar";
   Extbyte ext_unix[]= "\n\nfoo\nbar";
@@ -66,6 +70,20 @@ Test TO_EXTERNAL_FORMAT() and TO_INTERNAL_FORMAT()
   Lisp_Object string_latin1 = make_string (int_latin1, sizeof (int_latin1) - 1);
   int autodetect_eol_p =
     !NILP (Fsymbol_value (intern ("eol-detection-enabled-p")));
+  struct gcpro gcpro1, gcpro2, gcpro3, gcpro4, gcpro5;
+  struct gcpro ngcpro1, ngcpro2, ngcpro3;
+#ifdef MULE
+  struct gcpro ngcpro4;
+#endif
+
+  /* DFC conversion inhibits GC, but we have a call2() below which calls
+     Lisp, which can trigger GC, so we need to GC-protect everything here. */
+  GCPRO5 (string, opaque, conversion_result, opaque_dos, string_foo);
+#ifdef MULE
+  NGCPRO4 (string_latin2, opaque_latin, opaque0_latin, string_latin1);
+#else
+  NGCPRO3 (opaque_latin, opaque0_latin, string_latin1);
+#endif
 
   /* Check for expected strings before and after conversion.
      Conversions depend on whether MULE is defined. */
@@ -75,129 +93,175 @@ Test TO_EXTERNAL_FORMAT() and TO_INTERNAL_FORMAT()
 #ifdef MULE
 #define DFC_CHECK_DATA_COND_MULE(ptr,len,			\
 				 constant_string_mule,		\
-				 constant_string_non_mule)	\
-    DFC_CHECK_DATA (ptr, len, constant_string_mule)
+				 constant_string_non_mule,	\
+				 description)			\
+    DFC_CHECK_DATA (ptr, len, constant_string_mule, description)
 #define DFC_CHECK_DATA_COND_MULE_NUL(ptr,len,			\
 				     constant_string_mule,	\
-				     constant_string_non_mule)	\
-    DFC_CHECK_DATA_NUL (ptr, len, constant_string_mule)
+				     constant_string_non_mule,	\
+				     description)		\
+    DFC_CHECK_DATA_NUL (ptr, len, constant_string_mule, description)
 #else
 #define DFC_CHECK_DATA_COND_MULE(ptr,len,			\
 				 constant_string_mule,		\
-				 constant_string_non_mule)	\
-    DFC_CHECK_DATA (ptr, len, constant_string_non_mule)
+				 constant_string_non_mule,	\
+				 description)			\
+    DFC_CHECK_DATA (ptr, len, constant_string_non_mule, description)
 #define DFC_CHECK_DATA_COND_MULE_NUL(ptr,len,			\
 				     constant_string_mule,	\
-				     constant_string_non_mule)	\
-    DFC_CHECK_DATA_NUL (ptr, len, constant_string_non_mule)
+				     constant_string_non_mule,	\
+				     description)		\
+    DFC_CHECK_DATA_NUL (ptr, len, constant_string_non_mule, description)
 #endif
 
   /* These now only apply to base coding systems, and
      need to test `eol-detection-enabled-p' at runtime. */
-#define DFC_CHECK_DATA_COND_EOL(ptr,len,			\
-				constant_string_eol,		\
-				constant_string_non_eol) do {	\
-    if (autodetect_eol_p)					\
-      DFC_CHECK_DATA (ptr, len, constant_string_eol);		\
-    else							\
-      DFC_CHECK_DATA (ptr, len, constant_string_non_eol);	\
+#define DFC_CHECK_DATA_COND_EOL(ptr,len,				\
+				constant_string_eol,			\
+				constant_string_non_eol,		\
+				description) do {			\
+    if (autodetect_eol_p)						\
+      DFC_CHECK_DATA (ptr, len, constant_string_eol, description);	\
+    else								\
+      DFC_CHECK_DATA (ptr, len, constant_string_non_eol, description);	\
   } while (0)
-#define DFC_CHECK_DATA_COND_EOL_NUL(ptr,len,			\
-				    constant_string_eol,	\
-				    constant_string_non_eol) do {	\
-    if (autodetect_eol_p)					\
-      DFC_CHECK_DATA_NUL (ptr, len, constant_string_eol);	\
-    else							\
-      DFC_CHECK_DATA_NUL (ptr, len, constant_string_non_eol);	\
+#define DFC_CHECK_DATA_COND_EOL_NUL(ptr,len,				\
+				    constant_string_eol,		\
+				    constant_string_non_eol,		\
+				    description) do {			\
+    if (autodetect_eol_p)						\
+      DFC_CHECK_DATA_NUL (ptr, len, constant_string_eol, description);	\
+    else								\
+      DFC_CHECK_DATA_NUL (ptr, len, constant_string_non_eol, description); \
   } while (0)
 
   /* Check for expected strings before and after conversion. */
-#define DFC_CHECK_DATA(ptr,len, constant_string) do {	\
-    assert ((len) == sizeof (constant_string) - 1);	\
-    assert (!memcmp (ptr, constant_string, len));	\
+#define DFC_CHECK_DATA(ptr,len,constant_string,test) do {		\
+    DFC_INITIALIZE (test);						\
+    DFC_CHECK_LENGTH (len, sizeof (constant_string) - 1, test);	\
+    DFC_CHECK_CONTENT (ptr, constant_string, len, test);		\
+    DFC_RESULT_PASS (test);						\
   } while (0)
 
   /* Macro version that includes the trailing NULL byte. */
-#define DFC_CHECK_DATA_NUL(ptr,len,constant_string) do {\
-    assert ((len) == sizeof (constant_string));		\
-    assert (!memcmp (ptr, constant_string, len));	\
+#define DFC_CHECK_DATA_NUL(ptr,len,constant_string,test) do {	\
+    DFC_INITIALIZE (test);					\
+    DFC_CHECK_LENGTH (len, sizeof (constant_string), test);	\
+    DFC_CHECK_CONTENT (ptr, constant_string, len, test);	\
+    DFC_RESULT_PASS (test);					\
   } while (0)
 
+/* WARNING WARNING WARNING!
+   The following macros are NOT protected by "do { ... } while (0)"!!
+*/
+
+#define DFC_INITIALIZE(test_name) if (0)
+
+#define DFC_CHECK_LENGTH(len1,len2,str1)	\
+    else if ((len1) != (len2))			\
+      conversion_result =			\
+        Fcons (list3 (build_cistring(str1), Qnil, build_ascstring("wrong length")), \
+	       conversion_result)
+
+#define DFC_CHECK_CONTENT(str1,str2,len1,str3)	\
+    else if (memcmp (str1, str2, len1))		\
+      conversion_result =			\
+	Fcons (list3 (build_cistring(str3), Qnil,			\
+		      build_ascstring("octet comparison failed")),	\
+	       conversion_result)
+
+#define DFC_RESULT_PASS(str1)		\
+    else				\
+      conversion_result =		\
+	Fcons (list3 (build_cistring(str1), Qt, Qnil),	\
+	       conversion_result)
+
 #ifdef MULE
   ptr = NULL, len = rand();
   TO_EXTERNAL_FORMAT (DATA, (int_latin2, sizeof (int_latin2)),
 		      ALLOCA, (ptr, len),
 		      intern ("iso-8859-2"));
-  DFC_CHECK_DATA_NUL (ptr, len, ext_latin);
+  DFC_CHECK_DATA_NUL (ptr, len, ext_latin,
+		      "Latin-2 DATA, ALLOCA, Latin 2/NUL");
 
   ptr = NULL, len = rand();
   TO_EXTERNAL_FORMAT (LISP_STRING, string_latin2,
 		      ALLOCA, (ptr, len),
 		      intern ("iso-8859-2"));
-  DFC_CHECK_DATA (ptr, len, ext_latin);
+  DFC_CHECK_DATA (ptr, len, ext_latin,
+		  "Latin-2 Lisp string, ALLOCA, Latin 2");
 
   ptr = NULL, len = rand();
   TO_EXTERNAL_FORMAT (LISP_STRING, string_latin1,
 		      ALLOCA, (ptr, len),
 		      intern ("iso-latin-2-with-esc"));
-  DFC_CHECK_DATA (ptr, len, ext_latin12);
+  DFC_CHECK_DATA (ptr, len, ext_latin12,
+		  "Latin-1 Lisp string, ALLOCA, Latin 2/ESC");
 
   ptr = NULL, len = rand();
   TO_EXTERNAL_FORMAT (DATA, (int_latin2, sizeof (int_latin2) - 1),
 		      MALLOC, (ptr, len),
 		      intern ("iso-8859-2"));
-  DFC_CHECK_DATA (ptr, len, ext_latin);
-  xfree (ptr, void *);
+  DFC_CHECK_DATA (ptr, len, ext_latin, "Latin-2 DATA, MALLOC, Latin-2");
+  xfree (ptr);
 
   TO_EXTERNAL_FORMAT (DATA, (int_latin2, sizeof (int_latin2) - 1),
 		      LISP_OPAQUE, opaque,
 		      intern ("iso-8859-2"));
-  DFC_CHECK_DATA (XOPAQUE_DATA (opaque), XOPAQUE_SIZE (opaque), ext_latin);
+  DFC_CHECK_DATA (XOPAQUE_DATA (opaque), XOPAQUE_SIZE (opaque), ext_latin,
+		  "Latin-2 DATA, Lisp opaque, Latin-2");
 
   ptr = NULL, len = rand();
   TO_INTERNAL_FORMAT (DATA, (ext_latin, sizeof (ext_latin) - 1),
 		      ALLOCA, (ptr, len),
 		      intern ("iso-latin-2-with-esc"));
-  DFC_CHECK_DATA (ptr, len, int_latin2);
+  DFC_CHECK_DATA (ptr, len, int_latin2,
+		  "Latin-2/ESC, ALLOCA, Latin-1 DATA");
 
   ptr = NULL, len = rand();
   TO_INTERNAL_FORMAT (DATA, (ext_latin, sizeof (ext_latin) - 1),
 		      MALLOC, (ptr, len),
 		      intern ("iso-latin-2-with-esc"));
-  DFC_CHECK_DATA (ptr, len, int_latin2);
-  xfree (ptr, void *);
+  DFC_CHECK_DATA (ptr, len, int_latin2,
+		  "Latin-2/ESC, MALLOC, Latin-1 DATA");
+  xfree (ptr);
 
   TO_INTERNAL_FORMAT (DATA, (ext_latin, sizeof (ext_latin) - 1),
 		      LISP_STRING, string,
 		      intern ("iso-latin-2-with-esc"));
-  DFC_CHECK_DATA (XSTRING_DATA (string), XSTRING_LENGTH (string), int_latin2);
+  DFC_CHECK_DATA (XSTRING_DATA (string), XSTRING_LENGTH (string), int_latin2,
+		  "Latin-2/ESC, Lisp string, Latin-2");
 
   TO_INTERNAL_FORMAT (LISP_OPAQUE, opaque_latin,
 		      LISP_STRING, string,
 		      intern ("iso-latin-2-with-esc"));
-  DFC_CHECK_DATA (XSTRING_DATA (string), XSTRING_LENGTH (string), int_latin2);
+  DFC_CHECK_DATA (XSTRING_DATA (string), XSTRING_LENGTH (string), int_latin2,
+		  "Lisp opaque, Lisp string, Latin-2/ESC");
 
   TO_INTERNAL_FORMAT (LISP_OPAQUE, opaque0_latin,
 		      LISP_STRING, string,
 		      intern ("iso-latin-2-with-esc"));
-  DFC_CHECK_DATA_NUL (XSTRING_DATA (string), XSTRING_LENGTH (string), int_latin2);
+  DFC_CHECK_DATA_NUL (XSTRING_DATA (string), XSTRING_LENGTH (string), int_latin2,
+		      "Lisp opaque, Lisp string, Latin-2/ESC/NUL");
 
   TO_INTERNAL_FORMAT (LISP_OPAQUE, opaque0_latin,
 		      LISP_BUFFER, Fcurrent_buffer(),
 		      intern ("iso-latin-2-with-esc"));
   DFC_CHECK_DATA_NUL (BUF_BYTE_ADDRESS (current_buffer, BUF_PT (current_buffer)),
-		    sizeof (int_latin2), int_latin2);
+		    sizeof (int_latin2), int_latin2,
+		      "Lisp opaque, Lisp buffer, Latin-2/ESC/NUL");
 
   TO_INTERNAL_FORMAT (LISP_OPAQUE, opaque_latin,
 		      LISP_BUFFER, Fcurrent_buffer(),
 		      intern ("iso-8859-1"));
   DFC_CHECK_DATA (BUF_BYTE_ADDRESS (current_buffer, BUF_PT (current_buffer)),
-		  sizeof (int_latin1) - 1, int_latin1);
+		  sizeof (int_latin1) - 1, int_latin1,
+		  "Lisp opaque, Lisp buffer, Latin-1");
 
   TO_INTERNAL_FORMAT (DATA, (ext_latin12, sizeof (ext_latin12) - 1),
 		      ALLOCA, (ptr, len),
 		      intern ("iso-latin-2-with-esc"));
-  DFC_CHECK_DATA (ptr, len, int_latin1);
+  DFC_CHECK_DATA (ptr, len, int_latin1, "DATA, ALLOCA, Latin-1");
 
 #endif /* MULE */
 
@@ -205,127 +269,147 @@ Test TO_EXTERNAL_FORMAT() and TO_INTERNAL_FORMAT()
   TO_EXTERNAL_FORMAT (DATA, (int_latin1, sizeof (int_latin1) - 1),
 		      ALLOCA, (ptr, len),
 		      Qbinary);
-  DFC_CHECK_DATA_COND_MULE (ptr, len, ext_latin, int_latin1);
+  DFC_CHECK_DATA_COND_MULE (ptr, len, ext_latin, int_latin1,
+			    "Latin-1 DATA, ALLOCA, binary");
 
   ptr = NULL, len = rand();
   TO_EXTERNAL_FORMAT (DATA, (int_latin1, sizeof (int_latin1)),
 		      ALLOCA, (ptr, len),
 		      Qbinary);
-  DFC_CHECK_DATA_COND_MULE_NUL (ptr, len, ext_latin, int_latin1);
+  DFC_CHECK_DATA_COND_MULE_NUL (ptr, len, ext_latin, int_latin1,
+				"Latin-1 DATA, ALLOCA, binary/NUL");
 
   ptr = NULL, len = rand();
   TO_EXTERNAL_FORMAT (DATA, (int_latin2, sizeof (int_latin2) - 1),
 		      ALLOCA, (ptr, len),
 		      Qbinary);
-  DFC_CHECK_DATA_COND_MULE (ptr, len, ext_tilde, int_latin2);
+  DFC_CHECK_DATA_COND_MULE (ptr, len, ext_tilde, int_latin2,
+			    "Latin-2 DATA, ALLOCA, binary");
 
   ptr = NULL, len = rand();
   TO_EXTERNAL_FORMAT (DATA, (int_latin1, sizeof (int_latin1) - 1),
 		      ALLOCA, (ptr, len),
 		      intern ("iso-8859-1"));
-  DFC_CHECK_DATA_COND_MULE (ptr, len, ext_latin, int_latin1);
+  DFC_CHECK_DATA_COND_MULE (ptr, len, ext_latin, int_latin1,
+			    "Latin-1 DATA, ALLOCA, Latin-1");
 
 
   ptr = NULL, len = rand();
   TO_EXTERNAL_FORMAT (LISP_STRING, string_latin1,
 		      ALLOCA, (ptr, len),
 		      Qbinary);
-  DFC_CHECK_DATA_COND_MULE (ptr, len, ext_latin, int_latin1);
+  DFC_CHECK_DATA_COND_MULE (ptr, len, ext_latin, int_latin1,
+			    "Latin-1 Lisp string, ALLOCA, binary");
 
   ptr = NULL, len = rand();
   TO_EXTERNAL_FORMAT (LISP_STRING, string_latin1,
 		      ALLOCA, (ptr, len),
 		      Qbinary);
-  DFC_CHECK_DATA_COND_MULE (ptr, len, ext_latin, int_latin1);
+  DFC_CHECK_DATA_COND_MULE (ptr, len, ext_latin, int_latin1,
+			    "Latin-1 Lisp string, ALLOCA, binary");
 
   ptr = NULL, len = rand();
   TO_EXTERNAL_FORMAT (LISP_STRING, string_latin1,
 		      ALLOCA, (ptr, len),
 		      intern ("iso-8859-1"));
-  DFC_CHECK_DATA_COND_MULE (ptr, len, ext_latin, int_latin1);
+  DFC_CHECK_DATA_COND_MULE (ptr, len, ext_latin, int_latin1,
+			    "Latin-1 Lisp string, ALLOCA, Latin-1");
 
   ptr = NULL, len = rand();
   TO_EXTERNAL_FORMAT (DATA, (int_latin1, sizeof (int_latin1) - 1),
 		      MALLOC, (ptr, len),
 		      Qbinary);
-  DFC_CHECK_DATA_COND_MULE (ptr, len, ext_latin, int_latin1);
-  xfree (ptr, void *);
+  DFC_CHECK_DATA_COND_MULE (ptr, len, ext_latin, int_latin1,
+			    "Latin-1 DATA, MALLOC, binary");
+  xfree (ptr);
 
   ptr = NULL, len = rand();
   TO_EXTERNAL_FORMAT (DATA, (int_latin2, sizeof (int_latin2)),
 		      MALLOC, (ptr, len),
 		      Qbinary);
-  DFC_CHECK_DATA_COND_MULE_NUL (ptr, len, ext_tilde, int_latin2);
-  xfree (ptr, void *);
+  DFC_CHECK_DATA_COND_MULE_NUL (ptr, len, ext_tilde, int_latin2,
+				"Latin-2 DATA, MALLOC, binary/NUL");
+  xfree (ptr);
 
   ptr = NULL, len = rand();
   TO_EXTERNAL_FORMAT (DATA, (int_latin1, sizeof (int_latin1) - 1),
 		      MALLOC, (ptr, len),
 		      intern ("iso-8859-1"));
-  DFC_CHECK_DATA_COND_MULE (ptr, len, ext_latin, int_latin1);
-  xfree (ptr, void *);
+  DFC_CHECK_DATA_COND_MULE (ptr, len, ext_latin, int_latin1,
+			    "Latin-1 DATA, MALLOC, Latin-1");
+  xfree (ptr);
 
   TO_EXTERNAL_FORMAT (DATA, (int_latin1, sizeof (int_latin1) - 1),
 		      LISP_OPAQUE, opaque,
 		      Qbinary);
   DFC_CHECK_DATA_COND_MULE (XOPAQUE_DATA (opaque),
-			    XOPAQUE_SIZE (opaque), ext_latin, int_latin1);
+			    XOPAQUE_SIZE (opaque), ext_latin, int_latin1,
+			    "Latin-1 DATA, Lisp opaque, binary");
 
   TO_EXTERNAL_FORMAT (DATA, (int_latin2, sizeof (int_latin2)),
 		      LISP_OPAQUE, opaque,
 		      Qbinary);
   DFC_CHECK_DATA_COND_MULE_NUL (XOPAQUE_DATA (opaque),
-				XOPAQUE_SIZE (opaque), ext_tilde, int_latin2);
+				XOPAQUE_SIZE (opaque), ext_tilde, int_latin2,
+				"Latin-2 DATA, Lisp opaque, binary");
 
   TO_EXTERNAL_FORMAT (DATA, (int_latin1, sizeof (int_latin1) - 1),
 		      LISP_OPAQUE, opaque,
 		      intern ("iso-8859-1"));
   DFC_CHECK_DATA_COND_MULE (XOPAQUE_DATA (opaque),
-			    XOPAQUE_SIZE (opaque), ext_latin, int_latin1);
+			    XOPAQUE_SIZE (opaque), ext_latin, int_latin1,
+			    "Latin-1 DATA, Lisp opaque, Latin-1");
 
   ptr = NULL, len = rand();
   TO_INTERNAL_FORMAT (DATA, (ext_latin, sizeof (ext_latin) - 1),
 		      ALLOCA, (ptr, len),
 		      Qbinary);
-  DFC_CHECK_DATA_COND_MULE (ptr, len, int_latin1, ext_latin);
+  DFC_CHECK_DATA_COND_MULE (ptr, len, int_latin1, ext_latin,
+			    "Latin-1 DATA, ALLOCA, binary");
 
   ptr = NULL, len = rand();
   TO_INTERNAL_FORMAT (DATA, (ext_latin, sizeof (ext_latin)),
 		      ALLOCA, (ptr, len),
 		      intern ("iso-8859-1"));
-  DFC_CHECK_DATA_COND_MULE_NUL (ptr, len, int_latin1, ext_latin);
+  DFC_CHECK_DATA_COND_MULE_NUL (ptr, len, int_latin1, ext_latin,
+				"Latin-1 DATA, ALLOCA, Latin-1");
 
   ptr = NULL, len = rand();
   TO_INTERNAL_FORMAT (DATA, (ext_latin, sizeof (ext_latin)),
 		      MALLOC, (ptr, len),
 		      intern ("iso-8859-1"));
-  DFC_CHECK_DATA_COND_MULE_NUL (ptr, len, int_latin1, ext_latin);
-  xfree (ptr, void *);
+  DFC_CHECK_DATA_COND_MULE_NUL (ptr, len, int_latin1, ext_latin,
+				"Latin-1 DATA, MALLOC, Latin-1");
+  xfree (ptr);
 
   ptr = NULL, len = rand();
   TO_INTERNAL_FORMAT (DATA, (ext_latin, sizeof (ext_latin)),
 		      MALLOC, (ptr, len),
 		      Qnil);
-  DFC_CHECK_DATA_COND_MULE_NUL (ptr, len, int_latin1, ext_latin);
-  xfree (ptr, void *);
+  DFC_CHECK_DATA_COND_MULE_NUL (ptr, len, int_latin1, ext_latin,
+				"Latin-1 DATA, MALLOC, nil");
+  xfree (ptr);
 
   TO_INTERNAL_FORMAT (DATA, (ext_latin, sizeof (ext_latin) - 1),
 		      LISP_STRING, string,
 		      intern ("iso-8859-1"));
   DFC_CHECK_DATA_COND_MULE (XSTRING_DATA (string),
-			    XSTRING_LENGTH (string), int_latin1, ext_latin);
+			    XSTRING_LENGTH (string), int_latin1, ext_latin,
+			    "Latin-1 DATA, Lisp stirng, Latin-1");
 
   TO_INTERNAL_FORMAT (LISP_OPAQUE, opaque_latin,
 		      LISP_STRING, string,
 		      intern ("iso-8859-1"));
   DFC_CHECK_DATA_COND_MULE (XSTRING_DATA (string),
-			    XSTRING_LENGTH (string), int_latin1, ext_latin);
+			    XSTRING_LENGTH (string), int_latin1, ext_latin,
+			    "Latin-1 Lisp opaque, Lisp string, Latin-1");
 
   TO_INTERNAL_FORMAT (LISP_OPAQUE, opaque0_latin,
 		      LISP_STRING, string,
 		      intern ("iso-8859-1"));
   DFC_CHECK_DATA_COND_MULE_NUL (XSTRING_DATA (string),
-				XSTRING_LENGTH (string), int_latin1, ext_latin);
+				XSTRING_LENGTH (string), int_latin1, ext_latin,
+				"Latin-1 Lisp opaque, Lisp string, Latin-1/NUL");
 
   /* This next group used to use the COND_EOL macros, but with the new Mule,
      they all specify an EOL convention, and all XEmacsen can grok them. */
@@ -333,45 +417,47 @@ Test TO_EXTERNAL_FORMAT() and TO_INTERNAL_FORMAT()
   TO_EXTERNAL_FORMAT (DATA, (int_foo, sizeof (int_foo)),
 		      MALLOC, (ptr, len),
 		      Qbinary);
-  DFC_CHECK_DATA_NUL (ptr, len, ext_unix);
-  xfree (ptr, void *);
+  DFC_CHECK_DATA_NUL (ptr, len, ext_unix,
+		      "ASCII DATA, MALLOC, binary/LF/NUL");
+  xfree (ptr);
 
   ptr = NULL, len = rand();
   TO_EXTERNAL_FORMAT (DATA, (int_foo, sizeof (int_foo) - 1),
 		      LISP_OPAQUE, opaque,
 		      intern ("raw-text-mac"));
-  DFC_CHECK_DATA (XOPAQUE_DATA (opaque), XOPAQUE_SIZE (opaque), ext_mac);
+  DFC_CHECK_DATA (XOPAQUE_DATA (opaque), XOPAQUE_SIZE (opaque), ext_mac,
+		      "ASCII DATA, Lisp opaque, binary/CR");
 
   ptr = NULL, len = rand();
   TO_EXTERNAL_FORMAT (LISP_STRING, string_foo,
 		      ALLOCA, (ptr, len),
 		      intern ("raw-text-dos"));
-  DFC_CHECK_DATA (ptr, len, ext_dos);
+  DFC_CHECK_DATA (ptr, len, ext_dos, "ASCII Lisp string, ALLOCA, binary/CRLF");
 
   ptr = NULL, len = rand();
   TO_EXTERNAL_FORMAT (DATA, (int_foo, sizeof (int_foo) - 1),
 		      ALLOCA, (ptr, len),
 		      intern ("raw-text-unix"));
-  DFC_CHECK_DATA (ptr, len, ext_unix);
+  DFC_CHECK_DATA (ptr, len, ext_unix, "ASCII DATA, ALLOCA, binary/LF");
 
   ptr = NULL, len = rand();
   TO_EXTERNAL_FORMAT (LISP_STRING, string_foo,
 		      MALLOC, (ptr, len),
 		      intern ("no-conversion-mac"));
-  DFC_CHECK_DATA (ptr, len, ext_mac);
-  xfree (ptr, void *);
+  DFC_CHECK_DATA (ptr, len, ext_mac, "ASCII Lisp string, MALLOC, binary/CR");
+  xfree (ptr);
 
   ptr = NULL, len = rand();
   TO_EXTERNAL_FORMAT (DATA, (int_foo, sizeof (int_foo) - 1),
 		      ALLOCA, (ptr, len),
 		      intern ("no-conversion-dos"));
-  DFC_CHECK_DATA (ptr, len, ext_dos);
+  DFC_CHECK_DATA (ptr, len, ext_dos, "ASCII DATA, ALLOCA, binary/CRLF");
 
   ptr = NULL, len = rand();
   TO_EXTERNAL_FORMAT (DATA, (int_foo, sizeof (int_foo)),
 		      ALLOCA, (ptr, len),
 		      intern ("no-conversion-unix"));
-  DFC_CHECK_DATA_NUL (ptr, len, ext_unix);
+  DFC_CHECK_DATA_NUL (ptr, len, ext_unix, "ASCII DATA, ALLOCA, binary/LF/NUL");
 
   /* Oh, Lawdy, Lawdy, Lawdy, this done broke mah heart!
 
@@ -401,7 +487,8 @@ Test TO_EXTERNAL_FORMAT() and TO_INTERNAL_FORMAT()
 			LISP_BUFFER, Fcurrent_buffer(),
 			cs_to_use);
     DFC_CHECK_DATA (BUF_BYTE_ADDRESS (current_buffer, BUF_PT (current_buffer)),
-		    sizeof (ext_dos) - 1, ext_dos);
+		    sizeof (ext_dos) - 1, ext_dos,
+		    "DOS Lisp opaque, Lisp buffer, undecided-unix");
 
     /* Check eol autodetection works when enabled -- honest. */
     cs_to_use =
@@ -413,7 +500,8 @@ Test TO_EXTERNAL_FORMAT() and TO_INTERNAL_FORMAT()
 			LISP_BUFFER, Fcurrent_buffer(),
 			cs_to_use);
     DFC_CHECK_DATA (BUF_BYTE_ADDRESS (current_buffer, BUF_PT (current_buffer)),
-		    sizeof (int_foo) - 1, int_foo);
+		    sizeof (int_foo) - 1, int_foo,
+		    "DOS Lisp opaque, Lisp buffer, undecided");
     /* reset to default */
     XCODING_SYSTEM_EOL_TYPE (cs_to_use) =
       autodetect_eol_p ? EOL_AUTODETECT : EOL_LF;
@@ -427,18 +515,20 @@ Test TO_EXTERNAL_FORMAT() and TO_INTERNAL_FORMAT()
   if (autodetect_eol_p)
     DFC_CHECK_DATA (BUF_BYTE_ADDRESS (current_buffer,
 				      BUF_PT (current_buffer)),
-		    sizeof (int_foo) - 1, int_foo);
+		    sizeof (int_foo) - 1, int_foo,
+		    "DOS Lisp opaque, Lisp buffer, autodetect eol");
   else
     DFC_CHECK_DATA (BUF_BYTE_ADDRESS (current_buffer,
 				      BUF_PT (current_buffer)),
-		    sizeof (ext_dos) - 1, ext_dos);
+		    sizeof (ext_dos) - 1, ext_dos,
+		    "DOS Lisp opaque, Lisp buffer, no autodetect eol");
 
   TO_INTERNAL_FORMAT (DATA, (ext_mac, sizeof (ext_mac) - 1),
 		      LISP_STRING, string,
 		      intern ("iso-8859-1"));
   DFC_CHECK_DATA_COND_EOL (XSTRING_DATA (string),
-			   XSTRING_LENGTH (string), int_foo, ext_mac);
-
+			   XSTRING_LENGTH (string), int_foo, ext_mac,
+			   "Mac DATA, Lisp string, Latin-1/EOL");
   {
     Lisp_Object stream =
       make_fixed_buffer_input_stream (ext_dos, sizeof (ext_dos) - 1);
@@ -446,23 +536,26 @@ Test TO_EXTERNAL_FORMAT() and TO_INTERNAL_FORMAT()
 			LISP_STRING, string,
 			intern ("iso-8859-1"));
     DFC_CHECK_DATA_COND_EOL (XSTRING_DATA (string),
-			     XSTRING_LENGTH (string), int_foo, ext_dos);
+			     XSTRING_LENGTH (string), int_foo, ext_dos,
+			   "DOS lstream, Lisp string, Latin-1/EOL");
   }
 
   TO_INTERNAL_FORMAT (DATA, (ext_unix, sizeof (ext_unix) - 1),
 		      LISP_STRING, string,
 		      intern ("no-conversion"));
   DFC_CHECK_DATA_COND_EOL (XSTRING_DATA (string),
-			   XSTRING_LENGTH (string), int_foo, ext_unix);
-
+			   XSTRING_LENGTH (string), int_foo, ext_unix,
+			   "Unix DATA, Lisp string, no-conversion");
 
   ptr = NULL, len = rand();
   TO_EXTERNAL_FORMAT (LISP_OPAQUE, opaque_dos,
 		      ALLOCA, (ptr, len),
 		      Qbinary);
-  DFC_CHECK_DATA (ptr, len, ext_dos);
+  DFC_CHECK_DATA (ptr, len, ext_dos, "DOS Lisp opaque, ALLOCA, binary");
 
-  return intern ("PASS");
+  NUNGCPRO;
+  UNGCPRO;
+  return conversion_result;
 }
 
 
@@ -506,13 +599,21 @@ test_hash_tables_predicate (Lisp_Object key,
 
 
 DEFUN ("test-hash-tables", Ftest_hash_tables, 0, 0, "", /*
-Test C interface to hash tables.
+  Return list of results of testing C interface to hash tables.
+For use by the automated test suite.  See tests/automated/c-tests.
+
+Each element is a list (DESCRIPTION, STATUS, REASON).
+DESCRIPTION is a string describing the test.
+STATUS is a symbol, either t (pass) or nil (fail).
+REASON is nil or a string describing the failure (not required).
 */
        ())
 {
+  Lisp_Object hash_result = Qnil;
+
   test_hash_tables_data data;
   data.hash_table = make_lisp_hash_table (50, HASH_TABLE_NON_WEAK,
-					  HASH_TABLE_EQUAL);
+					  Qequal);
 
   Fputhash (make_int (1), make_int (2), data.hash_table);
   Fputhash (make_int (3), make_int (4), data.hash_table);
@@ -520,19 +621,28 @@ Test C interface to hash tables.
   data.sum = 0;
   elisp_maphash_unsafe (test_hash_tables_mapper,
 			data.hash_table, (void *) &data);
-  assert (data.sum == 2 + 4);
+  hash_result = Fcons (list3 (build_ascstring ("simple mapper"),
+				   (data.sum == 2 + 4) ? Qt : Qnil,
+				   build_ascstring ("sum != 2 + 4")),
+			    hash_result);
 
   data.sum = 0;
   elisp_maphash (test_hash_tables_modifying_mapper,
 		 data.hash_table, (void *) &data);
-  assert (data.sum == 2 + 4);
+  hash_result = Fcons (list3 (build_ascstring ("modifying mapper"),
+				   (data.sum == 2 + 4) ? Qt : Qnil,
+				   build_ascstring ("sum != 2 + 4")),
+			    hash_result);
 
   /* hash table now contains:  (1, 2) (3, 4) (-1, 2*2) (-3, 2*4) */
 
   data.sum = 0;
   elisp_maphash_unsafe (test_hash_tables_mapper,
 			data.hash_table, (void *) &data);
-  assert (data.sum == 3 * (2 + 4));
+  hash_result = Fcons (list3 (build_ascstring ("simple mapper"),
+				   (data.sum == 3 * (2 + 4)) ? Qt : Qnil,
+				   build_ascstring ("sum != 3 * (2 + 4)")),
+			    hash_result);
 
   /* Remove entries with negative keys, added by modifying mapper */
   elisp_map_remhash (test_hash_tables_predicate,
@@ -541,9 +651,52 @@ Test C interface to hash tables.
   data.sum = 0;
   elisp_maphash_unsafe (test_hash_tables_mapper,
 			data.hash_table, (void *) &data);
-  assert (data.sum == 2 + 4);
+  hash_result = Fcons (list3 (build_ascstring ("remove negatives mapper"),
+				   (data.sum == 2 + 4) ? Qt : Qnil,
+				   build_ascstring ("sum != 2 + 4")),
+			    hash_result);
 
-  return intern ("PASS");
+  return hash_result;
+}
+
+DEFUN ("test-store-void-in-lisp", Ftest_store_void_in_lisp, 0, 0, "", /*
+  Test STORE_VOID_IN_LISP and its inverse GET_VOID_FROM_LISP.
+Tests by internal assert(); only returns if it succeeds.
+*/
+       ())
+{
+  struct foobar { int x; int y; short z; void *q; } baz;
+
+#define FROB(val)							\
+do									\
+{									\
+  void *pval = (void *) (val);						\
+  assert (GET_VOID_FROM_LISP (STORE_VOID_IN_LISP (pval)) == pval);	\
+}									\
+while (0)
+  assert (INT_VALBITS >= 31);
+  FROB (&baz);
+  FROB (&baz.x);
+  FROB (&baz.y);
+  FROB (&baz.z);
+  FROB (&baz.q);
+  FROB (0);
+  FROB (2);
+  FROB (&Vtest_function_list);
+  FROB (0x00000080);
+  FROB (0x00008080);
+  FROB (0x00808080);
+  FROB (0x80808080);
+  FROB (0xCAFEBABE);
+  FROB (0xFFFFFFFE);
+#if INT_VALBITS >= 63
+  FROB (0x0000808080808080);
+  FROB (0x8080808080808080);
+  FROB (0XDEADBEEFCAFEBABE);
+  FROB (0XFFFFFFFFFFFFFFFE);
+#endif /* INT_VALBITS >= 63 */
+
+  return list1 (list3 (build_ascstring ("STORE_VOID_IN_LISP"), Qt, Qnil));
 }
 
 
@@ -572,6 +725,7 @@ syms_of_tests (void)
 
   TESTS_DEFSUBR (Ftest_data_format_conversion);
   TESTS_DEFSUBR (Ftest_hash_tables);
+  TESTS_DEFSUBR (Ftest_store_void_in_lisp);
   /* Add other test functions here with TESTS_DEFSUBR */
 }
 
@@ -583,3 +737,4 @@ List of all test functions defined in tests.c.
 For use by the automated test suite.  See tests/automated/c-tests.
 */ );
 }
+
