@@ -1,4 +1,8 @@
-/* I really wish this entire file could go away, but there is
+/* ui-byhand.c --- hand-coded GTK functions
+
+Copyright (C) 2000, 2001 William M. Perry
+
+   I really wish this entire file could go away, but there is
    currently no way to do the following in the Foreign Function
    Interface:
 
@@ -7,7 +11,21 @@
    So we have to code a few functions by hand.  Ick.
 
    William M. Perry 5/8/00
-*/
+
+This file is part of XEmacs.
+
+XEmacs is free software: you can redistribute it and/or modify it
+under the terms of the GNU General Public License as published by the
+Free Software Foundation, either version 3 of the License, or (at your
+option) any later version.
+
+XEmacs is distributed in the hope that it will be useful, but WITHOUT
+ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
+for more details.
+
+You should have received a copy of the GNU General Public License
+along with XEmacs.  If not, see <http://www.gnu.org/licenses/>. */
 
 #include "gui.h"
 
@@ -170,7 +188,7 @@ Returns the text from GtkCList OBJ cell at coordinates ROW, COLUMN.
 
   if (text)
     {
-      rval = build_string (text);
+      rval = build_cistring (text);
       /* NOTE: This is NOT a memory leak.  GtkCList returns a pointer
 	 to internally used memory, not a copy of it.
 	 g_free (text);
@@ -257,7 +275,7 @@ Return a list of (pixmap mask text) at ROW,COLUMN in CLIST.
 
   return (list3 (pixmap ? build_gtk_boxed (pixmap, GTK_TYPE_GDK_WINDOW) : Qnil,
 		 mask ? build_gtk_boxed (mask, GTK_TYPE_GDK_WINDOW) : Qnil,
-		 (text && text[0]) ? build_string (text) : Qnil));
+		 (text && text[0]) ? build_cistring (text) : Qnil));
 }
 
 /* void gtk_color_selection_get_color(GtkColorSelection *colorsel, gdouble *color); */
@@ -408,7 +426,7 @@ Return the text of LABEL.
 
   gtk_label_get (GTK_LABEL (XGTK_OBJECT (label)->object), &string);
 
-  return (build_string (string));
+  return (build_cistring (string));
 }
 
 DEFUN ("gtk-notebook-query-tab-label-packing", Fgtk_notebook_query_tab_label_packing, 2, 2, 0, /*
@@ -473,7 +491,7 @@ __generic_toolbar_callback (GtkWidget *UNUSED (item), gpointer user_data)
   Lisp_Object callback;
   Lisp_Object lisp_user_data;
 
-  callback = VOID_TO_LISP (user_data);
+  callback = GET_LISP_FROM_VOID (user_data);
 
   lisp_user_data = XCAR (callback);
   callback = XCDR (callback);
@@ -532,7 +550,7 @@ generic_toolbar_insert_item (Lisp_Object toolbar,
 	 (char*) XSTRING_DATA (tooltip_private_text),
 	 GTK_WIDGET (XGTK_OBJECT (icon)->object),
 	 GTK_SIGNAL_FUNC (__generic_toolbar_callback),
-	 LISP_TO_VOID (callback));
+	 STORE_LISP_IN_VOID (callback));
     }
   else
     {
@@ -542,7 +560,7 @@ generic_toolbar_insert_item (Lisp_Object toolbar,
 				   (char*) XSTRING_DATA (tooltip_private_text),
 				   GTK_WIDGET (XGTK_OBJECT (icon)->object),
 				   GTK_SIGNAL_FUNC (__generic_toolbar_callback),
-				   LISP_TO_VOID (callback),
+				   STORE_LISP_IN_VOID (callback),
 				   XINT (position));
     }
 
@@ -582,7 +600,7 @@ __emacs_gtk_ctree_recurse_internal (GtkCTree *ctree, GtkCTreeNode *node, gpointe
 {
   Lisp_Object closure;
 
-  closure = VOID_TO_LISP (user_data);
+  closure = GET_LISP_FROM_VOID (user_data);
 
   call3 (XCAR (closure),
 	 build_gtk_object (GTK_OBJECT (ctree)),
@@ -649,7 +667,7 @@ void gtk_ctree_pre_recursive_to_depth            (GtkCTree     *ctree,
 	(GTK_CTREE (XGTK_OBJECT (ctree)->object),
 	 NILP (node) ? NULL : (GtkCTreeNode *) XGTK_BOXED (node)->object,
 	 __emacs_gtk_ctree_recurse_internal,
-	 LISP_TO_VOID (closure));
+	 STORE_LISP_IN_VOID (closure));
     }
   else
     {
@@ -658,7 +676,7 @@ void gtk_ctree_pre_recursive_to_depth            (GtkCTree     *ctree,
 	 NILP (node) ? NULL : (GtkCTreeNode *) XGTK_BOXED (node)->object,
 	 XINT (depth),
 	 __emacs_gtk_ctree_recurse_internal,
-	 LISP_TO_VOID (closure));
+	 STORE_LISP_IN_VOID (closure));
     }
 
   UNGCPRO;
