@@ -3,10 +3,10 @@
 
 This file is part of XEmacs.
 
-XEmacs is free software; you can redistribute it and/or modify it
+XEmacs is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
-Free Software Foundation; either version 2, or (at your option) any
-later version.
+Free Software Foundation, either version 3 of the License, or (at your
+option) any later version.
 
 XEmacs is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
@@ -14,9 +14,7 @@ FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 for more details.
 
 You should have received a copy of the GNU General Public License
-along with XEmacs; see the file COPYING.  If not, write to the Free
-Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
-02111-1307, USA. */
+along with XEmacs.  If not, see <http://www.gnu.org/licenses/>. */
 
 #include <config.h>
 #include "lisp.h"
@@ -70,7 +68,7 @@ urlify_filename (Ibyte *filename)
 {
   Ibyte *pseudo_url;
   
-  WIN32_TO_LOCAL_FILE_FORMAT (filename, filename);
+  INTERNAL_MSWIN_TO_LOCAL_FILE_FORMAT (filename, filename);
   pseudo_url = xnew_array (Ibyte, 5 + qxestrlen (filename) + 1);
   qxestrcpy_ascii (pseudo_url, "file:");
   qxestrcat (pseudo_url, filename);
@@ -90,12 +88,10 @@ urlify_filename (Ibyte *filename)
 Lisp_Object
 tstr_to_local_file_format (Extbyte *path)
 {
-  Ibyte *ttlff;
+  Ibyte *pathint = TSTR_TO_ITEXT (path);
+  INTERNAL_MSWIN_TO_LOCAL_FILE_FORMAT (pathint, pathint);
 
-  TSTR_TO_C_STRING (path, ttlff);
-  WIN32_TO_LOCAL_FILE_FORMAT (ttlff, ttlff);
-
-  return build_intstring (ttlff);
+  return build_istring (pathint);
 }
 
 /* Normalize filename by converting all path separators to the specified
@@ -271,7 +267,7 @@ mswindows_lisp_error_1 (int errnum, int no_recurse)
 	break;
     }
 
-  TSTR_TO_C_STRING (lpMsgBuf, inres);
+  inres = TSTR_TO_ITEXT (lpMsgBuf);
   len = qxestrlen (inres);
   /* Messages tend to end with a period and newline */
   if (len >= 3 && !qxestrcmp_ascii (inres + len - 3, ".\r\n"))
@@ -289,7 +285,7 @@ mswindows_lisp_error (int errnum)
 }
 
 void
-mswindows_output_last_error (char *frob)
+mswindows_output_last_error (const Ascbyte *frob)
 {
   int errval = GetLastError ();
   Lisp_Object errmess = mswindows_lisp_error (errval);
@@ -299,11 +295,10 @@ mswindows_output_last_error (char *frob)
 }
 
 DOESNT_RETURN
-mswindows_report_process_error (const char *string, Lisp_Object data,
+mswindows_report_process_error (const Ascbyte *reason, Lisp_Object data,
 				int errnum)
 {
-  report_file_type_error (Qprocess_error, mswindows_lisp_error (errnum),
-			  string, data);
+  signal_error_2 (Qprocess_error, reason, mswindows_lisp_error (errnum), data);
 }
 
 DEFUN ("mswindows-shell-execute", Fmswindows_shell_execute, 2, 4, 0, /*
@@ -345,14 +340,14 @@ otherwise it is an integer representing a ShowWindow flag:
     Extbyte *doc = NULL;
 
     if (STRINGP (operation))
-      LISP_STRING_TO_TSTR (operation, opext);
+      opext = LISP_STRING_TO_TSTR (operation);
     /* #### What about path names, which may be links? */
     if (STRINGP (parameters))
-      LISP_STRING_TO_TSTR (parameters, parmext);
+      parmext = LISP_STRING_TO_TSTR (parameters);
     if (STRINGP (current_dir))
-      LOCAL_FILE_FORMAT_TO_TSTR (current_dir, path);
+      LISP_LOCAL_FILE_FORMAT_TO_TSTR (current_dir, path);
     if (STRINGP (document))
-      LOCAL_FILE_FORMAT_MAYBE_URL_TO_TSTR (document, doc);
+      LISP_LOCAL_FILE_FORMAT_MAYBE_URL_TO_TSTR (document, doc);
 
     ret = (int) qxeShellExecute (NULL, opext, doc, parmext, path,
 				 (INTP (show_flag) ?
@@ -404,8 +399,8 @@ No expansion is performed, all conversion is done by the cygwin runtime.
     return path;
 
   /* Use mule and cygwin-safe APIs top get at file data. */
-  LOCAL_TO_WIN32_FILE_FORMAT (p, p);
-  return build_intstring (p);
+  LOCAL_FILE_FORMAT_TO_INTERNAL_MSWIN (p, p);
+  return build_istring (p);
 }
 #endif
 
@@ -418,7 +413,7 @@ struct read_link_hash
 static Ibyte *
 mswindows_read_link_1 (const Ibyte *fname)
 {
-#ifdef NO_CYGWIN_COM_SUPPORT
+#if defined (NO_CYGWIN_COM_SUPPORT) || !defined (HAVE_MS_WINDOWS)
   return NULL;
 #else
   Ibyte *retval = NULL;
@@ -435,7 +430,7 @@ mswindows_read_link_1 (const Ibyte *fname)
 
   if (!mswindows_read_link_hash)
     mswindows_read_link_hash = make_string_hash_table (1000);
-  C_STRING_TO_TSTR (fname, fnameext);
+  fnameext = ITEXT_TO_TSTR (fname);
 
   /* See if we can find a cached value. */
 
@@ -521,7 +516,7 @@ mswindows_read_link_1 (const Ibyte *fname)
 	    {
 	      Extbyte *fname_unicode;
 	      WIN32_FIND_DATAW wfd;
-	      LPWSTR resolved = alloca_array (WCHAR, PATH_MAX_EXTERNAL + 1);
+	      LPWSTR resolved = alloca_array (WCHAR, PATH_MAX_TCHAR + 1);
 
 	      /* Always Unicode.  Not obvious from the
 		 IPersistFile documentation, but look under
@@ -546,13 +541,13 @@ mswindows_read_link_1 (const Ibyte *fname)
 		  /* Another Cygwin prototype error,
 		     fixed in v2.2 of w32api */
 		  XECOMCALL4 (psl, GetPath, (LPSTR) resolved,
-			      PATH_MAX_EXTERNAL, &wfd, 0)
+			      PATH_MAX_TCHAR, &wfd, 0)
 #else
 		  XECOMCALL4 (psl, GetPath, resolved,
-			      PATH_MAX_EXTERNAL, &wfd, 0)
+			      PATH_MAX_TCHAR, &wfd, 0)
 #endif
 		  == S_OK)
-		TSTR_TO_C_STRING_MALLOC (resolved, retval);
+		retval = TSTR_TO_ITEXT_MALLOC (resolved);
 
 	      XECOMCALL0 (ppf, Release);
 	    }
@@ -579,20 +574,19 @@ mswindows_read_link_1 (const Ibyte *fname)
 	    {
 	      Extbyte *fname_unicode;
 	      WIN32_FIND_DATAA wfd;
-	      LPSTR resolved = alloca_array (CHAR, PATH_MAX_EXTERNAL + 1);
+	      LPSTR resolved = alloca_array (CHAR, PATH_MAX_TCHAR + 1);
 
 	      /* Always Unicode.  Not obvious from the
 		 IPersistFile documentation, but look under
 		 "Shell Link" for example code. */
-	      C_STRING_TO_EXTERNAL (fname, fname_unicode,
-				    Qmswindows_unicode);
+	      fname_unicode = ITEXT_TO_EXTERNAL (fname, Qmswindows_unicode);
 
 	      if (XECOMCALL2 (ppf, Load,
 			      (LPWSTR) fname_unicode,
 			      STGM_READ) == S_OK
 		  && XECOMCALL4 (psl, GetPath, resolved,
-				 PATH_MAX_EXTERNAL, &wfd, 0) == S_OK)
-		TSTR_TO_C_STRING_MALLOC (resolved, retval);
+				 PATH_MAX_TCHAR, &wfd, 0) == S_OK)
+		retval = TSTR_TO_ITEXT_MALLOC (resolved);
 
 	      XECOMCALL0 (ppf, Release);
 	    }
@@ -603,7 +597,7 @@ mswindows_read_link_1 (const Ibyte *fname)
 
   /* Cache newly found value */
   if (rlh->resolved)
-    xfree (rlh->resolved, Ibyte *);
+    xfree (rlh->resolved);
   rlh->resolved = retval ? qxestrdup (retval) : NULL;
 
   return retval;
