@@ -1,4 +1,5 @@
 ;; Copyright (C) 1999 Free Software Foundation, Inc.
+;; Copyright (C) 2010 Ben Wing.
 
 ;; Author: Hrvoje Niksic <hniksic@xemacs.org>
 ;; Maintainers: Hrvoje Niksic <hniksic@xemacs.org>,
@@ -8,20 +9,18 @@
 
 ;; This file is part of XEmacs.
 
-;; XEmacs is free software; you can redistribute it and/or modify it
-;; under the terms of the GNU General Public License as published by
-;; the Free Software Foundation; either version 2, or (at your option)
-;; any later version.
+;; XEmacs is free software: you can redistribute it and/or modify it
+;; under the terms of the GNU General Public License as published by the
+;; Free Software Foundation, either version 3 of the License, or (at your
+;; option) any later version.
 
-;; XEmacs is distributed in the hope that it will be useful, but
-;; WITHOUT ANY WARRANTY; without even the implied warranty of
-;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-;; General Public License for more details.
+;; XEmacs is distributed in the hope that it will be useful, but WITHOUT
+;; ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+;; FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
+;; for more details.
 
 ;; You should have received a copy of the GNU General Public License
-;; along with XEmacs; see the file COPYING.  If not, write to the Free
-;; Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
-;; 02111-1307, USA.
+;; along with XEmacs.  If not, see <http://www.gnu.org/licenses/>.
 
 ;;; Synched up with: Not in FSF.
 
@@ -141,6 +140,27 @@ This is a naive implementation in Lisp.  "
     (kill-buffer nil))
   (delete-file test-file-name))
 
+(let ((existing-file-name 
+       (make-temp-file (expand-file-name "k7lCS2Mg" (temp-directory))))
+      (nonexistent-file-name
+       (make-temp-name (temp-directory))))
+  (find-file existing-file-name)
+  (Assert (not (eq 'undecided
+                   (coding-system-type buffer-file-coding-system))))
+  (kill-buffer nil)
+  (dolist (coding-system '(utf-8 windows-1251 macintosh big5))
+    (when (find-coding-system coding-system)
+      (find-file existing-file-name coding-system)
+      (Assert (eq (find-coding-system coding-system)
+                  buffer-file-coding-system))
+      (kill-buffer nil)
+      (find-file nonexistent-file-name coding-system)
+      (Assert (eq (find-coding-system coding-system)
+                  buffer-file-coding-system))
+      (set-buffer-modified-p nil)
+      (kill-buffer nil)))
+  (delete-file existing-file-name))
+  
 ;;-----------------------------------------------------------------
 ;; Test string modification functions that modify the length of a char.
 ;;-----------------------------------------------------------------
@@ -353,6 +373,29 @@ This is a naive implementation in Lisp.  "
     (loop for k in '(0 1 58 59) do
       (Assert (equal (substring string (* 94 k) (* 94 (1+ k))) ascii-string))))
 
+  ;;---------------------------------------------------------------
+  ;; Test string character conversion
+  ;;---------------------------------------------------------------
+
+  ;; #### This should test all coding systems!
+
+  (let ((all-octets (let ((s (make-string 256 ?\000)))
+		      (loop for i from (1- (length s)) downto 0 do
+			(aset s i (int-char i)))
+		      s))
+	(escape-quoted-result (let ((schar '(27 155 142 143 14 15))
+				    (s (make-string 262 ?\000))
+				    (pos 0))
+				(loop for ord from 0 to 255 do
+				  (when (member ord schar)
+				    (aset s pos ?\033)
+				    (incf pos))
+				  (aset s pos (int-char ord))
+				  (incf pos))
+				s)))
+    (Assert (string= (encode-coding-string all-octets 'escape-quoted)
+		     escape-quoted-result)))
+
   ;;---------------------------------------------------------------
   ;; Test file-system character conversion (and, en passant, file ops)
   ;;---------------------------------------------------------------
@@ -364,7 +407,8 @@ This is a naive implementation in Lisp.  "
 	 (file-name-coding-system
 	  ;; 'iso-8859-X doesn't work on darwin (as of "Panther" 10.3), it
 	  ;; seems to know that file-name-coding-system is definitely utf-8
-	  (if (string-match "darwin" system-configuration)
+	  (if (or (string-match "darwin" system-configuration)
+		  (featurep 'cygwin-use-utf-8))
 	      'utf-8
 	    'iso-8859-2))
          ;; make-temp-name does stat(), which on OS X requires that you
@@ -406,13 +450,16 @@ This is a naive implementation in Lisp.  "
   ;;---------------------------------------------------------------
   (let* ((scaron (make-char 'latin-iso8859-2 57)))
     ;; Used to try #x0000, but you can't change ASCII or Latin-1
-    (loop for code in '(#x0100 #x2222 #x4444 #xffff) do
+    (loop
+      for code in '(#x0100 #x2222 #x4444 #xffff)
+      with initial-unicode = (char-to-unicode scaron)
+      do
       (progn
 	(set-unicode-conversion scaron code)
 	(Assert (eq code (char-to-unicode scaron)))
-	(Assert (eq scaron (unicode-to-char code '(latin-iso8859-2))))))
-  
-    (Check-Error wrong-type-argument (set-unicode-conversion scaron -10000)))
+	(Assert (eq scaron (unicode-to-char code '(latin-iso8859-2)))))
+      finally (set-unicode-conversion scaron initial-unicode))
+    (Check-Error args-out-of-range (set-unicode-conversion scaron -10000)))
 
   (dolist (utf-8-char 
 	   '("\xc6\x92"		  ;; U+0192 LATIN SMALL LETTER F WITH HOOK
@@ -509,43 +556,40 @@ This is a naive implementation in Lisp.  "
 				      (loop for i from ?\x00 to ?\xFF
 					collect i))
     do
-    (when (and (coding-system-get coding-system '8-bit-fixed)
-	       ;; Don't check the coding systems with autodetect, they are
-	       ;; not round-trip compatible for the possible line-ending
-	       ;; characters.
-	       (string-match #r"-\(unix\|dos\|mac\)$"
-			     (symbol-name coding-system)))
+    (when (and (eq 'fixed-width (coding-system-type coding-system))
+	       ;; Don't check the coding systems with odd line endings
+	       ;; (maybe we should):
+	       (eq 'lf (coding-system-eol-type coding-system)))
       ;; These coding systems are round-trip compatible with themselves.
       (Assert (equal (encode-coding-string 
 		      (decode-coding-string all-possible-octets
 					    coding-system)
 		      coding-system)
-		     all-possible-octets))))
+		     all-possible-octets)
+              (format "checking %s is transparent" coding-system))))
 
   ;;---------------------------------------------------------------
   ;; Test charset-in-* functions
   ;;---------------------------------------------------------------
   (with-temp-buffer
     (insert-file-contents (locate-data-file "HELLO"))
-    (Assert (equal 
-             ;; The sort is to make the algorithm of charsets-in-region
-             ;; irrelevant.
-             (sort (charsets-in-region (point-min) (point-max))
-                   #'string<)
-             '(ascii chinese-big5-1 chinese-gb2312 cyrillic-iso8859-5
-               ethiopic greek-iso8859-7 hebrew-iso8859-8 japanese-jisx0208
-               japanese-jisx0212 jit-ucs-charset-0 katakana-jisx0201
-               korean-ksc5601 latin-iso8859-1 latin-iso8859-2 thai-xtis
-               vietnamese-viscii-lower)))
-    (Assert (equal 
-             (sort (charsets-in-string (buffer-substring (point-min)
-							 (point-max)))
-                   #'string<)
-             '(ascii chinese-big5-1 chinese-gb2312 cyrillic-iso8859-5
-               ethiopic greek-iso8859-7 hebrew-iso8859-8 japanese-jisx0208
-               japanese-jisx0212 jit-ucs-charset-0 katakana-jisx0201
-               korean-ksc5601 latin-iso8859-1 latin-iso8859-2 thai-xtis
-               vietnamese-viscii-lower))))
+    (let ((sorted-charsets-in-HELLO
+	   '(arabic-iso8859-6 ascii chinese-big5-1 chinese-gb2312
+	     cyrillic-iso8859-5 ethiopic greek-iso8859-7
+	     hebrew-iso8859-8 japanese-jisx0208 japanese-jisx0212
+	     katakana-jisx0201 korean-ksc5601 latin-iso8859-1
+	     latin-iso8859-2 vietnamese-viscii-lower)))
+      (Assert (equal 
+       ;; The sort is to make the algorithm of charsets-in-region
+       ;; irrelevant.
+       (sort (charsets-in-region (point-min) (point-max))
+	     #'string<)
+       sorted-charsets-in-HELLO))
+      (Assert (equal 
+       (sort (charsets-in-string (buffer-substring (point-min)
+						   (point-max)))
+	     #'string<)
+       sorted-charsets-in-HELLO))))
 
   ;;---------------------------------------------------------------
   ;; Language environments, and whether the specified values are sane.
@@ -554,6 +598,7 @@ This is a naive implementation in Lisp.  "
     for language in (mapcar #'car language-info-alist)
     with language-input-method = nil
     with native-coding-system = nil
+    with original-language-environment = current-language-environment
     do
     ;; s-l-e can call #'require, which says "Loading ..."
     (Silence-Message (set-language-environment language))
@@ -592,7 +637,8 @@ This is a naive implementation in Lisp.  "
       ;; We don't have the appropriate POSIX locales to test with a
       ;; native-coding-system that is a function.
       (unless (functionp coding-system)
-	(Assert (coding-system-p (find-coding-system coding-system))))))
+	(Assert (coding-system-p (find-coding-system coding-system)))))
+    finally (set-language-environment original-language-environment))
 
   (with-temp-buffer
     (flet
@@ -734,4 +780,28 @@ This is a naive implementation in Lisp.  "
      do (unless (get-range-table i exceptions)
           (read (format (if (> i #xFFFF) #r"?\U%08X" #r"?\u%04X") i)))
      finally return t))
-  )
+  (loop
+    for i from #x00 to #xff
+    do (Assert
+        (= 1 (length (decode-coding-string (format "%c" i) 'utf-8-unix)))
+        (format 
+         "checking Unicode coding systems behave well with short input, %02X"
+         i)))
+
+  ;;---------------------------------------------------------------
+  ;; Process tests
+  ;; #### Should do network too.
+  ;;---------------------------------------------------------------
+  (Skip-Test-Unless (and (file-exists-p "/dev/null")
+			 (fboundp 'executable-find)
+			 (executable-find "cat"))
+      "cat(1) or /dev/null missing"
+      "Test that default-process-coding-system can be nil."
+    (with-temp-buffer
+      (Assert (let (default-process-coding-system)
+		(shell-command "cat </dev/null >/dev/null")
+		t))))
+
+  ) ; end of tests that require MULE built in.
+
+;;; end of mule-tests.el
