@@ -7,20 +7,18 @@
 
 ;; This file is part of XEmacs.
 
-;; XEmacs is free software; you can redistribute it and/or modify it
-;; under the terms of the GNU General Public License as published by
-;; the Free Software Foundation; either version 2, or (at your option)
-;; any later version.
+;; XEmacs is free software: you can redistribute it and/or modify it
+;; under the terms of the GNU General Public License as published by the
+;; Free Software Foundation, either version 3 of the License, or (at your
+;; option) any later version.
 
-;; XEmacs is distributed in the hope that it will be useful, but
-;; WITHOUT ANY WARRANTY; without even the implied warranty of
-;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-;; General Public License for more details.
+;; XEmacs is distributed in the hope that it will be useful, but WITHOUT
+;; ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+;; FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
+;; for more details.
 
 ;; You should have received a copy of the GNU General Public License
-;; along with XEmacs; see the file COPYING.  If not, write to the Free
-;; Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
-;; 02111-1307, USA.
+;; along with XEmacs.  If not, see <http://www.gnu.org/licenses/>.
 
 ;;; Synched up with: Not in FSF.
 
@@ -192,8 +190,9 @@
     (Assert (backward-up-list-moves-point-from-to 20 3))
     (Known-Bug-Expect-Failure
      (Assert (backward-up-list-moves-point-from-to 22 3)))
-    (Known-Bug-Expect-Failure
-     (Assert (backward-up-list-moves-point-from-to 23 3)))
+    (Known-Bug-Expect-Error scan-error
+     (Assert (backward-up-list-moves-point-from-to 23 3))
+     )
     (Assert (backward-up-list-moves-point-from-to 24 3))
     ;; This is maybe a little tricky, since we don't expect the position
     ;; check to happen -- so use an illegal expected position
